/******************************************************************************
*                                                                             *
*  File: AutoConfig_i.h                                                       *
*                                                                             *
*  Description: Implementation of the Automatic Configuration Service.        *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.4 $
   $Log: AutoConfig_i.h,v $
   Revision 1.4  2000/10/09 20:26:19  f-kon
   New namespace for CORBA modules combined with Tomo's changes to support Windows.

   Revision 1.3  2000/09/30 18:24:33  yamane
   Patches for Windows

   Revision 1.2  2000/01/18 22:08:40  f-kon
   various optimizations and bug fixes

   Revision 1.1.1.3  1999/10/15 22:10:25  yamane
   Clean up for native exception support.

   Revision 1.1.1.2  1999/10/15 21:48:15  yamane
   Clean up for native exception support

   Revision 1.1.1.1  1999/10/12 00:42:46  f-kon
   Version compatible with TAO 1.0

   Revision 1.2  1999/08/11 13:57:28  f-kon
   improved version with most bugs fixed

   Revision 1.1  1999/08/03 22:47:48  f-kon
   Initial version of the Automatic Configuration Systems. It is not yet fully debugged.

******************************************************************************/

// teste
// Inclui 'TwoK2::' na frente de varias linhas que chamavam AutoConfig::Excessao,
// pois tava dando erro na compilacao

#ifndef AUTOCONFIG_I_H
#define AUTOCONFIG_I_H

#include "AutoConfigS.h"
#include "PrereqParser.h"

#include "AutoConfig_export.h"

class AUTOCONF_2K_Export AutoConfig_i : public  POA_TwoK2::AutoConfig
{
public:
  AutoConfig_i (); 
  void init (CORBA_Environment& ACE_TRY_ENV);

protected:  
  virtual TwoK::Configuration::ComponentConfigurator_ptr
             load_component (DynamicConfigurator_ptr implRep,
                             const TwoK2::CompSpec & compSpecification,
                             TwoK::Configuration::CCFactory_ptr factory,
                             CORBA::Environment &ACE_TRY_ENV = 
                             CORBA::Environment::default_environment ())
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      TwoK2::AutoConfig::NotFound,
      TwoK2::AutoConfig::LoadFailure
    ));

  virtual TwoK::Configuration::ComponentConfigurator_ptr 
         load_component_aux (const TwoK2::Name & compSpecification,
                             TwoK::Configuration::CCFactory_ptr factory,
                             CORBA::Environment &ACE_TRY_ENV = 
                             CORBA::Environment::default_environment ())
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      TwoK2::AutoConfig::NotFound,
      TwoK2::AutoConfig::LoadFailure
    ));

  virtual TwoK::Configuration::ComponentConfigurator_ptr 
         load_prereq (const TwoK2::Name & prereqSpecification,
                      TwoK::Configuration::CCFactory_ptr factory,
                      CORBA::Environment &ACE_TRY_ENV = 
                      CORBA::Environment::default_environment ())
    ACE_THROW_SPEC ((CORBA::SystemException,
                     TwoK2::AutoConfig::NotFound,
                     TwoK2::AutoConfig::LoadFailure
                     ));
  
  virtual void setParser (const char * impName,
                          const char * params,
                          CORBA::Environment &ACE_TRY_ENV = 
                          CORBA::Environment::default_environment ())
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      TwoK2::AutoConfig::NotFound,
      TwoK2::AutoConfig::LoadFailure
    ));
  

  virtual void setResolver (const char * impName,
                            const char * params,
                            CORBA::Environment &ACE_TRY_ENV = 
                            CORBA::Environment::default_environment ())
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      TwoK2::AutoConfig::NotFound,
      TwoK2::AutoConfig::LoadFailure
    ));
  
  virtual DynamicConfigurator_ptr getRepository (CORBA::Environment &ACE_TRY_ENV = 
                                                 CORBA::Environment::default_environment ())
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      TwoK2::AutoConfig::NotFound
    ));
  
private:
  DynamicConfigurator_var dynamicConfigurator_;
  PrereqParser *parser_;
  PrereqResolver *resolver_;
};

#endif
