/******************************************************************************
*                                                                             *
*  File: PrereqParser.h                                                       *
*                                                                             *
*  Description: Abstract class definition for prerequisite parsers.           *
*                                                                             *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.2 $
   $Log: PrereqParser.h,v $
   Revision 1.2  2000/10/09 20:26:17  f-kon
   New namespace for CORBA modules combined with Tomo's changes to support Windows.

   Revision 1.1.1.3  1999/10/15 22:10:32  yamane
   Clean up for native exception support.

   Revision 1.1.1.2  1999/10/15 21:49:48  yamane
   Clean up for native exception support

   Revision 1.1.1.1  1999/10/12 00:42:46  f-kon
   Version compatible with TAO 1.0

   Revision 1.2  1999/08/11 13:57:31  f-kon
   improved version with most bugs fixed

   Revision 1.1  1999/08/03 22:47:49  f-kon
   Initial version of the Automatic Configuration Systems. It is not yet fully debugged.

******************************************************************************/

#ifndef PREREQ_PARSER_H
#define PREREQ_PARSER_H

#include "AutoConfigExceptionsC.h"
#include "PrereqResolver.h"

class PrereqParser
{
public:
  inline PrereqParser () : resolver_(0) {};

  virtual void init (CORBA::Object_ptr repository, PrereqResolver *resolver,
                     CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC((NotFound, InitializationFailure));
  // Look at the CORBA ImplRep spec to see if we can pass a reference of that
  // type here instead of using a generic Object type.

  // Parses the prerequisite specification given by 'prereqs' on behalf of 'client'.
  // Subclasses, such as SPDFParser, load new components to resolve all the prerequisites and 
  // attach them to the client component.

  virtual void parse (PrereqSpec prereqs,
                      TwoK::Configuration::ComponentConfigurator_ptr client,
                     CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC((NotFound, ParseError)) = 0;

  static const char *const PREREQ_SUFFIX; // ".prq"

protected: 
  PrereqResolver *resolver_;
};


#endif
