/******************************************************************************
*                                                                             *
*  File: PrereqResolver.h                                                     *
*                                                                             *
*  Description: Abstract class definition for prerequisite resolvers.         *
*                                                                             *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.2 $
   $Log: PrereqResolver.h,v $
   Revision 1.2  2000/10/09 20:26:16  f-kon
   New namespace for CORBA modules combined with Tomo's changes to support Windows.

   Revision 1.1.1.3  1999/10/15 22:10:33  yamane
   Clean up for native exception support.

   Revision 1.1.1.2  1999/10/15 21:58:16  yamane
   Clean up for native exception support

   Revision 1.1.1.1  1999/10/12 00:42:46  f-kon
   Version compatible with TAO 1.0

   Revision 1.2  1999/08/11 13:57:32  f-kon
   improved version with most bugs fixed

   Revision 1.1  1999/08/03 22:47:49  f-kon
   Initial version of the Automatic Configuration Systems. It is not yet fully debugged.

******************************************************************************/

#ifndef PREREQ_RESOLVER_H
#define PREREQ_RESOLVER_H

#include "2k/ComponentConfigurator/ConfigurationC.h"
//#include "2k/Environment/envC.h"
#include "AutoConfigExceptionsC.h"

#include "AutoConfigC.h"

typedef const TwoK2::CompSpec PrereqSpec;

//typedef Object_ptr ImplRep;

class PrereqResolver
{
 public: 
  virtual void init (TwoK2::AutoConfig_ptr autoConfig,
                     DynamicConfigurator_ptr impRep, 
                     DynamicConfigurator_ptr localDynConf,
                     TwoK::Configuration::CCFactory_ptr configFactory,
                     CORBA::Environment& ACE_TRY_ENV) 
	ACE_THROW_SPEC((InitializationFailure)) = 0;

  virtual TwoK::Configuration::ComponentConfigurator_ptr
  resolvePrereq (TwoK::Configuration::ComponentConfigurator_ptr client,
                 const char *hookName,
                 PrereqSpec &component,
                 CORBA::Environment &ACE_TRY_ENV) ACE_THROW_SPEC((InitializationFailure,
                                                         NotFound,
                                                         Unsupported)) = 0;
};

#endif
