/******************************************************************************
*                                                                             *
*  File: SPDFParser.h                                                         *
*                                                                             *
*  Description: A prerequisite parser for the Simple Prerequisite Description *
*               Format (SPDF).                                                *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.2 $
   $Log: SPDFParser.h,v $
   Revision 1.2  2000/10/09 20:26:13  f-kon
   New namespace for CORBA modules combined with Tomo's changes to support Windows.

   Revision 1.1.1.4  2000/09/30 18:26:19  yamane
   Patches for Windows

   Revision 1.1.1.3  1999/10/15 22:10:35  yamane
   Clean up for native exception support.

   Revision 1.1.1.2  1999/10/15 21:58:37  yamane
   Clean up for native exception support

   Revision 1.1.1.1  1999/10/12 00:42:46  f-kon
   Version compatible with TAO 1.0

   Revision 1.2  1999/08/11 13:57:34  f-kon
   improved version with most bugs fixed

   Revision 1.1  1999/08/03 22:47:49  f-kon
   Initial version of the Automatic Configuration Systems. It is not yet fully debugged.

******************************************************************************/

#ifndef SPDF_PARSER_H
#define SPDF_PARSER_H

#include "PrereqParser.h"
#include "dynamicTAO/src/DynamicConfigurator/dynamicConfiguratorC.h"

class SPDFParser : public PrereqParser
{
public:
  void init (CORBA::Object_ptr repository, PrereqResolver *resolver,
             CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC((NotFound, InitializationFailure));
  void parse (PrereqSpec prereqs,
              TwoK::Configuration::ComponentConfigurator_ptr client,
             CORBA::Environment& ACE_TRY_ENV) 
    ACE_THROW_SPEC((NotFound, ParseError));
  
private: 
  void parse (char *buf, long length,
              TwoK::Configuration::ComponentConfigurator_ptr client,
             CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC((ParseError));

  char *locateLine (char *start, char *end, const char *prefix, char **nextLine);
  
  DynamicConfigurator_var repository_;

  // string constants for the SPDF.
  static const char *const HARDWARE_REQUIREMENTS_HEADER;
  static const char *const SOFTWARE_REQUIREMENTS_HEADER;
  static const char *const OPTIONAL_REQUIREMENTS;
};


#endif
