/******************************************************************************
*                                                                             *
*  File: SimpleResolver.h                                                     *
*                                                                             *
*  Description: A simple prerequisite resolver (with no caching).             *
*               Resolves prerequisites by loading components from a given     *
*               implementation repository accessible through a                *
*               DynamicConfigurator CORBA object.                             *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.3 $
   $Log: SimpleResolver.h,v $
   Revision 1.3  2000/10/09 20:26:11  f-kon
   New namespace for CORBA modules combined with Tomo's changes to support Windows.

   Revision 1.2  2000/01/18 22:08:44  f-kon
   various optimizations and bug fixes

   Revision 1.1.1.3  1999/10/15 22:10:38  yamane
   Clean up for native exception support.

   Revision 1.1.1.2  1999/10/15 21:58:56  yamane
   Clean up for native exception support

   Revision 1.1.1.1  1999/10/12 00:42:46  f-kon
   Version compatible with TAO 1.0

   Revision 1.2  1999/08/11 13:57:36  f-kon
   improved version with most bugs fixed

   Revision 1.1  1999/08/03 22:47:49  f-kon
   Initial version of the Automatic Configuration Systems. It is not yet fully debugged.

******************************************************************************/

#ifndef SIMPLE_RESOLVER_H
#define SIMPLE_RESOLVER_H

#include "tao/TAO.h"
#include "PrereqResolver.h"

#include "dynamicTAO/src/DynamicConfigurator/dynamicConfiguratorC.h"
#include "2k/ComponentConfigurator/ConfigurationC.h"
#include "AutoConfig_i.h"

class SimpleResolver : public PrereqResolver
{
 public:
  void init (TwoK2::AutoConfig_ptr autoConfig,
             DynamicConfigurator_ptr impRep, 
             DynamicConfigurator_ptr localDynConf,
             TwoK::Configuration::CCFactory_ptr configFactory,
             CORBA::Environment &ACE_TRY_ENV) 
    ACE_THROW_SPEC((InitializationFailure));

  TwoK::Configuration::ComponentConfigurator_ptr
  resolvePrereq (TwoK::Configuration::ComponentConfigurator_ptr client,
                 const char *hookName,
                 PrereqSpec &component,
                 CORBA::Environment &ACE_TRY_ENV) 
    ACE_THROW_SPEC((InitializationFailure,
                    NotFound,
                    Unsupported,
                    LoadFailure));
  
 protected: 
  TwoK::Configuration::ComponentConfigurator_ptr
  resolvePrereq (TwoK::Configuration::ComponentConfigurator_ptr client,
                 const char *hookName, 
                 TwoK2::Name compSpec,
                 CORBA::Environment &ACE_TRY_ENV) 
    ACE_THROW_SPEC((NotFound, LoadFailure));

  DynamicConfigurator_var localDynConf_;
  DynamicConfigurator_var remoteDynConf_;
  TwoK::Configuration::CCFactory_var configFactory_;
  TwoK2::AutoConfig_var autoConfig_;
};


#endif
