/******************************************************************************
*                                                                             *
*  File: client_i.cpp                                                         *
*                                                                             *
*  Description: A test client for the AutomaticConfigurationServer.           *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
 Revision Control Information
 $Revision: 1.3 $
 $Log: client_i.cpp,v $
 Revision 1.3  2000/10/09 20:26:10  f-kon
 New namespace for CORBA modules combined with Tomo's changes to support Windows.

 Revision 1.2  2000/01/12 21:56:16  f-kon
 *** empty log message ***

 Revision 1.1.1.3  1999/10/15 22:10:40  yamane
 Clean up for native exception support.

 Revision 1.1.1.2  1999/10/15 22:09:26  yamane
 Clean up for native exception support

 Revision 1.1.1.1  1999/10/12 00:42:46  f-kon
 Version compatible with TAO 1.0

 Revision 1.3  1999/08/11 13:57:37  f-kon
 improved version with most bugs fixed

 Revision 1.2  1999/08/10 15:49:09  f-kon
 little bug fixes

 Revision 1.1  1999/08/03 22:47:49  f-kon
 Initial version of the Automatic Configuration Systems. It is not yet fully debugged.

******************************************************************************/

#include "ace/Env_Value_T.h"
#include "ace/Read_Buffer.h"
#include "client_i.h"
#include "2k/ComponentConfigurator/ConfigurationC.h"
#include "tao/Timeprobe.h"

#if defined (ACE_ENABLE_TIMEPROBES)

static const char *CC_Client_Timeprobe_Description[] =
{
  "AC_Client::op - start",
  "AC_Client::op - end",
};

enum
{
  AC_CLIENT_OP_START = 10000,
  AC_CLIENT_OP_END,
};

#endif /* ACE_ENABLE_TIMEPROBES */

// Setup Timeprobes
ACE_TIMEPROBE_EVENT_DESCRIPTIONS (AC_Client_Timeprobe_Description,
                                  AC_CLIENT_OP_START)

// Constructor.
AC_Client::AC_Client (void)
  : ac_ior_ (0),
    shutdown_ (0)
{
}

int
AC_Client::read_ior (char *filename)
{
  ACE_HANDLE f_handle = ACE_OS::open (filename, 0);

  if (f_handle == ACE_INVALID_HANDLE)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "Unable to open %s for writing: %p\n",
                       filename),
                      -1);

  ACE_Read_Buffer ior_buffer (f_handle);

  ac_ior_ = ior_buffer.read ();

  ACE_OS::close (f_handle);

  if (ac_ior_ == 0)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "Unable to allocate memory to read ior: %p\n"),
                      -1);
   return 0;
}// read_ior


////////////////////////////////////////////////////////////////////////////////
// Parses the command line arguments and returns an error status.
////////////////////////////////////////////////////////////////////////////////
int
AC_Client::parse_args (int argc, char *argv[])
{
  ACE_Get_Opt get_opts (argc, argv, "dn:f:k:xs");
  int c;
  int result;

  while ((c = get_opts ()) != -1)
    switch (c)
      {
      case 'd':  // debug flag
        TAO_debug_level++;
        break;
       case 'f': // read the IOR from the file.
        result = read_ior (get_opts.optarg);
        if (result < 0)
          ACE_ERROR_RETURN ((LM_ERROR,
                             "Unable to read ior from %s : %p\n",
                             get_opts.optarg),
                            -1);
            break;
      case 'x':
        shutdown_ = 1;
        break;
      case '?':
      default:
        ACE_ERROR_RETURN ((LM_ERROR,
                           "usage:  %s"
                           " [-d]"
                           " [-f ac_ior_file]"
                           " [-x]"
                           "\n",
                           argv[0]),
                          -1);
      }

  // Indicates successful parsing of command line.
  return 0;
}// parse_args

void
AC_Client::test (void)
{
  TwoK2::CompSpec compSpec;
  TwoK2::Name specName;

  ACE_TRY_NEW_ENV
    {
      specName.length (2);
      specName[0] = CORBA::string_dup ("Hello");
      specName[1] = CORBA::string_dup ("LoadableHello");
        
      compSpec.comp_name (specName);
      DynamicConfigurator_var myDynConf =
        DynamicConfigurator::_narrow (orb_->resolve_initial_references ("DynamicConfigurator"));

      if (CORBA::is_nil (myDynConf.in ()))
          {
            ACE_ERROR ((LM_ERROR, "Could not locate my DynamicConfigurator\n"));
            return;
          }

      TwoK::Configuration::ComponentConfigurator_var cc = 
        autoConfig_->load_component (myDynConf.in (), // implementation repository
                                     compSpec,
                                     factory_ptr_,
                                     ACE_TRY_ENV);
                                     
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_ANY_EXCEPTION._tao_print_exception ("in AC_Client::test ()");
    }
  ACE_ENDTRY;
}// test ()


void
AC_Client::print_stats (const char *call_name,
                        ACE_Profile_Timer::ACE_Elapsed_Time &elapsed_time)
{
  ACE_DEBUG ((LM_INFO, "\n%s:\n", call_name));

  // elapsed_time.real_time is in units of seconds.
  double calls_per_sec = 1.0 / elapsed_time.real_time;
  
  ACE_DEBUG ((LM_INFO,
              "\treal_time\t= %0.06f ms, \n\t"
              "user_time\t= %0.06f ms, \n\t"
              "system_time\t= %0.06f ms\n"
              "\t%0.00f calls/second\n",
              elapsed_time.real_time < 0.0 ? 0.0
              : elapsed_time.real_time * ACE_ONE_SECOND_IN_MSECS,
              elapsed_time.user_time < 0.0 ? 0.0
              : elapsed_time.user_time * ACE_ONE_SECOND_IN_MSECS,
              elapsed_time.system_time < 0.0 ? 0.0
              : elapsed_time.system_time * ACE_ONE_SECOND_IN_MSECS,
              calls_per_sec < 0.0 ? 0.0 : calls_per_sec));

}// print_stats



// Execute client example code.
void
AC_Client::run ()
{
  ACE_Profile_Timer timer;
  ACE_Profile_Timer::ACE_Elapsed_Time elapsed_time;

  // We start an ACE_Profile_Timer here...
  timer.start ();

  test ();

  // stop the timer.
  timer.stop ();

  timer.elapsed_time (elapsed_time);
  // compute call average call time.
  print_stats ("ac test call", elapsed_time);

  if (shutdown_)
    {
      // tell the AutoConfigServer to shutdown
    }
  
  ACE_OS::printf("End client execution.\n");

}// run ()



AC_Client::~AC_Client (void)
{
  // Free resources

  if (ac_ior_ != 0)
    ACE_OS::free (ac_ior_);

  ACE_OS::printf("Destructed Client Object.\n");

}// destructor


int
AC_Client::init (int argc, char **argv)
{
  ACE_TRY_NEW_ENV
    {
      // Retrieve the ORB.
//       orb_ = CORBA::ORB_init (argc, argv,
//                               "AutoConfigClientTest",
//                               ACE_TRY_ENV);
//       ACE_TRY_CHECK;

      orb_manager_.init (argc,argv, "AutoConfigClientTest", ACE_TRY_ENV);
      ACE_TRY_CHECK;
      
      orb_ = orb_manager_.orb();
      
      // Parse command line and verify parameters.
      if (parse_args (argc, argv) == -1)
        return -1;

      // Call the init of <TAO_ORB_Manager> to initialize the ORB and
      // create a child POA under the root POA.
      if (this->orb_manager_.init_child_poa (argc, argv, "child_poa", ACE_TRY_ENV) == -1)
        ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "AutoConfigClient: init_child_poa"), -1);
      
       ACE_TRY_CHECK;

      factory_ptr_ = TwoK::Configuration::CCFactory::_nil ();
      if (1) 
        {
          PortableServer::ObjectId_var id = PortableServer::string_to_ObjectId ("factory");
          orb_manager_.child_poa ()->activate_object_with_id (id.in (), 
                                                              &this->factory_impl_, 
                                                              ACE_TRY_ENV);
          ACE_TRY_CHECK;

          CORBA::Object_var obj = orb_manager_.child_poa ()->id_to_reference (id.in (),
                                                                              ACE_TRY_ENV);
          ACE_TRY_CHECK;

          // The following call is required for clients,
          //  which are not going to enter "orb->run" loop, yet may want to
          //  service incoming requests.
          orb_manager_.activate_poa_manager (ACE_TRY_ENV);
          ACE_TRY_CHECK;
         
          factory_ptr_ = TwoK::Configuration::CCFactory::_narrow (obj, ACE_TRY_ENV);
          ACE_TRY_CHECK;
        }
      
      if (ac_ior_ == 0)
        ACE_ERROR_RETURN ((LM_ERROR,
                           "%s: no server IOR specified\n",
                           argv[0]),
                          -1);
      
      
      CORBA::Object_var ac_server =
        orb_->string_to_object (ac_ior_,
                                      ACE_TRY_ENV);
      ACE_TRY_CHECK;
      
      autoConfig_ = TwoK2::AutoConfig::_narrow (ac_server.in(), ACE_TRY_ENV);
      ACE_TRY_CHECK;
      
      if (CORBA::is_nil (autoConfig_.in ()))
        ACE_ERROR_RETURN ((LM_ERROR,
                           "invalid AutoConfigurationServer IOR <%s>\n",
                           ac_ior_),
                          -1);

      ACE_DEBUG ((LM_DEBUG, "Located AutomaticConfigurationServer.\n"));

    }
  ACE_CATCHANY
    {
      ACE_ANY_EXCEPTION._tao_print_exception ("AC_Client::init");
      return -1;
    }
  ACE_ENDTRY;

  return 0;

}// init ()

void
AC_Client::fini ()
{
    // Shutdown local client ORB.
    orb_->shutdown ();
}


#if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)
template class ACE_Env_Value<unsigned long>;
template class ACE_Env_Value<unsigned short>;
#elif defined (ACE_HAS_TEMPLATE_INSTANTIATION_PRAGMA)
#pragma instantiate ACE_Env_Value<unsigned long>
#pragma instantiate ACE_Env_Value<unsigned short>
#endif /* ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION */
