/******************************************************************************
*                                                                             *
*  File: Configuration_i.h                                                    *
*                                                                             *
*  Description: CORBA implementation of the ComponentConfigurator             *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.5 $
   $Log: Configuration_i.h,v $
   Revision 1.5  2000/10/09 21:02:35  f-kon
   Changes to use new TwoK:: namespace for 2K services integrated with Tomo's
   changes for Windows compilation.

   Revision 1.4  2000/09/30 18:28:56  yamane
   Patches for Windows

   Revision 1.3  2000/02/16 17:08:47  f-kon
   *** empty log message ***

   Revision 1.2  2000/02/02 21:04:04  f-kon
   minor updates

   Revision 1.1.1.3  1999/10/17 18:34:07  yamane
   Cleanup for native exception and linux support

   Revision 1.1.1.2  1999/10/15 21:22:46  yamane
   Clean up for native exception support

   Revision 1.1.1.1  1999/10/12 00:42:46  f-kon
   Version compatible with TAO 1.0

******************************************************************************/

// teste
// Inclui 'TwoK::' na frente de varias linhas que chamavam
// Configuration::ComponentConfigurator::Excessao
// pois estava dando erro na compilacao


#ifndef COMPONENT_CONFIGURATOR_I_H
#define COMPONENT_CONFIGURATOR_I_H

#include "ConfigurationS.h"
#include "Configuration_export.h"

// The implementation of the ComponentConfigurator interface
class CORBACONF_2K_Export ComponentConfigurator_i 
  : public POA_TwoK::Configuration::ComponentConfigurator
{
 public:
  ComponentConfigurator_i (PortableServer::POA_ptr poa);
  ComponentConfigurator_i (PortableServer::POA_ptr poa,
                           char *name, 
                           char *info,
                           CORBA::Object_ptr implementation);
  ~ComponentConfigurator_i (void);
  
  virtual void destroy (CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC ((CORBA::SystemException));
  
  virtual void addHook (const char * hookName,
                        CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC ((CORBA::SystemException,TwoK::Configuration::ComponentConfigurator::ElementExists));
  
  virtual void deleteHook (const char * hookName,
                           CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC ((
                     CORBA::SystemException,
                     TwoK::Configuration::ComponentConfigurator::NotFound
                     ));
  
  virtual void hook (const char * hookName, TwoK::Configuration::ComponentConfigurator_ptr cc,
                     CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC ((
                     CORBA::SystemException,
                     TwoK::Configuration::ComponentConfigurator::NotFound
                     ));
  
  virtual void unhook (const char * hookName,
                       CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC ((
                     CORBA::SystemException,
                     TwoK::Configuration::ComponentConfigurator::HookVacant,
                     TwoK::Configuration::ComponentConfigurator::NotFound
                     ));
  
  virtual void registerClient (TwoK::Configuration::ComponentConfigurator_ptr client,
                               const char * hookNameInClient,
                               CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC ((
                     CORBA::SystemException,
                     TwoK::Configuration::ComponentConfigurator::ElementExists
                     ));
  
  virtual void unregisterClient (TwoK::Configuration::ComponentConfigurator_ptr client,
                                 const char * hookNameInClient,
                                 CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC ((
                     CORBA::SystemException,
                     TwoK::Configuration::ComponentConfigurator::NotFound
                     ));
  
  virtual void eventFromHookedComponent (TwoK::Configuration::ComponentConfigurator_ptr hookedComponent,
                                         const TwoK::Configuration::Event & e,
                                         CORBA::UShort timeToLive,
                                         CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC ((
                     CORBA::SystemException
                     ));
  
  virtual void eventFromClient (TwoK::Configuration::ComponentConfigurator_ptr client,
                                const TwoK::Configuration::Event & e,
                                CORBA::UShort timeToLive,
                                CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC ((
                     CORBA::SystemException
                     ));

  virtual void eventFromHookedComponent_ow (TwoK::Configuration::ComponentConfigurator_ptr hookedComponent,
                                            const TwoK::Configuration::Event & e,
                                            CORBA::UShort timeToLive,
                                            CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
                     CORBA::SystemException
                     ));

  virtual void eventFromClient_ow (TwoK::Configuration::ComponentConfigurator_ptr client,
                                   const TwoK::Configuration::Event & e,
                                   CORBA::UShort timeToLive,
                                   CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
                     CORBA::SystemException
                     ));

  virtual char * name (CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC ((
                     CORBA::SystemException
                     ));
  
  virtual char * info (CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC ((
                     CORBA::SystemException
                     ));
  
  virtual TwoK::Configuration::DependencyList * 
  listHooks (CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC ((
                     CORBA::SystemException
                     ));
  
  virtual TwoK::Configuration::ComponentConfigurator_ptr 
  getHookedComponent (const char * hookName,
                      CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC ((
                     CORBA::SystemException
                     ));
  
  virtual CORBA::UShort numberOfClients (CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC ((
                     CORBA::SystemException
                     ));
  
  virtual TwoK::Configuration::DependencyList * 
  listClients (CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC ((
                     CORBA::SystemException
                     ));
  
  virtual CORBA::Object_ptr implementation (CORBA::Environment& ACE_TRY_ENV)
    ACE_THROW_SPEC ((
                     CORBA::SystemException
                     ));
 
 protected:
  const char *eventToString (TwoK::Configuration::EventType type);
  
  TwoK::Configuration::DependencySpecification *findHookedComponent 
                            (TwoK::Configuration::ComponentConfigurator_ptr cc);

  TwoK::Configuration::DependencySpecification *findHook (const char *hookName);

   /* A list containing the hooks for this component. */
  TwoK::Configuration::DependencyList hookList_;
  
  /* A list containing the clients for this component.  */
  TwoK::Configuration::DependencyList clientList_;
  
 private:  
  TwoK::Configuration::DependencySpecification *
  findClient (TwoK::Configuration::ComponentConfigurator_ptr client, 
              const char *name);
  
  PortableServer::POA_var poa_;
  
  /* The name of this component. */
  char *name_;
  
  /* Information string about this component. */
  char *info_;
  
  /* The implementating this component. */
  CORBA::Object_ptr implementation_;
  
  ACE_Lock *lock_; // WARNING: It's not being used yet!
  // Lock to serialize access to the underlying data structure.  This
  // is a lock adapter that hides the type of lock, which may be a
  // null lock if the ORB decides threading is not necessary.
};

#endif



