/******************************************************************************
*                                                                             *
*  File: factory_i.cpp                                                        *
*                                                                             *
*  Description: The implementation of a ComponentConfigurator factory.        *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
 Revision Control Information
 $Revision: 1.2 $
 $Log: Factory_i.cpp,v $
 Revision 1.2  2000/10/09 21:02:34  f-kon
 Changes to use new TwoK:: namespace for 2K services integrated with Tomo's
 changes for Windows compilation.

 Revision 1.1.1.2  2000/02/20 07:53:18  yamane
 Using Factory from Resource Manager

 Revision 1.1.1.1  1999/10/12 00:42:46  f-kon
 Version compatible with TAO 1.0

 Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
 This new 2K source tree contains the most up-to-date version of ACE and TAO
 and including the dynamicTAO code as well as some 2K tests and preliminary
 versions of some of its high-level systems.


 Revision 1.1  1999/02/15 18:03:18  f-kon
 *** empty log message ***


******************************************************************************/
#include "tao/corba.h"
#include "Factory_i.h"
#include "Configuration_i.h"

CCFactory_i::CCFactory_i (void)
  : poa_ (PortableServer::POA::_duplicate(this->_default_POA()))
{
}  // Constructor

CCFactory_i::~CCFactory_i (void)
{
}  // Destructor

////////////////////////////////////////////////////////////////////////////////
// Creates a new ComponentConfigurator.
////////////////////////////////////////////////////////////////////////////////
TwoK::Configuration::ComponentConfigurator_ptr 
CCFactory_i::newConfigurator (const char *name,
                            CORBA::Object_ptr implementation,
                            CORBA::Environment &env)
{
   TwoK::Configuration::ComponentConfigurator_var result;
   ComponentConfigurator_i *ccptr = 
      new ComponentConfigurator_i 
                 (poa_, CORBA::string_dup (name), (char *)NULL, 
                  CORBA::Object::_duplicate (implementation));
    
  if (ccptr == 0)
    {
      env.clear ();
      env.exception (new CORBA::NO_MEMORY ());
      return TwoK::Configuration::ComponentConfigurator::_nil ();
    }

  result = ccptr->_this (env); // Registers the new object with the POA.
  return result._retn ();
}// newConfigurator ()


////////////////////////////////////////////////////////////////////////////////
// Shut downs the ORB. Note that in the current implementation, all the
// ComponentConfigurators created with this factory are lost.
////////////////////////////////////////////////////////////////////////////////
void CCFactory_i::shutdown (CORBA::Environment &env)
{
    ACE_UNUSED_ARG (env);
    
    ACE_DEBUG ((LM_DEBUG,
                "%s\n",
                "Configuration::Factory is shutting down"));
    
    TAO_ORB_Core_instance ()->orb ()->shutdown ();

}// shutdown ()



void 
CCFactory_i::poa (PortableServer::POA_ptr poa)
{
  poa_ = poa;
}
