/******************************************************************************
*                                                                             *
*  File: client_i.cpp                                                         *
*                                                                             *
*  Description: A test client for the ComponentConfigurator object.           *
*               Based on the TAO Cubit example.                               *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
 Revision Control Information
 $Revision: 1.3 $
 $Log: client_i.cpp,v $
 Revision 1.3  2000/10/09 21:02:32  f-kon
 Changes to use new TwoK:: namespace for 2K services integrated with Tomo's
 changes for Windows compilation.

 Revision 1.2  1999/12/17 20:46:21  f-kon
 *** empty log message ***

 Revision 1.1.1.1  1999/10/12 00:42:46  f-kon
 Version compatible with TAO 1.0

 Revision 1.2  1999/04/23 22:32:07  f-kon
 *** empty log message ***

 Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
 This new 2K source tree contains the most up-to-date version of ACE and TAO
 and including the dynamicTAO code as well as some 2K tests and preliminary
 versions of some of its high-level systems.


 Revision 1.1  1999/02/15 18:03:18  f-kon
 *** empty log message ***


******************************************************************************/

#include "ace/Env_Value_T.h"
#include "ace/Read_Buffer.h"
#include "client_i.h"

#include "tao/Timeprobe.h"

#if defined (ACE_ENABLE_TIMEPROBES)

static const char *CC_Client_Timeprobe_Description[] =
{
  "CC_Client::op - start",
  "CC_Client::op - end",
};

enum
{
  CC_CLIENT_OP_START = 10000,
  CC_CLIENT_OP_END,
};

#endif /* ACE_ENABLE_TIMEPROBES */

// Setup Timeprobes
ACE_TIMEPROBE_EVENT_DESCRIPTIONS (CC_Client_Timeprobe_Description,
                                  CC_CLIENT_OP_START)

// Constructor.
CC_Client::CC_Client (void)
  : cc_factory_key_ (0),
    cc_key_ (ACE_OS::strdup ("key0")),
    loop_count_ (250),
    shutdown_ (0),
    cc_ (TwoK::Configuration::ComponentConfigurator::_nil ()),
    call_count_ (0),
    error_count_ (0),
    cc_factory_ior_file_ (0),
    f_handle_ (ACE_INVALID_HANDLE)
    //    use_naming_service_ (1)
{
}

int
CC_Client::read_ior (char *filename)
{
  // Open the file for reading.
  this->f_handle_ = ACE_OS::open (filename,0);

  if (this->f_handle_ == ACE_INVALID_HANDLE)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "Unable to open %s for writing: %p\n",
                       filename),
                      -1);
  ACE_Read_Buffer ior_buffer (this->f_handle_);
  this->cc_factory_key_ = ior_buffer.read ();

  if (this->cc_factory_key_ == 0)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "Unable to allocate memory to read ior: %p\n"),
                      -1);
   return 0;
}

// Parses the command line arguments and returns an error status.

int
CC_Client::parse_args (void)
{
  ACE_Get_Opt get_opts (argc_, argv_, "dn:f:k:xs");
  int c;
  int result;

  while ((c = get_opts ()) != -1)
    switch (c)
      {
      case 'd':  // debug flag
        TAO_debug_level++;
        break;
      case 'n':                 // loop count
        this->loop_count_ = (u_int) ACE_OS::atoi (get_opts.optarg);
        break;
      case 'f': // read the IOR from the file.
        result = this->read_ior (get_opts.optarg);
        if (result < 0)
          ACE_ERROR_RETURN ((LM_ERROR,
                             "Unable to read ior from %s : %p\n",
                             get_opts.optarg),
                            -1);
            break;
      case 'k': // read the cc IOR from the command-line.
        this->cc_factory_key_ =
          ACE_OS::strdup (get_opts.optarg);
        break;
      case 'x':
        this->shutdown_ = 1;
        break;
      case '?':
      default:
        ACE_ERROR_RETURN ((LM_ERROR,
                           "usage:  %s"
                           " [-d]"
                           " [-n loopcount]"
                           " [-f cc_factory-obj-ref-key-file]"
                           " [-k cc-obj-ref-key]"
                           " [-x]"
                           "\n",
                           this->argv_ [0]),
                          -1);
      }

  // Indicates successful parsing of command line.
  return 0;
}

void
CC_Client::test_dii (void)
{
}



void
CC_Client::simple_test (void)
{
  CORBA::UShort retv;
  {
    ACE_FUNCTION_TIMEPROBE (CC_CLIENT_OP_START);

    retv = cc_->numberOfClients (this->env_);
  }

  this->call_count_++;

  if (this->env_.exception () != 0)
    {
      this->env_.print_exception ("from simple_test");
      this->error_count_++;
    }
  else
    {
      ACE_OS::printf("number of clients = %d \n", retv);
    }
}// simple_test ()

void
CC_Client::complex_test (void)
{
  TwoK::Configuration::ComponentConfigurator_ptr a, b, c, d;
  int i;
  
  TAO_TRY
    {
      a = factory_->newConfigurator ("Einstein", CORBA::Object::_nil (), TAO_TRY_ENV);
      TAO_CHECK_ENV;
      b = factory_->newConfigurator ("Marx", CORBA::Object::_nil (), TAO_TRY_ENV);
      TAO_CHECK_ENV;
      c = factory_->newConfigurator ((CORBA::String) NULL, CORBA::Object::_nil (), TAO_TRY_ENV);
      TAO_CHECK_ENV;
      d = factory_->newConfigurator ("Freud", CORBA::Object::_nil (), TAO_TRY_ENV);
      TAO_CHECK_ENV;


      if (a->_is_equivalent (a, TAO_TRY_ENV))
        ACE_OS::printf("a == a\n");
      else
        ACE_OS::printf("a != a, BUG!\n");
      TAO_CHECK_ENV;
      if (a->_is_equivalent (b, TAO_TRY_ENV))
        ACE_OS::printf("a == b, BUG!\n");
      else
        ACE_OS::printf("a != b\n");
      TAO_CHECK_ENV;
 
      // normal cases    
      a->addHook ("LeftHand", TAO_TRY_ENV);  TAO_CHECK_ENV;
      a->addHook ("RightHand", TAO_TRY_ENV); TAO_CHECK_ENV;
      b->addHook ("LeftHand", TAO_TRY_ENV);  TAO_CHECK_ENV;
      b->addHook ("RightHand", TAO_TRY_ENV); TAO_CHECK_ENV;
      d->addHook ("LeftHand", TAO_TRY_ENV);  TAO_CHECK_ENV;
      d->addHook ("RightHand", TAO_TRY_ENV); TAO_CHECK_ENV;
      ACE_OS::printf ("added hooks.\n");

      // erroneous cases
      a->addHook ("LeftHand", TAO_TRY_ENV);  // should fail: repeating same hook
      if (TAO_TRY_ENV.exception () != 0)
        {
          TAO_TRY_ENV.print_exception ("expected exception");
          TAO_TRY_ENV.clear ();
        }
      else
        ACE_ERROR ((LM_ERROR, "(%P|%t) expected exception MISSING\n"));

      a->addHook ((CORBA::String) NULL, TAO_TRY_ENV);  // should fail: null name
      if (TAO_TRY_ENV.exception () != 0)
        {
          TAO_TRY_ENV.print_exception ("expected exception");
          TAO_TRY_ENV.clear ();
        }
      else
        ACE_ERROR ((LM_ERROR, "(%P|%t) expected exception MISSING\n"));
      
      // print number of clients in a and b
      ACE_OS::printf ("\nnumber of clients in %s: %d\n", a->name (TAO_TRY_ENV), 
              a->numberOfClients (TAO_TRY_ENV));
      ACE_OS::printf ("number of clients in %s: %d\n", b->name (TAO_TRY_ENV), 
              b->numberOfClients (TAO_TRY_ENV));

      // print list of hooks in a
      TwoK::Configuration::DependencyList *list = a->listHooks (TAO_TRY_ENV);  TAO_CHECK_ENV;
      
      ACE_OS::printf("\nList of hooks:\n");
      for (int i = 0; i < list->length (); i++)
        ACE_OS::printf("hook name = %s\n", (const char *)(*list)[i].name);
      ACE_OS::printf("End of list.\n");
      
      // back to normal cases
      ACE_OS::printf("\nRegistering clients.\n");
      a->hook ("LeftHand", b, TAO_TRY_ENV); TAO_CHECK_ENV;
      b->registerClient (a, "LeftHand", TAO_TRY_ENV); TAO_CHECK_ENV;
      
      b->hook ("RightHand", a, TAO_TRY_ENV); TAO_CHECK_ENV;
      a->registerClient (b, "RightHand", TAO_TRY_ENV); TAO_CHECK_ENV;
    
      a->hook ("LeftHand", b, TAO_TRY_ENV); TAO_CHECK_ENV; // repetition, OK.

      b->registerClient (c, "LeftHand", TAO_TRY_ENV); TAO_CHECK_ENV; // no repetition
      b->registerClient (d, "fictitious", TAO_TRY_ENV); TAO_CHECK_ENV; // no repetition
      b->registerClient (a, "LeftHand", TAO_TRY_ENV); // repetition
      if (TAO_TRY_ENV.exception () != 0)
        {
          TAO_TRY_ENV.print_exception ("expected exception");
          TAO_TRY_ENV.clear ();
        }
      else
        ACE_ERROR ((LM_ERROR, "(%P|%t) expected exception MISSING\n"));

      // print number of clients in a and b
      ACE_OS::printf ("\nnumber of clients in %s: %d\n", a->name (TAO_TRY_ENV), 
                      a->numberOfClients (TAO_TRY_ENV));
      ACE_OS::printf ("number of clients in %s: %d\n", b->name (TAO_TRY_ENV), 
                      b->numberOfClients (TAO_TRY_ENV));

      ACE_OS::printf ("\nTesting events...\n");
      
      TwoK::Configuration::Event e;
      e.type = TwoK::Configuration::RECONFIGURED;
      e.description = CORBA::string_dup ("reconf. desc.");
      ACE_OS::printf("first event.\n");
      b->eventFromClient(a, e, 1, TAO_TRY_ENV); TAO_CHECK_ENV; 

      e.type = TwoK::Configuration::FAILED;
      e.description = CORBA::string_dup ("failure desc.");
      ACE_OS::printf("second event.\n");
      b->eventFromClient(a, e, 2, TAO_TRY_ENV); TAO_CHECK_ENV; 

      e.type = TwoK::Configuration::UNKNOWN;
      e.description = CORBA::string_dup ("unknown desc.");
      ACE_OS::printf("third event.\n");
      b->eventFromHookedComponent(a, e, 5, TAO_TRY_ENV); TAO_CHECK_ENV; 

      ACE_OS::printf("\nUnregistering clients.\n");
      a->unregisterClient (b, "RightHand", TAO_TRY_ENV); TAO_CHECK_ENV;

      b->unregisterClient (a, "LeftHand", TAO_TRY_ENV); TAO_CHECK_ENV;
      b->unregisterClient (c, "LeftHand", TAO_TRY_ENV); TAO_CHECK_ENV;
      b->unregisterClient (d, "fictitious", TAO_TRY_ENV); TAO_CHECK_ENV;
      b->unregisterClient (a, "LeftHand", TAO_TRY_ENV); // repetition
      if (TAO_TRY_ENV.exception () != 0)
        {
          TAO_TRY_ENV.print_exception ("expected exception");
          TAO_TRY_ENV.clear ();
        }
      else
        ACE_ERROR ((LM_ERROR, "(%P|%t) expected exception MISSING\n"));

      // print number of clients in a and b
      ACE_OS::printf ("\nnumber of clients in %s: %d\n", a->name (TAO_TRY_ENV), 
                      a->numberOfClients (TAO_TRY_ENV));
      ACE_OS::printf ("number of clients in %s: %d\n", b->name (TAO_TRY_ENV), 
                      b->numberOfClients (TAO_TRY_ENV));
      
      // back to normal cases, hook some more guys to each other
      d->hook ("LeftHand", a, TAO_TRY_ENV);
      d->hook ("RightHand", a, TAO_TRY_ENV);
      a->hook ("RightHand", d, TAO_TRY_ENV);
      b->hook ("LeftHand", d, TAO_TRY_ENV);

      // test getHookedComponent ()
      ACE_OS::printf("\nleft of %s is <%s>, right of %s is <%s>\n",
                     a->name (TAO_TRY_ENV), 
                     a->getHookedComponent("LeftHand", TAO_TRY_ENV)->name (TAO_TRY_ENV),
                     a->name (TAO_TRY_ENV), 
                     a->getHookedComponent("RightHand", TAO_TRY_ENV)->name (TAO_TRY_ENV));

      ACE_OS::printf("left of %s is <%s>, right of %s is <%s>\n",
                     b->name (TAO_TRY_ENV), 
                     b->getHookedComponent("LeftHand", TAO_TRY_ENV)->name (TAO_TRY_ENV),
                     b->name (TAO_TRY_ENV), 
                     b->getHookedComponent("RightHand", TAO_TRY_ENV)->name (TAO_TRY_ENV));

      a->addHook ("gancho1",  TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho2", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho3", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho4", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho5", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho6", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho7", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho8", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho9", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho10", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho11", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho12", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho13", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho14", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho15", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho16", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho17", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho18", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho19", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho20", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho21", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho22", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->addHook ("gancho23", TAO_TRY_ENV); TAO_CHECK_ENV;
      ACE_OS::printf("added 23 extra hooks on %s.\n", a->name (TAO_TRY_ENV));
      
      a->deleteHook ("gancho1",  TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho2", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho3", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho4", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho5", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho6", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho7", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho8", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho9", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho10", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho11", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho12", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho13", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho14", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho15", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho16", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho17", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho18", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho19", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho20", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho21", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho22", TAO_TRY_ENV); TAO_CHECK_ENV;
      a->deleteHook ("gancho23", TAO_TRY_ENV); TAO_CHECK_ENV;
      ACE_OS::printf("deleted 23 hooks on %s.\n", a->name (TAO_TRY_ENV));

      // now, let's remove the hooks in a
      ACE_OS::printf("\ndeleting hooks.\n");
      a->deleteHook ("LeftHand", TAO_TRY_ENV);  TAO_CHECK_ENV;
      a->deleteHook ("RightHand", TAO_TRY_ENV); TAO_CHECK_ENV;

      a->deleteHook ("RightHand", TAO_TRY_ENV);
      if (TAO_TRY_ENV.exception () != 0)
        {
          TAO_TRY_ENV.print_exception ("expected exception");
          TAO_TRY_ENV.clear ();
        }
      else
        ACE_ERROR ((LM_ERROR, "(%P|%t) expected exception MISSING\n"));

      // print list of hooks in a
      list = a->listHooks (TAO_TRY_ENV);  TAO_CHECK_ENV;
      ACE_OS::printf("\nList of hooks after deleting:\n");
      for (i = 0; i < list->length (); i++)
        ACE_OS::printf("hook name = %s\n", (const char *)(*list)[i].name);
      ACE_OS::printf("End of list.\n");


      // destroy the objects in the server
      ACE_OS::printf("\nDestroying server objects.\n");
      a->destroy (TAO_TRY_ENV);  TAO_CHECK_ENV;
      b->destroy (TAO_TRY_ENV);  TAO_CHECK_ENV;
      c->destroy (TAO_TRY_ENV);  TAO_CHECK_ENV;
      d->destroy (TAO_TRY_ENV);  TAO_CHECK_ENV;
      ACE_OS::printf("Destroyed server objects.\n\n");

      // call a method on a destroyed object
      char *s = a->name (TAO_TRY_ENV);
      if (TAO_TRY_ENV.exception () != 0)
        {
           TAO_TRY_ENV.print_exception ("expected exception");
           TAO_TRY_ENV.clear ();
        }
      else ACE_ERROR ((LM_ERROR, "(%P|%t) expected exception MISSING\n"));
      
      // now release the objects.
      ACE_OS::printf("\nReleasing objects.\n");
      CORBA::release (a); // Is this supposed to delete the object in the server? 
      CORBA::release (b); // No, it does not delete it on the server...
      CORBA::release (c);
      CORBA::release (d);
    }
  TAO_CATCHANY
    {
      TAO_TRY_ENV.print_exception ("in CC_Client::complex_test ()");
    }
  TAO_ENDTRY;
}// complex_test ()


void
CC_Client::print_stats (const char *call_name,
                        ACE_Profile_Timer::ACE_Elapsed_Time &elapsed_time)
{
  ACE_DEBUG ((LM_INFO,
              "%s:\n",
              call_name));

  if (this->call_count_ > 0 && this->error_count_ == 0)
    {
#if defined (ACE_LACKS_FLOATING_POINT)
      // elapsed_time.real_time is in units of microseconds.
      const u_int calls_per_sec =
        this->call_count_ * 1000000u / elapsed_time.real_time;

      ACE_DEBUG ((LM_INFO,
                  "\treal_time\t= %u ms,\n"
                  "\t%u calls/second\n",
                  elapsed_time.real_time / 1000u,
                  calls_per_sec));
#else  /* ! ACE_LACKS_FLOATING_POINT */
      // elapsed_time.real_time is in units of seconds.
      double calls_per_sec = this->call_count_ / elapsed_time.real_time;

      ACE_DEBUG ((LM_INFO,
                  "\treal_time\t= %0.06f ms, \n\t"
                  "user_time\t= %0.06f ms, \n\t"
                  "system_time\t= %0.06f ms\n"
                  "\t%0.00f calls/second\n",
                  elapsed_time.real_time < 0.0 ? 0.0
                    : elapsed_time.real_time * ACE_ONE_SECOND_IN_MSECS,
                  elapsed_time.user_time < 0.0 ? 0.0
                    : elapsed_time.user_time * ACE_ONE_SECOND_IN_MSECS,
                  elapsed_time.system_time < 0.0 ? 0.0
                    : elapsed_time.system_time * ACE_ONE_SECOND_IN_MSECS,
                  calls_per_sec < 0.0 ? 0.0 : calls_per_sec));
#endif /* ! ACE_LACKS_FLOATING_POINT */
    }
  else
    ACE_ERROR ((LM_ERROR,
                "\tNo time stats printed.  Call count zero or error ocurred.\n"));

  ACE_DEBUG ((LM_INFO,
              "\t%d calls, %d errors\n",
              this->call_count_,
              this->error_count_));
}

// Execute client example code.

int
CC_Client::run (int testing_collocation)
{
  u_int i;

  ACE_Profile_Timer timer;
  ACE_Profile_Timer::ACE_Elapsed_Time elapsed_time;

  //  ACE_Time_Value before;

  // We start an ACE_Profile_Timer here...
  timer.start ();

  // Make the calls in a loop.
  for (i = 0; i < this->loop_count_; i++)
    {
      this->simple_test ();
    }

  // stop the timer.
  timer.stop ();

  timer.elapsed_time (elapsed_time);
  // compute call average call time.
  this->print_stats ("cc average call", elapsed_time);

  this->complex_test ();

  this->cc_->destroy (this->env_); 
  CORBA::release (this->cc_);

  if (this->shutdown_)
    {
      this->factory_->shutdown (this->env_);
      //dexc (this->env_, "server, please ACE_OS::exit");
      // see what this was supposed to do in the old version.
    }
  
  ACE_OS::printf("End client execution.\n");

  return this->error_count_ == 0 ? 0 : 1;

}// run ()



CC_Client::~CC_Client (void)
{
  // Free resources
  // Close the ior files
  if (this->cc_factory_ior_file_)
    ACE_OS::fclose (this->cc_factory_ior_file_);
  if (this->f_handle_ != ACE_INVALID_HANDLE)
    ACE_OS::close (this->f_handle_);

  if (this->cc_factory_key_ != 0)
    ACE_OS::free (this->cc_factory_key_);
  if (this->cc_key_ != 0)
    ACE_OS::free (this->cc_key_);

  ACE_OS::printf("Destructed Client Object.\n");
}


int
CC_Client::init (int argc, char **argv)
{
  int naming_result;
  this->argc_ = argc;
  this->argv_ = argv;

  TAO_TRY
    {
      // Retrieve the ORB.
      this->orb_ = CORBA::ORB_init (this->argc_,
                                    this->argv_,
                                    "internet",
                                    TAO_TRY_ENV);
      TAO_CHECK_ENV;

      // Parse command line and verify parameters.
      if (this->parse_args () == -1)
        return -1;

        {
          if (this->cc_factory_key_ == 0)
            ACE_ERROR_RETURN ((LM_ERROR,
                               "%s: no cc factory key specified\n",
                               this->argv_[0]),
                              -1);


          CORBA::Object_var factory_object =
            this->orb_->string_to_object (this->cc_factory_key_,
                                          TAO_TRY_ENV);
          TAO_CHECK_ENV;

          this->factory_ =
            TwoK::Configuration::CCFactory::_narrow (factory_object.in(), TAO_TRY_ENV);
          TAO_CHECK_ENV;

          if (CORBA::is_nil (this->factory_.in ()))
            ACE_ERROR_RETURN ((LM_ERROR,
                               "invalid factory key <%s>\n",
                               this->cc_factory_key_),
                              -1);
        }

      ACE_DEBUG ((LM_DEBUG, "Located TwoK::Configuration::CCFactory.\n"));

      // Now retrieve the cc obj ref corresponding to the key.
      this->cc_ = this->factory_->newConfigurator (this->cc_key_, 
                                                   CORBA::Object::_nil (),
                                                   TAO_TRY_ENV);
      TAO_CHECK_ENV;

      if (CORBA::is_nil (this->cc_))
        ACE_ERROR_RETURN ((LM_ERROR,
                           "null cc objref returned by factory\n"),
                          -1);
    }
  TAO_CATCHANY
    {
      TAO_TRY_ENV.print_exception ("CC_Client::init");
      return -1;
    }
  TAO_ENDTRY;

  return 0;
}

void
CC_Client::fini ()
{
    // Shutdown local client ORB.
    this->orb_->shutdown ();
}


#if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)
template class ACE_Env_Value<unsigned long>;
template class ACE_Env_Value<unsigned short>;
#elif defined (ACE_HAS_TEMPLATE_INSTANTIATION_PRAGMA)
#pragma instantiate ACE_Env_Value<unsigned long>
#pragma instantiate ACE_Env_Value<unsigned short>
#endif /* ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION */
