/******************************************************************************
*                                                                             *
*  File: client_i.h                                                           *
*                                                                             *
*  Description: A test client for the ComponentConfigurator object.           *
*               Based on the TAO Cubit example.                               *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
 Revision Control Information
 $Revision: 1.2 $
 $Log: client_i.h,v $
 Revision 1.2  2000/10/09 21:02:30  f-kon
 Changes to use new TwoK:: namespace for 2K services integrated with Tomo's
 changes for Windows compilation.

 Revision 1.1.1.1  1999/10/12 00:42:46  f-kon
 Version compatible with TAO 1.0

 Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
 This new 2K source tree contains the most up-to-date version of ACE and TAO
 and including the dynamicTAO code as well as some 2K tests and preliminary
 versions of some of its high-level systems.


 Revision 1.1  1999/02/15 18:03:18  f-kon
 *** empty log message ***


******************************************************************************/
#ifndef CC_CLIENT_I_H
#define CC_CLIENT_I_H

#include "ace/Get_Opt.h"
#include "ace/Profile_Timer.h"
#include "tao/corba.h"
#include "ConfigurationC.h" 

class CC_Client
{
public:
  // = Constructor and destructor.
  CC_Client (void);
  ~CC_Client (void);

  int run (int testing_collocation = 0);
  // Execute client example code.

  int init (int argc, char **argv);
  // Initialize the client communication endpoint with server.

  void fini ();
  // Shuts down the local ORB.

private:
  // int init_naming_service (void);
  // Function to initialize the naming service.

  int func (u_int i);
  // Simple function that returns the substraction of 117 from the
  // parameter.

  int read_ior (char *filename);
  // Function to read the cc factory ior from a file.

  int parse_args (void);
  // Parses the arguments passed on the command line.

  void print_stats (const char *call_name,
                    ACE_Profile_Timer::ACE_Elapsed_Time &elapsed_time);
  // Prints the time stats.

  void simple_test  (void);
  void complex_test (void);
  // Exercise the ComponentConfigurator

  void test_dii (void);
  // Exercise the ComponentConfigurator using DII.

  int argc_;
  // # of arguments on the command line.

  char **argv_;
  // arguments from command line.

  char *cc_factory_key_;
  // Key of factory obj ref.

  char *cc_key_;
  // Key of the obj ref to be retrieved via the factory.

  u_int loop_count_;
  // Number of times to do the operations.

  int shutdown_;
  // Flag to tell server to shutdown.

  TwoK::Configuration::CCFactory_var factory_;
  // factory pointer.

  CORBA::Environment env_;
  // Environment variable.

  TwoK::Configuration::ComponentConfigurator_ptr cc_;
  // Cubit obj ref.

  CORBA::ORB_var orb_;
  // Remember our orb.

  u_int call_count_;
  // # of calls made to functions.

  u_int error_count_;
  // # of errors incurred in the lifetime of the application.

  FILE *cc_factory_ior_file_;
  // File from which to obtain the IOR.

  ACE_HANDLE f_handle_;
  // File handle to read the IOR.

  // int use_naming_service_;
  // Flag to tell client not to use Namingservice to find the cubit
  // Factory.
};

#endif
