/******************************************************************************
*                                                                             *
*  File: server_i.cpp                                                         *
*                                                                             *
*  Description: A test server for the ComponentConfigurator object.           *
*               Based on the TAO Cubit example.                               *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
 Revision Control Information
 $Revision: 1.1.1.1 $
 $Log: server_i.cpp,v $
 Revision 1.1.1.1  1999/10/12 00:42:47  f-kon
 Version compatible with TAO 1.0

 Revision 1.2  1999/04/23 22:32:08  f-kon
 *** empty log message ***

 Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
 This new 2K source tree contains the most up-to-date version of ACE and TAO
 and including the dynamicTAO code as well as some 2K tests and preliminary
 versions of some of its high-level systems.


 Revision 1.1  1999/02/15 18:03:19  f-kon
 *** empty log message ***


******************************************************************************/

#include "server_i.h"

ComponentConfiguratorServer::ComponentConfiguratorServer (void)
    :
    ior_output_file_ (0)
{
}

int
ComponentConfiguratorServer::parse_args (void)
{
  ACE_Get_Opt get_opts (argc_, argv_, "dn:o:s");
  int c;

  while ((c = get_opts ()) != -1)
    switch (c)
      {
      case 'd':  // debug flag.
        TAO_debug_level++;
        break;
      case 'o': // output the IOR to a file.
        this->ior_output_file_ = ACE_OS::fopen (get_opts.optarg, "w");
        if (this->ior_output_file_ == 0)
          ACE_ERROR_RETURN ((LM_ERROR,
                             "Unable to open %s for writing: %p\n",
                             get_opts.optarg), -1);
        break;
      case '?':
      default:
        ACE_ERROR_RETURN ((LM_ERROR,
                           "usage:  %s"
                           " [-d]"
                           " [-o <ior_output_file>]"
                           "\n",
                           argv_ [0]),
                          1);
      }

  // Indicates successful parsing of command line.
  return 0;
}

int
ComponentConfiguratorServer::init (int argc,
                                   char** argv,
                                   CORBA::Environment& env)
{
  // Call the init of <TAO_ORB_Manager> to initialize the ORB and
  // create a child POA under the root POA.
  if (this->orb_manager_.init_child_poa (argc,
                                         argv,
                                         "child_poa",
                                         env) == -1)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "%p\n",
                       "init_child_poa"),
                      -1);

  TAO_CHECK_ENV_RETURN (env,-1);
  this->argc_ = argc;
  this->argv_ = argv;

  this->parse_args ();
  // @@ Check for the return value here.

  CORBA::String_var str  =
    this->orb_manager_.activate_under_child_poa ("factory",
                                                 &this->factory_impl_,
                                                 env);
  ACE_DEBUG ((LM_DEBUG,
              "The IOR is: <%s>\n",
              str.in ()));

  if (this->ior_output_file_)
    {
      ACE_OS::fprintf (this->ior_output_file_,
                       "%s",
                       str.in ());
      ACE_OS::fclose (this->ior_output_file_);
    }
  PortableServer::POA_var child_poa = this->orb_manager_.child_poa ();

  factory_impl_.poa (child_poa.in ());

  return 0;
}


int
ComponentConfiguratorServer::run (CORBA::Environment& env)
{
  if (this->orb_manager_.run (env) == -1)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "ComponentConfiguratorServer::run"),
                      -1);
  return 0;
}

ComponentConfiguratorServer::~ComponentConfiguratorServer (void)
{
}
