/******************************************************************************
*                                                                             *
*  File: ComponentRepository.h                                                *
*                                                                             *
*  Description: A Repository of Components built on top                       *
*               of the DynamicConfigurator.                                   *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.3 $
   $Log: ComponentRepository.h,v $
   Revision 1.3  2000/02/22 23:36:58  yamane
   Deleting Naming binding on the termination

   Revision 1.2  2000/01/27 17:43:24  f-kon
   added registration to the NameService

   Revision 1.1  2000/01/24 15:59:47  f-kon
   first working version

******************************************************************************/

# ifndef COMPONENT_REPOSITORY_H
# define COMPONENT_REPOSITORY_H

#include "tao/TAO.h"
#include "dynamicTAO/src/DynamicConfigurator/dynamicConfigurator_i_.h"
#include "orbsvcs/orbsvcs/CosNamingC.h"

class ComponentRepository;

class ComponentRepository_Shutdown : public ACE_Event_Handler
{
public:

  ComponentRepository_Shutdown (ComponentRepository& crepos);

  virtual int handle_signal (int, siginfo_t*, ucontext_t*);

private:
  int done;

  ComponentRepository& crepos_;
  ACE_Sig_Handler shutdown_;
};

class ComponentRepository
{
 public:
  ComponentRepository ();

  int init (int argc, char**argv);
  int run (CORBA_Environment& ACE_TRY_ENV);
  void terminate ();

 private:
  int parse_args (int argc, char *argv[]);

  int verbose_; // If set to 1, we print the server IOR on the stdout.
  FILE * ior_output_file_;
  size_t time_; // After how many seconds the server should stop listening to requests.
                // It's ignored if it's 0.

  TAO_ORB_Manager orbManager_;
  DynamicConfigurator_ptr localDynConf_;
  CosNaming::NamingContext_ptr nameServiceRoot_;

  CosNaming::Name query_name;
};

#endif
