/******************************************************************************
*                                                                             *
*  File: BaseCC.cpp                                                           *
*                                                                             *
*  Description: A base component configurator executing application           *
*               Based on the TAO Cubit example.                               *
*                                                                             *
*  Project: 2K - Resource Management                                          *
*  Author: Tomonori Yamane (yamane@uiuc.edu)                                  *
*                                                                             *
*******************************************************************************
 Revision 1.1  1999/02/15 18:03:19  f-kon
 *** empty log message ***


******************************************************************************/

// Inclui 'TwoK::' e ' TwoK2::' na frente de varias linhas que nao especificavam de 
// qual modulo era o tipo desejado, pois tava dando erro na compilacao
// Elas estao marcadas com // MOD


#include "BaseCC.h"
#include "dynamicTAO/src/ace/Dynamic_Service_Config.h"
#include "../../../Environment/envC.h" // MOD

// Constructor
BaseCC::BaseCC (TAO_ORB_Manager &manager_)
	: SimpleConfigurator("CORBAConfigurator",  0), orb_manager_(manager_) , orb_(orb_manager_.orb ())
{

//  ACE_OS::printf("Initializing BaseCC class\n");
  //
  // Create poa
  //
  poa_ = PortableServer::POA::_duplicate(orb_manager_.child_poa ());

  setup_domainconf();

  setup_dynconf();
  setup_autoconf();
  setup_cc_factory();
  setup_crepos();

  connect_components();
}

// Destructor
BaseCC::~BaseCC (void)
{
  // no-op
}

ComponentConfigurator_i *
BaseCC::add_component(const char *hook_name, const char *name,
                      CORBA::Object_ptr obj, CORBA::Environment& ACE_TRY_ENV)
{
  ACE_TRY {
	//
	// Create ComponentConfigurator
	//
	ComponentConfigurator_i *cc_impl_ =
		new ComponentConfigurator_i
                 (poa_, CORBA::string_dup (name), (char *)NULL,
                  CORBA::Object::_duplicate(obj));

        TwoK::Configuration::ComponentConfigurator_var result; // MOD

	result = cc_impl_->_this (ACE_TRY_ENV); // Registers the new object with the POA.
        ACE_TRY_CHECK;

	domain_cc_impl_->addHook(hook_name, ACE_TRY_ENV);
        ACE_TRY_CHECK;

	domain_cc_impl_->hook(hook_name, result.in(), ACE_TRY_ENV);
        ACE_TRY_CHECK;

	return (cc_impl_);
  }
  ACE_CATCH (CORBA::SystemException, sysex)
    {
      ACE_UNUSED_ARG (sysex);
      ACE_TRY_ENV.print_exception ("System Exception");
      exit(1);
    }
  ACE_CATCH (CORBA::UserException, userex)
    {
      ACE_UNUSED_ARG (userex);
      ACE_TRY_ENV.print_exception ("User Exception");
      exit(1);
    }
  ACE_ENDTRY;
  return (NULL);
}


//
// Setup Domain configurator
//
void
BaseCC::setup_domainconf()
{
  //
  // Create domainComponentConfigurator
  //
  domain_cc_impl_ =
		new ComponentConfigurator_i
                 (poa_, CORBA::string_dup ("domainConfigurator"), "domainConfigurator",
                  CORBA::Object::_nil());

  ACE_TRY_NEW_ENV {
//  	ACE_OS::printf("BaseCC::setup_domainconf()\n");

        // activate servant
        // ???
        CORBA::String_var str  =
            orb_manager_.activate_under_child_poa ("DomainConfigWorld",
                                                    domain_cc_impl_,
                                                   ACE_TRY_ENV);
        ACE_TRY_CHECK;

        CORBA::Object_var obj_ = orb_->string_to_object(str, ACE_TRY_ENV);
        ACE_TRY_CHECK;

        domainConfigurator_ = TwoK::Configuration::ComponentConfigurator::_duplicate(TwoK::Configuration::ComponentConfigurator::_narrow(obj_.in(),ACE_TRY_ENV)); // MOD

        ACE_TRY_CHECK;

  	(Dynamic_Service_Config::instance())->domainConfigurator()->addHook("CORBAConfigurator");
  	(Dynamic_Service_Config::instance())->domainConfigurator()->hook("CORBAConfigurator", this);
    }
  ACE_CATCH (CORBA::SystemException, sysex)
    {
      ACE_UNUSED_ARG (sysex);
      ACE_TRY_ENV.print_exception ("System Exception");
      exit(1);
    }
  ACE_CATCH (CORBA::UserException, userex)
    {
      ACE_UNUSED_ARG (userex);
      ACE_TRY_ENV.print_exception ("User Exception");
      exit(1);
    }
  ACE_ENDTRY;
}

//
// Setup dynamic configurator
//
void
BaseCC::setup_dynconf() {
  ACE_OS::printf("Loader_i::setup_dynconf()\n"); // teste

  // Find Dynamic Configurator
  CORBA::Object_var dyn_obj;
  dyn_obj = orb_->resolve_initial_references("DynamicConfigurator");

  ACE_TRY_NEW_ENV {
    dyn_conf_ = DynamicConfigurator::_duplicate(DynamicConfigurator::_narrow(dyn_obj.in(), ACE_TRY_ENV));
    ACE_TRY_CHECK;

#if 1 // teste Mudei para 1
    // List all categories
    DynamicConfigurator::stringList *cat_list, *impl_list;

    cat_list = dyn_conf_->list_categories(ACE_TRY_ENV);
    ACE_TRY_CHECK;

    ACE_OS::printf("\nNumber of categories:%d\n",cat_list->length());

    for (CORBA::ULong i=0; i < cat_list->length(); i++) {
      const char *category = ((*cat_list)[i]).in();
      ACE_OS::printf("Category =  %s\n",category);

      impl_list = dyn_conf_->list_implementations(category, ACE_TRY_ENV);
      ACE_TRY_CHECK;

      ACE_OS::printf("\nNumber of %s implementations:%d\n",
		     category, impl_list->length());

      for (CORBA::ULong j=0; j < impl_list->length(); j++) {
	ACE_OS::printf("%s\n",((*impl_list)[j]).in());
      }
    }
#endif
  }
  ACE_CATCH (CORBA::SystemException, sysex) {
    ACE_UNUSED_ARG (sysex);
    ACE_TRY_ENV.print_exception ("System Exception");
    exit(1);
  }
  ACE_CATCH (CORBA::UserException, userex) {
    ACE_UNUSED_ARG (userex);
    ACE_TRY_ENV.print_exception ("User Exception");
    exit(1);
  }
  ACE_ENDTRY;
}

//
// Setup Auto configurator
//
void
BaseCC::setup_autoconf() {
  //
  // Create AutoConfig_i
  //
  autoConfig_impl_ = new AutoConfig_i ();

  ACE_TRY_NEW_ENV {
    ACE_OS::printf("BaseCC::setup_autoconf()\n"); // teste

        // activate servant
        // ???
        CORBA::String_var str  =
            orb_manager_.activate_under_child_poa ("AutoConfigWorld",
                                                    autoConfig_impl_,
                                                   ACE_TRY_ENV);
        ACE_TRY_CHECK;

  	ACE_OS::printf("BaseCC::setup_autoconf() before autoConfig_impl_->init() done\n"); // teste
	autoConfig_impl_->init(ACE_TRY_ENV);
        ACE_TRY_CHECK;
  	ACE_OS::printf("BaseCC::setup_autoconf() after autoConfig_impl_->init() done\n"); // teste

        CORBA::Object_var obj_ = orb_->string_to_object(str, ACE_TRY_ENV);
        ACE_TRY_CHECK;

        auto_conf_ = TwoK2::AutoConfig::_duplicate(TwoK2::AutoConfig::_narrow(obj_.in(), ACE_TRY_ENV)); // MOD
        ACE_TRY_CHECK;
  }
  ACE_CATCH (CORBA::SystemException, sysex) {
    ACE_UNUSED_ARG (sysex);
    ACE_TRY_ENV.print_exception ("System Exception");
    exit(1);
  }
  ACE_CATCH (CORBA::UserException, userex) {
    ACE_UNUSED_ARG (userex);
    ACE_TRY_ENV.print_exception ("User Exception");
    exit(1);
  }
  ACE_ENDTRY;
}

void
BaseCC::setup_cc_factory()
{
  //
  // Create AutoConfig_i
  //
  cc_factory_impl_ = new CCFactory_i (); // MOD

  ACE_TRY_NEW_ENV {
//  	ACE_OS::printf("BaseCC::setup_cc_factory()\n");

        // activate servant
        // ???
        CORBA::String_var str  =
            orb_manager_.activate_under_child_poa ("CCFactoryWorld",
                                                    cc_factory_impl_,
                                                   ACE_TRY_ENV);
        ACE_TRY_CHECK;

	cc_factory_impl_->poa(poa_);
        ACE_TRY_CHECK;

        CORBA::Object_var obj_ = orb_->string_to_object(str, ACE_TRY_ENV);
        ACE_TRY_CHECK;

        cc_factory_ = TwoK::Configuration::CCFactory::_duplicate(TwoK::Configuration::CCFactory::_narrow(obj_.in(), ACE_TRY_ENV)); // MOD
        ACE_TRY_CHECK;

    }
  ACE_CATCH (CORBA::SystemException, sysex)
    {
      ACE_UNUSED_ARG (sysex);
      ACE_TRY_ENV.print_exception ("System Exception");
      exit(1);
    }
  ACE_CATCH (CORBA::UserException, userex)
    {
      ACE_UNUSED_ARG (userex);
      ACE_TRY_ENV.print_exception ("User Exception");
      exit(1);
    }
  ACE_ENDTRY;

}

void
BaseCC::setup_crepos() {
  CosNaming::Name query_name (3);
  int done = 0;

  ACE_TRY_NEW_ENV {
    ACE_OS::printf("BaseCC::setup_crepos()\n");

    CORBA::Object_var obj_ = orb_->resolve_initial_references ("NameService");
    if (CORBA::is_nil (obj_.in ())) {
      ACE_OS::printf("Cannot find Name Service\n");
      exit(0);
    }

    CosNaming::NamingContext_ptr nameServiceRoot_ = CosNaming::NamingContext::_narrow (obj_.in ());

    if (CORBA::is_nil (nameServiceRoot_)) {
      ACE_OS::printf("Cannot narrow Name Service\n");
      exit(0);
    }

    char arch[256];
    ACE_OS::sprintf(arch, 
		    // MOD
		    // de "%s:%s" para "%s-%s"
		    "%s-%s",
		    // FIM-MOD
		    ACE_2k_os::os_name(), ACE_2k_os::processor_name());

    ACE_OS::printf("Context = %s\n", arch);

    query_name.length(3);
    query_name[0].id = CORBA::string_dup ("services"); // is this leaking?
    query_name[0].kind = CORBA::string_dup ("");
    query_name[1].id = CORBA::string_dup ("ComponentRepository"); // is this leaking?
    query_name[1].kind = CORBA::string_dup ("");
    query_name[2].id = CORBA::string_dup (arch); // is this leaking?
    query_name[2].kind = CORBA::string_dup ("");
 
    CORBA::Object_var cr_obj_ = nameServiceRoot_->resolve(query_name, ACE_TRY_ENV);
    ACE_TRY_CHECK;

    ACE_OS::printf("ARCH = %s\n", arch);	
    ACE_OS::printf("resolve done\n");

    component_repos_ = DynamicConfigurator::_duplicate(DynamicConfigurator::_narrow(cr_obj_.in(), ACE_TRY_ENV));
    ACE_TRY_CHECK;
    done = 1;
  }
  ACE_CATCH (CORBA::SystemException, sysex) {
    ACE_UNUSED_ARG (sysex);
    ACE_TRY_ENV.print_exception ("System Exception");
    exit(1);
  }
  ACE_CATCH (CORBA::UserException, userex) {
    ACE_UNUSED_ARG (userex);
    ACE_TRY_ENV.print_exception ("User Exception");
  }
  ACE_ENDTRY;
  if (! done) {
    ACE_OS::printf("Component Repository is not available.\n We will use a local dynamic configurator instead\n");
    component_repos_ = DynamicConfigurator::_duplicate(DynamicConfigurator::_narrow(orb_->resolve_initial_references("DynamicConfigurator")));
  } else {
    ACE_OS::printf("Component repository found\n");
  }
}

//
// Setup CORBA component configurator
//
void
BaseCC::connect_components()
{
  ACE_TRY_NEW_ENV {
//  	ACE_OS::printf("BaseCC::connect_components()\n");

	//
	// Create ComponentConfigurator (AUTOCONFIG)
	//
	auto_cc_impl_ = add_component("AutoConfigurator",
                                     "AutoConfigurator",
                                     CORBA::Object::_duplicate(auto_conf_),
                                     ACE_TRY_ENV);
        ACE_TRY_CHECK;

	//
	// Create ComponentConfigurator (Dynamic Configurator)
	//
	dynconf_cc_impl_ = add_component("DynamicConfigurator",
                                        "DynamicConfigurator",
                                        CORBA::Object::_duplicate(dyn_conf_),
                                        ACE_TRY_ENV);
        ACE_TRY_CHECK;

	//
	// Create ComponentConfigurator (Component Configurator Factory)
	//
	ccf_cc_impl_ = add_component("CCFactory",
                                    "CCFactory",
                                    CORBA::Object::_duplicate(cc_factory_),
                                    ACE_TRY_ENV);
        ACE_TRY_CHECK;

	//
	// Create ComponentConfigurator (Component Repository)
	//
	cr_cc_impl_ = add_component("ComponentRepository",
                                   "ComponentRepository",
                                   CORBA::Object::_duplicate(component_repos_),
                                   ACE_TRY_ENV);
        ACE_TRY_CHECK;

  }
  ACE_CATCH (CORBA::SystemException, sysex)
    {
      ACE_UNUSED_ARG (sysex);
      ACE_TRY_ENV.print_exception ("System Exception");
      exit(1);
    }
  ACE_CATCH (CORBA::UserException, userex)
    {
      ACE_UNUSED_ARG (userex);
      ACE_TRY_ENV.print_exception ("User Exception");
      exit(1);
    }
  ACE_ENDTRY;

}

int
BaseCC::load_component(const char *hook_name,
                       const char *category,
                       const char *component,
                       const char *args)
{
#if 1
  ACE_OS::printf("BaseCC::load_component -- %s %s %s\n",
                  category,
                  component,
                  args);
#endif

  ACE_TRY_NEW_ENV {
    TwoK2::CompSpec compSpec; // MOD
    TwoK2::Name specName; // MOD
    //    EnvironmentService::CompSpec compSpec;
    //    EnvironmentService::Name specName;

	specName.length (3);
	specName[0] = CORBA::string_dup (category);
	specName[1] = CORBA::string_dup (component);
	specName[2] = CORBA::string_dup (args);
	compSpec.comp_name (specName);

	TwoK::Configuration::ComponentConfigurator_ptr cc_;
	cc_ = auto_conf_->load_component(component_repos_.in(), compSpec,
                                             cc_factory_.in(),
                                             ACE_TRY_ENV);
	ACE_TRY_CHECK;

	domain_cc_impl_->addHook(hook_name, ACE_TRY_ENV);
        ACE_TRY_CHECK;

	domain_cc_impl_->hook(hook_name, cc_, ACE_TRY_ENV);
	ACE_TRY_CHECK;
  }
  ACE_CATCH (TwoK2::AutoConfig::NotFound, not_found)
    {
      ACE_UNUSED_ARG (not_found);
      ACE_TRY_ENV.print_exception ("Not found");
      ACE_OS::printf("Component loading is failed: Not Found\n");
      return(-1);
    }
  ACE_CATCH (TwoK2::AutoConfig::LoadFailure, loadfailure)
    {
      ACE_UNUSED_ARG (loadfailure);
      ACE_TRY_ENV.print_exception ("Load Failure");
      ACE_OS::printf("Component loading is failed: LoadFailure\n");
      return(-1);
    }
  ACE_CATCH (CORBA::SystemException, sysex)
  ACE_CATCH (CORBA::SystemException, sysex)
    {
      ACE_UNUSED_ARG (sysex);
      ACE_TRY_ENV.print_exception ("System Exception");
      ACE_OS::printf("Component loading is failed\n");
      ACE_OS::printf("Component loading is failed\n");
      exit(1);
    }
  ACE_CATCH (CORBA::UserException, userex)
    {
      ACE_UNUSED_ARG (userex);
      ACE_TRY_ENV.print_exception ("User Exception");
      ACE_OS::printf("Component loading is failed\n");
      ACE_OS::printf("Component loading is failed\n");
      ACE_OS::printf("Component loading is failed\n");
      ACE_OS::printf("Component loading is failed\n");
      ACE_OS::printf("Component loading is failed\n");
      ACE_OS::printf("Component loading is failed\n");
      ACE_OS::printf("Component loading is failed\n");
      ACE_OS::printf("Component loading is failed\n");
      ACE_OS::printf("Component loading is failed\n");
      exit(1);
    }
  ACE_ENDTRY;

  return(0);
}

