// -*- C++ -*-
// $Id: Cloakroom_Helper.cpp,v 1.1 2000/02/19 20:31:00 yamane Exp $

// ============================================================================
//
// = LIBRARY
//    RMP/Lrm
//
// = FILENAME
//    Cloakroom_Helper.cpp
//
// = DESCRIPTION
//    This class implements the Cloakroom analogy
//
// = AUTHOR
//    Tomonori Yamane <yamane@uiuc.edu>
//
// ============================================================================

#include "ace/Synch.h"

#include "Cloakroom_Helper.h"

  // Constructor

template <class ITEM>
Cloakroom_Helper<ITEM>::Cloakroom_Helper(void)
{
	key_candidate = 1;
}

  // Copy Constructor added because g++ generates a 
  // bad one if not defined

template <class ITEM>
Cloakroom_Helper<ITEM>::Cloakroom_Helper(Cloakroom_Helper<ITEM> &a)
{
	ACE_UNUSED_ARG(a);
}

  // Destructor

template <class ITEM>
Cloakroom_Helper<ITEM>::~Cloakroom_Helper()
{
}

  // Check in the item and receive a unique tag

template <class ITEM>
long
Cloakroom_Helper<ITEM>::checkin(ITEM &item)
{
	long initial = key_candidate;
	//
	// find next key
	//
//	ACE_OS::printf("key_candidate = %d\n", key_candidate);

	while (1) {
		//
		// we should avoid corner case number (-1, 0, 1)
		//
		if (key_candidate < 2) {
			key_candidate = 2;
		}

		//
		//  make sure candidate is not used now
		//
		if (map.find(key_candidate) == -1) break;

		key_candidate++;

		if (key_candidate == initial) {
		//	we wrap around (not likely happen though)
//			ACE_OS::printf("We cannot find tag!!\n");
		}
	}

//	ACE_OS::printf("final key = %d\n", key_candidate);

	map.bind(key_candidate, item);

//	ACE_OS::printf("bound to the key\n");

	return(key_candidate++);
}

  // Pass the tag and read the item without checking out it

template <class ITEM>
ITEM
Cloakroom_Helper<ITEM>::lookup(long tag)
{
	ITEM h;
	int result;

	result = map.find(tag, h);

//	printf("Cloakroom_Helper::lookup(%ld) result = %d\n", tag, result);

	return(h);
}

  // Pass the tag and check out the item

template <class ITEM>
ITEM
Cloakroom_Helper<ITEM>::checkout(long tag)
{
	ITEM h;
	int result;

	result = map.find(tag, h);

//	printf("Cloakroom_Helper::lookup(%ld) result = %d\n", tag, result);

	map.unbind(tag);

	return(h);
}

  // Display all item

template <class ITEM>
void
Cloakroom_Helper<ITEM>::display_all()
{
        int i = 0;

	ITERATOR end = map.end ();
        for (ITERATOR iter = map.begin ();
                iter != end;
                ++iter)
        {
		ENTRY &entry = *iter;
                ACE_DEBUG ((LM_DEBUG,
                    "[%d] %d\n",
                    i,
                    entry.ext_id_));

                i++;
        }
}

