/******************************************************************************
*                                                                             *
*  File: CPUAPI_i.cpp                                                         *
*                                                                             *
*  Description: CORBA Wrapper for DSRT API                                          *
*                                                                             *
*  Project: 2K - Resource management                                          *
*  Author: Tomonori Yamane (yamane@uiuc.edu)                                  *
*                                                                             *
*******************************************************************************
 Revision Control Information
 $Revision: 1.2 $
 $Log: CpuApi_i.cpp,v $
 Revision 1.2  2000/02/24 06:00:41  yamane
 Bug fix for SunOS5.6

 Revision 1.1  2000/02/19 20:31:00  yamane
 Initial version


******************************************************************************/

#if !defined (CC_CPUAPI_I_CC)
#define CC_DSRT_I_CC

#include "CpuApi_i.h"

CpuApi_i::CpuApi_i(::CpuApi *cpu)
{
	cpu_ = cpu;
}

CpuApi_i::~CpuApi_i()
{
}

CORBA::Long
CpuApi_i::reserve (
        const ResourceMgmt::Cpu::CpuReserve& cr,
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	::CpuReserve cr_;
	CORBA::Long ret = 0;

	cr_.serviceClass = cr.serviceClass;
	cr_.period = cr.period;
	cr_.peakProcessingTime = cr.peakProcessingTime;
	cr_.sustainableProcessingTime = cr.sustainableProcessingTime;
	cr_.burstTolerance = cr.burstTolerance;
	cr_.peakProcessingUtil = cr.peakProcessingUtil;

	ret = cpu_->reserve(&cr_);

	return(ret);
}


CORBA::Long
CpuApi_i::modify (
        const ResourceMgmt::Cpu::CpuReserve& cr,
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	::CpuReserve cr_;
	CORBA::Long ret = 0;

	cr_.serviceClass = cr.serviceClass;
	cr_.period = cr.period;
	cr_.peakProcessingTime = cr.peakProcessingTime;
	cr_.sustainableProcessingTime = cr.sustainableProcessingTime;
	cr_.burstTolerance = cr.burstTolerance;
	cr_.peakProcessingUtil = cr.peakProcessingUtil;

	ret = cpu_->modify(&cr_);

	return(ret);
}

CORBA::Long
CpuApi_i::free (
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	CORBA::Long ret = 0;

	ret = cpu_->free();

	return(ret);
}

CORBA::Long
CpuApi_i::start (
        const ResourceMgmt::Cpu::timeval & begin,
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	::timeval begin_;
	CORBA::Long ret = 0;

	begin_.tv_sec = begin.tv_sec;
	begin_.tv_usec = begin.tv_usec;

	ret = cpu_->start(begin_);

	return(ret);
}

CORBA::Long
CpuApi_i::stop (
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	CORBA::Long ret = 0;

	ret = cpu_->stop();

	return(ret);
}

////// ????????????????????
CORBA::Long
CpuApi_i::destroy (
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	CORBA::Long ret = 0;
	return(ret);
}

void
CpuApi_i::yield (
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	cpu_->yield();
}

void
CpuApi_i::svcDestroy (
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	cpu_->svcDestroy();
}

CORBA::Long
CpuApi_i::setAdaptStrategy (
        const ResourceMgmt::Cpu::AdaptStrategy & as,
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	::AdaptStrategy as_;

	CORBA::Long ret = 0;

	as_.strategy = as.strategy;
	as_.windowSize = as.windowSize;
	as_.alpha = as.alpha;
	as_.max = as.max;
	as_.min = as.min;
	as_.nonconformFreq = as.nonconformFreq;

	ret = cpu_->setAdaptStrategy(&as_);

	return(ret);
}

CORBA::Long
CpuApi_i::setApDeadline (
        CORBA::Long deadline,
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	CORBA::Long ret = 0;

	int deadline_;

	deadline_ = deadline;

	ret = cpu_->setApDeadline(deadline_);

	return(ret);
}

CORBA::Long
CpuApi_i::probe (
        CORBA::Long period,
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	CORBA::Long ret = 0;

	int period_;

	period_ = period;

	ret = cpu_->probe(period_);

	return(ret);
}

CORBA::Long
CpuApi_i::probeEnd (
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	CORBA::Long ret = 0;

	ret = cpu_->probeEnd();

	return(ret);
}

CORBA::Long
CpuApi_i::probeMatch (
        ResourceMgmt::Cpu::CpuReserve_out cr,
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	::CpuReserve cr_;
	CORBA::Long ret = 0;

	ret = cpu_->probeMatch(&cr_);

	if (ret != 0) {
		cr.serviceClass = cr_.serviceClass;
		cr.period = cr_.period;
		cr.peakProcessingTime = cr_.peakProcessingTime;
		cr.sustainableProcessingTime = cr_.sustainableProcessingTime;
		cr.burstTolerance = cr_.burstTolerance;
		cr.peakProcessingUtil = cr_.peakProcessingUtil;
	}

	return(ret);
}

/// ????????
CORBA::Long
CpuApi_i::profileSave (
        const ResourceMgmt::Cpu::Args & args,
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	CORBA::Long ret = 0;
	return(ret);
}

CORBA::Long
CpuApi_i::profileGet (
        const ResourceMgmt::Cpu::Args & args,
        CORBA::Float_out util,
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	CORBA::Long ret = 0;

	
	return(ret);
}

CORBA::Long
CpuApi_i::setRecordProcStats (
        const char * fname,
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	CORBA::Long ret = 0;

	ret = cpu_->setRecordProcStats((char *)fname);
	return(ret);
}

CORBA::Long
CpuApi_i::setRecordSchedStats (
        const char * fname,
        CORBA::Long samplingInterval,
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	CORBA::Long ret = 0;

	ret = cpu_->setRecordSchedStats((char *)fname, samplingInterval);
	return(ret);
}

ResourceMgmt::Cpu::ProcStats
CpuApi_i::getProcStats (
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	ResourceMgmt::Cpu::ProcStats ps;

	::ProcStats *ps_;

	ps_ = cpu_->getProcStats();

	if (ps_ == NULL) {
		/// ???? raise exception ???
	} else {
		ps.clock.tv_sec = ps_->clock.tv_sec;
		ps.clock.tv_usec = ps_->clock.tv_usec;
		ps.usage = ps_->usage;
		ps.nonconformFlag = ps_->nonconformFlag;
		ps.overrunFlag = ps_->overrunFlag;
		ps.cpuReserve.serviceClass = ps_->cpuReserve.serviceClass;
		ps.cpuReserve.period = ps_->cpuReserve.period;
		ps.cpuReserve.peakProcessingTime = ps_->cpuReserve.peakProcessingTime;
		ps.cpuReserve.sustainableProcessingTime = ps_->cpuReserve.sustainableProcessingTime;
		ps.cpuReserve.burstTolerance = ps_->cpuReserve.burstTolerance;
		ps.cpuReserve.peakProcessingUtil = ps_->cpuReserve.peakProcessingUtil;
		ps.apDeadline =  ps_->apDeadline;
		ps.permNonconformFlag =  ps_->permNonconformFlag;
		ps.overrunItr =  ps_->overrunItr;
		ps.accUsage.tv_sec =  ps_->accUsage.tv_sec;
		ps.accUsage.tv_usec =  ps_->accUsage.tv_usec;
		ps.accNumNonconform =  ps_->accNumNonconform;
		ps.accNumOverrun =  ps_->accNumOverrun;
		ps.accNumPermNonconform =  ps_->accNumPermNonconform;
		ps.accNumDeadlineMiss =  ps_->accNumDeadlineMiss;
		ps.accItr =  ps_->accItr;

	}

	return(ps);
}

ResourceMgmt::Cpu::SchedStats
CpuApi_i::getSchedStats (
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	ResourceMgmt::Cpu::SchedStats sc;

	::SchedStats *sc_;

	sc_ = cpu_->getSchedStats();

	sc.clock.tv_sec = sc_->clock.tv_sec;
	sc.clock.tv_usec = sc_->clock.tv_usec;
	sc.numProcessor = sc_->numProcessor;
	sc.availPermUtil = sc_->availPermUtil;
	sc.overrunRatio = sc_->overrunRatio;
	sc.permRatio = sc_->permRatio;
	sc.tsRatio = sc_->tsRatio;
	
	sc.permUsage.tv_sec = sc_->permUsage.tv_sec;
	sc.permUsage.tv_usec = sc_->permUsage.tv_usec;

	sc.burstUsage.tv_sec = sc_->burstUsage.tv_sec;
	sc.burstUsage.tv_usec = sc_->burstUsage.tv_usec;

	sc.nonconformUsage.tv_sec = sc_->nonconformUsage.tv_sec;
	sc.nonconformUsage.tv_usec = sc_->nonconformUsage.tv_usec;

	sc.probeUsage.tv_sec = sc_->probeUsage.tv_sec;
	sc.probeUsage.tv_usec = sc_->probeUsage.tv_usec;

	sc.tsUsage.tv_sec = sc_->tsUsage.tv_sec;
	sc.tsUsage.tv_usec = sc_->tsUsage.tv_usec;

	return(sc);
}

CORBA::Long
CpuApi_i::isPermNonconform (
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

	CORBA::Long ret = 0;

	ret = cpu_->isPermNonconform();
	return(ret);
}

ResourceMgmt::Cpu::Process
CpuApi_i::getProcess (
        CORBA::Environment &ACE_TRY_ENV
      )
      ACE_THROW_SPEC ((
        CORBA::SystemException
      ))
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);
	ResourceMgmt::Cpu::Process ps;
	::Process *ps_;

	ps_ = cpu_->getProcess();
	
	ps.idx = ps_->idx;
	ps.pid = ps_->pid;
	ps.euid = ps_->euid;
	ps.egid = ps_->egid;
	ps.state = ps_->state;
	ps.cpu = ps_->cpu;

	ps.cpuReserve.serviceClass = ps_->cpuReserve.serviceClass;
	ps.cpuReserve.period = ps_->cpuReserve.period;
	ps.cpuReserve.peakProcessingTime = ps_->cpuReserve.peakProcessingTime;
	ps.cpuReserve.sustainableProcessingTime = ps_->cpuReserve.sustainableProcessingTime;
	ps.cpuReserve.burstTolerance = ps_->cpuReserve.burstTolerance;
	ps.cpuReserve.peakProcessingUtil = ps_->cpuReserve.peakProcessingUtil;

	ps.adaptStrategy.strategy = ps_->adaptStrategy.strategy;
	ps.adaptStrategy.windowSize = ps_->adaptStrategy.windowSize;
	ps.adaptStrategy.alpha = ps_->adaptStrategy.alpha;
	ps.adaptStrategy.max = ps_->adaptStrategy.max;
	ps.adaptStrategy.min = ps_->adaptStrategy.min;
	ps.adaptStrategy.nonconformFreq = ps_->adaptStrategy.nonconformFreq;

	ps.begin.tv_sec = ps_->begin.tv_sec;
	ps.begin.tv_usec = ps_->begin.tv_usec;

	ps.guaranteed = ps_->guaranteed;

	ps.probeFlag = ps_->probeFlag;
	ps.usage = ps_->usage;
	ps.usageOverrunItr = ps_->usageOverrunItr;
	ps.overrunFlag = ps_->overrunFlag;
	ps.nonconformFlag = ps_->nonconformFlag;
	ps.permNonconformFlag = ps_->permNonconformFlag;
	ps.overrunItr = ps_->overrunItr;

	ps.deadline.tv_sec = ps_->deadline.tv_sec;
	ps.deadline.tv_usec = ps_->deadline.tv_usec;

	ps.accUsage.tv_sec = ps_->accUsage.tv_sec;
	ps.accUsage.tv_usec = ps_->accUsage.tv_usec;

	ps.accNumNonconform = ps_->accNumNonconform;
	ps.accNumOverrun = ps_->accNumOverrun;
	ps.accNumPermNonconform = ps_->accNumPermNonconform;
	ps.accNumDeadlineMiss = ps_->accNumDeadlineMiss;
	ps.accItr = ps_->accItr;

	ps.yieldFlag = ps_->yieldFlag;
	ps.resetContSemFlag = ps_->resetContSemFlag;

	ps.ps.clock.tv_sec = ps_->ps.clock.tv_sec;
	ps.ps.clock.tv_usec = ps_->ps.clock.tv_usec;
	ps.ps.usage = ps_->ps.usage;
	ps.ps.nonconformFlag = ps_->ps.nonconformFlag;
	ps.ps.overrunFlag = ps_->ps.overrunFlag;
	ps.ps.cpuReserve.serviceClass = ps_->ps.cpuReserve.serviceClass;
	ps.ps.cpuReserve.period = ps_->ps.cpuReserve.period;
	ps.ps.cpuReserve.peakProcessingTime = ps_->ps.cpuReserve.peakProcessingTime;
	ps.ps.cpuReserve.sustainableProcessingTime = ps_->ps.cpuReserve.sustainableProcessingTime;
	ps.ps.cpuReserve.burstTolerance = ps_->ps.cpuReserve.burstTolerance;
	ps.ps.cpuReserve.peakProcessingUtil = ps_->ps.cpuReserve.peakProcessingUtil;
	ps.ps.apDeadline =  ps_->ps.apDeadline;
	ps.ps.permNonconformFlag =  ps_->ps.permNonconformFlag;
	ps.ps.overrunItr =  ps_->ps.overrunItr;
	ps.ps.accUsage.tv_sec =  ps_->ps.accUsage.tv_sec;
	ps.ps.accUsage.tv_usec =  ps_->ps.accUsage.tv_usec;
	ps.ps.accNumNonconform =  ps_->ps.accNumNonconform;
	ps.ps.accNumOverrun =  ps_->ps.accNumOverrun;
	ps.ps.accNumPermNonconform =  ps_->ps.accNumPermNonconform;
	ps.ps.accNumDeadlineMiss =  ps_->ps.accNumDeadlineMiss;
	ps.ps.accItr =  ps_->ps.accItr;

	ps.procFd = ps_->procFd;
	ps.wrShmAreaFd = ps_->wrShmAreaFd;

	return(ps);
}

#endif /* CC_CPUAPI_I_CC */
