/******************************************************************************
*                                                                             *
*  File: DSRT_Helper.cpp                                                      *
*                                                                             *
*  Description: Wrapper for DSRT API                                          *
*                                                                             *
*  Project: 2K - Resource management                                          *
*  Author: Tomonori Yamane (yamane@uiuc.edu)                                  *
*                                                                             *
*******************************************************************************
 Revision Control Information
 $Revision: 1.1 $
 $Log: DSRT_Helper.cpp,v $
 Revision 1.1  2000/02/19 20:31:00  yamane
 Initial version

 Revision 1.1.1.1  1999/08/30 04:41:29  col
 Initial check-in


 Revision 1.1  1999/02/22 22:29:16  f-kon
 *** empty log message ***


******************************************************************************/

#if !defined (CC_DSRT_HELPER_CC)
#define CC_DSRT_HELPER_CC

#include "DSRT_Helper.h"

CpuApi* DSRT_Helper::instance_ = 0;

static  CORBA::Boolean is = 0;

static  CpuReserve cr;
static  AdaptStrategy as;

CpuApi *
DSRT_Helper::instance(void)
{
	if (instance_) {
		return instance_;
        }

        int _fd = msgget(MSG_QUEUE_KEY, 0);
        if (_fd < 0) {
                //   DSRT not started
		return (CpuApi *)0;
        } else {
                //   DSRT started, OK to call CpuApi
                ACE_OS::printf("DSRT detected\n");
		instance_ = new CpuApi();
        }

	return instance_;
}

DSRT_Helper::DSRT_Helper()
{
}

int
DSRT_Helper::parse(CosTrading::PropertySeq& rs)
{
  CORBA::String_var serviceClass;
  CORBA::Long period = 0;
  CORBA::Long peakProcessingTime = 0;
  CORBA::Long sustainableProcessingTime;
  CORBA::Long burstTolerance;
  CORBA::Float peakProcessingUtil;
  CORBA::String_var strategy;
  CORBA::Long windowSize;
  CORBA::Float alpha;
  CORBA::Float max;
  CORBA::Float min;
  CORBA::Float nonconformFreq;

	char *s;
	for (CORBA::ULong i = 0 ; i < rs.length(); i++) {
		if (strcmp(rs[i].name.in(), "DSRT_serviceClass")==0) {
			s = 0;
			rs[i].value >>= s;
			if (strcmp(s, "CPU_RT_PCPT")==0) {
				cr.serviceClass = CPU_RT_PCPT;
			} else if (strcmp(s, "CPU_RT_PVPT")==0) {
				cr.serviceClass = CPU_RT_PVPT;
			} else if (strcmp(s, "CPU_RT_ACPU")==0) {
				cr.serviceClass = CPU_RT_ACPU;
			}
		}
//
//	CPU_RT_PCPT
//
		else
		if (strcmp(rs[i].name.in(), "DSRT_period")==0) {
			rs[i].value >>= period;
			cr.period = period * 1000;
		}
		else
		if (strcmp(rs[i].name.in(), "DSRT_peakProcessingTime")==0) {
			rs[i].value >>= peakProcessingTime;
			cr.peakProcessingTime = peakProcessingTime * 1000;
		}
		else
		if (strcmp(rs[i].name.in(), "DSRT_sustainableProcessingTime")
			==0) {
			rs[i].value >>= sustainableProcessingTime;
			cr.sustainableProcessingTime 
				= sustainableProcessingTime * 1000;
		}
		else
		if (strcmp(rs[i].name.in(), "DSRT_burstTolerance")==0) {
			rs[i].value >>= burstTolerance;
			cr.burstTolerance = burstTolerance * 1000;
		}
//
//	CPU_RT_ACPU
//
		else
		if (strcmp(rs[i].name.in(), "DSRT_peakProcessingUtil")==0) {
			rs[i].value >>= peakProcessingUtil;
			cr.peakProcessingUtil = peakProcessingUtil;
		}
//
//	AdaptStrategy
//
		else
		if (strcmp(rs[i].name.in(), "DSRT_AdaptStrategy")==0) {
			s = 0;
			rs[i].value >>= s;
			if (strcmp(s, "ADAPT_EXPO")==0) {
				as.strategy = ADAPT_EXPO;
			} else if (strcmp(s, "ADAPT_EXPO_UP")==0) {
				as.strategy = ADAPT_EXPO_UP;
			} else if (strcmp(s, "ADAPT_EXPO_RANGE")==0) {
				as.strategy = ADAPT_EXPO_RANGE;
			} else if (strcmp(s, "ADAPT_STAT")==0) {
				as.strategy = ADAPT_EXPO_RANGE;
			} else {
				as.strategy = ADAPT_NONE;
			}
		}
		else
		if (strcmp(rs[i].name.in(), "DSRT_windowSize")==0) {
			rs[i].value >>= windowSize;
			as.windowSize = windowSize;
		}
		else
		if (strcmp(rs[i].name.in(), "DSRT_alpha")==0) {
			rs[i].value >>= alpha;
			as.alpha = alpha;
		}
		else
		if (strcmp(rs[i].name.in(), "DSRT_max")==0) {
			rs[i].value >>= max;
			as.alpha = max;
		}
		else
		if (strcmp(rs[i].name.in(), "DSRT_min")==0) {
			rs[i].value >>= min;
			as.alpha = min;
		}
		else
		if (strcmp(rs[i].name.in(), "DSRT_nonconformFreq")==0) {
			rs[i].value >>= nonconformFreq;
			as.nonconformFreq = nonconformFreq;
		}
		else
		if (strcmp(rs[i].name.in(), "DSRT_CpuReserve")==0) {
			is = 1;
		}
        }
	return 0;
}

int
DSRT_Helper::reserve(void)
{
	if (!instance()) {
		return -1;
	}

	if (is) {
		if (!instance_->reserve(&cr, getpid())) {
			return -1;
		}

		if (!instance_->setAdaptStrategy(&as, getpid())) {
			return -1;
		}
	}
	return 0;
}

#endif /* CC_DSRT_HELPER_CC */
