/******************************************************************************
*                                                                             *
*  File: Loader_i.cpp                                                         *
*                                                                             *
*  Description: A server for executing service                                *
*               Based on the TAO Cubit example.                               *
*                                                                             *
*  Project: 2K - Resource Management                                          *
*  Author: Tomonori Yamane (yamane@uiuc.edu)                                *
*                                                                             *
*******************************************************************************
 Revision Control Information
 $Revision: 1.2 $
 $Log: Loader_i.cpp,v $
 Revision 1.2  2000/09/30 18:38:33  yamane
 Patches for Windows

 Revision 1.1  2000/02/19 20:31:00  yamane
 Initial version

 Revision 1.1.1.1  1999/08/30 04:41:29  col
 Initial check-in

 Revision 1.1.1.1  1999/04/01 18:16:37  f-kon
 This new 2K source tree contains the most up-to-date version of ACE and TAO
 and including the dynamicTAO code as well as some 2K tests and preliminary
 versions of some of its high-level systems.


 Revision 1.1  1999/02/22 22:29:16  f-kon
 *** empty log message ***

 Revision 1.1  1999/02/15 18:03:19  f-kon
 *** empty log message ***


******************************************************************************/

#include "Loader_i.h"

#include "Local_DB.h"

#if defined(HAS_DSRT2)
#include "DSRT_Helper.h"
#include "CpuApi_i.h"
#endif

#include <orbsvcs/Trader/Trader_Utils.h>

#if defined(HAS_AUTOCONFIG)
#include "2k/ComponentConfigurator/Configuration_i.h"
#endif

// #include "P_Timer.h"

CosNaming::NamingContext_ptr
Loader_i::create_context (const CosNaming::NamingContext_ptr inc,
                        const CosNaming::Name name)
{
	CosNaming::Name part_name;
	CosNaming::NamingContext_var nc;

        CORBA::Object_var obj;

        CORBA::Environment ACE_TRY_ENV;

//        ACE_OS::printf("create_context length = %d\n", name.length());

        for (CORBA::ULong i = 0; i < name.length(); i++) {
//                ACE_OS::printf("create context\n");

                part_name = name;
                part_name.length(i + 1);

                ACE_TRY {
                        nc = inc->bind_new_context(part_name, ACE_TRY_ENV);
                        ACE_TRY_CHECK;
                }
                ACE_CATCH (CosNaming::NamingContext::AlreadyBound, ex) {
			ACE_OS::printf("create_context() already bound\n");
			ACE_TRY_ENV.clear();
                        obj = inc->resolve(part_name);
                        nc = CosNaming::NamingContext::_narrow(obj);
                }
		ACE_CATCH (CORBA::SystemException, sysex)
		{
			ACE_UNUSED_ARG (sysex);
			ACE_OS::printf("create_context() system exception\n");
			ACE_TRY_ENV.print_exception ("System Exception");
			exit(1);
		}
		ACE_CATCH (CORBA::UserException, userex)
		{
			ACE_UNUSED_ARG (userex);
			ACE_OS::printf("create_context() user exception\n");
			ACE_TRY_ENV.print_exception ("User Exception");
			exit(1);
		}
                ACE_ENDTRY;
        }

//        ACE_OS::printf("create context ends\n");

        return nc._retn();
}

// Constructor
void
Loader_i::register_name (CORBA::Object *ptr)
{
	char *str = orb_->object_to_string(ptr);
//	ACE_OS::printf("Loader_i::register_name %s\n", str);
	CORBA::string_free(str);

	char buf[512];
	ACE_OS::hostname(buf, 511);

        //
        // Find nameserver
        //
        CORBA::Object_var obj;

	for (int i = 0 ; i < 3; ++i) {
        	obj = orb_->resolve_initial_references("NameService");
       		if (CORBA::is_nil(obj)) {
               		ACE_OS::printf("NamingService is nil. Try again (newComp)\n");
			ACE_OS::sleep(1);
		} else {
//        		ACE_OS::printf("NamingService resolved\n");
			break;
		}
	}

       	if (CORBA::is_nil(obj)) {
               	ACE_OS::printf("NamingService is nil.\n");
		exit(0);
               	return;
       	}

        CosNaming::NamingContext_var inc, nc;
        inc = CosNaming::NamingContext::_narrow(obj);


        CosNaming::Name name;
        name.length(3);
        name[0].id = CORBA::string_dup("local");
        name[0].kind = CORBA::string_dup("");
        name[1].id = CORBA::string_dup("machines");
        name[1].kind = CORBA::string_dup("");
        name[2].id = CORBA::string_dup(buf); // hostname
        name[2].kind = CORBA::string_dup("");

        nc = create_context(inc, name);

        name.length(1);
	ACE_OS::sprintf(buf, "newComponent:%d", getpid());
        name[0].id = CORBA::string_dup(buf); // process id
        name[0].kind = CORBA::string_dup("");

	nc->rebind(name, CORBA::Object::_duplicate(ptr));
}

// Constructor
Loader_i::Loader_i (void)
{
//  ACE_OS::printf("pid = %d\n", getppid());
  // no-op
}

// Destructor
Loader_i::~Loader_i (void)
{
  // no-op
}

//
//	send NACK
//
void
Loader_i::send_nack()
{
//	ACE_OS::printf("Loader_i::send_nack()\n");

	ResourceMgmt::Grm_var Grm_ = ResourceMgmt::Grm::_duplicate(req.grm_.in());
	ResourceMgmt::QueryPacket copy_req = req;
	copy_req.visited = Lrm_impl_->mark_here(req.visited.in());

//        ACE_OS::printf("Issue request to the remote\n");

        CORBA::Environment ACE_TRY_ENV;

	ACE_TRY {
        	if (Grm_.in() == Grm_->_nil()) {
	        	// Grm is not available. We should fail.
			ACE_OS::printf("Grm is not available. We should fail.\n");
			ResourceMgmt::Lrm_var Lrm_ = ResourceMgmt::Lrm::_duplicate(req.sender.in());

                	Lrm_->NACK(req.sem_tag, ACE_TRY_ENV);
		} else {
			Grm_->perform_query(copy_req , ACE_TRY_ENV);
		}
		ACE_TRY_CHECK;
	}
	ACE_CATCH (CORBA::SystemException, sysex)
	{
		ACE_UNUSED_ARG (sysex);
		ACE_OS::printf("send_nack() system exception\n");
		ACE_TRY_ENV.print_exception ("System Exception");
		exit(1);
	}
	ACE_CATCH (CORBA::UserException, userex)
	{
		ACE_UNUSED_ARG (userex);
		ACE_OS::printf("send_nack() user exception\n");
		ACE_TRY_ENV.print_exception ("User Exception");
		exit(1);
	}
	ACE_ENDTRY;
}


//
//	send ACK
//
void
Loader_i::send_ack(const CosTrading::OfferSeq &offers)
{
//	ACE_OS::printf("Loader_i::send_ack(%d)\n", req.sem_tag);

	ResourceMgmt::Lrm_var Lrm_ = ResourceMgmt::Lrm::_duplicate(req.sender.in());

//	ACE_OS::printf("Loader_i::send_ack() sender setup\n");

        CORBA::Environment ACE_TRY_ENV;

	ACE_TRY {
		Lrm_->ACK(req.sem_tag, offers, ACE_TRY_ENV);
//	ACE_OS::printf("Loader_i::send_ack() almost done\n");
		ACE_TRY_CHECK;
	}
	ACE_CATCH (CORBA::SystemException, sysex)
	{
		ACE_UNUSED_ARG (sysex);
		ACE_OS::printf("send_ack() system exception\n");
		ACE_TRY_ENV.print_exception ("System Exception");
		exit(1);
	}
	ACE_CATCH (CORBA::UserException, userex)
	{
		ACE_UNUSED_ARG (userex);
		ACE_OS::printf("send_ack() user exception\n");
		ACE_TRY_ENV.print_exception ("User Exception");
		exit(1);
	}
	ACE_ENDTRY;
}

void
Loader_i::setup_Lrm()
{
  // Create Lrm_i
  //
	Lrm_impl_ = new Lrm_i (argc_, argv_);

	// Set orb
        Lrm_impl_->orb(orb_.in());

  ACE_TRY_NEW_ENV {
//	ACE_OS::printf("Loader_i::setup_Lrm\n");

	// activate servant
	// ???
	CORBA::String_var str  =
	    orb_manager_.activate_under_child_poa ("LrmWorld",
	       	                                    Lrm_impl_,
	                                           ACE_TRY_ENV);
	ACE_TRY_CHECK;

//	Lrm_impl_->self_ = str;
	CORBA::Object_var obj_ = orb_->string_to_object(str, ACE_TRY_ENV);
	Lrm_impl_->self_ = ResourceMgmt::Lrm::_duplicate(ResourceMgmt::Lrm::_narrow(obj_.in()));
	ACE_TRY_CHECK;
  }
  ACE_CATCH (CORBA::SystemException, sysex)
    {
      ACE_UNUSED_ARG (sysex);
      ACE_TRY_ENV.print_exception ("System Exception");
      exit(1);
    }
  ACE_CATCH (CORBA::UserException, userex)
    {
      ACE_UNUSED_ARG (userex);
      ACE_TRY_ENV.print_exception ("User Exception");
      exit(1);
    }
  ACE_ENDTRY;
}

void
Loader_i::setup_Lrm_from_req()
{
//  ACE_OS::printf("Loader_i::setup_Lrm_from_req\n");

 	// Set Grm so far
  	Lrm_impl_->Grm(req.grm_.in());

}

void
Loader_i::setup_Lrm_from_arg()
{
  ACE_TRY_NEW_ENV {
//	ACE_OS::printf("Loader_i::setup_Lrm_from_arg\n");

  	//
	// Interpreting arguments
	//
  	Lrm_impl_->compose_packet(&req, argc_, argv_, ACE_TRY_ENV);
	ACE_TRY_CHECK;
 
  	// Set Grm so far
  	Lrm_impl_->Grm(req.grm_.in());

  }
  ACE_CATCH (CORBA::SystemException, sysex)
    {
      ACE_UNUSED_ARG (sysex);
      ACE_TRY_ENV.print_exception ("System Exception");
      exit(1);
    }
  ACE_CATCH (CORBA::UserException, userex)
    {
      ACE_UNUSED_ARG (userex);
      ACE_TRY_ENV.print_exception ("User Exception");
      exit(1);
    }
  ACE_ENDTRY;
}

//
//	Setup BaseProcess interface
//
void
Loader_i::setup_BaseProcess()
{
  // Create BaseProcess_i
  //
	BaseProcess_impl_ = new BaseProcess_i (argc_, argv_);

	// Set loader
        BaseProcess_impl_->loader(this);

  ACE_TRY_NEW_ENV {
//	ACE_OS::printf("Loader_i::setup_BaseProcess\n");

	// activate servant
	// ???
	CORBA::String_var str  =
	    orb_manager_.activate_under_child_poa ("BaseProcessWorld",
	       	                                    BaseProcess_impl_,
	                                           ACE_TRY_ENV);
	ACE_TRY_CHECK;

	CORBA::Object_var obj_ = orb_->string_to_object(str, ACE_TRY_ENV);
	BaseProcess_impl_->self_ = ResourceMgmt::BaseProcess::_duplicate(ResourceMgmt::BaseProcess::_narrow(obj_.in()));
	ACE_TRY_CHECK;
  }
  ACE_CATCH (CORBA::SystemException, sysex)
    {
      ACE_UNUSED_ARG (sysex);
      ACE_TRY_ENV.print_exception ("System Exception");
      exit(1);
    }
  ACE_CATCH (CORBA::UserException, userex)
    {
      ACE_UNUSED_ARG (userex);
      ACE_TRY_ENV.print_exception ("User Exception");
      exit(1);
    }
  ACE_ENDTRY;
}

//
// Setup auto configurator
//
#if defined(HAS_AUTOCONFIG)
void
Loader_i::setup_autoconf()
{
  CORBA::ORB_var orb;

  //
  // Create AutoConfig_i
  //
  base_cc_ = new BaseCC (orb_manager_);
}


//
// Setup CORBA component configurator
//
void
Loader_i::setup_components()
{
  CORBA::ORB_var orb;

  ACE_TRY_NEW_ENV {
//  	ACE_OS::printf("Loader_i::setup_components()\n");

#if defined(HAS_DSRT2)
	//
	// Create ComponentConfigurator (DSRT)
	//
	if (!CORBA::is_nil(CpuApi_.in())) {
		dsrt_cc_impl_ = base_cc_->add_component("CpuSvc",
                                                        "DSRT2",
                                                        CpuApi_.in(),
                                                        ACE_TRY_ENV);
	        ACE_TRY_CHECK;
	}

#endif /* HAS_DSRT2 */
  }
  ACE_CATCH (CORBA::SystemException, sysex)
    {
      ACE_UNUSED_ARG (sysex);
      ACE_TRY_ENV.print_exception ("System Exception");
      exit(1);
    }
  ACE_CATCH (CORBA::UserException, userex)
    {
      ACE_UNUSED_ARG (userex);
      ACE_TRY_ENV.print_exception ("User Exception");
      exit(1);
    }
  ACE_ENDTRY;

}

#endif /* HAS_AUTOCONFIG */

int
Loader_i::reserve_resource()
{
  //
  // parse resource specification for other purpose
  //
  CORBA::ULong length = req.resource_spec.length();

//  ACE_OS::printf("Loader_i::reserve_resource() resource_spec.length = %d\n", length);

  if (length == 0) return 0;

  for (CORBA::ULong i = 0 ; i < length; i++) {
//	ACE_OS::printf("name[%d] = %s\n", i, req.resource_spec[i].name.in());
  }

#if defined(HAS_DSRT2)
  //
  // parse resource specification (DSRT)
  //
  DSRT_Helper::parse(req.resource_spec);

  //
  // reserve resource
  //
  // Q: How can I release resource if we cannot reserve resource?
  //
  if (DSRT_Helper::reserve() != 0) {
	// If we cannot reserve resource
	send_nack();
	exit(1);
  }

  if (!DSRT_Helper::instance()) {
	return(0);
  }

  CORBA::ORB_var orb;

  //
  // Create DSRT Object
  //
  CpuApi_impl_ = new CpuApi_i(DSRT_Helper::instance());

  ACE_TRY_NEW_ENV {
//  	ACE_OS::printf("Loader_i::reserve_resource()\n");

        // activate servant
        // ???
        CORBA::String_var str  =
            orb_manager_.activate_under_child_poa ("CpuApiWorld",
                                                    CpuApi_impl_,
                                                   ACE_TRY_ENV);
        ACE_TRY_CHECK;

	orb = orb_manager_.orb ();

        CORBA::Object_var obj_ = orb_->string_to_object(str, ACE_TRY_ENV);
        ACE_TRY_CHECK;

        CpuApi_ = ResourceMgmt::Cpu::_duplicate(ResourceMgmt::Cpu::_narrow(obj_.in()));
        ACE_TRY_CHECK;
    }
  ACE_CATCH (CORBA::SystemException, sysex)
    {
      ACE_UNUSED_ARG (sysex);
      ACE_TRY_ENV.print_exception ("System Exception");
      exit(1);
    }
  ACE_CATCH (CORBA::UserException, userex)
    {
      ACE_UNUSED_ARG (userex);
      ACE_TRY_ENV.print_exception ("User Exception");
      exit(1);
    }
  ACE_ENDTRY;

#endif /* HAS_DSRT2 */

  return (0);
}

void
Loader_i::load_component()
{
//  ACE_OS::printf("Loader_i::load_component -- %s %s\n", req.category.in(), req.component.in());

  ACE_TRY_NEW_ENV {
#if defined(HAS_AUTOCONFIG)
	if (base_cc_->load_component("Application",
                                  req.category.in(),
                                  req.component.in(),
                                  req.args.in()) != 0) {
		send_nack();
		exit(1);
	}
#endif /* HAS_AUTOCONFIG */
	ACE_TRY_CHECK;
//      ACE_OS::printf("load done()\n");
  }
  ACE_CATCH (CORBA::SystemException, sysex)
    {
      ACE_UNUSED_ARG (sysex);
      ACE_TRY_ENV.print_exception ("System Exception");
      exit(1);
    }
  ACE_CATCH (CORBA::UserException, userex)
    {
      ACE_UNUSED_ARG (userex);
      ACE_TRY_ENV.print_exception ("User Exception");
      ACE_OS::printf("Service Error -- ?????\n");
      ACE_OS::printf("Service Error -- ?????\n");
      ACE_OS::printf("Service Error -- ?????\n");
      ACE_OS::printf("Service Error -- ?????\n");
      ACE_OS::printf("Service Error -- ?????\n");
      ACE_OS::printf("Service Error -- ?????\n");
      ACE_OS::printf("Service Error -- ?????\n");
      ACE_OS::printf("Service Error -- ?????\n");
      ACE_OS::printf("Service Error -- ?????\n");
      ACE_OS::printf("Service Error -- ?????\n");
      ACE_OS::printf("Service Error -- ?????\n");
      exit(1);
    }
  ACE_ENDTRY;
}

void
Loader_i::compose_offers(CosTrading::OfferSeq& offers)
{
  CORBA::ULong offer_len = 1;
  offers.length(offer_len);

  //
  //
  //	Use TAO_Property_Filter to process desired properties
  //
  //
  CosTrading::Offer offer;

#if defined(HAS_AUTOCONFIG)
  offer.reference = CORBA::Object::_duplicate(base_cc_->CORBA_Object());
#else
//  offer.reference = Lrm_impl_->_this();
//  offer.reference = orb_->string_to_object(Lrm_impl_->self_);
  offer.reference = ResourceMgmt::Lrm::_duplicate(Lrm_impl_->self_);
//  offer.reference = ResourceMgmt::Lrm::_narrow(obj_var);
#endif

//  char *str = orb_->object_to_string(offer.reference);
//  ACE_OS::printf("offer.reference = %s\n", str);
//  CORBA::string_free(str);

  register_name(offer.reference);

  Local_DB::getResources(offer.properties);

  ACE_TRY_NEW_ENV {
  	TAO_Property_Filter ppf(req.desired_props, ACE_TRY_ENV);
	ACE_TRY_CHECK;
	ppf.filter_offer(&offer, offers[0]);
  }
  ACE_CATCH (CosTrading::DuplicatePropertyName,  pname)
  {
	ACE_OS::printf("TAO_Property_Filter() duplicate property name %s\n", pname.name.in());
	ACE_TRY_ENV.print_exception ("Exception");
	ACE_TRY_ENV.clear();
  }
  ACE_CATCH (CosTrading::IllegalPropertyName,  pname2)
  {
	ACE_OS::printf("TAO_Property_Filter() IllegalPropertyName %s\n", pname2.name.in());
	ACE_TRY_ENV.print_exception ("Exception");
	ACE_TRY_ENV.clear();
  }
  ACE_CATCH (CORBA::SystemException, sysex)
  {
	ACE_UNUSED_ARG (sysex);
	ACE_OS::printf("create_context() system exception\n");
	ACE_TRY_ENV.print_exception ("System Exception");
	exit(1);
  }
  ACE_CATCH (CORBA::UserException, userex)
  {
	ACE_UNUSED_ARG (userex);
	ACE_OS::printf("create_context() user exception\n");
	ACE_TRY_ENV.print_exception ("User Exception");
	exit(1);
  }
  ACE_ENDTRY;

#if defined(HAS_DSRT2)
  if (!CORBA::is_nil(CpuApi_.in())) {
	offer_len++;
	offers.length(offer_len);
	offers[offer_len-1].reference
		 = ResourceMgmt::Cpu::_duplicate((CpuApi_.in()));
	offers[offer_len-1].properties = offers[0].properties;
  }
#endif /* HAS_DSRT2 */
}

int
Loader_i::init (int argc,
                                   char** argv,
                                   CORBA::Environment& ACE_TRY_ENV)
{
  argc_ = argc;
  argv_ = argv;

//  ACE_OS::printf("Loader_i:init()\n");
  // Call the init of <TAO_ORB_Manager> to initialize the ORB and
  // create a child POA under the root POA.
  if (orb_manager_.init_child_poa (argc,
                                   argv,
                                   "child_poa",
                                   ACE_TRY_ENV) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "init_child_poa"),
                      -1);

  ACE_CHECK_RETURN (-1);
  //
  // start orb
  orb_ = orb_manager_.orb();

      CORBA::Object_var poa_object =
        orb_->resolve_initial_references("RootPOA");
      if (CORBA::is_nil (poa_object.in ()))
        ACE_ERROR_RETURN ((LM_ERROR,
                           " (%P|%t) Unable to initialize the POA.\n"),
                          1);

      PortableServer::POA_var root_poa =
        PortableServer::POA::_narrow (poa_object.in (), ACE_TRY_ENV);
      ACE_CHECK_RETURN(-1);

      PortableServer::POAManager_var poa_manager =
        root_poa->the_POAManager (ACE_TRY_ENV);
      ACE_CHECK_RETURN(-1);

       poa_manager->activate (ACE_TRY_ENV);
      ACE_CHECK_RETURN(-1);

#if defined(HAS_AUTOCONFIG)
  setup_autoconf();
//  ACE_OS::printf("Loader_i::setup_autoconf() done\n");
#endif

  setup_Lrm();

  if (parse_args(argc, argv) ) {
	// prelaunch BaseProcess

	setup_BaseProcess();

  	CosTrading::OfferSeq offers;
  	offers.length(1);

  	CosTrading::Offer offer;

	offer.reference =  CORBA::Object::_duplicate(BaseProcess_impl_->self_);
	offers[0] = offer;
        send_ack(offers);

	ACE_OS::sleep(3);

 	ACE_OS::printf("prelaunched() done!\n");
	ACE_OS::printf("prelaunched() done!\n");
	ACE_OS::printf("prelaunched() done!\n");
	ACE_OS::printf("prelaunched() done!\n");

	return 0;
  } else {
    // launch BaseProcess on demand
    setup_Lrm_from_arg();


//        CORBA::String_var str  =
//            orb_manager_.activate_under_child_poa ("LrmWorld",
//                                                   Lrm_impl_,
//                                                   ACE_TRY_ENV);
    ACE_OS::printf("setup_configurators done!\n");
	//	ACE_OS::printf("setup_configurators done!\n");
	//	ACE_OS::printf("setup_configurators done!\n");
	//	ACE_OS::printf("setup_configurators done!\n");
	//	ACE_OS::printf("setup_configurators done!\n");

    //
    //	Reserve resource
    //
    reserve_resource();

    //
    //	load component
    //
    //  P_Timer pt;
    //  pt.start();

    load_component();

    ACE_OS::printf("load_component() done!\n"); // teste
//  ACE_OS::printf("load_component() done!\n");
//  ACE_OS::printf("load_component() done!\n");
//  ACE_OS::printf("load_component() done!\n");
//  ACE_OS::printf("load_component() done!\n");

#if defined(HAS_AUTOCONFIG)
    setup_components();
#endif  

//  pt.stop();
//  pt.print();

    //
    //   Composing Offer
    //
    CosTrading::OfferSeq offers;

    compose_offers(offers);

    send_ack(offers);

    ACE_OS::printf("send_ack() done!\n"); // teste
//  ACE_OS::printf("send_ack() done!\n");
//  ACE_OS::printf("send_ack() done!\n");
//  ACE_OS::printf("send_ack() done!\n");
  return 0;
  }
}

int
Loader_i::run (CORBA::Environment& ACE_TRY_ENV)
{
//  ACE_OS::printf("Loader_i::run()\n");

  if (orb_manager_.run (ACE_TRY_ENV) == -1)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "LrmServer::run"),
                      -1);
  return 0;
}

int
Loader_i::parse_args (int argc, char **argv)
{
        ACE_Get_Opt get_opts (argc, argv, "dt:p:y:");

        int c;

        while ((c = get_opts ()) != -1)
        switch (c)
        {
        case 'd':  // debug flag.
                TAO_debug_level++;
                break;
        case 't':  // tag
//		ACE_OS::printf("tag flag\n");
                req.sem_tag = ACE_OS::atoi(get_opts.optarg);
//		ACE_OS::printf("tag flag done\n");
		break;
        case 'p':  // prelaunch flag.
//		ACE_OS::printf("Prelaunch flag\n");
                req.sender = ResourceMgmt::Lrm::_narrow(orb_->string_to_object(get_opts.optarg));
		prelaunch_flag = 1;
		return 1;
		break;
        case 'y': // encoded arguments
		return 0;
        }
	return -1;
}

// ------------------------ implement BaseProcess interface to start newComponent
// Constructor
BaseProcess_i::BaseProcess_i (int argc, char **argv)
{
}

BaseProcess_i::BaseProcess_i (BaseProcess_i &)
{
}

// Destructor
BaseProcess_i::~BaseProcess_i (void)
{
}

void
BaseProcess_i::load_component (
     const ResourceMgmt::QueryPacket & req_,
     CORBA::Environment &ACE_TRY_ENV)
{
//  ACE_OS::printf("BaseProcess::load_component() done!\n");

  loader_->req = req_;
 
  loader_->setup_Lrm_from_req();

  //
  //	Reserve resource
  //
  loader_->reserve_resource();

  //
  //	load component
  //
  loader_->load_component();

//  ACE_OS::printf("load_component() done!\n");
//  ACE_OS::printf("load_component() done!\n");
//  ACE_OS::printf("load_component() done!\n");
//  ACE_OS::printf("load_component() done!\n");
//  ACE_OS::printf("load_component() done!\n");

#if defined(HAS_AUTOCONFIG)
  loader_->setup_components();
#endif  

  //
  //   Composing Offer
  //
  CosTrading::OfferSeq offers;

  loader_->compose_offers(offers);

  loader_->send_ack(offers);

//  ACE_OS::printf("send_ack() done!\n");
//  ACE_OS::printf("send_ack() done!\n");
//  ACE_OS::printf("send_ack() done!\n");
//  ACE_OS::printf("send_ack() done!\n");
}

void
BaseProcess_i::loader (
     Loader_i *ldr
)
{
	loader_ = ldr;
}

