/******************************************************************************
*                                                                             *
*  File: Loader_i.h                                                           *
*                                                                             *
*  Description: A test server for the Loader                                  *
*               Based on the TAO Cubit example.                               *
*                                                                             *
*  Project: 2K - Resource management                                          *
*  Author: Tomonori Yamane (yamane@uiuc.edu)                                  *
*                                                                             *
*******************************************************************************
 Revision Control Information
 $Revision: 1.1 $
 $Log: Loader_i.h,v $
 Revision 1.1  2000/02/19 20:31:01  yamane
 Initial version

 Revision 1.1.1.1  1999/08/30 04:41:29  col
 Initial check-in


 Revision 1.1  1999/02/22 22:29:16  f-kon
 *** empty log message ***


******************************************************************************/

#if !defined (CC_LOADER_I_H)
#define CC_LOADER_I_H

#include "tao/TAO.h"
#include "orbsvcs/CosNamingS.h"
#include "orbsvcs/CosNamingC.h"
#include "orbsvcs/Naming/Naming_Utils.h"

#include "Lrm_i.h"

#if defined(HAS_AUTOCONFIG)
#include "BaseCC.h"
#endif

#if defined(HAS_DSRT2)
#include "CpuApi_i.h"
#endif

class BaseProcess_i;

class Loader_i
{
public:
  // = Initialization and termination methods.
  Loader_i (void);
  // Constructor

  ~Loader_i (void);
  // Destructor

  int init(int argc, char **argv, CORBA::Environment& env);
  // Initialize servant

  int run (CORBA::Environment& env);
  // Run the orb 

  int parse_args(int argc, char **argv);
  // check if prelaunch mode or not

  friend class BaseProcess_i;

private:
  int prelaunch_flag;

  void send_ack(const CosTrading::OfferSeq &offers);
  // send ack to the Lrm

  void send_nack(void);
  // send nack to the Grm

  void setup_Lrm(void);
  // common setup for Lrm

  void setup_Lrm_from_req(void);
  // setup Lrm based on the req structure

  void setup_Lrm_from_arg(void);
  // setup Lrm based on the command line

  void setup_BaseProcess(void);
  // setup BaseProcess

#if defined(HAS_AUTOCONFIG)
  void setup_autoconf(void);
  // setup Auto Configurator based on the command line

  void setup_components(void);
#endif

  int reserve_resource();
  // reserve a resource as specified

  void load_component();
  // load specified component

  CosNaming::NamingContext_ptr
  create_context (const CosNaming::NamingContext_ptr inc,
			const CosNaming::Name name);

  void register_name(CORBA::Object *obj);
  // register newComponent to the name server

  void compose_offers(CosTrading::OfferSeq &offers);
  // create ACK packet

  TAO_ORB_Manager orb_manager_;
  // The ORB manager.

  int argc_;
  // Number of commandline arguments.

  char **argv_;
  // commandline arguments.

  CORBA::Environment env_;
  // Environment variable.

  CORBA::ORB_var orb_;
  // Remember our orb.

  ResourceMgmt::QueryPacket req; 
  // request

  Lrm_i *Lrm_impl_;
  // Lrm implementation to decode argumanets

  BaseProcess_i *BaseProcess_impl_;

#if defined(HAS_AUTOCONFIG)
  BaseCC* base_cc_;

  ComponentConfigurator_i *dsrt_cc_impl_;

#endif /* HAS_AUTOCONFIG */

#if defined(HAS_DSRT2)
  ResourceMgmt::Cpu_var CpuApi_;
  CpuApi_i *CpuApi_impl_; // DSRT Object
#endif /* HAS_DSRT2 */
};

class BaseProcess_i : public POA_ResourceMgmt::BaseProcess
{
public:
  BaseProcess_i (int argc, char **argv);
  // Constructor

  BaseProcess_i (BaseProcess_i &);

  ~BaseProcess_i (void);
  // Destructor

  virtual void load_component (
     const ResourceMgmt::QueryPacket & req,
     CORBA::Environment &ACE_TRY_ENV);

  void loader(Loader_i *loader);

  ResourceMgmt::BaseProcess_var self_;

private:
  Loader_i *loader_;
};

#endif /* CC_LOADER_H */
