// $Id: Local_DB.cpp,v 1.1 2000/02/19 20:31:01 yamane Exp $

#include <math.h>
char * cuserid (char *string);
#include "ace/OS.h"
#include "ace/High_Res_Timer.h"
#include "Local_DB.h"

ACE_RCSID(Local_DB, Local_DB, "$Id: Local_DB.cpp,v 1.1 2000/02/19 20:31:01 yamane Exp $")

#if defined(HAS_DSRT2)
#include "DSRT_Helper.h"
#endif /* HAS_DSRT2 */

//
//	Platform dependent layer should be moved to ACE_2k_OS
//
#if defined(sun)
// #include <kstat.h>
#include <sys/systeminfo.h>
#include <sys/sysinfo.h>
#define MAXSTRSIZE 80

// for statvfs()
#include <sys/types.h>
#include <sys/statvfs.h>

// for loadavg()
#if !defined(__SunOS_5_6)
#include <sys/loadavg.h>
#endif

// for processor_info()
#include <sys/processor.h>

// for swapctl()
#include <sys/stat.h>
#include <sys/swap.h>

#elif defined(linux)
#include <sys/sysinfo.h>

// for statfs()
#include <sys/vfs.h>

//
//	Special class to measure CPU utilization on Linux
//	(start thread to measure utilization periodically.)
//

static
void *
util_update_thread(void *ptr)
{
Linux_processor_util *pu = (Linux_processor_util *)ptr;

	char buff[512];
	int fd;
	unsigned long us, us_low, sy, id;
	unsigned long c_total, c_used;

	char* p;

	while(1) {
		//
		//	We need to 
		//	(1) read "/proc/stat"
		//	(2) wait for a while (currently 2 sec)
		//	(3) read "/proc/stat"
		//	(4) calculate diff of (3) and (1)
		//
		if ((fd = ACE_OS::open("/proc/stat", O_RDONLY, 0)) != -1) {
			buff[512-1] = 0;  /* ensure null termination in buffer */
			ACE_OS::read(fd,buff,512-1);
			ACE_OS::close(fd);
			p = ACE_OS::strstr(buff, "cpu ");

			////////////////////////////////////////////////////////
			//
			// Warning:   ACE_OS does not have sscanf() 
			//

			sscanf(p, "cpu  %lu %lu %lu %lu", &us, &us_low, &sy, &id);
			////////////////////////////////////////////////////////
	
//		ACE_OS::printf("us = %lu, us_low = %lu, sys = %lu, idle = %lu\n", us, us_low, sy, id);

			c_total = us + us_low + sy + id;
			c_used  = us + us_low + sy;

			pu->sem->acquire();

			pu->util_ = (c_used - pu->used);
			pu->util_ *= 100;
			pu->util_ /= (c_total - pu->total);

			pu->total = c_total;
			pu->used = c_used;

//	ACE_OS::printf("util = %e total = %ld used = %ld\n", pu->util_ , pu->total, pu->used);

			pu->sem->release();
		}
		ACE_OS::sleep(5);
	}
}

Linux_processor_util::Linux_processor_util(void)
: total(0), used(0), util_(0.0)
{
	sem = new ACE_Semaphore(1);

        ACE_Thread_Manager::instance ()->spawn(util_update_thread, (void *)this);
}

Linux_processor_util::~Linux_processor_util()
{
}

double
Linux_processor_util::util()
{
	double cpu_utilization;

	sem->acquire();
	cpu_utilization = util_;
	sem->release();

	return (cpu_utilization);
}

#endif

#if defined(ACE_WIN32)
#include <pdh.h> 

// Search resedit HKEY_LOCAL_MACHINE
#define PERFENUMKEY "System\\CurrentControlSet\\Control\\PerfStats\\Enum"

// concatenate the object and couter key names for CPU Usage
#define CPU_USAGE_PAIR "KERNEL\\CPUUsage"

BOOL is_windows_nt()
{
  OSVERSIONINFO osvi;
  osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

  GetVersionEx (&osvi);

  if (osvi.dwPlatformId == VER_PLATFORM_WIN32_NT) {
	  return (TRUE);
  } else {
	  return (FALSE);
  }
}

//
//  PDH Helper (Performance helper helper)
//  PDH is only available for NT
//  To keep same binary for 95/98/NT, we cannot link
//  PDH.dll directly
//  So we will load library explicitly
//  On NT Platform
//

extern "C" {

typedef PDH_STATUS (CALLBACK* PdhOpenQuery_p)(LPCSTR,DWORD,HQUERY*);
typedef PDH_STATUS (CALLBACK* PdhAddCounter_p)(HQUERY,LPCSTR,DWORD,HCOUNTER*);
typedef PDH_STATUS (CALLBACK* PdhRemoveCounter_p)(HCOUNTER);
typedef PDH_STATUS (CALLBACK* PdhCollectQueryData_p)(HQUERY);
typedef PDH_STATUS (CALLBACK* PdhCloseQuery_p)(HQUERY);
typedef PDH_STATUS (CALLBACK* PdhGetFormattedCounterValue_p)(HCOUNTER,DWORD,LPDWORD,PPDH_FMT_COUNTERVALUE);

};

class PDH_Helper {
private:

	PdhOpenQuery_p PdhOpenQuery_;
	PdhAddCounter_p PdhAddCounter_;
    PdhRemoveCounter_p PdhRemoveCounter_;
    PdhCollectQueryData_p PdhCollectQueryData_;
	PdhCloseQuery_p PdhCloseQuery_;
    PdhGetFormattedCounterValue_p PdhGetFormattedCounterValue_;

	HINSTANCE hDLL;               // Handle to DLL



	static void init();

public:
	static PDH_Helper *_instance;

    static
	PDH_STATUS PdhOpenQuery ( 
    IN      LPCSTR      szDataSource,
    IN      DWORD       dwUserData,
    IN      HQUERY      *phQuery
	);

	static
    PDH_STATUS
    PdhAddCounter (
    IN      HQUERY      hQuery,
    IN      LPCSTR      szFullCounterPath,
    IN      DWORD       dwUserData,
    IN      HCOUNTER    *phCounter
    );

    static
	PDH_STATUS
    PdhRemoveCounter (
    IN      HCOUNTER    hCounter
    );

    static
	PDH_STATUS
    PdhCollectQueryData (
    IN      HQUERY      hQuery
    );

    static
	PDH_STATUS
    PdhCloseQuery (
    IN      HQUERY      hQuery
    );

    static
	PDH_STATUS
    PdhGetFormattedCounterValue (
    IN      HCOUNTER                hCounter,
    IN      DWORD                   dwFormat,
    IN      LPDWORD                 lpdwType,
    IN      PPDH_FMT_COUNTERVALUE   pValue
    );
};

PDH_Helper* PDH_Helper::_instance = NULL;

void
PDH_Helper::init()
{
	ACE_OS::printf("PDH_Helper::init()\n");
	_instance = new PDH_Helper();

    _instance->hDLL = LoadLibrary("Pdh.DLL");

	if (_instance->hDLL != NULL){
        _instance->PdhOpenQuery_ = (PdhOpenQuery_p)GetProcAddress(_instance->hDLL,
                                           "PdhOpenQueryA");
		if (!_instance->PdhOpenQuery_) {
			ACE_OS::printf("Cannot find PdhOpenQueryA() entry\n");
		}

		_instance->PdhAddCounter_ = (PdhAddCounter_p)GetProcAddress(_instance->hDLL,
                                           "PdhAddCounterA");
		if (!_instance->PdhAddCounter_) {
			ACE_OS::printf("Cannot find PdhAddCounterA() entry\n");
		}

		_instance->PdhRemoveCounter_ = (PdhRemoveCounter_p)GetProcAddress(_instance->hDLL,
                                           "PdhRemoveCounter");
		if (!_instance->PdhRemoveCounter_) {
			ACE_OS::printf("Cannot find PdhRemoveCounter() entry\n");
		}

		_instance->PdhCollectQueryData_ = (PdhCollectQueryData_p)GetProcAddress(_instance->hDLL,
                                           "PdhCollectQueryData");
		if (!_instance->PdhCollectQueryData_) {
			ACE_OS::printf("Cannot find PdhCollectQueryData() entry\n");
		}

		_instance->PdhCloseQuery_ = (PdhCloseQuery_p)GetProcAddress(_instance->hDLL,
                                           "PdhCloseQuery");

 		if (!_instance->PdhCloseQuery_) {
			ACE_OS::printf("Cannot find PdhClose() entry\n");
		}

		_instance->PdhGetFormattedCounterValue_ = (PdhGetFormattedCounterValue_p)GetProcAddress(_instance->hDLL,
                                           "PdhGetFormattedCounterValue");
		if (!_instance->PdhGetFormattedCounterValue_) {
			ACE_OS::printf("Cannot find PdhGetFormattedCounterValue() entry\n");
		}

	} else {
			ACE_OS::printf("Cannot load PDH.dll\n");
			delete _instance;
			_instance = 0;
	}
}

PDH_STATUS
PDH_Helper::PdhOpenQuery ( 
    IN      LPCSTR      szDataSource,
    IN      DWORD       dwUserData,
    IN      HQUERY      *phQuery)
{
	if(_instance == 0) init();
	return (_instance->PdhOpenQuery_)(szDataSource, dwUserData, phQuery);
}

PDH_STATUS
PDH_Helper::PdhAddCounter (
    IN      HQUERY      hQuery,
    IN      LPCSTR      szFullCounterPath,
    IN      DWORD       dwUserData,
    IN      HCOUNTER    *phCounter)
{
	if(_instance == 0) init();
	return (_instance->PdhAddCounter_)(hQuery, szFullCounterPath, dwUserData, phCounter);
}

PDH_STATUS
PDH_Helper::PdhRemoveCounter (
    IN      HCOUNTER    hCounter)
{
	if(_instance == 0) init();
	return (_instance->PdhRemoveCounter_)(hCounter);
}


PDH_STATUS
PDH_Helper::PdhCollectQueryData (
    IN      HQUERY      hQuery)
{
	if(_instance == 0) init();
	return (_instance->PdhCollectQueryData_)(hQuery);
}

PDH_STATUS
PDH_Helper::PdhCloseQuery (
    IN      HQUERY      hQuery)
{
	if(_instance == 0) init();
	return (_instance->PdhCloseQuery_)(hQuery);
}

PDH_STATUS
PDH_Helper::PdhGetFormattedCounterValue (
    IN      HCOUNTER                hCounter,
    IN      DWORD                   dwFormat,
    IN      LPDWORD                 lpdwType,
    IN      PPDH_FMT_COUNTERVALUE   pValue)
{
	if(_instance == 0) init();
	return (_instance->PdhGetFormattedCounterValue_)(hCounter, dwFormat, lpdwType, pValue);
}


BOOL
CollectPerfData_NT(DWORD *pdwData)
{
    BOOL bSuccess = TRUE;
    HQUERY     hQuery;
    HCOUNTER   arCounter[1] = {0}; 
    PDH_STATUS pdhStatus; 
    PDH_FMT_COUNTERVALUE   arFmtValue[1]; 
    DWORD           ctrType; 

    // 
    // First step in using PDH is to open a "query"... 
    // 
    pdhStatus = PDH_Helper::PdhOpenQuery (0, 0, &hQuery); 
    if (pdhStatus != ERROR_SUCCESS) { 
		ACE_OS::printf("Cannot open PDH\n"); 
        bSuccess = FALSE;
		return(bSuccess);
    } 

    // 
    // With the query handle, we now add one "counters"... 
    // 
 
    pdhStatus = PDH_Helper::PdhAddCounter (hQuery, 
                               "\\System\\% Total Processor Time",
                               0, 
                               &arCounter[0]); 
    if (pdhStatus != ERROR_SUCCESS) { 
		ACE_OS::printf("Cannot add PDH counter\n"); 
        bSuccess = FALSE;
        pdhStatus = PDH_Helper::PdhCloseQuery (hQuery);
		return(bSuccess);
    }

    // 
    // Now that everything is setup, actually do the query to bind 
    //  values to the counters... 
    // 
 
    pdhStatus = PDH_Helper::PdhCollectQueryData (hQuery); 
    if (pdhStatus != ERROR_SUCCESS) { 
   		ACE_OS::printf("Cannot collect PDH counter\n"); 
        bSuccess = FALSE;
        pdhStatus = PDH_Helper::PdhCloseQuery (hQuery);
		return(bSuccess);
    } 
 
    // 
    // And finally, get the formatted values... 
    // 
 
    pdhStatus = PDH_Helper::PdhGetFormattedCounterValue (arCounter[0], 
                                             PDH_FMT_LONG, 
                                             &ctrType, 
                                             &arFmtValue[0]); 
 
    if (pdhStatus != ERROR_SUCCESS) { 
   		ACE_OS::printf("Cannot get PDH formatted value\n"); 
        bSuccess = FALSE;
        pdhStatus = PDH_Helper::PdhCloseQuery (hQuery);
		return(bSuccess);
    } 
 
    *pdwData = arFmtValue[0].longValue; 

	PDH_Helper::PdhRemoveCounter( arCounter[0] ); 
    pdhStatus = PDH_Helper::PdhCloseQuery (hQuery); 
    return(bSuccess);
}

BOOL
EnablePerfDataCollection()
{
    HKEY hOpen;
    DWORD cbData;
    DWORD dwType;
    DWORD rc;
    BOOL bSuccess = TRUE;
    DWORD perf_value;

	if (is_windows_nt()) {
	}

    if ( (rc = RegOpenKeyEx(HKEY_DYN_DATA,"PerfStats\\StartStat", 0,
                           KEY_READ, &hOpen)) == ERROR_SUCCESS)    {

	  cbData = sizeof(DWORD);
		// query to get data size
       if ( (rc = RegQueryValueEx(hOpen,CPU_USAGE_PAIR,NULL,&dwType,
             (LPBYTE)&perf_value, &cbData )) == ERROR_SUCCESS ) {

       }
       else {
        	  ACE_OS::printf("********** RegQueryValueEx() Failed\n");
         bSuccess = FALSE;
       }
       CloseHandle(hOpen);
    }
    else {
	   ACE_OS::printf("**** RegisOpenKeyEx() error ****\n");
        	bSuccess = FALSE;
    }
  return bSuccess;
}
	
BOOL
CollectPerfData(DWORD *pdwData)
{
    HKEY hOpen;
    DWORD dwType;
    DWORD cbData;
    DWORD rc;
    BOOL bSuccess = TRUE;

    // open performance data key
    if ( (rc = RegOpenKeyEx(HKEY_DYN_DATA,"PerfStats\\StatData", 0,
         KEY_READ, &hOpen)) == ERROR_SUCCESS)
    {
      cbData = sizeof(DWORD);
      // retrieve performance data which is of size of DWORD
      rc=RegQueryValueEx(hOpen,CPU_USAGE_PAIR,0,&dwType,
            (LPBYTE)pdwData,&cbData );

      // let the caller know if it worked or not
      bSuccess = rc == ERROR_SUCCESS;
      RegCloseKey(hOpen);
    }
    else
      bSuccess = FALSE;

    return bSuccess;
}

BOOL
DisablePerfDataCollection()
{
    HKEY hOpen;
    DWORD cbData;
    DWORD dwType;
    DWORD rc;
    BOOL bSuccess = TRUE;
    DWORD perf_value;

    if ( (rc = RegOpenKeyEx(HKEY_DYN_DATA,"PerfStats\\StopStat", 0,
                           KEY_READ, &hOpen)) == ERROR_SUCCESS)
    {
	  cbData = sizeof(DWORD);
      // query to get data size
      if ( (rc = RegQueryValueEx(hOpen,CPU_USAGE_PAIR,NULL,&dwType,
            (LPBYTE)&perf_value, &cbData )) == ERROR_SUCCESS )
      {
	  ACE_OS::printf("DisablePerfDataCollection() cbData = %d\n", cbData);

      }
      else
         bSuccess = FALSE;

      CloseHandle(hOpen);
    }
    else
      bSuccess = FALSE;

    return bSuccess;
}

//
// For Windows-NT only
//
DWORD
NT_Processor_MHz()
{
    HKEY hOpen;
    DWORD cbData;
    DWORD dwType;
    DWORD rc;
    BOOL bSuccess = TRUE;
    DWORD clocks = 0;

    if ( (rc = RegOpenKeyEx(HKEY_LOCAL_MACHINE,"Hardware\\Description\\System\\CentralProcessor\\0", 0,
                           KEY_READ, &hOpen)) == ERROR_SUCCESS)
    {
	  cbData = sizeof(DWORD);
      // query to get data size
      if ( (rc = RegQueryValueEx(hOpen,"~MHz",NULL,&dwType,
            (LPBYTE)&clocks, &cbData )) == ERROR_SUCCESS )
      {
	  ACE_OS::printf("Processor MHz() cloks = %d\n", clocks);
      }
      else
		  ACE_OS::printf("It is not possible to get MHz\n");
	  
      CloseHandle(hOpen);
    }
    else
		ACE_OS::printf("We failed to open Processor registory\n");

    return clocks;
}

#endif /* ACE_WIN32 */


//
// Initialize dummy Lrm data
//
static
void
dummy_resource (CosTrading::PropertySeq &list)
{
static CORBA::Long Total_number = 0;
CORBA::ULong ind = list.length();

//  ACE_OS::printf("Initialize dummy resource\n");
  Total_number++;

  list.length(ind+1);
  list[0].name = CORBA::string_dup("Total_number");
  list[0].value <<= Total_number;

  ind++;
}

Local_static_resource::Local_static_resource(void)
{
char buf[512];

  //
  // hostname
  //

  if (ACE_OS::hostname(buf, 511) == 0) {
#if defined(ACE_WIN32)
	  // hostname to the lower case letters
	  for (size_t i = 0 ; i < ACE_OS::strlen(buf); ++i) {
		buf[i] = ACE_OS::to_lower(buf[i]);
	  }
#endif
#if defined(sun) /* For U of I DCL only */
	if (ACE_OS::strchr(buf, '.')==NULL) {
		ACE_OS::strcat(buf, ".cs.uiuc.edu");
	}
#endif
	this->hostname = CORBA::string_dup(buf);
  } else {
	this->hostname = CORBA::string_dup("");
  }

#if defined(sun)
  //
  // Operation system
  //
  ACE_OS::sysinfo(SI_SYSNAME, buf, 511);
  this->os_name = CORBA::string_dup(buf);

  //
  // Release
  //
  ACE_OS::sysinfo(SI_RELEASE, buf, 511);
  this->os_version = CORBA::string_dup(buf);

  //
  // Processor
  //
  ACE_OS::sysinfo(SI_ARCHITECTURE, buf, 511);
  this->processor_name = CORBA::string_dup(buf);

  //
  // Number of processors
  //
  this->processors = sysconf(_SC_NPROCESSORS_CONF);

  //
  // CPU speed
  //

  // We only check the primary CPU
  int cpu_no = 0;
  processor_info_t info;
  CORBA::ULong cpu_mhz;

  processor_info(cpu_no, &info);

  this->processor_MHz = info.pi_clock;

  //
  // Total memory installed (MB)
  //
  this->RAM_total = ACE_OS::sysconf(_SC_PHYS_PAGES) 
			* (ACE_OS::sysconf(_SC_PAGE_SIZE) / 1024) / 1024 + 1;

  // Total VM installed (MB)

  {
        swaptbl_t      *s;
        int            i, n, num;
        char           *strtab;    /* string table for path names */
	long sz;
  	this->VM_total = 0;
        sz = sysconf(_SC_PAGE_SIZE);

     while (1) {
         if ((num = swapctl(SC_GETNSWP, 0)) == -1) {
	     perror("swapctl: GETNSWP");
             break;
         }
         if (num == 0) {
             ACE_OS::printf("No Swap Devices Configured\n");
             break;
         }
         /* allocate swaptable for num+1 entries */
         if ((s = (swaptbl_t *)
             malloc(num * sizeof(swapent_t) +
                 sizeof(struct swaptable))) ==
             (void *) 0) {
             ACE_OS::printf("Malloc Failed\n");
             break;
         }
         /* allocate num+1 string holders */
         if ((strtab = (char *)
             malloc((num + 1) * MAXSTRSIZE)) == (void *) 0) {
             ACE_OS::printf("Malloc Failed\n");
             break;
         }

         /* initialize string pointers */
         for (i = 0; i < (num + 1); i++) {
             s->swt_ent[i].ste_path = strtab + (i * MAXSTRSIZE);
         }

         s->swt_n = num + 1;
         if ((n = swapctl(SC_LIST, s)) < 0) {
             perror("swapctl");
             break;
         }
         if (n > num) {        /* more were added */
             free(s);
             free(strtab);
             continue;
         }

         for (i = 0; i < n; i++) {
  	     this->VM_total += (s->swt_ent[i].ste_pages) / ((1024*1024)/sz);
         }

         free(s);
         free(strtab);
         break;
     }
  }

#elif defined(linux)
  struct utsname host_info;
  ACE_OS::uname (&host_info);

  //
  // Operation system
  //
  this->os_name = CORBA::string_dup(host_info.sysname);

  //
  // Release
  //
  this->os_version = CORBA::string_dup(host_info.release);

  //
  // Processor
  //
  this->processor_name = CORBA::string_dup(host_info.machine);

  //
  //  # of processors
  //  Clock Speed of processors
  //
  //  We don't think about SMP machine
  //  Only the first CPU utilization will be registered.
  //
  unsigned long cpu_MHz = ACE_High_Res_Timer::get_cpuinfo ();
//  ACE_OS::printf("Estimated cpu MHz = %ld\n", cpu_MHz);

  long n_processors = 1;

  FILE *cpuinfo = ACE_OS::fopen ("/proc/cpuinfo", "r");

  if (cpuinfo != 0)
  {
	  n_processors = 0;
	  char buf[128];
	  int found_MHz_entry = 0;

          // ACE_DEBUG ((LM_DEBUG, ASYS_TEXT ("\nReading /proc/cpuinfo...")));

          while (ACE_OS::fgets (buf, sizeof buf, cpuinfo))
          {
              ACE_UINT32 cpu_no;
              if (::sscanf (buf,
                           "processor : %d\n",
                            &cpu_no) == 1)
              {
//                     ACE_OS::printf("Found processor no = %d\n", cpu_no);
		     n_processors++;
              }

              ACE_UINT32 MHz;
              if (::sscanf (buf,
                           "cpu MHz : %d\n",
                            &MHz) == 1)
              {
//                     ACE_OS::printf("Found cpu MHz = %d\n", MHz);
	             cpu_MHz = MHz;
                     found_MHz_entry = 1;
              }

              if (::sscanf (buf,
                           "bogomips : %d\n",
                            &MHz) == 1 && !found_MHz_entry)
              {
//                     ACE_OS::printf("Found bogomips = %d\n", MHz);
	             cpu_MHz = MHz+1;
              }
          }

          ACE_OS::fclose (cpuinfo);
  }

  this->processor_MHz = cpu_MHz;

  this->processors = n_processors;

  //
  // Total memory installed
  //
  this->RAM_total =  get_phys_pages()
			* (ACE_OS::sysconf(_SC_PAGE_SIZE) / 1024) / 1024 +1;

  struct sysinfo info;
  sysinfo(&info);

  for (int i = 0 ; i < 3 ; i++) {
//	ACE_OS::printf("loads[%d] = %d\n", i, info.loads[i]);
  }

//  	ACE_OS::printf("totalram = %d\n", info.totalram/1024);
//	ACE_OS::printf("freeram = %d\n", info.freeram/1024);
//	ACE_OS::printf("totalswap = %d\n", info.totalswap/1024);
//	ACE_OS::printf("freeswap = %d\n", info.freeswap/1024);

  this->VM_total = info.totalswap/1024/1024 + 1;

#elif defined (ACE_WIN32)
  // OS version information

  char ver[1024];
  OSVERSIONINFO osvi;
  osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

  GetVersionEx (&osvi);

  switch(osvi.dwPlatformId) {
  case VER_PLATFORM_WIN32s:
	  this->os_name = CORBA::string_dup("windows");
	  ACE_OS::strcpy(ver, "3.1");
	  break;
  case VER_PLATFORM_WIN32_WINDOWS:
	  this->os_name = CORBA::string_dup("windows");
	  if (osvi.dwMinorVersion == 0) {
		  ACE_OS::sprintf(ver, "95 (%s)", osvi.szCSDVersion);		  
	  } else {
		  ACE_OS::sprintf(ver, "98 (%s)", osvi.szCSDVersion);
	  }
	  break; 
  case VER_PLATFORM_WIN32_NT:
	  this->os_name = CORBA::string_dup("windows-nt");
	  ACE_OS::sprintf(ver, "%d.%d (Build %d %s)", osvi.dwMajorVersion, osvi.dwMajorVersion, osvi.dwBuildNumber, osvi.szCSDVersion);
	  break;
  }

  os_version = CORBA::string_dup(ver);

  // Processor name
  SYSTEM_INFO si;

  GetSystemInfo(&si);

  char processor_name_[1024];
  processor_name_[0] = 0;
  if (ACE_OS::strcmp(os_name, "windows-nt")==0) {
	if (si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_INTEL) {
		switch (si.wProcessorLevel) {
		case 3:	ACE_OS::strcpy(processor_name_,"i386");
			    break;
		case 4: ACE_OS::strcpy(processor_name_,"i486");
				break;
		case 5: ACE_OS::strcpy(processor_name_, "i586");
				break;
		default:	ACE_OS::strcpy(processor_name_,"intel");
				break;
		}
	} else if (si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_MIPS) {
		ACE_OS::strcpy(processor_name_,"mips");
	} else if (si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_ALPHA) {
		ACE_OS::sprintf(processor_name_, "Alpha%d", si.wProcessorLevel);
	} else if (si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_PPC) {
		ACE_OS::sprintf(processor_name_, "PPC 6%02d", si.wProcessorLevel);
	}
  } else {
        if (si.dwProcessorType == PROCESSOR_INTEL_386) {
		strcpy(processor_name_,"i386");
	} else if (si.dwProcessorType == PROCESSOR_INTEL_486) {
		strcpy(processor_name_,"i486");
	} else if (si.dwProcessorType == PROCESSOR_INTEL_PENTIUM) {
		strcpy(processor_name_,"i586");
	} else ACE_OS::strcpy(processor_name_,"intel");
  }

  this->processor_name = CORBA::string_dup(processor_name_);

  // How many Processors?

  this->processors = si.dwNumberOfProcessors;

  // MHz?
  if (ACE_OS::strcmp(os_name, "windows-nt")==0)
  {
        this->processor_MHz = NT_Processor_MHz();
  }

  // Memory status

  MEMORYSTATUS mem_status;

  GlobalMemoryStatus(&mem_status);

  this->RAM_total = (mem_status.dwTotalPhys/1024/1024);

  this->VM_total = (mem_status.dwTotalVirtual/1024/1024);
#endif /* ACE_WIN32 */

}

Local_static_resource::~Local_static_resource(void)
{
	CORBA::string_free(hostname);
	CORBA::string_free(os_name);
	CORBA::string_free(os_version);
	CORBA::string_free(processor_name);
}

void
Local_static_resource::query(CosTrading::PropertySeq& list)
{
CORBA::ULong ind = list.length();

  //
  // hostname
  //
  list.length(ind+1);
  list[ind].name = CORBA::string_dup("hostname");
//  ACE_OS::printf("before Hostname to any!\n"); 
  list[ind].value <<= CORBA::string_dup(this->hostname);
  ind++;

  //
  // Operation system
  //
  list.length(ind+1);
  list[ind].name = CORBA::string_dup("os_name");
  list[ind].value <<= CORBA::string_dup(this->os_name);
  ind++;

  //
  // Release
  //
  list.length(ind+1);
  list[ind].name = CORBA::string_dup("os_version");
  list[ind].value <<= CORBA::string_dup(this->os_version);
  ind++;

  //
  // Processor type
  //
  list.length(ind+1);
  list[ind].name = CORBA::string_dup("processor_name");
  list[ind].value <<= CORBA::string_dup(this->processor_name);
  ind++;

  //
  // Number of processors
  //
  list.length(ind+1);
  list[ind].name = CORBA::string_dup("processors");
  list[ind].value <<= this->processors;
  ind++;

  //
  // CPU speed
  //
  if (ACE_OS::strcmp(this->os_name, "windows")!=0) {
  	list.length(ind+1);
	list[ind].name = CORBA::string_dup("processor_MHz");
	list[ind].value <<= this->processor_MHz;
	ind++;
  }

  //
  // Total memory installed
  //
  list.length(ind+1);
  list[ind].name = CORBA::string_dup("RAM_total");
  list[ind].value <<= this->RAM_total;
  ind++;

  //
  // Total VM installed
  //
  list.length(ind+1);
  list[ind].name = CORBA::string_dup("VM_total");
  list[ind].value <<= this->VM_total;
  ind++;
}

Local_dynamic_resource::Local_dynamic_resource(Local_static_resource *ls)
{
	ls_ = ls;

#if defined(ACE_WIN32)
	EnablePerfDataCollection();
#endif /* ACE_WIN32 */

	probe();
	update();
}

Local_dynamic_resource::~Local_dynamic_resource(void)
{
}

void
Local_dynamic_resource::update(void)
{
	last = current;
}

#if defined(HAS_DYNAMICCONFIG)
void
Local_dynamic_resource::set_dynconf(CORBA::Object_var& obj)
{
	CORBA::Environment ACE_TRY_ENV;

	ACE_TRY {

	ACE_OS::printf("Local_dynamic_resource::set_dynconf() narrrowing\n");
        DynamicConfigurator_var dyn_var =
           DynamicConfigurator::_narrow (obj.in (),ACE_TRY_ENV);
	ACE_TRY_CHECK;

	if (CORBA::is_nil (dyn_var.in ())) {
		ACE_OS::printf("Unable to resolve the DynConf Service.\n");
		return;
	}

	ACE_OS::printf("dynconf is not null\n");

	dyn_conf_ =
		DynamicConfigurator::_duplicate (dyn_var.in ());

	ACE_OS::printf("dynconf narrow done\n");

	// List all categories
	DynamicConfigurator::stringList *cat_list, *impl_list;

	cat_list = dyn_conf_->list_categories(ACE_TRY_ENV);
	ACE_TRY_CHECK;

	ACE_OS::printf("\nNumber of categories:%d\n",cat_list->length());

	for(CORBA::ULong i=0; i < cat_list->length(); i++) {
		const char *category = ((*cat_list)[i]).in();
		ACE_OS::printf("Category =  %s\n",category);

		impl_list =
			dyn_conf_->list_implementations(category, ACE_TRY_ENV);
		ACE_TRY_CHECK;

       		ACE_OS::printf("\nNumber of %s implementations:%d\n",
				category, impl_list->length());

		for (CORBA::ULong j=0; j < impl_list->length(); j++) {
               		ACE_OS::printf("%s\n",((*impl_list)[j]).in());
		}
	}

#if 0
	DynamicConfigurator::stringList *hook_list;
        hook_list= dyn_conf_->list_hooks("TAO",ACE_TRY_ENV);
	ACE_TRY_CHECK;
 
        ACE_OS::printf("\nNumber of hooks in TAO:%d\n",hook_list->length());
 
        for(CORBA::ULong k=0; k < hook_list->length(); k++) {
		ACE_OS::printf("%s\n",((*hook_list)[k]).in());
	}
#endif

	}
  ACE_CATCH (CORBA::SystemException, sysex)
    {
      ACE_UNUSED_ARG (sysex);
      ACE_TRY_ENV.print_exception ("System Exception");
      exit(1);
    }
  ACE_CATCH (CORBA::UserException, userex)
    {
      ACE_UNUSED_ARG (userex);
      ACE_TRY_ENV.print_exception ("User Exception");
      exit(1);
    }
  ACE_ENDTRY;
}

#endif /* HAS_DYNAMICCONFIG */

void
Local_dynamic_resource::probe(void)
{
#if defined(sun)
  //
  // Available memory
  // 
  current.RAM_free = ACE_OS::sysconf(_SC_AVPHYS_PAGES) 
			* (ACE_OS::sysconf(_SC_PAGE_SIZE)/1024)/1024 + 1;

  // Current VM available (MB)
  {
        swaptbl_t      *s;
        int            i, n, num;
        char           *strtab;    /* string table for path names */
	long sz;
  	current.VM_free = 0;
        sz = sysconf(_SC_PAGE_SIZE);

     while (1) {
         if ((num = swapctl(SC_GETNSWP, 0)) == -1) {
	     perror("swapctl: GETNSWP");
             break;
         }
         if (num == 0) {
             ACE_OS::printf("No Swap Devices Configured\n");
             break;
         }
         /* allocate swaptable for num+1 entries */
         if ((s = (swaptbl_t *)
             malloc(num * sizeof(swapent_t) +
                 sizeof(struct swaptable))) ==
             (void *) 0) {
             ACE_OS::printf("Malloc Failed\n");
             break;
         }
         /* allocate num+1 string holders */
         if ((strtab = (char *)
             malloc((num + 1) * MAXSTRSIZE)) == (void *) 0) {
             ACE_OS::printf("Malloc Failed\n");
             break;
         }

         /* initialize string pointers */
         for (i = 0; i < (num + 1); i++) {
             s->swt_ent[i].ste_path = strtab + (i * MAXSTRSIZE);
         }

         s->swt_n = num + 1;
         if ((n = swapctl(SC_LIST, s)) < 0) {
             perror("swapctl");
             break;
         }
         if (n > num) {        /* more were added */
             free(s);
             free(strtab);
             continue;
         }

         for (i = 0; i < n; i++) {
  	     current.VM_free += (s->swt_ent[i].ste_free) / ((1024*1024)/sz);
         }

         free(s);
         free(strtab);
         break;
     }
  }

  //
  //  CPU utilization
  //
  //  We don't think about SMP machine
  //  Only the first CPU utilization will be registered.
  //
  double cpu_utilization = 0.0;
  {
#if !defined(__SunOS_5_6)
	double av[LOADAVG_NSTATS];
#endif
	int ret;
	int n_processors;
	int i;

	n_processors = sysconf(_SC_NPROCESSORS_CONF);

#if !defined(__SunOS_5_6)
	ret = getloadavg(av, LOADAVG_NSTATS);
//	ACE_OS::printf("ret = %d\n", ret);


        for (i = 0 ; i < LOADAVG_NSTATS ; i++) {
//                ACE_OS::printf("av[%d] = %g\n", i, av[i]);
        }

	cpu_utilization = av[LOADAVG_1MIN] * 100 / n_processors;
	if (cpu_utilization > 100.0) {
		cpu_utilization = 100.0;
	}
#endif
  }

  current.processor_util = cpu_utilization;

  struct statvfs vfs;
  int ret;

  ret = statvfs(".", &vfs);

  if (ret == 0) {

	// Think about large file system (more than 32bit)
	const char *fmt_blks, *fmt_files;

	if (sizeof(fsblkcnt_t) == sizeof(ulong_t)) {
		fmt_blks = "%ld";
	} else {
		fmt_blks = "%lld";
	}

	if (sizeof(fsfilcnt_t) == sizeof(ulong_t)) {
		fmt_files = "%ld";
	} else {
		fmt_files = "%lld";
	}

	fsblkcnt_t fs_total, fs_free;

	current.FS_total = (vfs.f_blocks - (vfs.f_bfree - vfs.f_bavail)) 
					/ 1024 * vfs.f_bsize / 1024 + 1;

	current.FS_free = vfs.f_bavail / 1024 * vfs.f_bsize / 1024 + 1;

	// total files (for non-super user)

	current.files_total = (vfs.f_files - (vfs.f_ffree - vfs.f_favail));

	// avail files (for non-super user)

	current.files_free = vfs.f_favail;

  }
 
#elif defined(linux)
  //
  // Available memory & Available swap
  // 
  current.RAM_free = get_avphys_pages()
			* (ACE_OS::sysconf(_SC_PAGE_SIZE) / 1024) / 1024 + 1;

  //
  //  VM availability
  //
  {
  	struct sysinfo info;
  	sysinfo(&info);

  	for (int i = 0 ; i < 3 ; i++) {
//		ACE_OS::printf("loads[%d] = %d\n", i, info.loads[i]);
  	}

//  	ACE_OS::printf("totalram = %d\n", info.totalram/1024);
//	ACE_OS::printf("freeram = %d\n", info.freeram/1024);
//	ACE_OS::printf("totalswap = %d\n", info.totalswap/1024);
//	ACE_OS::printf("freeswap = %d\n", info.freeswap/1024);

	current.VM_free =  info.freeswap/1024/1024 + 1;

  } 

  //
  //  CPU utilization
  //
  //  CPU workload
  //  It will take a while to get the value
  //
  //  We don't think about SMP machine
  //  Only the first CPU utilization will be registered.
  //
  double cpu_utilization = 0.0;

#if 1
  cpu_utilization = linux_processor_util_.util();
#else
  {
	char buff[512];
	int fd;
	unsigned long us, us_low, sy, id;

	unsigned long total = 0;
	unsigned long used = 0;
	char* p;

	//
	//	We need to 
	//	(1) read "/proc/stat"
	//	(2) wait for a while (currently 2 sec)
	//	(3) read "/proc/stat"
	//	(4) calculate diff of (3) and (1)
	//
	if ((fd = ACE_OS::open("/proc/stat", O_RDONLY, 0)) != -1) {
		buff[512-1] = 0;  /* ensure null termination in buffer */
		ACE_OS::read(fd,buff,512-1);
		ACE_OS::close(fd);
		p = ACE_OS::strstr(buff, "cpu ");

		////////////////////////////////////////////////////////
		//
		// Warning:   ACE_OS does not have sscanf() 
		//

		sscanf(p, "cpu  %lu %lu %lu %lu", &us, &us_low, &sy, &id);
		////////////////////////////////////////////////////////
	
//		ACE_OS::printf("us = %lu, us_low = %lu, sys = %lu, idle = %lu\n", us, us_low, sy, id);
		total = us + us_low + sy + id;
		used = us + us_low + sy;

	}

	ACE_OS::sleep(2);

	if ((fd = ACE_OS::open("/proc/stat", O_RDONLY, 0)) != -1) {
		buff[512-1] = 0;  /* ensure null termination in buffer */
		ACE_OS::read(fd,buff,512-1);
		ACE_OS::close(fd);
		p = ACE_OS::strstr(buff, "cpu ");

		////////////////////////////////////////////////////////
		//
		// Warning:   ACE_OS does not have sscanf() 
		//

		sscanf(p, "cpu  %lu %lu %lu %lu", &us, &us_low, &sy, &id);
		////////////////////////////////////////////////////////
	
//		ACE_OS::printf("us = %lu, us_low = %lu, sys = %lu, idle = %lu\n", us, us_low, sy, id);

		// Calculate diff
		total = us + us_low + sy + id - total;
		used = us + us_low + sy - used;

		ACE_OS::printf("total = %lu, used = %lu\n", total, used);
		cpu_utilization = used;
		cpu_utilization *= 100;
		cpu_utilization /= total;
	}
  }
#endif

  current.processor_util = cpu_utilization;

  //
  //  Disk  utilization
  //
  struct statfs vfs;
  int ret;

  ret = statfs(".", &vfs);

  if (ret == 0) {

	// Don't Think about large file system (more than 32bit)
	char *fmt_blks, *fmt_files;

	fmt_blks = "%ld";
	fmt_files = "%ld";

	// file size

  current.FS_total = (vfs.f_blocks - (vfs.f_bfree - vfs.f_bavail)) 
			/ 1024 * vfs.f_bsize / 1024 + 1;

  current.FS_free = vfs.f_bavail / 1024 * vfs.f_bsize / 1024 + 1;

	// total files

  current.files_total = vfs.f_files;

	// avail files

  current.files_free  = vfs.f_ffree;


  }
#elif defined(ACE_WIN32)
  
   DWORD perf_value = 0;
   if (is_windows_nt()) {
	   CollectPerfData_NT(&perf_value);
   } else {
		CollectPerfData(&perf_value);
   }
   ACE_OS::printf("CollectPerfData returns %d\n", perf_value);

  double cpu_utilization = perf_value;
  current.processor_util = cpu_utilization;

  // Memory status

  MEMORYSTATUS mem_status;

  GlobalMemoryStatus(&mem_status);

  current.RAM_free = (mem_status.dwAvailPhys/1024/1024);

  current.VM_free = (mem_status.dwAvailVirtual/1024/1024);

  //
  //  Disk  utilization
  //  Here we assuming Windows95 OSR2 or later
  //  We can check availability of GetDiskSpaceEx() later
  //

  ULARGE_INTEGER FreeBytesAvailableToCaller;
  ULARGE_INTEGER TotalNumberOfBytes;
  ULARGE_INTEGER TotalNumberOfFreeBytes;
  CORBA::ULong FS_total;
  CORBA::ULong FS_free;

  if (GetDiskFreeSpaceEx(
	                 (LPCTSTR)NULL,
					 &FreeBytesAvailableToCaller,
					 &TotalNumberOfBytes,
					 &TotalNumberOfFreeBytes)) {
	  ACE_OS::printf("avail bytes high = 0x%x\n", FreeBytesAvailableToCaller.HighPart);
	  ACE_OS::printf("avail bytes low = 0x%x\n", FreeBytesAvailableToCaller.LowPart);
	  ACE_OS::printf("total bytes high = 0x%x\n", TotalNumberOfBytes.HighPart);
	  ACE_OS::printf("total bytes low = 0x%x\n", TotalNumberOfBytes.LowPart);
	  ACE_OS::printf("free bytes high = 0x%x\n", TotalNumberOfFreeBytes.HighPart);
	  ACE_OS::printf("free bytes low = 0x%x\n", TotalNumberOfFreeBytes.LowPart);

      FS_total = (TotalNumberOfBytes.LowPart >> 20) + (TotalNumberOfBytes.HighPart << 12);
      FS_free  = (FreeBytesAvailableToCaller.LowPart >> 20) + (FreeBytesAvailableToCaller.HighPart << 12);
	  ACE_OS::printf("FS_total = 0x%x\n", FS_total);
	  ACE_OS::printf("FS_free = 0x%x\n", FS_free);
	  ACE_OS::printf("FS_total = %d\n", FS_total);
	  ACE_OS::printf("FS_free = %d\n", FS_free);

           current.FS_total = FS_total;
		   current.FS_free  = FS_free;
  }

#endif

#if defined(HAS_DSRT2)
	//
	//	All the resource are not equally important.
	//	We need to decide which should be advertised.
	//
	if (DSRT_Helper::instance()) {
		current.DSRT2 = 1;

		SchedStats *s = DSRT_Helper::instance()->getSchedStats();

		// availPermUtil
		current.DSRT2_availPermUtil = s->availPermUtil;

		// overrunRatio
		current.DSRT2_overrunRatio = s->overrunRatio;

		// permRatio
		current.DSRT2_permRatio = s->permRatio;

		// tsRatio
		current.DSRT2_tsRatio = s->tsRatio;

		// permUsage ???
		current.DSRT2_permUsage = Timeval::toInt(&s->permUsage);

		// burstUsage ???
		current.DSRT2_burstUsage = Timeval::toInt(&s->burstUsage);

		// nonconformUsage ???
		current.DSRT2_nonconformUsage = Timeval::toInt(&s->nonconformUsage);

		// probeUsage ???
		current.DSRT2_probeUsage = Timeval::toInt(&s->probeUsage);

		// tsUsage ???
		current.DSRT2_tsUsage = Timeval::toInt(&s->tsUsage);
	} else {
		current.DSRT2 = 0;
	}
#endif /* HAS_DSRT2 */

#if defined(HAS_DYNAMICCONFIG)
	//
	//	Creat a list of components
	//
	//	Format:  category/component , catefory/component , ...
	//
	DynamicConfigurator::stringList_var comp_list 
			= new DynamicConfigurator::stringList();
	CORBA::ULong comp_size = 0;

	if (CORBA::is_nil (dyn_conf_.in ())) {
		ACE_OS::printf("probe() Unable to resolve the DynConf Service.\n");
		current.ComponentsList = comp_list;
		return;
	}

	CORBA::Environment ACE_TRY_ENV;

	ACE_TRY {

		// List all categories
		DynamicConfigurator::stringList *cat_list, *impl_list;

		cat_list = dyn_conf_->list_categories(ACE_TRY_ENV);
		ACE_TRY_CHECK;

		ACE_OS::printf("\nNumber of categories:%d\n", cat_list->length());

		for(CORBA::ULong i=0; i < cat_list->length(); i++) {
			const char *category = ((*cat_list)[i]).in();
			ACE_OS::printf("Category =  %s\n",category);

			impl_list =
				dyn_conf_->list_implementations(category, ACE_TRY_ENV);
			ACE_TRY_CHECK;

			for (CORBA::ULong j=0; j < impl_list->length(); j++) {
				const char *i_name = ((*impl_list)[j]).in();
				char *s = CORBA::string_alloc(
						ACE_OS::strlen(category)
						+ 1
						+ ACE_OS::strlen(i_name)
						+ 1);
				ACE_OS::sprintf(s, "%s/%s", category, i_name);

				ACE_OS::printf("%s\n", s);

				comp_list->length(comp_size+1);
				comp_list[comp_size] = s;
				comp_size++;
			}
		}
	}
	ACE_CATCH (CORBA::SystemException, sysex)
	{
		ACE_UNUSED_ARG (sysex);
		ACE_TRY_ENV.print_exception ("System Exception");
		exit(1);
	}
	ACE_CATCH (CORBA::UserException, userex)
	{
		ACE_UNUSED_ARG (userex);
		ACE_TRY_ENV.print_exception ("User Exception");
		exit(1);
	}
	ACE_ENDTRY;

	current.ComponentsList = comp_list;
	ACE_OS::printf("current.ComponentsList length = %d\n", current.ComponentsList->length());
#endif

}

int Local_dynamic_resource::query_significant_change() {
  int significant = 0;

  if (fabs(((double)current.RAM_free - last.RAM_free) * 100 / ls_->RAM_total) > 10.0) {
    ACE_OS::printf("current.RAM_free = %ld\n", current.RAM_free);
    ACE_OS::printf("last.RAM_free    = %ld\n", last.RAM_free);
    ACE_OS::printf("RAM_free (significant change)\n");
    significant = 1;
  }

  if (fabs(((double)current.VM_free - last.VM_free) * 100 / ls_->VM_total) > 10.0) {
    ACE_OS::printf("current.VM_free = %ld\n", current.VM_free);
    ACE_OS::printf("last.VM_free    = %ld\n", last.VM_free);
    ACE_OS::printf("VM_free (significant change)\n");
    significant = 1;
  }

  if (fabs(current.processor_util - last.processor_util) > 10.0) {
    ACE_OS::printf("processor_util (significant change)\n");
    significant = 1;
  }

#if defined(HAS_DSRT2)
  //
  //	We just think DSRT start message is important
  //	We need to think about how can we detect the down of DSRT.
  //
  if (current.DSRT2 == 0 && last.DSRT2 == 1) {
    ACE_OS::printf("DSRT2 stopped (significant change)\n");
    significant = 1;
  }

  if (current.DSRT2 == 1 && last.DSRT2 == 0) {
    ACE_OS::printf("DSRT2 start (significant change)\n");
    significant = 1;
  }
#endif /* HAS_DSRT2 */

#if defined(HAS_DYNAMICCONFIG)
  CORBA::ULong comp_length = current.ComponentsList->length();
  
  if (!last.ComponentsList.ptr()) {
    significant = 1;
  } else if (comp_length != last.ComponentsList->length()) {
    ACE_OS::printf("ComponentsList changed (significant change)\n");
    significant = 1;
  } else {
    for (CORBA::ULong i = 0 ; i < comp_length ; i++) {
      if (ACE_OS::strcmp(current.ComponentsList[i].in(),
			 last.ComponentsList[i].in())
	  != 0) {
	ACE_OS::printf("ComponentsList changed (significant change)\n");
	significant = 1;
	break;
      }
    }
  }
#endif /* HAS_DYNAMICCONFIG */
  return (significant);
}

//
// Initialize dynamic Lrm data
//
void Local_dynamic_resource::query(CosTrading::PropertySeq& list)
{
CORBA::ULong ind = list.length();

  //
  // Available memory
  // 
  list.length(ind+1);
  list[ind].name = CORBA::string_dup("RAM_free");
  list[ind].value <<= current.RAM_free;
  ind++;

  list.length(ind+1);
  list[ind].name = CORBA::string_dup("VM_free");
  list[ind].value <<= current.VM_free;
  ind++;

  //
  //  CPU utilization
  //
  // Windows-NT always return 0

  list.length(ind+1);
  list[ind].name = CORBA::string_dup("processor_util");

  list[ind].value <<= current.processor_util;

  ind++;

  //
  // Disk space 
  //

  list.length(ind+1);
  list[ind].name = CORBA::string_dup("FS_total");
  list[ind].value <<= current.FS_total;
  ind++;

  list.length(ind+1);
  list[ind].name = CORBA::string_dup("FS_free");
  list[ind].value <<= current.FS_free;
  ind++;

#if !defined(ACE_WIN32)
  // total files (for non-super user)

  list.length(ind+1);
  list[ind].name = CORBA::string_dup("files_total");
  list[ind].value <<= current.files_total;
  ind++;

  // avail files (for non-super user)

  list.length(ind+1);
  list[ind].name = CORBA::string_dup("files_free");
  list[ind].value <<= current.files_free;
  ind++;

#endif /* !defined(WIN32) */
 
#if defined(HAS_DSRT2)
	//
	//	All the resource are not equally important.
	//	We need to decide which should be advertised.
	//
	if (current.DSRT2) {
  		list.length(ind+1);
  		list[ind].name = CORBA::string_dup("DSRT2");
  		list[ind].value <<= CORBA::Any::from_boolean(1);
		ind++;

		// availPermUtil
  		list.length(ind+1);
  		list[ind].name = CORBA::string_dup("DSRT2_availPermUtil");
  		list[ind].value <<= current.DSRT2_availPermUtil;
		ind++;

		// overrunRatio
  		list.length(ind+1);
  		list[ind].name = CORBA::string_dup("DSRT2_overrunRatio");
  		list[ind].value <<= current.DSRT2_overrunRatio;
		ind++;

		// permRatio
  		list.length(ind+1);
  		list[ind].name = CORBA::string_dup("DSRT2_permRatio");
  		list[ind].value <<= current.DSRT2_permRatio;
		ind++;

		// tsRatio
  		list.length(ind+1);
  		list[ind].name = CORBA::string_dup("DSRT2_tsRatio");
  		list[ind].value <<= current.DSRT2_tsRatio;
		ind++;

		// permUsage ???
  		list.length(ind+1);
  		list[ind].name = CORBA::string_dup("DSRT2_permUsage");
  		list[ind].value <<= current.DSRT2_permUsage;
		ind++;

		// burstUsage ???
  		list.length(ind+1);
  		list[ind].name = CORBA::string_dup("DSRT2_burstUsage");
  		list[ind].value <<= current.DSRT2_burstUsage;
		ind++;

		// nonconformUsage ???
  		list.length(ind+1);
  		list[ind].name = CORBA::string_dup("DSRT2_nonconformUsage");
  		list[ind].value <<= current.DSRT2_nonconformUsage;
		ind++;

		// probeUsage ???
  		list.length(ind+1);
  		list[ind].name = CORBA::string_dup("DSRT2_probeUsage");
  		list[ind].value <<= current.DSRT2_probeUsage;
		ind++;

		// tsUsage ???
  		list.length(ind+1);
  		list[ind].name = CORBA::string_dup("DSRT2_tsUsage");
  		list[ind].value <<= current.DSRT2_tsUsage;
		ind++;

	}
#endif /* HAS_DSRT2 */

#if defined(HAS_DYNAMICCONFIG)
  // List of available components

  if (current.ComponentsList.ptr()) {
	  ACE_OS::printf("inserting ComponentsList\n");
	  list.length(ind+1);
	  list[ind].name = CORBA::string_dup("ComponentsList");

	  ResourceMgmt::ComponentsList *components = new ResourceMgmt::ComponentsList();
	  components->length(current.ComponentsList->length());

	  for (CORBA::ULong i = 0 ; i < current.ComponentsList->length() ; i++) {
		(*components)[i] = CORBA::string_dup(current.ComponentsList[i]);
 	 }

	  list[ind].value <<= components;
	  ind++;
  }
#endif /* HAS_DYNAMICCONFIG */

}

// Copy  Constructor
Local_DB::Local_DB (Local_DB &lrm)
{
  ACE_UNUSED_ARG(lrm);
  // no-op

}

// Destructor

Local_DB::~Local_DB (void)
{
  // no-op
#if defined(ACE_WIN32)
	DisablePerfDataCollection();
#endif /* ACE_WIN32 */
}

// get resource

static Local_static_resource *lsr = 0;
static Local_dynamic_resource *ldr = 0;

void
Local_DB::init()
{
  if (!lsr) {
	lsr = new Local_static_resource;
	ldr = new Local_dynamic_resource(lsr);
  }
}

void
Local_DB::getResources (CosTrading::PropertySeq &ps)
{
//  ACE_OS::printf("Local_DB::getResource() before calling static_resource()\n");

  init();

  dummy_resource(ps);
  lsr->query(ps);
  ldr->query(ps);

#if 0
  for (CORBA::ULong i = 0 ; i < ps.length() ; i++) {
	ACE_OS::printf("property[%d] name = %s\t",
			i,
			ps[i].name.in());
	CORBA::Any::dump(ps[i].value);
	ACE_OS::printf("\n");
  }
#endif
}

int
Local_DB::poll()
{
  ldr->probe();
  if (ldr->query_significant_change()) {
  	ldr->update();
	return 1;
  } else {
	return 0;
  }
}

CosTrading::PropertySeq *
Local_DB::getResources (void)
{
#if 1 // Tomonori Yamane
  CosTrading::PropertySeq_var h(new CosTrading::PropertySeq(1));

  Local_DB::getResources(h.inout());

//  ACE_OS::printf("Local_DB::getResources() done\n");
  return h._retn();

#else
  ACE_OS::printf("Local_DB::getResource() before calling static_resource()\n");
  Local_DB::display_all();
  dummy_resource();
  ACE_OS::printf("Local_DB::getResource() before calling dynamic_resource()\n");
  Local_DB::display_all();
  dynamic_resource();
  static_resource();

  CosTrading::PropertySeq *h = new CosTrading::PropertySeq(stock.properties);

  ACE_OS::printf("Local_DB::getResource() before calling display_all()\n");
  Local_DB::display_all();

  ACE_OS::printf("Local_DB::getResource() after calling display_all()\n");
  return h;
#endif
}

#if defined(HAS_DYNAMICCONFIG)

void
Local_DB::set_dynconf(CORBA::Object_var& dyn_var)
{
	init();
	ldr->set_dynconf(dyn_var);
}

#endif /* HAS_DYNAMICCONFIG */

void
Local_DB::display_all()
{
	return;
#if 1
	CosTrading::PropertySeq_var h = Local_DB::getResources();

	for (CORBA::ULong i = 0 ; i < h->length() ; i++) {
		ACE_OS::printf("property[%d] name = %s\t",
			i,
			h[i].name.in());
		CORBA::Any::dump(h[i].value);
	}
#else
	for (CORBA::ULong i = 0 ; i < stock.properties.length() ; i++) {
		ACE_OS::printf("property[%d] name = %s\t",
			i,
			stock.properties[i].name.in());
		CORBA::Any::dump(stock.properties[i].value);
	}
#endif
}

// Constructor
Local_DB::Local_DB (void)
{
#if 0
	stock.reference = CosTrading::TraderComponents::_nil();
	stock.properties.length(0);
#endif
	ACE_OS::printf("Local_DB constructor\n");

	ACE_OS::printf("sizeof(Property) = %d\n", sizeof(CosTrading::Property));

}

//
//	Instanciate local_DB
//
static Local_DB lc;


