// -*- C++ -*-
// $Id: Local_DB.h,v 1.1 2000/02/19 20:31:01 yamane Exp $

// ============================================================================
//
// = LIBRARY
//    RMP/Lrm
//
// = FILENAME
//    Local_DB.h
//
// = DESCRIPTION
//    This class implements the Local Resource Database.
//
// = AUTHOR
//    Tomonori Yamane <yamane@uiuc.edu>
//
// ============================================================================

#ifndef LOCAL_DB_I_H
#define LOCAL_DB_I_H

#include "ace/OS.h"
#include "orbsvcs/CosTradingC.h"
#include "orbsvcs/Trader/Trader_Utils.h"

#if defined(HAS_DSRT2)
#include "CpuApi.H"
#endif

#if defined(HAS_DYNAMICCONFIG)
#include "dynamicTAO/src/DynamicConfigurator/dynamicConfigurator_i_.h"
#endif

#if defined(linux)
//
//	Special class to measure CPU utilization on Linux
//	(start thread to measure utilization periodically.)
//

class Linux_processor_util
{
public:
	Linux_processor_util(void);
	~Linux_processor_util();

	double util(void);

	ACE_Semaphore *sem;
	// Sepahore to update utilizarion value;

	unsigned long total;
	unsigned long used;
  double util_;
};

#endif /* defined(linux) */

class Local_static_resource
{
public:
	Local_static_resource(void);

	~Local_static_resource(void);

	void query(CosTrading::PropertySeq& list);

	char	*hostname;
	char	*os_name;
	char	*os_version;
	char	*processor_name;
	CORBA::ULong	processors;
	CORBA::ULong	processor_MHz;
	CORBA::ULong	RAM_total; // MB
	CORBA::ULong	VM_total;
};

class Local_dynamic_resource
{
public:
	Local_dynamic_resource(Local_static_resource *ls);

	~Local_dynamic_resource(void);

	void probe(void); // check data and store to current

	void update(void); // current ==> last

#if defined(HAS_DYNAMICCONFIG)
	void set_dynconf(CORBA::Object_var& obj);
#endif

 	// If there is no significant change between current and last return 1
	int query_significant_change();

	void query(CosTrading::PropertySeq& list);

private:
	Local_static_resource    *ls_;

#if defined(HAS_DYNAMICCONFIG)
	DynamicConfigurator_var dyn_conf_;
#endif

	typedef struct {
		CORBA::ULong	RAM_free;
		CORBA::ULong	VM_free;
		CORBA::ULong	FS_total;
		CORBA::ULong	FS_free;
		CORBA::ULong	files_total;
		CORBA::ULong	files_free;
		CORBA::Double	processor_util;
#if defined(HAS_DSRT2)
		int		DSRT2; // bool
		CORBA::Float	DSRT2_availPermUtil;
		CORBA::Float	DSRT2_overrunRatio;
		CORBA::Float	DSRT2_permRatio;
		CORBA::Float	DSRT2_tsRatio;
		CORBA::Long	DSRT2_permUsage;
		CORBA::Long	DSRT2_burstUsage;
		CORBA::Long	DSRT2_nonconformUsage;
		CORBA::Long	DSRT2_probeUsage;
		CORBA::Long	DSRT2_tsUsage;
#endif /* HAS_DSRT2 */

#if defined(HAS_DYNAMICCONFIG)
		DynamicConfigurator::stringList_var ComponentsList;
#endif /* HAS_DYNAMICCONFIG */
	} DYN_Resource;

	DYN_Resource last; /* last one reported */
	DYN_Resource current;

	CORBA::Long	timeout;
	time_t		last_check;

#if defined(linux)
	Linux_processor_util	linux_processor_util_;
#endif /* defined(linux) */
};

class Local_DB
{
  // = TITLE
  //    Local Resource Data Base
  //
  // = DESCRIPTION
  //    Implementation of a local resource database.
  //
public:
  // = Initialization and termination methods.
  Local_DB (void);
  // Constructor

  Local_DB (Local_DB &);
  // Copy Constructor added because g++ generates a 
  // bad one if not defined

  ~Local_DB (void);
  // Destructor
 
  static void getResources (CosTrading::PropertySeq &ps);
  // Get resource of Lrm
  
  static CosTrading::PropertySeq *getResources ();
  // Get resource of Lrm
  
  static void display_all();
  
#if defined(HAS_DYNAMICCONFIG)
  static void set_dynconf(CORBA::Object_var& obj);
#endif

  static int poll();
  // Polling dynamic resource, if significant change happens, return 1 otherwise 0

private:
  static void init();

};

#endif /* LOCAL_DB_I_H */
