// $Id: Lrm_i.cpp,v 1.3 2000/09/30 18:39:06 yamane Exp $

#include "Lrm_i.h"
#include "Offer_Sem.h"
#include "Local_DB.h"

// #include "P_Timer.h"

#include "orbsvcs/Trader/Constraint_Interpreter.h"
#include "orbsvcs/Trader/Constraint_Visitors.h"

#if defined(HAS_DYNAMICCONFIG)
#include "dynamicTAO/src/DynamicConfigurator/dynamicConfigurator_i_.h"
#endif

ACE_RCSID(Lrm, Lrm_i, "$Id: Lrm_i.cpp,v 1.3 2000/09/30 18:39:06 yamane Exp $")

// Constructor
Lrm_i::Lrm_i (int argc, char **argv)
: prelaunch_flag(0)
{
	argc__ = argc;
	argv__ = argv;

	//
	// hostname
	//
#if 1
	Local_static_resource *lsr = new Local_static_resource();
	hostname_ = CORBA::string_dup(lsr->hostname);
	delete lsr;
#else
	char buf[512];

	if (ACE_OS::hostname(buf, 511) == 0) {
        	hostname_ = CORBA::string_dup(buf);
	}
#endif

	//
	// default interval = 60 sec
	//
	update_interval = 1200;

	//
	// Initialize semaphore for Local_DB
	//
	Local_DB_Sem = new ACE_Semaphore(1);

	//
	// Initialize semaphore for Local_DB
	//
	prelaunch_ready_Sem = new ACE_Semaphore(0);
	prelaunch_request_Sem = new ACE_Semaphore(0);

	//
	// default Threshold
	//
	threshold_ = CORBA::string_dup("processor_util < 90.0");
}

// Copy  Constructor
// Lrm_i::Lrm_i (Lrm_i &lrm)
// {
  // no-op
// }

// Destructor

Lrm_i::~Lrm_i (void)
{
  // delete semaphore
  delete Local_DB_Sem;
}

// Set the ORB pointer.

void
Lrm_i::orb (CORBA::ORB_ptr o)
{
  orb_ = CORBA::ORB::_duplicate (o);

#if defined(HAS_DYNAMICCONFIG)

  CORBA::Object_var dynconf_obj =
       orb_->resolve_initial_references ("DynamicConfigurator");

  if (CORBA::is_nil (dynconf_obj.in ())) {
       ACE_OS::printf("Cannot find DynamicConfigurator\n");
       return;
  }

//  ACE_OS::printf("Lrm_i::orb() Setting up for DynamicConfigurator\n");
  Local_DB::set_dynconf(dynconf_obj); 

#endif /* defined(HAS_DYNAMICCONFIG) */

  char buf[1024];
  buf[0] = 0;
  u_short name_port = orb_->orb_core()->orb_params()->name_service_port();
  u_short trading_port = orb_->orb_core()->orb_params()->trading_service_port();

  if (name_port && trading_port) {
	ACE_OS::sprintf(buf, " -ORBnameserviceport %d -ORBtradingserviceport %d", name_port, trading_port);
  } else if (name_port) {
	ACE_OS::sprintf(buf, " -ORBnameserviceport %d", name_port);
  } else if (trading_port) {
	ACE_OS::sprintf(buf, " -ORBtradingserviceport %d", trading_port);
  }

  ior_options_ = CORBA::string_dup(buf);

//  ACE_OS::printf("ior_options_ = %s\n", buf);
}

CORBA::ORB_ptr
Lrm_i::orb (void)
{
  CORBA::ORB_var o_;

  ACE_DECLARE_NEW_CORBA_ENV;
  ACE_TRY
    {
//        ACE_OS::printf("Getting orb()\n");

	int argc_ = argc__;
	char** argv_ = argv__;

        o_ = CORBA::ORB_init (argc_, argv_, "", ACE_TRY_ENV);
        ACE_TRY_CHECK;

//        ACE_OS::printf("orb initiated\n");
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION,
                           "MT_Client: exception raised");
    }
  ACE_ENDTRY;
  return o_._retn();
}

// set prelaunch mode ON
void
Lrm_i::set_prelaunch_mode (void)
{
      prelaunch_flag = 1;
}

void
Lrm_i::issue_query(const ResourceMgmt::QueryPacket & req,
		   CORBA::Environment &ACE_TRY_ENV) {
  ACE_OS::printf("Issue request to the remote\n"); // teste

  if (Grm_.in() != Grm_->_nil()) {
    ResourceMgmt::QueryPacket copy_req = req;

    copy_req.visited = mark_here(req.visited);

    Grm_->perform_query(copy_req, ACE_TRY_ENV);
    return;
  }

  // Grm is not available. We should fail.
  ACE_OS::printf("Grm is not available. We should fail.\n");

  ResourceMgmt::Lrm_var Lrm_ = ResourceMgmt::Lrm::_duplicate(req.sender.in());

  ACE_TRY_NEW_ENV {
    Lrm_->NACK(req.sem_tag, ACE_TRY_ENV);
    ACE_TRY_CHECK;
  }
  ACE_CATCH (CORBA::SystemException, sysex) {
    ACE_UNUSED_ARG (sysex);
    ACE_TRY_ENV.print_exception ("System Exception");
    exit(1);
  }
  ACE_CATCH (CORBA::UserException, userex) {
    ACE_UNUSED_ARG (userex);
    ACE_TRY_ENV.print_exception ("User Exception");
    exit(1);
  }
  ACE_ENDTRY;
}

CORBA::Boolean
Lrm_i::eval_constraint(
	const char *constraint,
	CosTrading::PropertySeq_var& prop_var,
	CORBA::Environment &ACE_TRY_ENV
	)
{
//	ACE_OS::printf("Lrm_i::eval_constraint(%s)\n", constraint);

	CORBA::Boolean flag = false;
	
//	ACE_OS::printf("Lrm_i::eval_constraint()  constraint interpreter is ready\n");

	ACE_TRY {

	CosTradingRepos::ServiceTypeRepository::TypeStruct type_struct;
	type_struct.props.length(prop_var->length());

  	CosTrading::Offer offer;
	offer.properties.length(prop_var->length());

//	offer.reference = this->_this();

//	CORBA::ORB_var o_ = orb();
//	offer.reference = o_->string_to_object(self_);
	offer.reference = CORBA::Object::_duplicate(self_);

	for(CORBA::ULong i = 0 ; i < prop_var->length() ; i++) {
		type_struct.props[i].name = prop_var[i].name;
		type_struct.props[i].value_type = (prop_var[i].value).type();
		type_struct.props[i].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;
		offer.properties[i].name = prop_var[i].name;
		offer.properties[i].value = prop_var[i].value;
	}

//	ACE_OS::printf("before constructing validator\n");
	TAO_Constraint_Validator validator(type_struct);

//	ACE_OS::printf("before constructing interpreter\n");
	TAO_Constraint_Interpreter constr_inter(validator, constraint, ACE_TRY_ENV);
	ACE_TRY_CHECK;

//	ACE_OS::printf("Lrm_i::eval_constraint()  offer.properties.maximum() = %d\n", offer.properties.maximum());

//	for(CORBA::ULong i = 0 ; i < offer.properties.length() ; i++) {
//		ACE_OS::printf("properties name = %s\n", offer.properties[i].name.in());
//	}
 
//	ACE_OS::printf("Lrm_i::eval_constraint()  properties are ready length = %d\n", offer.properties.length());
//	ACE_OS::printf("before evaluation\n");

	flag = constr_inter.evaluate(&offer);

//	ACE_OS::printf("after evaluation\n");
//	ACE_OS::printf("Lrm_i::eval_constraint()   returns %d\n", flag);

	}
        ACE_CATCH (CosTrading::IllegalServiceType, ill_serv)
	{
                ACE_UNUSED_ARG (ill_serv);
		ACE_OS::printf("eval_constraint() IllegalServiceType\n");
                ACE_TRY_ENV.print_exception ("IllegalServiceType Exception");
		exit(1);
	}
        ACE_CATCH (CosTrading::UnknownServiceType, unknown_serv)
	{
                ACE_UNUSED_ARG (unknown_serv);
		ACE_OS::printf("eval_constraint() UnknownServiceType\n");
                ACE_TRY_ENV.print_exception ("UnknownServiceType Exception");
		exit(1);
	}
        ACE_CATCH (CosTrading::IllegalConstraint, ill_constr)
	{
		// If we fail to the constraint, just return False
                ACE_UNUSED_ARG (ill_constr);
		ACE_OS::printf("eval_constraint() IllegalConstraint\n");
                ACE_TRY_ENV.print_exception ("IllegalConstraint Exception");
		ACE_TRY_ENV.clear();
		flag = 0;
	}
        ACE_CATCH (CosTrading::Lookup::IllegalPreference, ill_pref)
	{
                ACE_UNUSED_ARG (ill_pref);
		ACE_OS::printf("eval_constraint() Lookup::IllegalPreference\n");
                ACE_TRY_ENV.print_exception ("Lookup::IllegalPreference Exception");
                exit(1);
	}
        ACE_CATCH (CosTrading::Lookup::IllegalPolicyName, ill_poly)
	{
                ACE_UNUSED_ARG (ill_poly);
		ACE_OS::printf("eval_constraint() Lookup::IllegalPolicyName\n");
                ACE_TRY_ENV.print_exception ("Lookup::IllegalPolicyName Exception");
                exit(1);
	}
        ACE_CATCH (CosTrading::Lookup::PolicyTypeMismatch, mismatch_poly)
	{
                ACE_UNUSED_ARG (mismatch_poly);
		ACE_OS::printf("eval_constraint() Lookup::PolicyTypeMismatch\n");
                ACE_TRY_ENV.print_exception ("Lookup::PolicyTypeMismatch Exception");
                exit(1);
	}
        ACE_CATCH (CosTrading::Lookup::InvalidPolicyValue, invalid_poly)
	{
                ACE_UNUSED_ARG (invalid_poly);
		ACE_OS::printf("eval_constraint() Lookup::InvalidPolicyValue\n");
                ACE_TRY_ENV.print_exception ("Lookup::InvalidPolicyValue Exception");
                exit(1);
	}
        ACE_CATCH (CosTrading::IllegalPropertyName, ill_property)
	{
                ACE_UNUSED_ARG (ill_property);
		ACE_OS::printf("eval_constraint() IllegalPropertyName\n");
                ACE_TRY_ENV.print_exception ("IllegalPropertyName Exception");
                exit(1);
	}
        ACE_CATCH (CosTrading::DuplicatePropertyName, dup_property)
	{
                ACE_UNUSED_ARG (dup_property);
		ACE_OS::printf("eval_constraint() DuplicatePropertyName\n");
                ACE_TRY_ENV.print_exception ("DuplicatePropertyName Exception");
                exit(1);
	}
        ACE_CATCH (CosTrading::DuplicatePolicyName, dup_policy)
	{
                ACE_UNUSED_ARG (dup_policy);
		ACE_OS::printf("eval_constraint() DuplicatePolicyName\n");
                ACE_TRY_ENV.print_exception ("DuplicatePolicyName Exception");
                exit(1);
	}
        ACE_CATCH (CORBA::SystemException, sysex)
        {
                ACE_UNUSED_ARG (sysex);
		ACE_OS::printf("eval_constraint() SystemException\n");
                ACE_TRY_ENV.print_exception ("System Exception");
                exit(1);
        }
        ACE_CATCH (CORBA::UserException, userex)
        {
                ACE_UNUSED_ARG (userex);
		ACE_OS::printf("eval_constraint() UserException\n");
                ACE_TRY_ENV.print_exception ("User Exception");
                exit(1);
        }
        ACE_ENDTRY;

	return flag;
}

//
//	Common query processing from local and remote
//
void
Lrm_i::common_query(const ResourceMgmt::QueryPacket & req, CORBA::Environment &ACE_TRY_ENV) {
  // Common Query
  ACE_OS::printf("Lrm_i::commonQuery()\n");

  ACE_OS::printf("\tconstraint = %s\n", (const char *)req.constr);
  ACE_OS::printf("\tpref = %s\n", (const char *)req.pref);
  ACE_OS::printf("\tvisited    = %s\n", (const char *)req.visited);

  Local_DB_Sem->acquire();
  CosTrading::PropertySeq_var prop_var = Local_DB::getResources();
  Local_DB_Sem->release();
  
  ACE_TRY {

    // If we exceed threshold constraint, go remote
    if (!eval_constraint(threshold_.in(), prop_var, ACE_TRY_ENV)) {
      ACE_OS::printf("\tthreshold out\n");
      ACE_TRY_CHECK;
      ACE_OS::printf("\tthreshold out no exception\n");
      issue_query(req, ACE_TRY_ENV);
      ACE_TRY_CHECK;
    }

    // If we cannot satisfy requested constraint locally, go remote.
    else {
      if (! eval_constraint(req.constr, prop_var, ACE_TRY_ENV)) {
	ACE_TRY_CHECK;
	ACE_OS::printf("\tconstraints out\n");
	issue_query(req, ACE_TRY_ENV);
	ACE_TRY_CHECK;
      }

    // Or else execute component locally
      else {
	ACE_TRY_CHECK;
	if (! prelaunch_flag) {
	  ACE_OS::printf("\ttry spawn()\n"); // teste

	  ACE_Process_Options options(1, 4096);
	  
	  ACE_OS::printf("!!!!!!!!!!!! Estimated command_line length = %d\n", command_line_length(&req)); // teste

	  compose_args(options, &req, ACE_TRY_ENV);
	  ACE_TRY_CHECK;

	  ACE_Process p;

	  pid_t result = p.spawn(options); // teste
	  
	  ACE_OS::printf("spawn() result = %d\n", result); // teste
	} else {
	  prelaunch_ready_Sem->acquire();
	  ACE_OS::printf("\tCall load_component method()\n"); // teste
	  pre_process_->load_component(req, ACE_TRY_ENV);
	  
	  prelaunch_request_Sem->release();
	  ACE_TRY_CHECK;
	}
      }
    }
  }
  ACE_CATCH (CosTrading::IllegalConstraint, ill_constr)	{
    ACE_UNUSED_ARG (ill_constr);
    ACE_OS::printf("common_query() CosTrading::IlliegalConstraint\n");
    ACE_TRY_ENV.print_exception ("IllegalConstraint Exception");
    ACE_TRY_ENV.clear();
    issue_query(req, ACE_TRY_ENV);
  }
  ACE_CATCH (CORBA::SystemException, sysex) {
    ACE_UNUSED_ARG (sysex);
    ACE_OS::printf("common_query() SystemException\n");
    ACE_TRY_ENV.print_exception ("System Exception");
    exit(1);
  }
  ACE_CATCH (CORBA::UserException, userex) {
    ACE_UNUSED_ARG (userex);
    ACE_OS::printf("common_query() UserException\n");
    ACE_TRY_ENV.print_exception ("User Exception");
    exit(1);
  }
  ACE_ENDTRY;
}

//
//	Additional constraint (Excluding hosts already visited)
//
//	""
//	"hostname != 'xxxx'"
//	"hostname != 'xxxx' and hostname != 'yyyy'" 
//	...
//
char *
Lrm_i::mark_here(const char *visited) {
  //
  // Calculate new string length
  //
  int len = ACE_OS::strlen(visited);
  int host_len = ACE_OS::strlen(hostname_.in());

  int additional_len;

  if (len == 0) {
    additional_len = ACE_OS::strlen( "hostname != ''" );
  } else {
    additional_len = ACE_OS::strlen( " and hostname != ''" );
  }
  
  int total_len = len + 1 + host_len + additional_len;
  
  // buffer
  CORBA::String buf = CORBA::string_alloc(total_len);
  
  if (len == 0) {
    ACE_OS::sprintf(buf, "hostname != '%s'", 
		    hostname_.in());
  } else {
    ACE_OS::sprintf(buf, "%s and hostname != '%s'", 
		    visited,
		    hostname_.in());
  }
  
  return buf;
}

// Get resource of Lrm (from remote host)

void
Lrm_i::perform_query(const ResourceMgmt::QueryPacket & req,
		     CORBA::Environment &ACE_TRY_ENV) {
	common_query(req, ACE_TRY_ENV);
}

CosTrading::OfferSeq *
Lrm_i::execute_application(const char * component,
		           const char * args,
		           const CosTrading::PropertySeq& resource_spec,
		           const char * constr,
		           const char * pref,
		           /* const CosTrading::PolicySeq& policies, */
		           const CosTrading::Lookup::SpecifiedProps& desired_props,
		           CORBA::Environment& ACE_TRY_ENV) {
  ResourceMgmt::QueryPacket req;

  ACE_OS::printf("Lrm_i::execute_application()\n"); // teste

  ACE_OS::printf("category = %s\n", category); // teste
  req.category = category;

  ACE_OS::printf("component = %s\n", component); // teste
  req.component = component;

  ACE_OS::printf("args = %s\n", args); // teste
  req.args = args;

  ACE_OS::printf("length of resource_spec = %d\n", resource_spec.length()); // teste
  req.resource_spec = resource_spec;

  ACE_OS::printf("constr = %s\n", constr); // teste
  req.constr = constr;

  req.visited = CORBA::string_dup("");

  ACE_OS::printf("pref = %s\n", pref); // teste
  req.pref = pref;

  // req.policies = policies;
  req.desired_props = desired_props;

  return execute_application(req, ACE_TRY_ENV);
}

//
// terminate the server
//
void
Lrm_i::shutdown (
	CORBA::Environment &ACE_TRY_ENV
	)
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);
	ACE_OS::exit(0);
}

//
// Create Process and Get resource (request from localhost)
//
CosTrading::OfferSeq *
Lrm_i::execute_application(ResourceMgmt::QueryPacket & req, CORBA::Environment &ACE_TRY_ENV) {
  ACE_OS::printf("execute_application() --- packet version\n"); // teste

  CosTrading::OfferSeq *offers_;

  ACE_TRY {

    //
    //	Set sender
    //
//		req.sender = this->_this();
//		CORBA::Object_var obj_var = orb_->string_to_object(self_.in());
//		req.sender = ResourceMgmt::Lrm::_narrow(obj_var);
    req.sender = ResourceMgmt::Lrm::_duplicate(self_.in());

    char *str = orb_->object_to_string(req.sender.in());
//		ACE_OS::printf("sender = %s\n", str);
    CORBA::string_free(str);

    ACE_TRY_CHECK;

    //
    //	Set Grm
    //
    req.grm_ = ResourceMgmt::Grm::_duplicate(Grm_.in());
    ACE_TRY_CHECK;

    // checkin the offer pointer and get the tag

    // semaphore tag
    long tag = Offer_Sem_Manager::checkin(&offers_);
    req.sem_tag = tag;

    common_query(req, ACE_TRY_ENV);
    ACE_TRY_CHECK;

    // wait until we receive reply
    ACE_OS::printf("swait(%d) before\n", tag); // teste
    Offer_Sem_Manager::swait(tag);
    ACE_OS::printf("swait(%d) after\n", tag); // teste

    if (offers_->length() > 0) {
      ACE_OS::printf("offers_->length() = %d\n", offers_->length()); // teste
      ACE_OS::printf("ref = %s\n", orb_->object_to_string((*offers_)[0].reference)); // teste
      ACE_OS::printf("property length = %d\n", (*offers_)[0].properties.length()); // teste
    }

  }
  ACE_CATCH (CORBA::SystemException, sysex) {
    ACE_UNUSED_ARG (sysex);
    ACE_OS::printf("execute_application SystemException\n");
    ACE_TRY_ENV.print_exception ("System Exception");
    exit(1);
  }
  ACE_CATCH (CORBA::UserException, userex) {
    ACE_UNUSED_ARG (userex);
    ACE_OS::printf("execute_application UserException\n");
    ACE_TRY_ENV.print_exception ("User Exception");
    exit(1);
  }
  ACE_ENDTRY;

  return offers_;
}

//
// Prelaunch BaseProcess
//
ResourceMgmt::BaseProcess *
Lrm_i::prelaunch_process(CORBA::Environment &ACE_TRY_ENV) {
  ACE_OS::printf("prelaunch_process()\n");

  CosTrading::OfferSeq *offers_;

  ResourceMgmt::QueryPacket req;

  ACE_TRY {

    //
    //	Set sender
    //

    char *sender = orb_->object_to_string(self_.in());
    //		ACE_OS::printf("sender = %s\n", sender);
    
    ACE_TRY_CHECK;

    //
    //	Set Grm
    //
    req.grm_ = ResourceMgmt::Grm::_duplicate(Grm_.in());
    ACE_TRY_CHECK;

    // checkin the offer pointer and get the tag

    // semaphore tag
    long tag = Offer_Sem_Manager::checkin(&offers_);
    req.sem_tag = tag;

    //		ACE_OS::printf("\ttry spawn()\n");
    
    ACE_Process_Options options(1, 4096);
    ACE_TRY_CHECK;
    
    options.command_line("../Lrm/BaseProcess %s -t %d -p %s", ior_options_.in(), tag, sender);

    CORBA::string_free(sender);
    
    ACE_Process p;

    //		pid_t result = p.spawn(options);

    //		ACE_OS::printf("prelaunch spawn() result = %d\n", result);

    // wait until we receive reply
    //		ACE_OS::printf("swait(%d) before\n", tag);
    Offer_Sem_Manager::swait(tag);
    //		ACE_OS::printf("swait(%d) after\n", tag);
    
    if (offers_->length() > 0) {
      //		ACE_OS::printf("prelaunch ref = %s\n", orb_->object_to_string((*offers_)[0].reference));
    }

    pre_process_ = ResourceMgmt::BaseProcess::_narrow(CORBA::Object::_duplicate(((*offers_)[0].reference.in())));

    return (ResourceMgmt::BaseProcess::_narrow(CORBA::Object::_duplicate(((*offers_)[0].reference.in()))));
  }
  ACE_CATCH (CORBA::SystemException, sysex) {
    ACE_UNUSED_ARG (sysex);
    ACE_OS::printf("execute_application SystemException\n");
    ACE_TRY_ENV.print_exception ("System Exception");
    exit(1);
  }
  ACE_CATCH (CORBA::UserException, userex) {
    ACE_UNUSED_ARG (userex);
    ACE_OS::printf("execute_application UserException\n");
    ACE_TRY_ENV.print_exception ("User Exception");
    exit(1);
  }
  ACE_ENDTRY;

  // MOD
  return NULL; // above return is inside a try block. 
               // This is just to the compilator stops complaining about it
  // FIM-MOD
}

void
Lrm_i::ACK (
	CORBA::Long sem_tag,
	const CosTrading::OfferSeq & offers,
	CORBA::Environment &ACE_TRY_ENV
	)
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);
//	ACE_OS::printf("Lrm_i::ACK(%d) received\n", sem_tag);
//	ACE_OS::printf("offer received = %s\n", orb_->object_to_string(offers[0].reference));
//	ACE_OS::printf("offer property length = %d\n", offers[0].properties.length());

        if (sem_tag == 0) {
                ACE_OS::printf("Not signal semaphore\n");
                return;
        }

//        ACE_OS::printf("Signal semaphore\n");
        //
        // Post semaphore
        //
	CosTrading::OfferSeq *seq = new CosTrading::OfferSeq(offers);
//        ACE_OS::printf("Copy done\n");

        Offer_Sem_Manager::checkout((long)sem_tag, seq);


//        ACE_OS::printf("Lrm_i::ACK() done\n");
}
  
void
Lrm_i::NACK (CORBA::Long sem_tag, CORBA::Environment &ACE_TRY_ENV) {
  ACE_UNUSED_ARG(ACE_TRY_ENV);
  ACE_OS::printf("Lrm_i::NACK(%d) received\n", sem_tag);

  CosTrading::OfferSeq *seq = new CosTrading::OfferSeq(1);
  seq->length(0);

  Offer_Sem_Manager::checkout((long)sem_tag, seq);
}

void
Lrm_i::trigger_update (
	ResourceMgmt::Grm_ptr grm,
	CORBA::Long distribution,
	CORBA::Long period,
	CORBA::Environment &ACE_TRY_ENV
	)
{
	ACE_UNUSED_ARG(grm);
	ACE_UNUSED_ARG(distribution);
	ACE_UNUSED_ARG(period);
	ACE_UNUSED_ARG(ACE_TRY_ENV);
}

// static	CORBA::Any a_value;
//static int n_thresh;

char *
Lrm_i::set_threshold (
	const char * constr,
	CORBA::Environment &ACE_TRY_ENV
	)
{
	ACE_UNUSED_ARG(ACE_TRY_ENV);

//	ACE_OS::printf("Lrm_i::set_threshold(%s)\n", constr);
	CosTrading::PropertyNameSeq test1_seq(1); // OK
	CosTrading::Property pp; // OK

	CosTrading::PropertySeq test2_seq(); // Not OK
//	ACE_OS::printf("Lrm_i::set_threshold 2\n");

//	CORBA::Any a;
//	CORBA::ULong l = 123;
//	a_value <<= l;

//	n_thresh++;

//	ACE_OS::printf("before assigning  a_value = a  n_thresh = %d\n", n_thresh);
//	a_value = a;
//	ACE_OS::printf("a_value.value() = 0x%08x\n", a_value.value());
//	ACE_OS::printf("after assigning a_value = a\n");

//	CosTrading::PropertySeq_var prop_var = Local_DB::getResources();

	CORBA::String_var old_constraint = threshold_._retn();
//	ACE_OS::printf("finish set_threshold() 0\n");
	threshold_ = CORBA::string_dup(constr);

//	Local_DB::display_all();

//	ACE_OS::printf("finish set_threshold() 1\n");
	return (old_constraint._retn());
}

// Set Grm pointer
void Lrm_i::Grm(ResourceMgmt::Grm_ptr grm) {
  Grm_ = ResourceMgmt::Grm::_duplicate(grm);
}

//
// encode/decode a string <==> hexdecimal format
//
char *
Lrm_i::encode(const char *original_str,	size_t len) {
  size_t i;

  if (len == 0) {
    ACE_OS::printf("Lrm_i::encode(%s)\n", original_str); // teste
    len = ACE_OS::strlen(original_str);
  }

  // special handling for Null string
  if (len == 0) {
    len = 1; // 00 will be coded
  }

  CORBA::String buf = CORBA::string_alloc(len*2+1);

  for (i = 0 ; i < len ; i++) {
    buf[i*2] = ACE::nibble2hex (original_str[i] >> 4);
    buf[i*2+1] = ACE::nibble2hex (original_str[i]);
  }

  buf[len*2] = '\0';

//////////////////////////////////////////////
// Assertion
//////////////////////////////////////////////
  char *decoded = decode(buf);
  for (i = 0 ; i < len ; i++) {
    if (original_str[i] != decoded[i]) {
      ACE_OS::printf("encode[%d] = %d /decode = %d pair wrong!!!!!!!!!!!!!!!!\n", i, original_str[i], decoded[i]); // teste
    }
  }
  CORBA::string_free(decoded);

  return buf;
}

char *
Lrm_i::decode(
	const char *hexdecimal_str
	)
{
//	ACE_OS::printf("Lrm_i::decode(%s)\n", hexdecimal_str);

	int len = ACE_OS::strlen(hexdecimal_str) / 2;

	CORBA::String buf = CORBA::string_alloc(len+1);

	for (int i = 0 ; i < len ; i++) {
		u_char b;
		b = (u_char) (ACE::hex2byte(hexdecimal_str[i*2]) << 4);
		b |= ACE::hex2byte(hexdecimal_str[i*2+1]);
		buf[i] = b;
	}

	buf[len] = '\0';

//	ACE_OS::printf("Lrm_i::decode() returns %s\n", buf);

	return buf;
}

// create command line options and return command line length
size_t
Lrm_i::command_line_length(const ResourceMgmt::QueryPacket *p) {
//
//	We will use CDR + simple encoding (byte to hex_string) to pass complicated
//	data structure as a commond argumanets
//
	//
	// CDR Encode
	//

	CORBA::Environment ACE_TRY_ENV;
	TAO_OutputCDR stream;

	ACE_TRY {
		stream.encode (ResourceMgmt::_tc_QueryPacket, (void *)p, 0, ACE_TRY_ENV);
		ACE_TRY_CHECK;
	}
	ACE_CATCH (CORBA::SystemException, sysex)
	{
		ACE_UNUSED_ARG (sysex);
		ACE_OS::printf("command_line_length()System Exception\n");
		ACE_TRY_ENV.print_exception ("System Exception");
		exit(1);
	}
	ACE_CATCH (CORBA::UserException, userex)
	{
		ACE_UNUSED_ARG (userex);
		ACE_OS::printf("command_line_length() encode User Exception\n");
		ACE_TRY_ENV.print_exception ("User Exception");
		exit(1);
	}
	ACE_ENDTRY;


	size_t total_len = stream.total_length();
//	ACE_OS::printf("CDR encode: total_len = %d\n", total_len);

	size_t command_len = total_len * 2 + ACE_OS::strlen("../Lrm/BaseProcess  -y ") + ACE_OS::strlen(ior_options_.in()) + 1 + 100;

	if ( command_len >= ACE_Process_Options::DEFAULT_COMMAND_LINE_BUF_LEN) {
//		ACE_OS::printf("!!!!!!!!!!!! Command line buffer overrun\n");
	}

	return (command_len);
}

// create command line options
void
Lrm_i::compose_args(
	ACE_Process_Options &options,
	const ResourceMgmt::QueryPacket *p,
	CORBA::Environment &ACE_TRY_ENV
	)
{
#if 1
//
//	We will use CDR + simple encoding (byte to hex_string) to pass complicated
//	data structure as a commond argumanets
//
	//
	// CDR Encode
	//

	TAO_OutputCDR stream;

	ACE_TRY {
		stream.encode (ResourceMgmt::_tc_QueryPacket, (void *)p, 0, ACE_TRY_ENV);
		ACE_TRY_CHECK;
	}
	ACE_CATCH (CORBA::SystemException, sysex)
	{
		ACE_UNUSED_ARG (sysex);
		ACE_OS::printf("compose_args encode System Exception\n");
		ACE_TRY_ENV.print_exception ("System Exception");
		exit(1);
	}
	ACE_CATCH (CORBA::UserException, userex)
	{
		ACE_UNUSED_ARG (userex);
		ACE_OS::printf("compose_args encode User Exception\n");
		ACE_TRY_ENV.print_exception ("User Exception");
		exit(1);
	}
	ACE_ENDTRY;

	size_t total_len = stream.total_length();
//	ACE_OS::printf("CDR encode: total_len = %d\n", total_len);

	size_t command_len = total_len * 2 + 100;

	size_t current_len = 0;
	CORBA::String cmd_arg = CORBA::string_alloc(command_len);
	cmd_arg[0] = 0;

	const ACE_Message_Block *mblk = stream.begin();

	for (const ACE_Message_Block *i = mblk;
		i != 0;
		i = i->cont ())
	{
		// Make sure there is something to send!
		if (i->length () > 0)
		{
			char *ptr = i->rd_ptr ();
			size_t mb_len = i->length ();
//			ACE_OS::printf("message block length = %d\n", mb_len);

			char *encoded = encode(ptr, mb_len);
			ACE_OS::strcat(cmd_arg, encoded);
			current_len += ACE_OS::strlen(encoded);

			CORBA::string_free(encoded);
		}
	}

	options.command_line("../Lrm/BaseProcess %s -y %s", ior_options_.in(), cmd_arg);


#else

	/*
		createdProcess command

		-y sender         (stringified object) 
		-t semaphore_tag
		-n grm            (stringified object)
		-v Lrm_visted     (hex decimal coded)
		-c category_name
		-o component_name
		-a args           (HEx decimal coded)
		-r resource_spec  (hex decimal coded)
		-s constraint     (hex decimal coded)
		-f preference     (hex decimal coded)
		-p policies       (hex decimal coded)
		-z desired_props  (hex decimal coded)
	*/

        char *sender_str = orb_->object_to_string(p->sender);

        char *grm_str = orb_->object_to_string(p->grm);

        char *visited_str = encode(p->visited.in());

        char *args_str = encode(p->args.in());

//
//	??? resource spec
//		-r resource_spec  (hex decimal coded)
//

	char *constr_str = encode(p->constr.in());

        char *pref_str = encode(p->pref.in());

//		-p policies       (hex decimal coded)
//		-z desired_props  (hex decimal coded)

//
//	We need to pass nameserviceport somehow.
//
	options.command_line("../Lrm/BaseProcess %s -y %s -t %d -n %s -v %s -c %s -o %s -a %s -s %s -f %s",
		ior_options_.in(),
		sender_str,
		p->sem_tag,
		grm_str,
		visited_str,
 		p->category.in(),
		p->component.in(),
		args_str,
		constr_str,
		pref_str);

	//
	// releasing string
	//
	CORBA::string_free(sender_str);
	CORBA::string_free(grm_str);
	CORBA::string_free(visited_str);
	CORBA::string_free(args_str);
	CORBA::string_free(constr_str);
        CORBA::string_free(pref_str);
#endif
}

// create Objects from the command line options
void
Lrm_i::compose_packet(
	ResourceMgmt::QueryPacket *p,
	int argc_,
	char** argv_,
	CORBA::Environment &ACE_TRY_ENV
	)
{
#if 1
//	ACE_OS::printf("compose_packet\n");

	/*
		-y encded_args
	*/
	ACE_Get_Opt get_opts (argc_, argv_, "dy:");

	int c;
	char *decoded = 0;
	char *encoded = 0;

	while ((c = get_opts ()) != -1)
	switch (c)
	{
	case 'd':  // debug flag.
		TAO_debug_level++;
		break;

	case 'y': // encoded arguments
		encoded = CORBA::string_dup(get_opts.optarg);
		decoded = decode(encoded);
		break;
	}

	if (decoded == 0) return;

	//
	// CDR Decode
	//
	size_t decode_len = ACE_OS::strlen (encoded) / 2;
	ACE_Message_Block mb (decode_len + 1 + ACE_CDR::MAX_ALIGNMENT + 1);

	ACE_CDR::mb_align (&mb);

	char *buffer = mb.rd_ptr ();
//	ACE_OS::printf("Message Block buffer len = %d\n", mb.length());
//	ACE_OS::printf("Estimated decode length = %d\n", decode_len);
	ACE_OS::memcpy(buffer, decoded, decode_len);
	mb.rd_ptr((size_t)0);
	mb.wr_ptr(decode_len);

//	ACE_OS::printf("creating stream\n");
	TAO_InputCDR stream(&mb);

//	ACE_OS::printf("decoding CDR stream\n");

	ACE_TRY {

		if (stream.decode(ResourceMgmt::_tc_QueryPacket, p, 0, ACE_TRY_ENV) == CORBA::TypeCode::TRAVERSE_CONTINUE) {
//			ACE_OS::printf("Decoded CDR stream properly\n");
		} else {
			ACE_OS::printf("Decoded CDR stream ????????\n");
		}
		ACE_TRY_CHECK;

	}
	ACE_CATCH (CORBA::SystemException, sysex)
	{
		ACE_UNUSED_ARG (sysex);
		ACE_OS::printf("Decoded CDR stream System Exception\n");
		ACE_TRY_ENV.print_exception ("System Exception");
		exit(1);
	}
	ACE_CATCH (CORBA::UserException, userex)
	{
		ACE_UNUSED_ARG (userex);
		ACE_OS::printf("Decoded CDR stream User Exception\n");
		ACE_TRY_ENV.print_exception ("User Exception");
		exit(1);
	}
	ACE_ENDTRY;

	CORBA::string_free(encoded);
	CORBA::string_free(decoded);

//	ACE_OS::printf("constraint    : %s\n", p->constr.in());
//	ACE_OS::printf("semaphore tag = %d\n", p->sem_tag);
//	ACE_OS::printf("visited       : %s\n", p->visited.in());
//	ACE_OS::printf("category      : %s\n", p->category.in());
//	ACE_OS::printf("component     : %s\n", p->component.in());
//	ACE_OS::printf("args          : %s\n", p->args.in());

	char *sender = orb_->object_to_string(p->sender);
//	ACE_OS::printf("sender        : %s\n", sender);
	CORBA::string_free(sender);

#else
	/*

		-y sender         (stringified object)         
		-t semaphore_tag
		-n grm            (stringified object)
		-v Lrm_visted     (hex decimal coded)
		-c category_name
		-o component_name
		-a args           (HEx decimal coded)
		-r resource_spec  (hex decimal coded)
		-s constraint     (hex decimal coded)
		-f preference     (hex decimal coded)
		-p policies       (hex decimal coded)
		-z desired_props  (hex decimal coded)
	*/
	ACE_Get_Opt get_opts (argc_, argv_, "dy:t:n:v:c:o:a:r:s:f:p:z:");

	int c;

	while ((c = get_opts ()) != -1)
	switch (c)
	{
	case 'd':  // debug flag.
		TAO_debug_level++;
		break;

	case 'y': // read ACK IOR from the command line.
		obj = orb_->string_to_object(get_opts.optarg);
		p->sender = Lrm::_narrow(obj);
		break;

	case 't': // semaphore tag
		p->sem_tag = ACE_OS::atoi(get_opts.optarg);
		break;

	case 'n': // read NACK IOR from the command line.
		obj = orb_->string_to_object(get_opts.optarg);
		p->grm = Grm::_narrow(obj);
		break;

	case 'v': // Lrm_visited
		p->visited = decode(get_opts.optarg);
		break;

	case 'c': // category_name
		p->category = CORBA::string_dup(get_opts.optarg);
		break;

	case 'o': // component_name
		p->component = CORBA::string_dup(get_opts.optarg);
		break;

	case 'a': // args
		p->args = decode(get_opts.optarg);
		break;

//
//		-r resource_spec  (hex decimal coded)
//

	case 's': // constraint
		p->constr = decode(get_opts.optarg);
		break;

	case 'f': // preference
		p->pref = decode(get_opts.optarg);
		break;

//
//		-p policies       (hex decimal coded)
//		-z desired_props  (hex decimal coded)
//
	}
#endif
}

/////////////////// Send update ///////////////////////////////
void Lrm_i::discover_Grm(CORBA::ORB_var& orb_var,
			 CosTrading::Lookup_var& lookup_if_var,
			 ResourceMgmt::Grm_var& Grm_var) {
  //
  // Find Trader server
  //
  CORBA::Object_var trading_obj;

  for (int i = 0 ; i < 5 ; ++i) {
    trading_obj = orb_var->resolve_initial_references("TradingService");
    if (CORBA::is_nil(trading_obj.in())) {
      ACE_OS::printf("TradingService is nil. Try again\n");
      ACE_OS::sleep(1);
    } else {
      ACE_OS::printf("TradingService resolved\n"); // teste
      break;
    }
  }

  if (CORBA::is_nil(trading_obj.in())) {
    ACE_OS::printf("TradingService is nil.\n");
    exit(0);
    return;
  }

  CORBA::Environment ACE_TRY_ENV;

  ACE_TRY {
    lookup_if_var = CosTrading::Lookup::_narrow(trading_obj.in(), ACE_TRY_ENV);
    ACE_TRY_CHECK;
    ACE_OS::printf("lookup_if narrowed\n"); // teste

    CosTrading::PolicySeq policies;	// Empty sequence

    CosTrading::Lookup::SpecifiedProps desired_props;	// Don't return props
    desired_props._d(CosTrading::Lookup::none);

    CosTrading::PolicyNameSeq_var       policies_applied;  // out param
    CosTrading::OfferSeq_var            offers;            // out param
    CosTrading::OfferIterator_var       iterator;          // out param

    // Run query
    lookup_if_var->query("ResourceMgmt_Grm",
			 "TRUE",
			 "",
			 policies,
			 desired_props,
			 1,
			 offers,
			 iterator,
			 policies_applied,
			 ACE_TRY_ENV);
    ACE_OS::printf("query returned (ResourceMgmt_Grm)\n"); // teste
    ACE_TRY_CHECK;
    ACE_OS::printf("query returned without exception\n"); // teste

    if (offers->length() == 0) {
      ACE_OS::printf("No matching Grm\n");
      Grm_var = ResourceMgmt::Grm::_nil();
    } else {
      Grm_var = ResourceMgmt::Grm::_narrow(offers[0].reference);
      if (CORBA::is_nil(Grm_var.ptr())) {
	ACE_OS::printf("No matching Grm\n");
      } else {
	ACE_OS::printf("Grm found\n");
      }
    }

    // Clean up
    if (!CORBA::is_nil(iterator.ptr()))
      iterator->destroy();
  }
  ACE_CATCH (CosTrading::UnknownServiceType, unknown_serv) {
    ACE_UNUSED_ARG(unknown_serv);
    ACE_OS::printf("discover_Grm() UnknownServiceType\n");
    ACE_TRY_ENV.print_exception ("UnknownServiceType Exception");
    exit(1);
  }
  ACE_CATCH (CosTrading::IllegalServiceType, ill_serv) {
    ACE_UNUSED_ARG(ill_serv);
    ACE_OS::printf("discover_Grm() IllegalServiceType\n");
    ACE_TRY_ENV.print_exception ("IllegalServiceType Exception");
    exit(1);
  }
  ACE_CATCH (CosTrading::IllegalConstraint, ill_constr)	{
    // If we fail to the constraint, just return False
    ACE_UNUSED_ARG (ill_constr);
    ACE_OS::printf("discover_Grm() IllegalConstraint\n");
    ACE_TRY_ENV.print_exception ("IllegalConstraint Exception");
    exit(1);
  }
  ACE_CATCH (CosTrading::Lookup::IllegalPreference, ill_pref) {
    ACE_UNUSED_ARG (ill_pref);
    ACE_OS::printf("discover_Grm() Lookup::IllegalPreference\n");
    ACE_TRY_ENV.print_exception ("Lookup::IllegalPreference Exception");
    exit(1);
  }
  ACE_CATCH (CosTrading::Lookup::IllegalPolicyName, ill_poly) {
    ACE_UNUSED_ARG (ill_poly);
    ACE_OS::printf("discover_Grm() Lookup::IllegalPolicyName\n");
    ACE_TRY_ENV.print_exception ("Lookup::IllegalPolicyName Exception");
    exit(1);
  }
  ACE_CATCH (CosTrading::Lookup::PolicyTypeMismatch, mismatch_poly) {
    ACE_UNUSED_ARG (mismatch_poly);
    ACE_OS::printf("discover_Grm() Lookup::PolicyTypeMismatch\n");
    ACE_TRY_ENV.print_exception ("Lookup::PolicyTypeMismatch Exception");
    exit(1);
  }
  ACE_CATCH (CosTrading::Lookup::InvalidPolicyValue, invalid_poly) {
    ACE_UNUSED_ARG (invalid_poly);
    ACE_OS::printf("discover_Grm() Lookup::InvalidPolicyValue\n");
    ACE_TRY_ENV.print_exception ("Lookup::InvalidPolicyValue Exception");
    exit(1);
  }
  ACE_CATCH (CosTrading::IllegalPropertyName, ill_property) {
    ACE_UNUSED_ARG (ill_property);
    ACE_OS::printf("discover_Grm() IllegalPropertyName\n");
    ACE_TRY_ENV.print_exception ("IllegalPropertyName Exception");
    exit(1);
  }
  ACE_CATCH (CosTrading::DuplicatePropertyName, dup_property) {
    ACE_UNUSED_ARG (dup_property);
    ACE_OS::printf("discover_Grm() DuplicatePropertyName\n");
    ACE_TRY_ENV.print_exception ("DuplicatePropertyName Exception");
    exit(1);
  }
  ACE_CATCH (CosTrading::DuplicatePolicyName, dup_policy) {
    ACE_UNUSED_ARG (dup_policy);
    ACE_OS::printf("discover_Grm() DuplicatePolicyName\n");
    ACE_TRY_ENV.print_exception ("DuplicatePolicyName Exception");
    exit(1);
  }
  ACE_CATCH(CORBA::SystemException, sysex) {
    ACE_UNUSED_ARG(sysex);
    ACE_OS::printf("discover_Grm SystemException()\n");
    ACE_TRY_ENV.print_exception("System Exception");
    exit(1);
  }
  ACE_CATCH(CORBA::UserException, userex) {
    ACE_UNUSED_ARG(userex);
    ACE_OS::printf("discover_Grm() UserException\n");
    ACE_TRY_ENV.print_exception("User Exception");
    exit(1);
  }
  ACE_ENDTRY;
}

void Lrm_i::register_Lrm(CORBA::ORB_var& o_, CosTrading::Lookup_var& lookup_) {
  ACE_UNUSED_ARG(o_);

  ACE_TRY_NEW_ENV {
    regis_if = CosTrading::Register::_duplicate(lookup_->register_if(ACE_TRY_ENV));
    ACE_TRY_CHECK;

    ACE_OS::printf("Got register_if()\n"); // teste

    if (CORBA::is_nil(regis_if.in())) {
      ACE_OS::printf("register_Lrm() No regiser_if() ???\n");
      ACE_OS::exit(1);
    }

    CosTrading::PropertySeq_var prop_var;

    Local_DB_Sem->acquire();
    prop_var = Local_DB::getResources();
    Local_DB_Sem->release();

    //    CORBA::Object_var reference =  o_->string_to_object(self_.in(), ACE_TRY_ENV);
    CORBA::Object_var reference =  ResourceMgmt::Lrm::_duplicate(self_.in());
    //    ACE_TRY_CHECK;

    ACE_OS::printf("register_Lrm() before _narrow()\n"); // teste

    ResourceMgmt::Lrm_var L_ =  ResourceMgmt::Lrm::_narrow(reference, ACE_TRY_ENV);
    ACE_TRY_CHECK;
    if (CORBA::is_nil(L_.in())) {
      ACE_OS::printf("register_Lrm() Cannot narrow to Lrm\n");
      ACE_OS::exit(1);
    }

    ACE_OS::printf("register_Lrm() before contacting register interface\n"); // teste

    OfferId_var = regis_if->_cxx_export (reference.in(),
					 "ResourceMgmt_Lrm",
					 prop_var.in(),
					 ACE_TRY_ENV);
    //    ACE_OS::printf("register_Lrm() done and need check\n");
    ACE_TRY_CHECK;
    
    ACE_OS::printf("register_Lrm() done\n"); // teste
  }
  ACE_CATCHANY {
    ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, "Lrm_i::_cxx_export");
    return;
  }
  ACE_ENDTRY;

  // MOD 
  // notifica o Grm da mudanca significativa
  /*
  ACE_OS::printf("Lrm_i::register_Lrm: before calling Grm_->update_notify()\n");
  CORBA::Environment ACE_TRY_ENV;
  Grm_->update_notify(ACE_TRY_ENV);
  ACE_OS::printf("Lrm_i::register_Lrm: after calling Grm_->update_notify()\n");
  */
  // FIM-MOD
}

void
Lrm_i::shutdown(void)
{
	ACE_TRY_NEW_ENV {
		ACE_OS::printf("Lrm_i::shutdown()\n");

		if (CORBA::is_nil(regis_if.in())) {
			ACE_OS::printf("register_Lrm() No regiser_if() ???\n");
			ACE_OS::exit(1);
		}

		regis_if->withdraw(OfferId_var.in(), ACE_TRY_ENV);
		ACE_TRY_CHECK;

	}
        ACE_CATCHANY
        {
		ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, "send_Update()");
		return;
        }
        ACE_ENDTRY;
}

void Lrm_i::send_Update(CosTrading::Lookup_var& lookup_) {
  ACE_TRY_NEW_ENV {
    CosTrading::Register_var regis_if_ = lookup_->register_if();

    if (CORBA::is_nil(regis_if_.in())) {
      ACE_OS::printf("register_Lrm() No regiser_if() ???\n");
      ACE_OS::exit(1);
    }

    CosTrading::PropertyNameSeq del_list;
    del_list.length(0);

    CosTrading::PropertySeq_var prop_var;

    Local_DB_Sem->acquire();
    prop_var = Local_DB::getResources();
    Local_DB_Sem->release();

    CORBA::ULong length = prop_var->length();

    // O trecho abaixo esta fazendo retornar uma excessao PropertyTypeMismatch
    // Nao parece dar problemas
    prop_var->length(length + 1);
    prop_var[length].name = CORBA::string_dup("timeout");
    prop_var[length].value <<= (CORBA::ULong)update_interval;

    ACE_OS::printf("Send update\n"); // teste

    // teste
    for (CORBA::ULong i = 0; i < length; i++) {
      ACE_OS::printf("property[%d] name = %s\t", i, prop_var[i].name.in());
      CORBA::Any::dump(prop_var[i].value);
      ACE_OS::printf("\n");
    }
    // fim-teste

    regis_if_->modify(OfferId_var.in(),
		      del_list,
		      prop_var.in(),
		      ACE_TRY_ENV);
    
    ACE_TRY_CHECK;
    ACE_OS::printf("Send update done\n"); // teste
 
    // MOD
    ACE_OS::printf("Calling Grm update\n"); // teste
    Grm_->update_notify(ACE_TRY_ENV);
    ACE_OS::printf("Grm update done\n"); // teste
    // FIM-MOD
  }
  // teste
  ACE_CATCH (CosTrading::PropertyTypeMismatch, propt) {
    ACE_PRINT_EXCEPTION(propt, "CosTrading::PropertyTypeMismatch Exception");
    ACE_OS::printf("** Tipo: %s, nome: %s\n", propt.type.in(), propt.prop.name.in());
    CORBA::Any::dump(propt.prop.value);
    ACE_OS::printf("\n");
    return;
  }
  // fim-teste
  ACE_CATCHANY {
    ACE_PRINT_EXCEPTION(ACE_ANY_EXCEPTION, "send_Update()");
    return;
  }
  ACE_ENDTRY;
}

/////////////////// update thread ///////////////////////////////
static
void *
Update_thread(void *ptr)
{
Lrm_i *lrm_impl = (Lrm_i *)ptr;

  CORBA::ORB_var o_;
  CosTrading::Lookup_var lookup_;
  ResourceMgmt::Grm_var Grm_var;

//  P_Timer pt;

  ACE_DECLARE_NEW_CORBA_ENV;
  ACE_TRY {
    ACE_OS::printf("Update_thread() started 1\n");

    int argc_ = lrm_impl->argc__;
    char** argv_ = lrm_impl->argv__;

    o_ = CORBA::ORB_init (argc_, argv_, "", ACE_TRY_ENV);
    ACE_TRY_CHECK;

    ACE_OS::printf("Update_thread() orb initialized 1\n");

    lrm_impl->discover_Grm(o_, lookup_, Grm_var);
    ACE_OS::printf("Update_thread() discover_Grm() done\n");

    lrm_impl->register_Lrm(o_, lookup_);
    
    ACE_OS::printf("Update_thread started 2\n");

    while(1) {
      // Local resource probe interval 10 sec
      //      int probe_interval = 30;
      int probe_interval = 600; // MOD mudei pra nao ficar esperando muito

      int remaining_quantum = lrm_impl->update_interval;

      while(remaining_quantum > 0) {
	remaining_quantum -= probe_interval;
	ACE_OS::sleep(probe_interval);
	
	//			pt.start();
	if (Local_DB::poll() || (remaining_quantum <= 0)) {
	  //				pt.stop();
	  //				pt.print();
	  //				pt.start();
	  lrm_impl->send_Update(lookup_);
	  //				pt.stop();
	  //				pt.print();
	  remaining_quantum = lrm_impl->update_interval;
	} else {
	  //				pt.stop();
	  //				pt.print();
	  ACE_OS::printf("No significant change in the local resource\n");
	}
      }
    }
  }
  ACE_CATCHANY {
    ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, "Update_thread: exception raised");
  }
  ACE_ENDTRY;
  return 0;
}

////////////////////////////////////////////////////////////////

static
void *
prelaunch_thread(void *ptr)
{
Lrm_i *lrm_impl = (Lrm_i *)ptr;

  CORBA::ORB_var o_;
  CosTrading::Lookup_var lookup_;
  ResourceMgmt::Grm_var Grm_var;

  ACE_DECLARE_NEW_CORBA_ENV;
  ACE_TRY
    {
        ACE_OS::printf("prelaunch_thread() started\n");

	int argc_ = lrm_impl->argc__;
	char** argv_ = lrm_impl->argv__;

        o_ = CORBA::ORB_init (argc_, argv_, "", ACE_TRY_ENV);
        ACE_TRY_CHECK;

        ACE_OS::printf("prelaunch_thread() orb initialized 1\n");

	lrm_impl->prelaunch_process(ACE_TRY_ENV);
       	ACE_TRY_CHECK;

	lrm_impl->prelaunch_ready_Sem->release();

	while(1) {
		lrm_impl->prelaunch_request_Sem->acquire();

//               	ACE_OS::sleep(2);
		lrm_impl->prelaunch_process(ACE_TRY_ENV);
        	ACE_TRY_CHECK;

		lrm_impl->prelaunch_ready_Sem->release();
        }
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION,
                           "Update_thread: exception raised");
    }
  ACE_ENDTRY;
  return 0;
}

////////////////////////////////////////////////////////////////

void Lrm_i::start_Update(CORBA::Long interval, CORBA::Environment & ACE_TRY_ENV) {
  ACE_UNUSED_ARG(ACE_TRY_ENV);

  static int started = 0;

//	started = 1;

  if (!started) {
    ACE_Thread_Manager::instance ()->spawn(Update_thread, (void *)this);

    if (prelaunch_flag)
      ACE_Thread_Manager::instance ()->spawn(prelaunch_thread, (void *)this);
  }

  started = 1;

  if (interval > 0) update_interval = interval;
}

