// -*- C++ -*-
// $Id: Lrm_i.h,v 1.2 2000/09/30 18:39:46 yamane Exp $
// ============================================================================
//
// = LIBRARY
//    RMP/Lrm
//
// = FILENAME
//    Lrm_i.h
//
// = DESCRIPTION
//    This class implements the RMP Lrm IDL interface.
//
// = AUTHOR
//    Tomonori Yamane <yamane@uiuc.edu>
//
// ============================================================================

#ifndef LRM_I_H
#define LRM_I_H

#include "ace/Thread_Manager.h"
#include "ace/Process.h"
#include "ace/Get_Opt.h"


#include "orbsvcs/ResourceMgmtS.h"
#include "orbsvcs/ResourceMgmtC.h"

#include "Cloakroom_Helper.h"

class Lrm_i: public POA_ResourceMgmt::Lrm
{
  // = TITLE
  //    Lrm Object Implementation
  //
  // = DESCRIPTION
  //    Implementation of a simple object that has all methods of Lrm.
  //
public:
  // = Initialization and termination methods.
  Lrm_i (int argc, char **argv);
  // Constructor

  Lrm_i (Lrm_i &);
  // Copy Constructor added because g++ generates a 
  // bad one if not defined

  ~Lrm_i (void);
  // Destructor
 
  char * mark_here (
	const char *visited
	);  
  // Add my hostname to the visited list

  virtual void perform_query (
	const ResourceMgmt::QueryPacket & req,
	CORBA::Environment &ACE_TRY_ENV
	);  
  // Accept query from the remote machine
  
  virtual void ACK (
	CORBA::Long sem_tag,
	const CosTrading::OfferSeq & offers,
	CORBA::Environment &ACE_TRY_ENV
	);  

  virtual void NACK (
	CORBA::Long sem_tag,
	CORBA::Environment &ACE_TRY_ENV
	);

  virtual void trigger_update (
	ResourceMgmt::Grm_ptr grm,
	CORBA::Long distribution,
	CORBA::Long period,
	CORBA::Environment &ACE_TRY_ENV
	);

  virtual char * set_threshold (
	const char * constr,
	CORBA::Environment &ACE_TRY_ENV
	);

  virtual CosTrading::OfferSeq * execute_application (
	const char * category,
	const char * component,
	const char * args,
	const CosTrading::PropertySeq & resource_spec,
	const char * constr,
	const char * pref,
	/* const CosTrading::PolicySeq & policies, */
	const CosTrading::Lookup::SpecifiedProps & desired_props,
	CORBA::Environment &ACE_TRY_ENV
	);

  virtual void shutdown (
  	CORBA::Environment &ACE_TRY_ENV
  	);
  
  void orb (CORBA::ORB_ptr o);
  // Set the ORB pointer.

  CORBA::ORB_ptr orb (void);
  // Get the ORB pointer.
 
  void Grm(ResourceMgmt::Grm_ptr grm);

  CosTrading::OfferSeq * execute_application (
	ResourceMgmt::QueryPacket & req,
	CORBA::Environment &ACE_TRY_ENV
	); 
  // CreateComponent it is not exported as an IDL interface

  ResourceMgmt::BaseProcess * prelaunch_process(
        CORBA::Environment &ACE_TRY_ENV
        );
  // Prelaunch createdProcess

  
  size_t command_line_length(
	const ResourceMgmt::QueryPacket *packet
	);
  // estimate command line buffer size

  void compose_args(
	ACE_Process_Options &options,
	const ResourceMgmt::QueryPacket *packet,
	CORBA::Environment &ACE_TRY_ENV
	);
  // create command line options

  void compose_packet(
	ResourceMgmt::QueryPacket *packet,
	int argc,
	char** argv,
	CORBA::Environment& ACE_TRY_ENV
	);
  // create Objects from the command line options

  void discover_Grm(CORBA::ORB_var& orb_var, 
			CosTrading::Lookup_var& lookup_if_var,
			ResourceMgmt::Grm_var& Grm_var);
  // find Grm

  void register_Lrm (CORBA::ORB_var& o_, CosTrading::Lookup_var& lookup_);
  // send report to the Grm

  void send_Update (CosTrading::Lookup_var& lookup_);
  // send report to the Grm

  void start_Update (CORBA::Long interval, CORBA::Environment &env);
  // Start Update thread

  short update_interval;
  // report interval --- should not be public

  ResourceMgmt::Lrm_var self_;
  // Lrm itself

  int argc__;
  // Number of commandline arguments.

  char **argv__;
  // commandline arguments.

  void shutdown(void);
  // Unregister Lrm

  void set_prelaunch_mode (void);
  // activate prelaunch mode

  ACE_Semaphore *prelaunch_ready_Sem;
  // Sepahore to notify prelaunch is done

  ACE_Semaphore *prelaunch_request_Sem;
  // Sepahore to request prelaunch

private:
  virtual void issue_query (
	const ResourceMgmt::QueryPacket & req,
	CORBA::Environment& ACE_TRY_ENV
	);  
  // Send query to the Grm

  CORBA::Boolean eval_constraint(
	const char *constraint,
	CosTrading::PropertySeq_var& prop_var,
	CORBA::Environment& ACE_TRY_ENV
	);
 
  void common_query (
	const ResourceMgmt::QueryPacket & req,
	CORBA::Environment &ACE_TRY_ENV
	); 
  // Check local resource or issue remote query

  char *encode(
	const char *original,
	size_t	len = 0
	);
  // encode a string to its hexdecimal representation
  // If we don't specify the length, we will use strlen() to get the length

  char *decode(
	const char *original
	);
  // decode a string to its original representation

  // Get hexdecimal representation of the string

  CORBA::ORB_var orb_;
  // ORB pointer.

  CORBA::String_var hostname_;

  ResourceMgmt::Grm_var	Grm_;
  // Grm to issue update request

  CosTrading::Register_var regis_if;
  // Register interface of the trader

  CosTrading::OfferId_var OfferId_var;
  // Lrm OfferId to issue update request

  CORBA::String_var threshold_;
  // Constraint to determine the threshold

  ACE_Semaphore *Local_DB_Sem;
  // Sepahore to protect Local_DB

  CORBA::String_var ior_options_;
  // "-ORBtradingserviceport xxx -ORBtradingserviceiport xxx"

  int prelaunch_flag;

  ResourceMgmt::BaseProcess_var pre_process_;
  // prelaunched process
};

#endif /* LRM_I_H */
