/******************************************************************************
*                                                                             *
*  File: Lrm_server_i.cpp                                                     *
*                                                                             *
*  Description: A test server for the Lrm test.                               *
*               Based on the TAO Cubit example.                               *
*                                                                             *
*  Project: 2K - Resource Management                                          *
*  Author: Tomonori Yamane (yamane@cs.uiuc.edu)                               *
*                                                                             *
*******************************************************************************
 Revision Control Information
 $Revision: 1.1 $
 $Log: Lrm_server_i.cpp,v $
 Revision 1.1  2000/02/19 20:31:01  yamane
 Initial version


 Revision 1.1  1999/02/22 22:29:16  f-kon
 *** empty log message ***

******************************************************************************/

#include "ace/Read_Buffer.h"

#include "Lrm_server_i.h"

LrmServer_Shutdown::LrmServer_Shutdown (LrmServer& lrmserver)
  : done(0), lrmserver_ (lrmserver) {
  if (this->shutdown_.register_handler (SIGINT, this) == -1) {
    ACE_ERROR ((LM_ERROR, "%p\n", "register_handler"));
  }

  if (this->shutdown_.register_handler (SIGTERM, this) == -1) {
    ACE_ERROR ((LM_ERROR, "%p\n", "register_handler"));
  }
}

int
LrmServer_Shutdown::handle_signal (int signum, siginfo_t* sinfo, ucontext_t* ucon) {
  ACE_UNUSED_ARG (signum);
  ACE_UNUSED_ARG (sinfo);
  ACE_UNUSED_ARG (ucon);
  if (!done) {
    done = 1;
    this->lrmserver_.~LrmServer ();
  }
  exit (0);
  return 0;
}


LrmServer::LrmServer (void)
 : prelaunch_flag(0) {
}

int
LrmServer::parse_args (void) {
  ACE_Get_Opt get_opts (argc_, argv_, "dpg");
  int c;

  while ((c = get_opts ()) != -1) {
    switch (c)
      {
      case 'd':  // debug flag.
        TAO_debug_level++;
        break;
      case 'p':  // prelaunch flag.
	ACE_OS::printf("Prelaunch flag in LrmServer class\n");
        prelaunch_flag = 1;
        break;
      case 'g': break;
      case '?':
      default:
        ACE_ERROR_RETURN ((LM_ERROR,
                           "usage:  %s"
                           " [-d]"
                           "\n",
                           argv_ [0]),
                          1);
      }
  }

  // Indicates successful parsing of command line.
  return 0;
}

CosNaming::NamingContext_ptr
LrmServer::create_context (const CosNaming::NamingContext_ptr inc,
			   const CosNaming::Name name) {
  CosNaming::Name part_name;
  CosNaming::NamingContext_var nc;

  CORBA::Object_var obj;

  CORBA::Environment ACE_TRY_ENV;

  for (CORBA::ULong i = 0; i < name.length(); i++) {
    ACE_OS::printf("create context\n");
    
    part_name = name;
    part_name.length(i + 1);

    ACE_TRY {
      nc = inc->bind_new_context(part_name, ACE_TRY_ENV);
      ACE_TRY_CHECK;
    }
    ACE_CATCH (CosNaming::NamingContext::AlreadyBound, ex) {
      ACE_TRY_ENV.clear();
      obj = inc->resolve(part_name);
      nc = CosNaming::NamingContext::_narrow(obj);
    }
    ACE_ENDTRY;
  }

  ACE_OS::printf("create context ends\n");

  return nc._retn();
}

//
// register object to the name server
//
void
LrmServer::name_register (CORBA::ORB_ptr o)
{
        if ((TAO_default_environment()).exception()) {
		ACE_OS::printf("name_register() somebody already set exception\n");
        }
	char buf[512];
	ACE_OS::hostname(buf, 511);

	//
	// Find nameserver
	//
	CORBA::Object_var obj;

	for (int i = 0 ; i < 5 ; ++i) {
		obj = o->resolve_initial_references("NameService");
		if (CORBA::is_nil(obj.in())) {
			ACE_OS::printf("NamingService is nil. Try again\n");
			ACE_OS::sleep(1);
		} else {
			ACE_OS::printf("NamingService resolved\n");
			break;
		}
	}

	if (CORBA::is_nil(obj.in())) {
		ACE_OS::printf("NamingService is nil.\n");
		exit(0);
		return;
	}

	CosNaming::NamingContext_var inc, nc;
	inc = CosNaming::NamingContext::_narrow(obj);

	CosNaming::Name name;
	name.length(4);
	name[0].id = CORBA::string_dup("SRG");
	name[0].kind = CORBA::string_dup("");
	name[1].id = CORBA::string_dup("services");
	name[1].kind = CORBA::string_dup("");
	name[2].id = CORBA::string_dup("RMP");
	name[2].kind = CORBA::string_dup("");
	name[3].id = CORBA::string_dup("Lrm");
	name[3].kind = CORBA::string_dup("");

	nc = create_context(inc, name);

	name.length(1);

	name[0].id = CORBA::string_dup(buf);
	name[0].kind = CORBA::string_dup("");

	nc->rebind(name, Lrm_impl_->_this());
}

int
LrmServer::init(int argc, char** argv, CORBA::Environment& ACE_TRY_ENV) {
  int ret;
  argc_ = argc;
  argv_ = argv;

  ACE_OS::printf("LrmServer::init()\n");

  // Call the init of <TAO_ORB_Manager> to initialize the ORB and
  // create a child POA under the root POA.
  if (orb_manager_.init_child_poa(argc, argv, "child_poa", ACE_TRY_ENV) == -1) {
    ACE_OS::printf("Looks like we already have Lrm running on the same host\n");
    ACE_OS::exit (1);
//    ACE_ERROR_RETURN ((LM_ERROR,
//                       "%p\n",
//                       "init_child_poa"),
//                      -1);
   
  }
  ACE_CHECK_RETURN (-1);

  ret = parse_args ();
  if (ret != 0)
    return ret;

  //
  // start ORB
  //
  CORBA::ORB_var orb = orb_manager_.orb();

  ACE_TRY {
    CORBA::Object_var poa_object = orb->resolve_initial_references("RootPOA");
    if (CORBA::is_nil (poa_object.in ())) {
      ACE_ERROR_RETURN ((LM_ERROR,
			 " (%P|%t) Unable to initialize the POA.\n"),
			1);
    }

    PortableServer::POA_var root_poa = PortableServer::POA::_narrow(poa_object.in (), ACE_TRY_ENV);
    ACE_TRY_CHECK;

    PortableServer::POAManager_var poa_manager = root_poa->the_POAManager(ACE_TRY_ENV);
    ACE_TRY_CHECK;

    // Create a Lrm server to receive message from Grm
    Lrm_impl_ = new Lrm_i(argc_, argv_);

    ACE_OS::printf("Lrm_impl_ = 0x%08x\n", Lrm_impl_);

    ACE_OS::printf("We are here 0\n");

    // Set orb
    Lrm_impl_->orb(orb);

    ACE_OS::printf("We are here 1\n");

    ACE_OS::printf("We are here 2\n");

    ior_ = orb_manager_.activate_under_child_poa ("LrmWorld",
						  Lrm_impl_,
						  ACE_TRY_ENV);

    ACE_OS::printf("We are here 3\n");
    ACE_TRY_CHECK;

    poa_manager->activate (ACE_TRY_ENV);
    ACE_TRY_CHECK;

    if ((TAO_default_environment()).exception()) {
      ACE_OS::printf("before _narrow() somebody already set exception\n");
    }

    //	Lrm_impl_->self_ = ior_;
    //	CORBA::Object_var obj_ = orb->string_to_object(ior_, ACE_TRY_ENV);
    //	Lrm_impl_->self_ = ResourceMgmt::Lrm::_duplicate(ResourceMgmt::Lrm::_narrow(obj_.in()));
    Lrm_impl_->self_ = Lrm_impl_->_this(ACE_TRY_ENV);
    ACE_TRY_CHECK;

    ACE_DEBUG ((LM_DEBUG,
		"The Lrm IOR is: <%s>\n",
		ior_.in ()));

    ACE_OS::printf("We are here 4\n");

    // Make the Naming Service locatable through iioploc.
    if (orb->_tao_add_to_IOR_table("LrmService", Lrm_impl_->self_.in ()) == -1) {
      ACE_DEBUG ((LM_DEBUG, "LrmService: cannot add to ior table.\n"));
      return -1;
    }

    name_register(orb.in());

    CosTrading::Lookup_var lookup_;
    ResourceMgmt::Grm_var Grm_var;
    Lrm_impl_->discover_Grm(orb, lookup_, Grm_var);
    Lrm_impl_->Grm(Grm_var);

    if (prelaunch_flag) {
      Lrm_impl_->set_prelaunch_mode();
    }

    ACE_OS::printf("We are here 5\n");

    // MOD
    // NOTA: fazer algo mais elegante para usar essa opcao com
    // o nome do GrmExt
    /*
    if (ACE_OS::strcmp("nao", argv_[1]) != 0) {
      ACE_OS::printf("Lrm_server_i::init() GRmExt name - %s\n", argv_[1]);
      
      ResourceMgmt::GrmExt_var grmExt;
      CORBA::Object_var name_obj = orb->resolve_initial_references("NameService");

      if (CORBA::is_nil(name_obj)) {
	ACE_OS::printf("Lrm_server_i::init(): NameService is nil.\n");
      }
      else {
	ACE_OS::printf("AAAAAAAAAAAAAAAAAAAaa\n");
	CosNaming::Name name;
	CORBA::Object_var grm_ext_obj;
	ResourceMgmt::GrmExt_var grm_ext;
	CosNaming::NamingContext_var inc = CosNaming::NamingContext::_narrow(name_obj);
	
	name.length(2);
	name[0].id = CORBA::string_dup("grm_ext");
	name[1].id = CORBA::string_dup(argv_[1]);

	grm_ext_obj = inc->resolve(name);
	grm_ext = ResourceMgmt::GrmExt::_narrow(grm_ext_obj);
	grm_ext->test("cccccccccccccc\n");
      }
    }
    */
   /* 
    ResourceMgmt::GrmExt_var grmExt;
    CORBA::Object_var objGrmExt = orb->string_to_object("file:///tmp/IOR_GrmExt");
    if (CORBA::is_nil(objGrmExt)) {
      ACE_OS::printf("Lrm_server_i::init: NULL\n");
      Grm_var->setParent(ResourceMgmt::GrmExt::_nil(), ACE_TRY_ENV);
    } else {
      grmExt = ResourceMgmt::GrmExt::_narrow(objGrmExt);
      grmExt->test("cccccccccccccc\n");
      Grm_var->setParent(ResourceMgmt::GrmExt::_duplicate(grmExt), ACE_TRY_ENV);
    }
    */
    // FIM-MOD

    ACE_OS::printf("We are here 6\n");

    Lrm_impl_->start_Update(0, ACE_TRY_ENV);

  }
  ACE_CATCH(CORBA::SystemException, sysex2) {
    ACE_UNUSED_ARG(sysex2);
    ACE_TRY_ENV.print_exception("System Exception");
    exit(1);
  }
  ACE_CATCH(CORBA::UserException, userex2) {
    ACE_UNUSED_ARG(userex2);
    ACE_TRY_ENV.print_exception("User Exception");
    exit(1);
  }
  ACE_ENDTRY;
 
  ACE_OS::printf("Lrm_server_i::init() ends\n");

  return 0;
}

int
LrmServer::run (CORBA::Environment& env)
{

  LrmServer_Shutdown lrmserver_shutdown (*this);

  if (orb_manager_.run (env) == -1)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "LrmServer::run"),
                      -1);
  return 0;
}

LrmServer::~LrmServer (void)
{
	Lrm_impl_->shutdown();
}
