/******************************************************************************
*                                                                             *
*  File: Lrm_server_i.h                                                       *
*                                                                             *
*  Description: A test server for the Lrm experiment.                         *
*               Based on the TAO Cubit example.                               *
*                                                                             *
*  Project: 2K - Resource management                                          *
*  Author: Tomonori Yamane (yamane@uiuc.edu)                                  *
*                                                                             *
*******************************************************************************
 Revision Control Information
 $Revision: 1.1 $
 $Log: Lrm_server_i.h,v $
 Revision 1.1  2000/02/19 20:31:01  yamane
 Initial version

 Revision 1.1.1.2  1999/09/07 13:22:55  col
 *** empty log message ***

 Revision 1.1.1.1  1999/04/01 18:16:37  f-kon
 This new 2K source tree contains the most up-to-date version of ACE and TAO
 and including the dynamicTAO code as well as some 2K tests and preliminary
 versions of some of its high-level systems.


 Revision 1.1  1999/02/22 22:29:16  f-kon
 *** empty log message ***


******************************************************************************/

#if !defined (LRM_SERVER_I_H)
#define LRM_SERVER_I_H

#include "ace/Get_Opt.h"
#include "tao/TAO.h"
#include "ace/Auto_Ptr.h"

#include "orbsvcs/CosNamingS.h"
#include "orbsvcs/CosNamingC.h"
#include "orbsvcs/Naming/Naming_Utils.h"



#include "Lrm_i.h"

class LrmServer;

class LrmServer_Shutdown : public ACE_Event_Handler
{
public:

  LrmServer_Shutdown (LrmServer& lrmserver);

  virtual int handle_signal (int, siginfo_t*, ucontext_t*);

private:
  int done;

  LrmServer& lrmserver_;
  ACE_Sig_Handler shutdown_;
};

class LrmServer
{
public:
  // = Initialization and termination methods.
  LrmServer (void);
  // Default constructor

  ~LrmServer (void);
  // Destructor

  CosNaming::NamingContext_ptr create_context(
	const CosNaming::NamingContext_ptr inc,
	const CosNaming::Name name);
  // Create specified context if it does not exist

  void name_register (CORBA::ORB_ptr o);
  // register to the name service

  int init (int argc,
            char **argv,
	    CORBA::Environment& env);

  int run (CORBA::Environment& env);
  // Run the orb 

private:
  int parse_args (void);
  // Parses the commandline arguments.

  TAO_ORB_Manager orb_manager_;
  // The ORB manager.

  Lrm_i *Lrm_impl_;

  int argc_;
  // Number of commandline arguments.

  char **argv_;
  // commandline arguments.

  ResourceMgmt::Grm_var Grm_;
  // Grm Rup pointer.

  CORBA::Environment env_;
  // Environment variable.

  CORBA::ORB_var orb_;
  // Remember our orb.

  CORBA::String_var ior_;
  // IOR of the trader kept around for handiness purposes.

  int prelaunch_flag;
};

#endif /* LRM_SERVER_I_H */
