// $Id: Offer_Sem.cpp,v 1.1 2000/02/19 20:31:51 yamane Exp $
// ============================================================================
//
// = LIBRARY
//    RMP/Lrm
//
// = FILENAME
//    Offer_Sem.cpp
//
// = DESCRIPTION
//    This class implements the oneway helper.
//
// = AUTHOR
//    Tomonori Yamane <yamane@uiuc.edu>
//
// ============================================================================

#include "Offer_Sem.h"
#include "Cloakroom_Helper.h"
#include "Cloakroom_Helper.cpp"	// for debugging

struct Offer_Sem {
	ACE_Semaphore *sem_ptr;
	CosTrading::OfferSeq **offers_ptr;
};

static Cloakroom_Helper<Offer_Sem> sem_list ;

#if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)

template class Cloakroom_Helper<Offer_Sem>;

template class ACE_Map_Manager<long, Offer_Sem, ACE_Null_Mutex>;
template class ACE_Map_Iterator_Base<long, Offer_Sem, ACE_Null_Mutex>;
template class ACE_Map_Iterator<long, Offer_Sem, ACE_Null_Mutex>;
template class ACE_Map_Reverse_Iterator<long, Offer_Sem, ACE_Null_Mutex>;
template class ACE_Map_Entry<long, Offer_Sem>;

#elif defined(ACE_HAS_TEMPLATE_INSTANTIATION_PRAGMA)

#pragma instantiate ACE_Map_Manager<long, Offer_Sem, ACE_Null_Mutex>;
#pragma instantiate ACE_Map_Iterator_Base<long, Offer_Sem, ACE_Null_Mutex>;
#pragma instantiate ACE_Map_Iterator<long, Offer_Sem, ACE_Null_Mutex>;
#pragma instantiate ACE_Map_Reverse_Iterator<long, Offer_Sem, ACE_Null_Mutex>;
#pragma instantiate ACE_Map_Entry<long, Offer_Sem>;

#endif

static ACE_Semaphore Offer_Global_Sem(1);

Offer_Sem_Manager::Offer_Sem_Manager(void)
{
}

Offer_Sem_Manager::~Offer_Sem_Manager(void)
{
}

long
Offer_Sem_Manager::checkin(CosTrading::OfferSeq **offers)
{
	ACE_Semaphore *sem_ptr = new ACE_Semaphore(0);
	Offer_Sem osem;
	long tag;

	osem.sem_ptr = sem_ptr;
	osem.offers_ptr = offers;

	ACE_OS::printf("Offer_Sem_Manager::checkin() sem_ptr = 0x%08x offers = 0x%08x\n", sem_ptr, offers); // teste

	Offer_Global_Sem.acquire();

	tag = sem_list.checkin(osem);

	Offer_Global_Sem.release();

	ACE_OS::printf("Offer_Sem_Manager::checkin() returns %d\n", tag); // teste
	return(tag);
}

void
Offer_Sem_Manager::checkout(long sem_tag, const CosTrading::OfferSeq *offers)
{
	Offer_Sem osem;

	ACE_OS::printf("Offer_Sem_Manager::checkout(%d)\n", sem_tag); // teste

	Offer_Global_Sem.acquire();

	osem = sem_list.lookup(sem_tag);
//	osem = sem_list.checkout(sem_tag);

	Offer_Global_Sem.release();

	(*osem.offers_ptr) = new CosTrading::OfferSeq(*offers);

	// resume waitor
	osem.sem_ptr->release();
}

void
Offer_Sem_Manager::swait(const long sem_tag)
{
	ACE_Semaphore *sem_ptr;

	Offer_Sem osem;
	ACE_OS::printf("Offer_Sem_Manager::swait(%d)\n", sem_tag); // teste

	Offer_Global_Sem.acquire();

	osem = sem_list.lookup(sem_tag);

	Offer_Global_Sem.release();

	sem_ptr = osem.sem_ptr;

	sem_ptr->acquire();

	sem_list.checkout(sem_tag);

	ACE_OS::printf("Offer_Sem_Manager::swait() recovered from swait\n"); // teste
	delete sem_ptr;
}
