#ifndef _CPU_API_H
#define _CPU_API_H
#define BATCH_SIZE 100

#include "Def.H"
#include "Shm.H"
#include "MsgQueue.H"
#include "ProcStatsBuf.H"
#include "SchedStatsBuf.H"
#include "Timeval.H"
#include "CpuServiceClass.H"

/*
 * See the "README" file for API definition.
 */
class CpuApi {
private:
  ShmCli* _shm;  // read-only shared memory with the server. 
  int _pid;  
  char* _psFname;  // file name for writing out the process stats.
  CliProcStatsBuf* _cpsBuf;  // process stats buffer.
  SchedStatsBuf* _ssBuf;  // server stats buffer.
  WrShmArea* _wrShmArea;  // writable shared memory with the server.
  Process* _p;  // the corresponding Process structure.
  MsgQueue* _msgQueue;  // message queue for communicating to the server.
  Msg _msg; 
  void _msgCall();
  int _ssSamplingInterval;
  int _probeCptFlag;

public:
  CpuApi();
  ~CpuApi();

  // see the README file for the API definition 
  int reserve(CpuReserve* cr, int pid=0);
  int modify(CpuReserve* cr, int pid=0);
  int free(int pid=0);
  int start(timeval begin = TIMEVAL_INIT, int pid = 0);
  int stop(int pid=0);
  int destroy(int pid=0);
  void yield();
  void svcDestroy();
  int setAdaptStrategy(AdaptStrategy* as, int pid=0);
  int setApDeadline(int deadline);
  int probe(int period = 0);
  int probeEnd();
  int probeMatch(CpuReserve* cr);
  int profileSave(int argc, char* argv);
  int profileGet(int argc, char* argv, float* util);

  int setRecordProcStats(char* fname=NULL);
  int setRecordSchedStats(char* fname=NULL, int samplingInterval=1000000);

  ProcStats* getProcStats(int pid=0);
  SchedStats* getSchedStats() { return(_shm->getSchedStats()); };
  int isPermNonconform(int pid=0);
  Process* getProcess(int pid=0);
  int getPidTab(int* pidTab, int len);
  ShmArea* getShmArea() { return(_shm->getArea()); };
  
  static int parseReserveArg(CpuReserve* cr, char* argv[], int mark);
  static void printReserveUsage();
  static int parseAdaptArg(AdaptStrategy* as, char* argv[], int mark);
  static void printAdaptUsage();
};

#endif _CPU_API_H
