/* 
 * Released Date: March 7 99
 * This is the Dynamic Soft CPU Scheduler II release by the 
 *   Monet Group, 
 *   http://cairo.cs.uiuc.edu/
 *   Department of Computer Science, 
 *   University of Illinois at Urbana Champaign.  
 * 
 * This release has been tested under SUN OS 5.6.
 *
 * Contacting Authors:
 *  Hao-hua Chu (h-chu3@cs.uiuc.edu)
 *  Klara Nahrstedt (klara@cs.uiuc.edu)
 */

#ifndef CPU_SERVICE_CLASS_H
#define CPU_SERVICE_CLASS_H

#include "Def.H"
#include "CpuReserve.H"


class CpuServiceClass
{
public:
  CpuServiceClass() {};
  int resetDeadline(Process* p, timeval* clock);
  virtual void adaptExpo(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf) = 0;
  virtual void adaptExpoUp(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf) = 0;
  virtual void adaptExpoRange(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf) = 0;
  virtual void adaptStat(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf) = 0;
  virtual void leakingBucketInit(Process* p, CpuReserve* cr) = 0;
  virtual void leakingBucketAdjust(Process* p, CpuReserve* cr) = 0;
  virtual int leakingBucketFill(Process* p, int usage) 
    { p->leakingBucket->fill(usage); return(p->leakingBucket->isNonconform()); };
  virtual int leakingBucketFlush(Process* p, int n=1);
  virtual void leakingBucketEmpty(Process* p) { p->leakingBucket->empty(); };
  static int isValid(CpuReserve* cr);
  static void print(CpuReserve* cr);
  static void print(AdaptStrategy* as);
  static void newReservation(Process* p);
  static float getUtil(CpuReserve* cr);
};



class CpuPcptClass : public CpuServiceClass
{
public:
  CpuPcptClass() {};
  void adaptExpo(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf);
  void adaptExpoUp(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf);
  void adaptExpoRange(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf);
  void adaptStat(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf);
  void leakingBucketInit(Process* p, CpuReserve* cr);
  void leakingBucketAdjust(Process* p, CpuReserve* cr);
};


class CpuPccptClass : public CpuServiceClass
{
public:
  CpuPccptClass() {};
  void adaptExpo(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf);
  void adaptExpoUp(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf);
  void adaptExpoRange(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf);
  void adaptStat(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf);
  void leakingBucketInit(Process* p, CpuReserve* cr);
  void leakingBucketAdjust(Process* p, CpuReserve* cr);
};



class CpuPvptClass : public CpuServiceClass
{
public:
  CpuPvptClass() {};
  void adaptExpo(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf);
  void adaptExpoUp(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf);
  void adaptExpoRange(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf);
  void adaptStat(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf);
  void leakingBucketInit(Process* p, CpuReserve* cr);
  void leakingBucketAdjust(Process* p, CpuReserve* cr);
  int leakingBucketFill(Process* p, int usage);
  int leakingBucketFlush(Process* p, int n=1);
  void leakingBucketEmpty(Process* p) { p->leakingBucket->empty(); p->leakingBucket2->empty(); };
};


class CpuAcptClass : public CpuServiceClass
{
public:
  CpuAcptClass() {};
  void adaptExpo(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf);  
  void adaptExpoUp(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf);
  void adaptExpoRange(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf);
  void adaptStat(CpuReserve* cr, CpuReserve* newCr, AdaptStrategy* as, ProcStatsBuf* psBuf);
  void leakingBucketInit(Process* p, CpuReserve* cr);  
  void leakingBucketAdjust(Process* p, CpuReserve* cr); 
};


extern CpuPcptClass cpuPcptClass;
extern CpuPvptClass cpuPvptClass;
extern CpuAcptClass cpuAcptClass;


#endif CPU_SERVICE_CLASS_H
