/* 
 * Released Date: March 7 99
 * This is the Dynamic Soft CPU Scheduler II release by the 
 *   Monet Group, 
 *   http://cairo.cs.uiuc.edu/
 *   Department of Computer Science, 
 *   University of Illinois at Urbana Champaign.  
 * 
 * This release has been tested under SUN OS 5.6.
 *
 * Contacting Authors:
 *  Hao-hua Chu (h-chu3@cs.uiuc.edu)
 *  Klara Nahrstedt (klara@cs.uiuc.edu)
 */

#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include "MsgQueue.H"


MsgQueue::MsgQueue(int svcFlag, int pid) 
{
  _svcFlag = svcFlag;
  _pid = pid;
  _fd = NULL;

  if (_svcFlag) {  // svc 
    if ((_fd = msgget(MSG_QUEUE_KEY, 0666 | IPC_CREAT)) < 0) {
      perror("msgget svc");
    }
  }
  else {  // cli
    if ((_fd = msgget(MSG_QUEUE_KEY, 0)) <0) {
      perror("msgget cli");
    }
  }
}


MsgQueue::~MsgQueue() 
{  
  if (_svcFlag) {  // svc
    if (msgctl(_fd, IPC_RMID, 0) < 0) {
      perror("msgctl IPC_RMID");
      _fd = NULL;
    }
  }
  else // cli
    close(_fd);
}



void MsgQueue::send(Msg* msg) 
{
  if (msgsnd(_fd, (char*)msg, sizeof(Msg), 0) != 0)
    perror("msgsnd");
}


int MsgQueue::recv(Msg* msg, int mtype, int blockFlag) 
{
  int size = msgrcv(_fd, (char*)msg, sizeof(Msg), 
		    mtype, 
		    blockFlag ? 0 : IPC_NOWAIT);
  if (size == -1) {
    if (errno == ENOMSG)
      return(0);
    else {
      perror("msgrcv");
      return(0);
    }
  }
  else
    return(1);
}

