/* 
 * Released Date: March 7 99
 * This is the Dynamic Soft CPU Scheduler II release by the 
 *   Monet Group, 
 *   http://cairo.cs.uiuc.edu/
 *   Department of Computer Science, 
 *   University of Illinois at Urbana Champaign.  
 * 
 * This release has been tested under SUN OS 5.6.
 *
 * Contacting Authors:
 *  Hao-hua Chu (h-chu3@cs.uiuc.edu)
 *  Klara Nahrstedt (klara@cs.uiuc.edu)
 */

#ifndef PROC_STATS_BUF_H
#define PROC_STATS_BUF_H

#include <stdio.h>
#include "Def.H"

typedef struct ProcStatsLink
{
  ProcStats* blk;
  ProcStatsLink* next;
} ProcStatsLink;


class ProcStatsBuf
{
protected:
  ProcStats* _blk;  // the array storing the processing time.
  int _idx;         // the current recording position in the array.
  int _blkSize;     // the size of the array  

public:
  ProcStatsBuf(int blkSize);
  ~ProcStatsBuf();
  void  setBlkSize(int blkSize);
  int   getBlkSize() { return(_blkSize); };
  void  reset() { _idx = 0; };
  int   getIdx() { return(_idx); };
  void  insert(ProcStats* ps);

  int   findAvgUsage(int start, int end);
  int   findUsageAtFreq(int start, int end, float nonconformFreq);
  float findAvgUtil(int start, int end);
  float findUtilAtFreq(int start, int end, float nonconformFreq);
  float findNonconformFreq(int start, int end);
  int   isPermNonconform(int start, int end, float nonconformFreq) 
    { return(findNonconformFreq(start, end) > nonconformFreq); }
};


class CliProcStatsBuf : public ProcStatsBuf
{
private:
  ProcStatsLink* _head;
  ProcStatsLink* _tail;

public:
  CliProcStatsBuf();
  ~CliProcStatsBuf();
  
  void insert(ProcStats* ps);
  void writeFile(char* fname);
  
  int findPeriod();
  int findAvgUsage();
  float findAvgUtil();
  int findPeakUsage();
  int findBurstTolerance(int sustainableProcessingTime, float maxNonconformFreq);
  void print();
};




#endif PROC_STATS_BUF_H
