/* 
 * Released Date: March 7 99
 * This is the Dynamic Soft CPU Scheduler II release by the 
 *   Monet Group, 
 *   http://cairo.cs.uiuc.edu/
 *   Department of Computer Science, 
 *   University of Illinois at Urbana Champaign.  
 * 
 * This release has been tested under SUN OS 5.6.
 *
 * Contacting Authors:
 *  Hao-hua Chu (h-chu3@cs.uiuc.edu)
 *  Klara Nahrstedt (klara@cs.uiuc.edu)
 */

#include <stdlib.h>
#include "Timeval.H"
#include "SchedStatsBuf.H"


SchedStatsBuf::SchedStatsBuf(int size, char* fname)
{
  _size = size;
  _idx = 0;
  _buf = (SchedStats*)calloc(_size, sizeof(SchedStats));
  _fd = NULL;
  _begin = TIMEVAL_INIT;
  if (fname != NULL) {
    _fd = fopen(fname, "w");
    fprintf(_fd, "#%11s %10s %5s %5s %5s %10s %5s %10s %5s %5s %5s %5s\n",
	    "clock", 
	    "PermUsage", "total", "resv", "avail",
	    "OverUsage", "total", 
	    "TsUsage", "total",
	    "Burst", "Nonco", "Probe");
  }
}


SchedStatsBuf::~SchedStatsBuf()
{
  if (_fd)
    _writeToFile();
  fclose(_fd);
  free(_buf);
}


void SchedStatsBuf::_writeToFile()
{
  if (Timeval::isZero(&_begin))
    _begin = _buf[0].clock;
  for (int i=1; i<_idx; i++) {
    float t = (float)Timeval::subr(&(_buf[i].clock), &(_buf[i-1].clock));
    
    int permUsage = Timeval::subr(&_buf[i].permUsage, &_buf[i-1].permUsage);
    int tsUsage = Timeval::subr(&_buf[i].tsUsage, &_buf[i-1].tsUsage);
    int burstUsage = Timeval::subr(&_buf[i].burstUsage, &_buf[i-1].burstUsage);
    int nonconformUsage = Timeval::subr(&_buf[i].nonconformUsage, &_buf[i-1].nonconformUsage);
    int probeUsage = Timeval::subr(&_buf[i].probeUsage, &_buf[i-1].probeUsage);
    int overrunUsage = burstUsage + nonconformUsage + probeUsage;
    
    fprintf(_fd, "%12d %10.3f %5.3f %5.3f %5.3f %10.3f %5.3f %10.3f %5.3f %5.3f %5.3f %5.3f\n",
	    Timeval::subr(&(_buf[i].clock), &_begin),
	    (float)permUsage / t, _buf[i].permRatio, _buf[i].permRatio - _buf[i].availPermUtil, _buf[i].availPermUtil,
	    (float)overrunUsage / t, _buf[i].overrunRatio,
	    (float)tsUsage / t, _buf[i].tsRatio,
	    (float)burstUsage / t,
	    (float)nonconformUsage / t, 
	    (float)probeUsage / t);
  }
}


int SchedStatsBuf::update(SchedStats* ss)
{
  _buf[_idx] = *ss;
  _idx++;
  if (_idx == _size) {
    if (_fd)
      _writeToFile();
    _idx = 0;
    return(1);
  }
  else
    return(0);
}
