/* 
 * Released Date: March 7 99
 * This is the Dynamic Soft CPU Scheduler II release by the 
 *   Monet Group, 
 *   http://cairo.cs.uiuc.edu/
 *   Department of Computer Science, 
 *   University of Illinois at Urbana Champaign.  
 * 
 * This release has been tested under SUN OS 5.6.
 *
 * Contacting Authors:
 *  Hao-hua Chu (h-chu3@cs.uiuc.edu)
 *  Klara Nahrstedt (klara@cs.uiuc.edu)
 */

#include <stdio.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include "Shm.H"

extern "C" {
  int shm_open(const char *name, int oflag, mode_t mode);
};


void Shm::makeShmFname(char* fname, int pid)
{
  sprintf(fname, "/tmp/srt-%d.shm", pid);
};


int ShmSvc::init()
{
  int shmFd;
  
  if ((shmFd = open(SHM_NAME, O_CREAT|O_EXCL|O_RDWR, 00644)) == -1) {
    perror("shm_open");
    if (errno == EEXIST) {
      printf("Check if another CpuSvc process is running\n");
      printf("Or you can remove the lock file %s\n", SHM_NAME); 
    }
    return(0);
  }
  
  if (fchmod(shmFd, 00644)) {
    perror("fchmod");
    return(0);
  };

  if (ftruncate(shmFd, sizeof(ShmArea)) < 0) {
    perror("ftruncate\n");
    return(0);
  }
  if ((_area = (ShmArea*)mmap(0, sizeof(ShmArea), PROT_READ|PROT_WRITE, 
			      MAP_SHARED, shmFd, (long)0)) == NULL) {
    perror("mmap");
    return(0);
  }
  for (int i=0; i<MAX_PROCESS; i++)
    _area->pArrayFlag[i] = 0;
  _free = MAX_PROCESS;
  close(shmFd);
  return(1);
}


void ShmSvc::destroy()
{
  if (munmap((caddr_t)_area, sizeof(ShmArea)) != 0)
    perror("munmap");
  unlink(SHM_NAME);
}



void ShmSvc::freeProcess(int idx) 
{ 
  Process* p = &(_area->pArray[idx]);
  if (munmap((caddr_t)p->wrShmArea, sizeof(WrShmArea)) != 0)
    perror("munmap");
  
  close(p->wrShmAreaFd);
  p->wrShmArea = NULL;
  p->wrShmAreaFd = 0;

  char shmFname[20];
  makeShmFname(shmFname, p->pid);  
  unlink(shmFname);

  _area->pArrayFlag[idx] = 0; 
  _free++; 
};


int ShmSvc::setWrShmAreaPcred(Process* p)
{
  if (fchown(p->wrShmAreaFd, p->euid, p->egid)) {
    perror("fchown");
    return(0);
  }
  return(1);
}


Process* ShmSvc::allocProcess(int pid)
{
  Process* p;
  if (_free == 0)
    return(NULL);
  for (int i=0; i<MAX_PROCESS; i++)
    if (_area->pArrayFlag[i] == 0) {
      _area->pArray[i].idx = i;
      _area->pArrayFlag[i] = 1;
      _free--;
      p = &(_area->pArray[i]);
      break;
    }
  
  p->pid = pid;
  char shmFname[20];
  makeShmFname(shmFname, pid);  
  if ((p->wrShmAreaFd = open(shmFname, O_CREAT|O_RDWR|O_TRUNC, 00644)) == -1) {
    perror("shm_open");
    return(0);
  }

  if (fchmod(p->wrShmAreaFd, 00644)) {
    perror("fchmod");
    return(0);
  }

  if (ftruncate(p->wrShmAreaFd, sizeof(WrShmArea)) < 0) {
    perror("ftruncate\n");
    return(0);
  }
  if ((p->wrShmArea = (WrShmArea*)mmap(0, sizeof(WrShmArea), PROT_READ|PROT_WRITE, 
				       MAP_SHARED, p->wrShmAreaFd, (long)0)) == NULL) {
    perror("mmap");
    return(0);
  }
  return(p);
}


Process* ShmCli::attach(int pid, int idx)
{
  int shmFd;
  if ((shmFd = open(SHM_NAME, O_RDONLY)) == -1) {
    perror("shm_open");
    return(NULL);
  }
  if ((_area = (ShmArea*)mmap(0, sizeof(ShmArea), PROT_READ, 
			      MAP_SHARED, shmFd, (long)0)) == NULL) {
    perror("mmap shmArea");
    return(NULL);
  }
  close(shmFd);
  
  char shmFname[20];
  makeShmFname(shmFname, pid);
  if ((shmFd = open(shmFname, O_RDWR)) == -1) {
    perror("shm_open");
    return(NULL);
  }
  if ((_wrShmArea = (WrShmArea*)mmap(0, sizeof(WrShmArea), PROT_READ|PROT_WRITE, 
				   MAP_SHARED, shmFd, (long)0)) == NULL) {
    perror("mmap wrShmArea");
    return(NULL);
  }
  close(shmFd);
  return(getProcessWithIdx(idx));
}


void ShmCli::detach()
{
  if (munmap((caddr_t)_area, sizeof(ShmArea)) != 0)
    perror("munmap shmArea");
  if (munmap((caddr_t)_wrShmArea, sizeof(WrShmArea)) != 0)
    perror("munmap wrShmArea");
}



