#ifndef TIMEVAL_H
#define TIMEVAL_H

#include <stdio.h>
#include <sys/types.h>

#ifdef SunOS
#include <math.h>
#endif

#ifdef IRIX
#include <sys/time.h>
#endif

class Timeval {
    public:

    static int toInt(timeval* t) {
	return(t->tv_usec + ((t->tv_sec)%100)*1000000);
    }

    static long long toLong(timeval* t) {
	return((long long)t->tv_usec + (long long)(t->tv_sec)*1000000);
    }

    static timeval toTimeval(int t) {
	timeval r;
	r.tv_usec = t % 1000000;
	r.tv_sec = t / 1000000;
	return(r);
    }

    static void incr(timeval* t1, int usec) {
	t1->tv_usec += usec;
	t1->tv_sec += (t1->tv_usec)/1000000;
	t1->tv_usec = (t1->tv_usec)%1000000;
	if (t1->tv_usec < 0) {
	    t1->tv_usec = 1000000 - t1->tv_usec;
	    (t1->tv_sec)--;
	}
    }

    static void print(timeval* t) { 
	printf("%3ld:%6ld", (t->tv_sec)%100, t->tv_usec); 
    }

    static void println(char* s) {
	timeval t;
	gettimeofday(&t, NULL);
	printf("%s:%2ld:%6ld\n", s, t.tv_sec%100, t.tv_usec);
    }

    static void println(timeval* t) {
	print(t);
	printf("\n");
    }

    static int subr(timeval* t1, timeval* t2) {
	return((t1->tv_sec - t2->tv_sec)*1000000 + 
	       (t1->tv_usec - t2->tv_usec));
    }

    static timeval subr2(timeval*  t1, timeval* t2) {
	timeval r;
	r.tv_sec = t1->tv_sec - t2->tv_sec;
	r.tv_usec = t1->tv_usec - t2->tv_usec;
	if (r.tv_usec < 0) {
	    r.tv_usec += 1000000;
	    r.tv_sec--;
	}
	return(r);
    }

  
    static int compare(timeval* t1, int t2) {
	timeval r2 = toTimeval(t2);
	return(compare(t1, &r2));
    }


    static int compare(timeval* t1, timeval* t2) {
	if (t1->tv_sec > t2->tv_sec) 
	    return(1);
	else if (t1->tv_sec == t2->tv_sec)
	    if (t1->tv_usec > t2->tv_usec)
		return(1);
	    else if (t1->tv_usec == t2->tv_usec)
		return(0);
	    else
		return(-1);
	else
	    return(-1);
    }

    static int isZero(timeval* t) {
	if (t->tv_sec == 0 && t->tv_usec == 0)
	    return(1);
	else
	    return(0);
    }

    static int incrTillGreater(timeval* clk, timeval* t, int incr) {
	int n = 0;
	if ((Timeval::compare(clk, t)) >= 0) {
	    n = subr(clk, t) / incr + 1; 
	    Timeval::incr(t, n*incr);
	}
	return(n);
    }
};


#endif TIMEVAL_H
