#if !defined (GRMEXT_I_C)
#define GRMEXT_I_C

#include "GrmExt_i.h"
#include "orbsvcs/CosTradingC.h"
#include "orbsvcs/CosTradingReposC.h"
//#include "orbsvcs/CosTradingDynamicC.h"


const CORBA::ULong
GrmExt_i::NUM_STANDARD_PROPS = 9;

const char *
GrmExt_i::standard_props[] = {
  "RAM_total",
  "VM_total",
  "RAM_free",
  "VM_free",
  "FS_total",
  "FS_free",
  "files_total",
  "files_free",
  "processor_util"
};


GrmExt_i::GrmExt_i(CORBA::ORB_ptr o) {
  for (CORBA::ULong i = 0; i < NUM_STANDARD_PROPS; i++) {
    average_values[i] = 0;
  }
  this->id_prop = 0;
  this->level_ = 2;
  this->offer_id = CORBA::string_dup("");
  this->num_sons = 0;
  this->max_sons_ = 10;
  this->orb = CORBA::ORB::_duplicate(o);
  init_trader(orb);
  ACE_OS::printf("GrmExt_i constructed\n"); // teste
}


GrmExt_i::~GrmExt_i(void) {
}


void GrmExt_i::init_trader(CORBA::ORB_ptr orb) {
  ACE_OS::printf("GrmExt_i::init_trader()\n");
  CORBA::Object_var trading_obj = orb->resolve_initial_references("TradingService");

  // o servidor nao deve rodar sem trader!
  if (CORBA::is_nil(trading_obj.in())) {
    ACE_OS::printf("GrmExt_i::init_trader(): trading service is null\n");
    exit(1);
  }

  CORBA::Environment ACE_TRY_ENV;

  ACE_TRY {
    //
    // Obtem referencias de algumas interfaces do trader
    //
    this->lookup_if = CosTrading::Lookup::_narrow(trading_obj.in(), ACE_TRY_ENV);
    ACE_TRY_CHECK;

    this->regis = this->lookup_if->register_if();

    CosTrading::TypeRepository_var repos_obj = lookup_if->type_repos(ACE_TRY_ENV);
    ACE_TRY_CHECK;

    this->repos = CosTradingRepos::ServiceTypeRepository::_narrow(repos_obj.in(), ACE_TRY_ENV);
    ACE_TRY_CHECK;

    if (CORBA::is_nil(repos)) {
      ACE_OS::printf("GrmExt_i::init_trader(): CosTradingRepos::ServiceTypeRepository_var is null\n");
      return;
    }

    //
    // registra as propriedades no trader
    //
    CosTradingRepos::ServiceTypeRepository::PropStructSeq props;
    CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq base_types;

    CORBA::ULong i;
    for (i = 0; i < NUM_STANDARD_PROPS; i++) {
      props.length(i + 1);
      ACE_OS::printf("GrmExt_i::init_trader(): prop.name %s\n", standard_props[i]); // teste
      props[i].name = CORBA::string_dup(standard_props[i]); 
      props[i].value_type = CORBA::TypeCode::_duplicate(CORBA::_tc_ulong);
      props[i].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;
    }
    // inclui 'id' do filho como propriedade
    props.length(i + 1);
    props[i].name = CORBA::string_dup("id");
    props[i].value_type = CORBA::TypeCode::_duplicate(CORBA::_tc_ulong);
    props[i].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;

    repos->add_type("ResourceMgmt::GRMs",
		    "IDL:twoK/ResourceMgmt/Grm:1.0",
		    props,
		    base_types);
    ACE_TRY_CHECK;
    ACE_OS::printf("GrmExt_i::init_trader(): type included\n");

    // teste
    //
    // imprime o que foi os tipos registrados no trader
    //
    CosTradingRepos::ServiceTypeRepository::SpecifiedServiceTypes sst;
    sst.all_(1);
    CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq_var type_names =
      repos->list_types(sst, ACE_TRY_ENV);
    ACE_TRY_CHECK;


    for (int i = type_names->length() - 1; i >= 0; i--) {
      ACE_DEBUG ((LM_DEBUG, "GrmExt_i::init_trader(): type name: %s\n",
		  ACE_static_cast(const char *, type_names[i])));
    }
    // fim-teste
  }
  ACE_CATCH (CosTrading::IllegalServiceType, a) {
    ACE_UNUSED_ARG(a);
    ACE_TRY_ENV.print_exception("GrmExt_i::init_trader(): Exception CosTrading::IllegalServiceType");
  }
  ACE_CATCH (CosTradingRepos::ServiceTypeRepository::ServiceTypeExists, b) {
    ACE_UNUSED_ARG(b);
    ACE_TRY_ENV.print_exception("GrmExt_i::init_trader(): Exception CosTradingRepos::ServiceTypeRepository::ServiceTypeExists");
  }
  ACE_CATCH (CosTradingRepos::ServiceTypeRepository::ServiceTypeExists, c) {
    ACE_UNUSED_ARG(c);
    ACE_TRY_ENV.print_exception("GrmExt_i::init_trader(): Exception CosTradingRepos::ServiceTypeRepository::ServiceTypeExists");
  }
  ACE_CATCH (CosTrading::IllegalPropertyName, d) {
    ACE_UNUSED_ARG(d);
    ACE_TRY_ENV.print_exception("GrmExt_i::init_trader(): Exception CosTrading::IllegalPropertyName");
  }
  ACE_CATCH (CosTradingRepos::ServiceTypeRepository::InterfaceTypeMismatch, e) {
    ACE_UNUSED_ARG(e);
    ACE_TRY_ENV.print_exception("GrmExt_i::init_trader(): Exception CosTradingRepos::ServiceTypeRepository::InterfaceTypeMismatch");
  }
  ACE_CATCH (CosTrading::IllegalPropertyName, f) {
    ACE_UNUSED_ARG(f);
    ACE_TRY_ENV.print_exception("GrmExt_i::init_trader(): Exception CosTrading::IllegalPropertyName");
  }
  ACE_CATCH (CosTrading::DuplicatePropertyName, g) {
    ACE_UNUSED_ARG(g);
    ACE_TRY_ENV.print_exception("GrmExt_i::init_trader(): Exception CosTrading::DuplicatePropertyName");
  }
  ACE_CATCH (CosTradingRepos::ServiceTypeRepository::ValueTypeRedefinition, h) {
    ACE_UNUSED_ARG(h);
    ACE_TRY_ENV.print_exception("GrmExt_i::init_trader(): Exception CosTradingRepos::ServiceTypeRepository::ValueTypeRedefinition");
  }
  ACE_CATCH (CosTradingRepos::ServiceTypeRepository::ValueTypeRedefinition, i) {
    ACE_UNUSED_ARG(i);
    ACE_TRY_ENV.print_exception("GrmExt_i::init_trader(): Exception CosTradingRepos::ServiceTypeRepository::ValueTypeRedefinition");
  }
  ACE_CATCH (CosTrading::UnknownServiceType, j) {
    ACE_UNUSED_ARG(j);
    ACE_TRY_ENV.print_exception("GrmExt_i::init_trader(): Exception CosTrading::UnknownServiceType");
  }
  ACE_CATCH (CosTradingRepos::ServiceTypeRepository::DuplicateServiceTypeName, k) {
    ACE_UNUSED_ARG(k);
    ACE_TRY_ENV.print_exception("GrmExt_i::init_trader(): Exception CosTradingRepos::ServiceTypeRepository::DuplicateServiceTypeName");
  }
  ACE_CATCHANY {
    ACE_PRINT_EXCEPTION(ACE_ANY_EXCEPTION, "GrmExt_i::init_trader(): Exception");
  } ACE_ENDTRY;
}

void GrmExt_i::test(const char *str,
		    CORBA::Environment &ACE_TRY_ENV) {
  ACE_UNUSED_ARG(ACE_TRY_ENV);
  ACE_OS::printf("GrmExt_i::test(): %s\n", str);
}


void GrmExt_i::register_parent(const ResourceMgmt::GrmExt_ptr parent, 
			       CORBA::Environment &ACE_TRY_ENV) {
  ACE_UNUSED_ARG(ACE_TRY_ENV);
  if (! CORBA::is_nil(parent)) {
    this->parent = parent;

    // TODO Tratamento de erro para o caso de o pai nao aceitar
    // TODO mais filhos;
    // TODO _duplicate(this).
    // Avisa seu pai que ele possui um novo filho
    // parent->register_sibling(ResourceMgmt::GrmExt::_duplicate(???), ACE_TRY_ENV);
    ACE_OS::printf("GrmExt_i::register_parent\n"); // teste
  }
}


CosTrading::OfferId GrmExt_i::register_sibling_grm(const ResourceMgmt::Grm_ptr sib,
						   const CosTrading::PropertySeq &props,
						   const CORBA::ULong_out id_prop,
						   CORBA::Environment &ACE_TRY_ENV) {
// NOTA: POR QUE DA' UNUSED PARAMETER QUANDO COMPILA USANDO ACE_TRY_ENV?
  ACE_UNUSED_ARG(ACE_TRY_ENV); 
  ACE_OS::printf("GrmExt_i::register_sibling_grm()\n");
  CosTrading::OfferId_var offer_id;

  ACE_TRY {
    ResourceMgmt::Grm_var grm_ref = ResourceMgmt::Grm::_duplicate(sib);
    CosTrading::PropertySeq copy_props = props;

    offer_id = regis->_cxx_export(grm_ref.in(),
				  "ResourceMgmt::GRMs",
				  copy_props);

    ACE_OS::printf("GrmExt_i::register_sibling_grm(): Grm registered with id %s\n",
		   offer_id.in());
  }
  ACE_CATCHANY {
    // TODO incluir todas as excessoes lancadas por _cxx_export
    ACE_PRINT_EXCEPTION(ACE_ANY_EXCEPTION, "GrmExt_i::register_sibling_grm()");
  } 
  ACE_ENDTRY;

  // ganhou mais um filho
  num_sons++;

  son_is_grm = 1;

  id_prop = num_sons;

  return offer_id._retn();
}


void GrmExt_i::register_sibling(const ResourceMgmt::GrmExt_ptr sib,
				CORBA::Environment &ACE_TRY_ENV) {
  ACE_UNUSED_ARG(sib);
  ACE_UNUSED_ARG(ACE_TRY_ENV);
  ACE_OS::printf("GrmExt_i::register_sibling()\n");
}


void GrmExt_i::search(ResourceMgmt::QueryPacket &query_pkt,
		      CORBA::Environment &ACE_TRY_ENV) {
  ACE_OS::printf("GrmExt_i::search()\n");

  CosTrading::PolicySeq policies; // Empty sequence
  CosTrading::Lookup::SpecifiedProps desired_props; // return hostname
  CosTrading::PropertyNameSeq name_seq;

  desired_props._d(CosTrading::Lookup::some);
  name_seq.length(0);
  desired_props.prop_names(name_seq);

  CosTrading::PolicyNameSeq_var   policies_applied; // out param
  CosTrading::OfferSeq_var        offers;           // out param
  CosTrading::OfferIterator_var   iterator;         // out param

  //
  // cria novas restricoes e preferencias
  //
  int str_length;
  int total_length;
  CORBA::String_var str;
  
  //
  // cria nova restricao para procurar no proximo filho
  //
  // seleciona os filhos do nivel imediatamente abaixo
  CORBA::ULong ids_next_level[this->max_sons_];
  CORBA::ULong num_ids = get_next_level_ids(query_pkt.ids, ids_next_level);

  str_length = ACE_OS::strlen(query_pkt.constr.in());
  if (str_length == 0) {
    
    total_length = ACE_OS::strlen("id != ") * num_ids // queremos o *proximo* candidato
      + ACE_OS::strlen(" and ") * (num_ids - 1)
      + sizeof(CORBA::ULong)
      + 1;
    str = CORBA::string_dup("id == %d");
  }
  else {
    total_length = str_length 
      + ACE_OS::strlen("() and (id != )") 
      + sizeof(CORBA::ULong) 
      + 1;
    str = CORBA::string_dup("(%s) and (id > %d)");
  }
  CORBA::String_var combined_constr = CORBA::string_alloc(total_length);
  ACE_OS::sprintf(combined_constr.inout(),
		  str.in(),
		  query_pkt.constr.in(),
		  query_pkt.id_prop);
  ACE_OS::printf("GrmExt_i::search(): combined constraint - %s\n", combined_constr.in()); // teste

  // cria nova preferencia para que as ofertas voltem ordenadas pelo id
  str_length = ACE_OS::strlen(query_pkt.pref.in());
  if (str_length == 0) {
    total_length = ACE_OS::strlen("min id") + 1;
    str = CORBA::string_dup("min id");
  } else {
    total_length = str_length 
      + ACE_OS::strlen("() and (min id)") 
      + 1;
    str = CORBA::string_dup("(%s) and (min id)");
  }
  CORBA::String_var combined_pref = CORBA::string_alloc(total_length);
  ACE_OS::sprintf(combined_pref.inout(),
		  str.in(),
		  query_pkt.pref.in(),
		  query_pkt.id_prop + 1); // queremos o *proximo* candidato
  ACE_OS::printf("GrmExt_i::search(): combined preference - %s\n", combined_pref.in()); // teste

  ACE_TRY {
    lookup_if->query("ResourceMgmt::GRMs",
		     combined_constr.in(),
		     combined_pref.in(),
		     policies,
		     desired_props,
		     1,
		     offers,
		     iterator,
		     policies_applied,
		     ACE_TRY_ENV);
    ACE_OS::printf("GrmExt_i::search(): query returned\n"); // teste
    ACE_TRY_CHECK;
    ACE_OS::printf("GrmExt_i::search(): query returned without exception\n"); // teste
    ACE_OS::printf("GrmExt_i::search(): number of offres returned = %d\n", offers->length()); // teste

    ACE_OS::printf("GrmExt_i::search(): ResourceMgmt::QueryPacket.component = %s\n",
		   query_pkt.component.in()); // teste



    if (offers->length() != 0) {
      // manda pra baixo
      if (son_is_grm) {
	ACE_OS::printf("GrmExt_i::search(): sending the query down to Grm\n"); // teste
	ResourceMgmt::Grm_var grm;
	
	grm = ResourceMgmt::Grm::_narrow(offers[0].reference.in(), ACE_TRY_ENV);
	
	grm->perform_query(query_pkt, ACE_TRY_ENV);
      }
      else {
	ACE_OS::printf("GrmExt_i::search(): sending the query down to GrmExt\n");
	ResourceMgmt::GrmExt_var grm_ext;

	query_pkt.id_prop = 0; // para indicar pro filho que a chamada foi feita pelo pai
	
	grm_ext = ResourceMgmt::GrmExt::_narrow(offers[0].reference.in(), ACE_TRY_ENV);
	grm_ext->search(query_pkt, ACE_TRY_ENV);
      }
    }
    else {
      if (this->id_prop != 0) {
	ACE_OS::printf("GrmExt_i::search(): sending the query up to GrmExt father\n");
	// manda pra cima
	query_pkt.id_prop = this->id_prop;
	parent->search(query_pkt);
      }
      else if (this->id_prop == 0) {
	// veio de cima
      }
    }

  }
  ACE_CATCHANY {
    ACE_OS::printf("GrmExt::search(): Exception!!!!!!!!!\n");
  }
  ACE_ENDTRY;
}


void GrmExt_i::update_props(const char *id,
			    const CosTrading::PropertySeq &modify_list,
			    CORBA::Environment &ACE_TRY_ENV) {
  ACE_OS::printf("GrmExt_i::update_props()\n");

  CosTrading::PropertySeq copy_modify_list = modify_list;
  CosTrading::PropertyNameSeq del_list;
  del_list.length(0);

  for (CORBA::ULong i = 0; i < modify_list.length(); i++) {
    ACE_OS::printf("prop[%d].name = %s\n", i, modify_list[i].name.in());
    CORBA::Any::dump(modify_list[i].value);
  }


  ACE_OS::printf("GrmExt_i::update_props(): modifying offer id %s\n", id);
  ACE_TRY {
    this->regis->modify(id, del_list, copy_modify_list, ACE_TRY_ENV);
    ACE_TRY_CHECK;
  }
  ACE_CATCHANY {
    // TODO incluir todas as excessoes lancadas por modify()
    ACE_PRINT_EXCEPTION(ACE_ANY_EXCEPTION, "GrmExt_i::update()");
  }
  ACE_ENDTRY;
}


CORBA::ULong GrmExt_i::get_next_level_ids(const ResourceMgmt::QueryPacket &query_pkt,
					  CORBA::ULong *ids_next_level,
					  CORBA::Environment &ACE_TRY_ENV) {
  ACE_UNUSED_ARG(ACE_TRY_ENV);
  CORBA::ULong num_ids = 0;

  for (CORBA::ULong i = 0; i < query_pkt.ids.length(); i++) {
    // se o id for do filho, insere no vetor
    if (query_pkt.ids[i].level == (this->level_ - 1)) {
      ids_next_level[num_ids++] = (CORBA::ULong) query_pkt.ids[i].id;
    }
  }

  return num_ids;
}

#endif /* GRMEXT_I_C */


