// ========================================================================
// $Id: Grm_i.cpp,v 1.2 2000/02/20 08:48:49 yamane Exp $
//
// = LIBRARY
//    orbsvcs
//
// = FILENAME
//    Grm_i.cpp 
//
// = AUTHOR
//    Tomonori Yamane <yamane@cs.uiuc.edu>
//
// ========================================================================

#if !defined (GRM_I_C)
#define GRM_I_C

#include "Grm_i.h"

ACE_RCSID(Grm, Grm_i, "$Id: Grm_i.cpp,v 1.2 2000/02/20 08:48:49 yamane Exp $")

template <class TRADER_LOCK_TYPE, class MAP_LOCK_TYPE>
const CORBA::ULong 
Grm_i<TRADER_LOCK_TYPE, MAP_LOCK_TYPE>::
NUM_STANDARD_PROPS = 9;

template <class TRADER_LOCK_TYPE, class MAP_LOCK_TYPE>
const char* 
Grm_i<TRADER_LOCK_TYPE, MAP_LOCK_TYPE>::
standard_props[] = {
  "RAM_total",
    "VM_total",
    "RAM_free",
    "VM_free",
    "FS_total",
    "FS_free",
    "files_total",
    "files_free",
    "processor_util"
};

// Constructor

template <class TRADER_LOCK_TYPE, class MAP_LOCK_TYPE>
Grm_i<TRADER_LOCK_TYPE, MAP_LOCK_TYPE>::
Grm_i (TAO_Trader<TRADER_LOCK_TYPE, MAP_LOCK_TYPE> &trader)
  : trader_(trader), shared_db_ (trader_.sharedprop_database()) {
  ACE_OS::printf("Grm_i constructed\n"); // teste

  // MOD
  this->id_prop = 0;

  this->level_ = 1; // todos os GRMs estao no nivel 1

  this->offer_id = CORBA::string_dup("");

  this->self_ = this->_this();

  for (CORBA::ULong i = 0; i < NUM_STANDARD_PROPS; i++) {
    average_values[i] = 0;
  }

  this->num_Lrms = 1;
  ACE_OS::printf("Grm_i: number_of_Lrms = %d\n", num_Lrms); // teste
  // FIM-MOD
}

// Destructor

template <class TRADER_LOCK_TYPE, class MAP_LOCK_TYPE>
Grm_i<TRADER_LOCK_TYPE, MAP_LOCK_TYPE>::
~Grm_i (void) {
  // no-op
}


// MOD
template <class TRADER_LOCK_TYPE, class MAP_LOCK_TYPE>
void
Grm_i<TRADER_LOCK_TYPE, MAP_LOCK_TYPE>::
setParent(const ResourceMgmt::GrmExt_ptr grmExt,
	  CORBA::Environment &ACE_TRY_ENV) {
  ACE_OS::printf("Grm_i::setParent()\n");
  CosTrading::PropertySeq props;

  grmExt_parent = ResourceMgmt::GrmExt::_duplicate(grmExt);

  if (! CORBA::is_nil(grmExt_parent)) {
    // cria as propriedades a serem oferecidas
    props.length(NUM_STANDARD_PROPS);
    for (CORBA::ULong i = 0; i < NUM_STANDARD_PROPS; i++) {
      props[i].name = CORBA::string_dup(standard_props[i]);
      props[i].value <<= (CORBA::ULong) average_values[i];
    }

    CORBA::ULong id;
    this->offer_id = grmExt_parent->register_sibling_grm(this->_this(), props, id, ACE_TRY_ENV);
    this->id_prop = id;
    ACE_OS::printf("Grm_i::setParent(): offer_id returned with value %s and prop_id = %d\n",
		   this->offer_id.in(),
		   this->id_prop);
  }
}
// FIM-MOD


template <class TRADER_LOCK_TYPE, class MAP_LOCK_TYPE>
void
Grm_i<TRADER_LOCK_TYPE, MAP_LOCK_TYPE>::
perform_query(const ResourceMgmt::QueryPacket& req, CORBA::Environment &ACE_TRY_ENV) {

// Find the candidate
//
//	servvice	"Lrm"
//      constraint ------ p.constr && p.visited
//      preferences ----- p.pref
//        in_policies ------ p.policies
//        desired_prop ----- p.desired
//        how_many    ------ 1
//        return_offers
//        return_offer_iterator
//        policyname_seq_out
//
//

//
//	Currently we cannot use federation of Trader to create new process
//
//
//
  ACE_OS::printf("Grm_i<>::perform_query()\n");
  ACE_OS::printf("Grm_i::eval_constraint()\n");
  ACE_OS::printf("---------- constr  = %s\n", req.constr.in());
  ACE_OS::printf("---------- pref  = %s\n", req.pref.in());
  ACE_OS::printf("---------- visited = %s\n", req.visited.in());
 
  //
  //	Merge user constraint and visited constraint
  //
  int constr_len = ACE_OS::strlen(req.constr.in());

  int total_len;
  if (constr_len == 0) {
    total_len = ACE_OS::strlen(req.visited.in()) + 1;
  } else {
    total_len = constr_len
      + ACE_OS::strlen(req.visited.in())
      + ACE_OS::strlen("() and ()")
      + 1;
  }

  CORBA::String_var combined_constr = CORBA::string_alloc(total_len);

  if (constr_len == 0) {
    strcpy(combined_constr.inout(), req.visited.in());
  } else {
    ACE_OS::sprintf(combined_constr.inout(),
		    "(%s) and (%s)",
		    req.visited.in(),
		    req.constr.in());
  }

  ACE_OS::printf("------- combined_constr = %s\n", combined_constr.in()); // teste
  // execute query to find Lrm

  ResourceMgmt::Lrm_var lrm_;

  CosTrading::Lookup_ptr lookup_if = this->trader_.trading_components().lookup_if();

  CosTrading::Register_ptr register_if = this->trader_.trading_components().register_if();

  CosTrading::PolicySeq policies; // Empty sequence
  CosTrading::Lookup::SpecifiedProps desired_props; // return hostname
  CosTrading::PropertyNameSeq name_seq;

  //        desired_props._d(CosTrading::Lookup::none);
  desired_props._d(CosTrading::Lookup::some);
  name_seq.length(1);
  name_seq[0] = CORBA::string_dup("hostname");
  desired_props.prop_names(name_seq);

  CosTrading::PolicyNameSeq_var   policies_applied; // out param
  CosTrading::OfferSeq_var        offers;           // out param
  CosTrading::OfferIterator_var   iterator;         // out param

 retry_query:

  ACE_OS::printf("query start\n"); // teste

  ACE_TRY {
    // Run query
    lookup_if->query("ResourceMgmt_Lrm",
		     combined_constr.in(),
		     req.pref.in(),
		     policies,
		     desired_props,
		     1,
		     offers,
		     iterator,
		     policies_applied,
		     ACE_TRY_ENV);
    ACE_OS::printf("perform_query() query returned\n"); // teste
    ACE_TRY_CHECK;
    ACE_OS::printf("query returned without exception\n"); // teste

  }
  ACE_CATCH (CosTrading::IllegalConstraint, ill_constr) {
    // If we fail to the constraint, just return False
    ACE_UNUSED_ARG (ill_constr);
    ACE_OS::printf("perform_query() IllegalConstraint\n");
    ACE_TRY_ENV.print_exception ("IllegalConstraint Exception");
    ACE_TRY_ENV.clear();

    lrm_ = req.sender.in();
    lrm_->NACK(req.sem_tag, ACE_TRY_ENV);
  }
  ACE_CATCH (CosTrading::Lookup::IllegalPreference, ill_pref) {
    ACE_UNUSED_ARG (ill_pref);
    ACE_OS::printf("perform_query() Lookup::IllegalPreference\n");
    ACE_TRY_ENV.print_exception ("Lookup::IllegalPreference Exception");
    exit(1);
  }
  ACE_CATCH (CosTrading::Lookup::IllegalPolicyName, ill_poly) {
    ACE_UNUSED_ARG (ill_poly);
    ACE_OS::printf("perform_query() Lookup::IllegalPolicyName\n");
    ACE_TRY_ENV.print_exception ("Lookup::IllegalPolicyName Exception");
    exit(1);
  }
  ACE_CATCH (CosTrading::Lookup::PolicyTypeMismatch, mismatch_poly) {
    ACE_UNUSED_ARG (mismatch_poly);
    ACE_OS::printf("perform_query() Lookup::PolicyTypeMismatch\n");
    ACE_TRY_ENV.print_exception ("Lookup::PolicyTypeMismatch Exception");
    exit(1);
  }
  ACE_CATCH (CosTrading::Lookup::InvalidPolicyValue, invalid_poly) {
    ACE_UNUSED_ARG (invalid_poly);
    ACE_OS::printf("perform_query() Lookup::InvalidPolicyValue\n");
    ACE_TRY_ENV.print_exception ("Lookup::InvalidPolicyValue Exception");
    exit(1);
  }
  ACE_CATCH (CosTrading::IllegalPropertyName, ill_property) {
    ACE_UNUSED_ARG (ill_property);
    ACE_OS::printf("perform_query() IllegalPropertyName\n");
    ACE_TRY_ENV.print_exception ("IllegalPropertyName Exception");
    ACE_TRY_ENV.clear();

    lrm_ = req.sender.in();
    lrm_->NACK(req.sem_tag, ACE_TRY_ENV);
  }
  ACE_CATCH (CosTrading::DuplicatePropertyName, dup_property) {
    ACE_UNUSED_ARG (dup_property);
    ACE_OS::printf("perform_query() DuplicatePropertyName\n");
    ACE_TRY_ENV.print_exception ("DuplicatePropertyName Exception");
    ACE_TRY_ENV.clear();
    
    lrm_ = req.sender.in();
    lrm_->NACK(req.sem_tag, ACE_TRY_ENV);
  }
  ACE_CATCH (CosTrading::DuplicatePolicyName, dup_policy) {
    ACE_UNUSED_ARG (dup_policy);
    ACE_OS::printf("perform_query() DuplicatePolicyName\n");
    ACE_TRY_ENV.print_exception ("DuplicatePolicyName Exception");
    exit(1);
  }
  ACE_CATCH(CORBA::SystemException, sysex) {
    ACE_UNUSED_ARG(sysex);
    ACE_OS::printf("perform_query() SystemException()\n");
    ACE_TRY_ENV.print_exception("System Exception");
    
    exit(1);
  }
  ACE_CATCH(CORBA::UserException, userex) {
    ACE_UNUSED_ARG(userex);
    ACE_OS::printf("perform_query() UserException\n");
    ACE_TRY_ENV.print_exception("User Exception");
    exit(1);
  }
  ACE_ENDTRY;

  ACE_TRY_EX(CHECK_AVAILABLE) {
    // Clean up iterator
    if (! CORBA::is_nil(iterator.ptr())) {
      iterator->destroy();
    }


    /*
    if (offers->length() == 0) {
      // MOD
      ACE_OS::printf("No matching Lrm in this cluster.\n"); // teste
      //
      // procura no GrmExt pai.
      //
      if (id_prop != 0) { // tem pai?
	ResourceMgmt::QueryPacket copy_query = req;
	// marca este GRM para nao ser procurado no trader de seu pai
	CORBA::ULong ids_length = copy_query.ids.length();
	copy_query.ids.length(ids_length + 1);
	copy_query.ids[ids_length].id = this->id_prop;
	copy_query.ids[ids_length].level = this->level_;

	  //	copy_query.id_prop = this->id_prop;
	// limpa os LRMs visitados. NOTA: PRECISA LIBERAR MEMORIA???
	//	copy_query.visited = CORBA::string_dup("");

	grmExt_parent->search(copy_query, ACE_TRY_ENV);
      }
      // FIM-MOD

      // nao achou em nenhum lugar
      ACE_OS::printf("No matching Lrm in anywhere\n"); // teste
      lrm_ = ResourceMgmt::Lrm::_duplicate(req.sender.in());
      lrm_->NACK(req.sem_tag, ACE_TRY_ENV);
      ACE_TRY_CHECK_EX(CHECK_AVAILABLE);
      return;
    }
    */

    // ORIGINAL
    if (offers->length() == 0) {
      ACE_OS::printf("No matching Lrm\n"); // teste
      lrm_ = ResourceMgmt::Lrm::_duplicate(req.sender.in());
      lrm_->NACK(req.sem_tag, ACE_TRY_ENV);
      ACE_TRY_CHECK_EX(CHECK_AVAILABLE);
      return;
    }
    
  }
  ACE_CATCH(CORBA::SystemException, sysex2) {
    ACE_UNUSED_ARG(sysex2);
    ACE_OS::printf("perform_query() check_available SystemException()\n");
    ACE_TRY_ENV.print_exception("System Exception");

    exit(1);
  }
  ACE_CATCH(CORBA::UserException, userex2) {
    ACE_UNUSED_ARG(userex2);
    ACE_OS::printf("perform_query() check_available UserException\n");
    ACE_TRY_ENV.print_exception("User Exception");
    exit(1);
  }
  ACE_ENDTRY;


  // extract  hostname for error handling
  char *host;
  offers[0].properties[0].value >>= host;

  ACE_OS::printf("matched host is %s\n", host); // teste

  char withdraw_constraint[1024];
  ACE_OS::sprintf(withdraw_constraint, "hostname == '%s'", host);

  //
  // Contact to the candidate Lrm
  // If it fails, drop the offer from the databse
  // and try query again
  //
  ACE_TRY_EX(FORWARDING) {
    lrm_ = ResourceMgmt::Lrm::_narrow(offers[0].reference.in(), ACE_TRY_ENV);
    ACE_TRY_CHECK_EX(FORWARDING);
    
    if (CORBA::is_nil(lrm_.in())) {
      ACE_OS::printf("Candidate cannot be narrowed\n");
    } else {
      ACE_OS::printf("We found candidate.\n"); // teste

      lrm_->perform_query(req, ACE_TRY_ENV);
      ACE_TRY_CHECK_EX(FORWARDING);
      ACE_OS::printf("Request forwarded to Lrm\n"); // teste
    }
  }
  ACE_CATCH(CORBA::SystemException, sysex3) {
    ACE_UNUSED_ARG(sysex3);
    ACE_OS::printf("perform_query() forwarding SystemException()\n");
    ACE_TRY_ENV.print_exception("System Exception");
    
    ACE_TRY_ENV.clear();

    register_if->withdraw_using_constraint("ResourceMgmt_Lrm",
					   withdraw_constraint,
					   ACE_TRY_ENV);

    ACE_TRY_ENV.clear();

    goto retry_query;
  }
  ACE_CATCH(CORBA::UserException, userex3) {
    ACE_UNUSED_ARG(userex3);
    ACE_OS::printf("perform_query() forwarding UserException\n");
    ACE_TRY_ENV.print_exception("User Exception");
    
    ACE_TRY_ENV.clear();

    register_if->withdraw_using_constraint("ResourceMgmt_Lrm",
					   withdraw_constraint,
					   ACE_TRY_ENV);
    
    ACE_TRY_ENV.clear();
    
    goto retry_query;
  }
  ACE_ENDTRY;
}


// MOD
template <class TRADER_LOCK_TYPE, class MAP_LOCK_TYPE>
void
Grm_i<TRADER_LOCK_TYPE, MAP_LOCK_TYPE>::
update_notify(CORBA::Environment &ACE_TRY_ENV) {
  static unsigned long num_calls = 0; // teste aval

  ACE_OS::printf("Grm_i::update_notify()\n");
  ACE_OS::printf("Grm_i::update_notify()\n");
  num_calls++; // teste aval
  ACE_OS::printf("Grm_i::update_notify(): num_calls = %ld\n", num_calls); // teste aval

  CosTrading::PropertySeq props;

  if (significant_change(props, ACE_TRY_ENV)) {
    ACE_OS::printf("Grm_i::update_notify(): significant change detected\n"); // teste

    if (! CORBA::is_nil(grmExt_parent)) {
      CosTrading::OfferId id = this->offer_id;

      ACE_OS::printf("Grm_i::update_notify(): number of props to modify = %d\n", 
		     props.length()); // teste
      grmExt_parent->update_props(id, props, ACE_TRY_ENV);
    }
  }
  else {
    ACE_OS::printf("Grm_i::update_notify(): no significant change detected\n"); // teste
  }
}
// FIM-MOD

// MOD
template <class TRADER_LOCK_TYPE, class MAP_LOCK_TYPE>
CosTrading::OfferSeq *
Grm_i<TRADER_LOCK_TYPE, MAP_LOCK_TYPE>::
get_offers(CORBA::Environment &ACE_TRY_ENV) {
  ACE_OS::printf("Grm_i::get_offers()\n");

  CosTrading::Lookup_ptr lookup_if = this->trader_.trading_components().lookup_if();

  CosTrading::PolicySeq policies; // Vazio
  CosTrading::Lookup::SpecifiedProps desired_props;
  CosTrading::PropertyNameSeq name_seq;

  desired_props._d(CosTrading::Lookup::all); // queremos que todas as propriedades sejam retornadas(all)
  // inclui os nomes das propriedades
  for (CORBA::ULong i = 0; i < NUM_STANDARD_PROPS; i++) {
    name_seq.length(i+1);
    name_seq[i] = CORBA::string_dup(standard_props[i]);
  }

  desired_props.prop_names(name_seq);

  CosTrading::OfferSeq_var        offers;           // out param
  CosTrading::PolicyNameSeq_var   policies_applied; // out param
  CosTrading::OfferIterator_var   iterator;         // out param

  ACE_TRY {
    // poderia ter usado describe(offer_id, props) do Register, 
    // se tivesse o offer_id...
    lookup_if->query("ResourceMgmt_Lrm",
		     "TRUE",             // sem restricao
		     "",                 // sem preferencia de ordem
		     policies,
		     desired_props,
		     (CORBA::ULong) this->num_Lrms,
		     offers,
		     iterator,
		     policies_applied.out(),
		     ACE_TRY_ENV);
    ACE_OS::printf("Grm_i::get_offers(): query returned\n"); // teste
    ACE_TRY_CHECK;
    ACE_OS::printf("Grm_i::get_offers(): query returned without exception\n"); // teste
  }
  ACE_CATCH (CosTrading::Lookup::IllegalPreference, ill_pref) {
    ACE_UNUSED_ARG (ill_pref);
    ACE_OS::printf("get_offers() Lookup::IllegalPreference\n");
    ACE_TRY_ENV.print_exception ("Lookup::IllegalPreference Exception");
  }
  ACE_CATCH (CosTrading::Lookup::IllegalPolicyName, ill_poly) {
    ACE_UNUSED_ARG (ill_poly);
    ACE_OS::printf("get_offers() Lookup::IllegalPolicyName\n");
    ACE_TRY_ENV.print_exception ("Lookup::IllegalPolicyName Exception");
  }
  ACE_CATCH (CosTrading::Lookup::PolicyTypeMismatch, mismatch_poly) {
    ACE_UNUSED_ARG (mismatch_poly);
    ACE_OS::printf("get_offers() Lookup::PolicyTypeMismatch\n");
    ACE_TRY_ENV.print_exception ("Lookup::PolicyTypeMismatch Exception");
  }
  ACE_CATCH (CosTrading::Lookup::InvalidPolicyValue, invalid_poly) {
    ACE_UNUSED_ARG (invalid_poly);
    ACE_OS::printf("get_offers() Lookup::InvalidPolicyValue\n");
    ACE_TRY_ENV.print_exception ("Lookup::InvalidPolicyValue Exception");
  }
  ACE_CATCH (CosTrading::IllegalPropertyName, ill_property) {
    ACE_UNUSED_ARG (ill_property);
    ACE_OS::printf("get_offers() IllegalPropertyName\n");
    ACE_TRY_ENV.print_exception ("IllegalPropertyName Exception");
  }
  ACE_CATCH (CosTrading::DuplicatePropertyName, dup_property) {
    ACE_UNUSED_ARG (dup_property);
    ACE_OS::printf("get_offers() DuplicatePropertyName\n");
    ACE_TRY_ENV.print_exception ("DuplicatePropertyName Exception");
  }
  ACE_CATCH (CosTrading::DuplicatePolicyName, dup_policy) {
    ACE_UNUSED_ARG (dup_policy);
    ACE_OS::printf("get_offers() DuplicatePolicyName\n");
    ACE_TRY_ENV.print_exception ("DuplicatePolicyName Exception");
  }
  ACE_CATCH(CORBA::SystemException, sysex) {
    ACE_UNUSED_ARG(sysex);
    ACE_OS::printf("get_offers() SystemException()\n");
    ACE_TRY_ENV.print_exception("System Exception");
  }
  ACE_CATCH(CORBA::UserException, userex) {
    ACE_UNUSED_ARG(userex);
    ACE_OS::printf("get_offers() UserException\n");
    ACE_TRY_ENV.print_exception("User Exception");
  }
  ACE_ENDTRY;

  return offers._retn();
}
// FIM-MOD


// MOD
template <class TRADER_LOCK_TYPE, class MAP_LOCK_TYPE>
int
Grm_i<TRADER_LOCK_TYPE, MAP_LOCK_TYPE>::
significant_change(CosTrading::PropertySeq &props, CORBA::Environment &ACE_TRY_ENV) {
  ACE_OS::printf("Grm_i::significant_change()\n");

  CosTrading::OfferSeq_var offers = get_offers(ACE_TRY_ENV);

  ACE_OS::printf("Grm_i::significant_change(): offers->length = %d\n", offers->length());

  //
  // Calcula a soma de cada recurso
  //
  CORBA::ULong sum[NUM_STANDARD_PROPS];
  for (CORBA::ULong i = 0; i < NUM_STANDARD_PROPS; i ++) {
    sum[i] = 0;
  }

  for (CORBA::ULong i = 0; i < offers->length(); i++) {
    /*
    ACE_OS::printf("Grm_i::significant_change(): Lrm offer %d, number of props = %d\n", 
		   i,
		   offers[i].properties.length()); // teste
    */
    for (CORBA::ULong j = 0; j < offers[i].properties.length(); j++) {
      /*
      ACE_OS::printf("Grm_i::significant_change(): prop[%d].name = %s\n", j, offers[i].properties[j].name.in()); // teste

      CORBA::Any::dump(offers[i].properties[j].value); // teste
      */
      // soma o valor encontrado aa posicao correspondente
      // com um Hash_Map seria mais elegante, facil e muito mais
      for (CORBA::ULong k = 0; k < NUM_STANDARD_PROPS; k++) {
	if (ACE_OS::strcmp(standard_props[k], offers[i].properties[j].name.in()) == 0) {
	  /*
	   ACE_OS::printf("Grm_i::significant_change(): Found %s, position %d\n",
			 standard_props[k],
			 k); // teste
	  */
	  CORBA::ULong value;
	  if (ACE_OS::strcmp(standard_props[k], "processor_util") == 0) {
	    CORBA::Double v;

	    offers[i].properties[j].value >>= v;
	    value <<= (CORBA::ULong) v;
	  }
	  else {
	    offers[i].properties[j].value >>= value;
	  }
	  sum[k] += value;
	  break;
	}
      }
      //ACE_OS::printf("\n"); // teste
    }
  }

  int changed = 0; // indica se alguma propriedade mudou significativamente

  // compara as medias antiga e nova e inclui como propriedades
  // a serem modificadas
  int index = 0; 
  for (CORBA::ULong i = 0; i < NUM_STANDARD_PROPS; i++) {
    CORBA::ULong prop_average = sum[i] / offers->length();
    CORBA::ULong old_prop_average = average_values[i];

    /*
    // teste
    if (ACE_OS::strcmp(standard_props[i], "processor_util") == 0) {
      ACE_OS::printf("&&&&&&&&&&&& %d, %d\n", old_prop_average, prop_average);
    }
    // fim-teste
    */

    if ((prop_average > (1.1 * old_prop_average)) || 
	(prop_average < (0.9 * old_prop_average))) {
      /*
      ACE_OS::printf("Grm_i::significant_change(): changed position %d - %s\n",
		     i,
		     standard_props[i]); // teste
      */
      // media ultrapassou os limites, sera modificada
      props.length(index+1);
      props[index].name = CORBA::string_dup(standard_props[i]);


      // NOTA: MUDANCA PARA TESTE NUMA MESMA MAQUINA
      if ((this->id_prop == 2) && (strcmp(standard_props[i], "RAM_free") == 0)) {
	prop_average = (CORBA::ULong) 300000;
	ACE_OS::printf("Grm_i::significant_change(): RAM_free = %d\n", prop_average);
      }
      

      props[index].value <<= (CORBA::ULong) prop_average;
      index++;
      // muda media da propriedade no GRM
      average_values[i] = prop_average;

      changed = 1;
    }
  }

  return changed;
}
// FIM-MOD

#endif /* GRM_I_C */

