// -*- C++ -*-
// $Id: Grm_i.h,v 1.2 2000/09/30 18:40:39 yamane Exp $

// ============================================================================
//
// = LIBRARY
//    RMP/Grm
//
// = FILENAME
//    Grm_i.h
//
// = DESCRIPTION
//    This class implements the RMP Grm IDL interface.
//
// = AUTHOR
//    Tomonori Yamane <yamane@uiuc.edu>
//
// ============================================================================

#ifndef GRM_I_H
#define GRM_I_H

#include "orbsvcs/ResourceMgmtS.h"
#include "orbsvcs/ResourceMgmtC.h"

#include "SharedProp_Database.h"

#include "Trader_Utils.h"

template <class TRADER_LOCK_TYPE, class MAP_LOCK_TYPE> class Grm_i;

#include "Trader_T.h"

template <class TRADER_LOCK_TYPE, class MAP_LOCK_TYPE>
class Grm_i: public POA_ResourceMgmt::Grm
{
  // = TITLE
  //    Grm Object Implementation
  //
  // = DESCRIPTION
  //    Implementation of a simple object that has all methods of Grm.
  //
public:
  // = Initialization and termination methods.
  Grm_i (TAO_Trader<TRADER_LOCK_TYPE, MAP_LOCK_TYPE> &trader);
  // Constructor

  ~Grm_i (void);
  // Destructor

  virtual void perform_query (
      const ResourceMgmt::QueryPacket & req,
      CORBA::Environment &ACE_TRY_ENV
	);

  // MOD
  virtual void setParent(const ResourceMgmt::GrmExt_ptr grmExt,
			 CORBA::Environment &ACE_TRY_ENV);
  virtual void update_notify(CORBA::Environment &ACE_TRY_ENV);
  // FIM-MOD

private:

  TAO_Trader<TRADER_LOCK_TYPE, MAP_LOCK_TYPE> &trader_;

  SharedProp_Database<MAP_LOCK_TYPE>  &shared_db_;

  TRADER_LOCK_TYPE lock_;
  // Lock to secure the set of request ids.

  // MOD
  ResourceMgmt::Grm_var self_;

  ResourceMgmt::GrmExt_ptr grmExt_parent; // pai deste GRM

  unsigned int num_Lrms;

  static const CORBA::ULong NUM_STANDARD_PROPS;
  static const char *standard_props[];
  CORBA::ULong average_values[NUM_STANDARD_PROPS];

  CORBA::ULong id_prop; // id deste GRM no seu pai

  CORBA::ULong level_; // nivel deste GRM na arvore

  CosTrading::OfferId_var offer_id;

  virtual CosTrading::OfferSeq *get_offers(//CosTrading::OfferSeq &offers,
			   CORBA::Environment &ACE_TRY_ENV);

  virtual int significant_change(CosTrading::PropertySeq &props,
				 CORBA::Environment &ACE_TRY_ENV);
  // FIM-MOD

  // = Disallow these operations.
  ACE_UNIMPLEMENTED_FUNC (void operator= (const Grm_i<TRADER_LOCK_TYPE,MAP_LOCK_TYPE> &))
  ACE_UNIMPLEMENTED_FUNC (Grm_i (const Grm_i<TRADER_LOCK_TYPE,MAP_LOCK_TYPE> &))

};

#if defined (ACE_TEMPLATES_REQUIRE_SOURCE)
#include "Grm_i.cpp"
#endif /* ACE_TEMPLATES_REQUIRE_SOURCE */

#endif /* GRM_I_H */
