/* -*- C++ -*- */

// ========================================================================
// $Id: SharedProp_Database.cpp,v 1.2 2000/02/20 08:48:59 yamane Exp $
//
// = LIBRARY
//    RMP
//
// = FILENAME
//    SharedProp_Database.cpp
//
// = AUTHOR
//    Tomonori Yamane <yamane@cs.uiuc.edu>
//
// ========================================================================

#ifndef SHAREDPROP_DATABASE_C
#define SHAREDPROP_DATABASE_C

#include "SharedProp_Database.h"
#include "Trader_Utils.h"
#include "Constraint_Interpreter.h"
#include "Constraint_Visitors.h"

template <class LOCK_TYPE>
SharedProp_Database<LOCK_TYPE>::SharedProp_Database (void)
: TAO_Offer_Database<LOCK_TYPE>()
{
//	ACE_OS::printf("SharedProp_Database constructed\n");
//	ACE_OS::printf("SharedProp_Database this = 0x%08x\n", this);
//	ACE_OS::printf("SharedProp_Database host_db_ = 0x%08x\n", &this->host_db_);
//	ACE_OS::printf("SharedProp_Database orig_db_ = 0x%08x\n", &this->orig_db_);
}

template <class LOCK_TYPE>
SharedProp_Database<LOCK_TYPE>::~SharedProp_Database (void)
{
  ACE_WRITE_GUARD (LOCK_TYPE, ace_mon, this->host_db_lock_);

  for (ACE_TYPENAME Property_Database::iterator type_iter (this->host_db_);
       ! type_iter.done ();
       type_iter++)
    {
	CosTrading::PropertySeq* seq_p = (*type_iter).int_id_;

	delete seq_p;
    }
}

template <class LOCK_TYPE> CosTrading::OfferId
SharedProp_Database<LOCK_TYPE>::
insert_offer (const char* type, CosTrading::Offer* offer)
{
#if 0
  ACE_OS::printf("SharedProp_Database<>::insert_offer2(%s)\n", type);
#endif

  ACE_WRITE_GUARD_RETURN (LOCK_TYPE, ace_mon, this->host_db_lock_, 0);

  CosTrading::OfferId offer_id;

  if (ACE_OS::strcmp(type, "ResourceMgmt_Lrm")==0) {

        CosTrading::PropertyNameSeq del_list;
        CosTrading::PropertySeq modify_list = offer->properties;

	char *host = this->hostname(offer);
	if (host) update(host, del_list, modify_list);

	offer_id
		= TAO_Offer_Database<LOCK_TYPE>::insert_offer (type, offer);

  } else if (is_derived_type(type, "ResourceMgmt_base")) {

#if 0
  	ACE_OS::printf("It's derived from ResourceMgmt_base\n");
#endif
//	CosTrading::Offer *orig_offer = new CosTrading::Offer;
//	orig_offer->reference = CORBA::Object::_duplicate(offer->reference);
//	orig_offer->properties = offer->properties;

	offer_id
		= TAO_Offer_Database<LOCK_TYPE>::insert_offer (type, offer);

//        TAO_String_Hash_Key offer_id_key ((const char *)offer_id);
//	orig_db_.bind(offer_id_key, orig_offer);

	convert_offer(offer_id);

  } else {

#if 0
  	ACE_OS::printf("It's not derived from ResourceMgmt_base\n");
#endif

	offer_id
		= TAO_Offer_Database<LOCK_TYPE>::insert_offer (type, offer);
  }

  return (offer_id);
}

template <class LOCK_TYPE> int
SharedProp_Database<LOCK_TYPE>::
remove_offer (const CosTrading::OfferId offer_id,
              CORBA::Environment& TAO_IN_ENV)
  ACE_THROW_SPEC ((CosTrading::IllegalOfferId,
                  CosTrading::UnknownOfferId))
{
#if 0
	printf("SharedProp_Database::remove_offer(%s)\n", (char *)offer_id);
#endif
	const char *type = parse_service_type_name((char *)offer_id);

	//
	// Prevent user from removing Grm offered type
	//
	if (ACE_OS::strcmp(type, "ResourceMgmt_Grm")==0) {
		return 0;
	}

	// Temporary cure
	if (ACE_OS::strcmp(type, "ResourceMgmt_Lrm")==0) {
		CORBA::Environment ACE_TRY_ENV;
		ACE_OS::printf("Removing Lrm\n");
		CosTrading::Offer *old_offer 
			= lookup_offer (offer_id, ACE_TRY_ENV);
		ACE_CHECK_RETURN(0);

		char *host = this->hostname(old_offer);
		ACE_OS::printf("\thostname = %s\n", host);
		if (host) withdraw(host, ACE_TRY_ENV);
		ACE_OS::printf("\tReturned from SharedProp_Datbase::withdraw()\n");
		ACE_CHECK_RETURN(0);
		ACE_OS::printf("\tNo exception from SharedProp_Datbase::withdraw()\n");
	}


	CosTrading::Offer *orig_offer;

  	TAO_String_Hash_Key offer_id_key(CORBA::string_dup((char *)offer_id)); 

        ACE_WRITE_GUARD_RETURN (LOCK_TYPE, ace_mon, this->host_db_lock_, 0);

	if (orig_db_.unbind(offer_id_key, orig_offer)!= -1) {
		ACE_OS::printf("\tremove original database entry\n");
		delete orig_offer;
	}

	int ret_value = TAO_Offer_Database<LOCK_TYPE>::remove_offer (
						offer_id, TAO_IN_ENV);

	return ret_value;
}

// convert offer id to long integer
// It depends on the TAO Trader implementation

template <class LOCK_TYPE>
CORBA::ULong
SharedProp_Database<LOCK_TYPE>::parse_offer_id(const char *offer_id)
{
//	See <orbsvcs/Trader/Offer_Database.cpp>

	char buf[17];

	ACE_OS::memcpy(buf, offer_id, 16);
	buf[16] = 0;

	CORBA::ULong value = ACE_OS::atoi(buf);

	return(value);
}

template <class LOCK_TYPE>
const char *
SharedProp_Database<LOCK_TYPE>::parse_service_type_name(const char *offer_id)
{
  // Get service type: it is everything from 17th character to the end.
  const char *service_type = (char *) offer_id + 16;

  return(service_type);
}  

template <class LOCK_TYPE>
void
SharedProp_Database<LOCK_TYPE>::update(
	const char *host,
        const CosTrading::PropertyNameSeq & del_list,
        const CosTrading::PropertySeq & modify_list)
{
#if 0 // Tomonori Yamane
  ACE_OS::printf("***SharedProp_Database::update(this = 0x%08x, %s mod_list = %d)\n", this, host, modify_list.length());
#endif

  CosTrading::PropertySeq *prop_p = 0;

  char *type_name = 0;

  CORBA::Environment ACE_TRY_ENV;

  CosTradingRepos::ServiceTypeRepository::TypeStruct type_struct;
  type_struct.props.length(0);

//  ACE_OS::printf("SharedProp_Database::update() 2\n");

  ACE_WRITE_GUARD (LOCK_TYPE, ace_mon, this->host_db_lock_);

  CosTrading::PropertySeq* new_property_map_entry;

//  ACE_OS::printf("SharedProp_Database::update() 3\n");

//  size_t curr = this->host_db_.current_size();
//  ACE_OS::printf("SharedProp_Database::update() 4 curr = %d\n", curr);

//  size_t total = this->host_db_.total_size();
//  ACE_OS::printf("SharedProp_Database::update() 4 total = %d\n", total);

  int ret;
  TAO_String_Hash_Key service_host (host);
  ret = host_db_.find (service_host, prop_p);

//  ACE_OS::printf("SharedProp_Database::update() 4 ret = %d\n", ret);

  ACE_TRY {

  if (ret != -1) {
        CosTrading::Offer offer;

//	ACE_OS::printf("Existing entry\n");
  	this->host_db_.unbind (service_host);
  	offer.properties =  *prop_p;
	delete prop_p;

//      ACE_OS::printf("constructor of TAO_Offer_Modifier\n");
       	TAO_Offer_Modifier offer_mod(type_name, type_struct , &offer);
	ACE_TRY_CHECK;

//	ACE_OS::printf("delete of TAO_Offer_Modifier\n");
	offer_mod.delete_properties (del_list, ACE_TRY_ENV);
	ACE_TRY_CHECK;

//	ACE_OS::printf("merge of TAO_Offer_Modifier\n");
	offer_mod.merge_properties (modify_list, ACE_TRY_ENV);
	ACE_TRY_CHECK;

//	ACE_OS::printf("commit of TAO_Offer_Modifier\n");
	offer_mod.affect_change (modify_list);
//	ACE_OS::printf("done of TAO_Offer_Modifier\n");

  	new_property_map_entry = new CosTrading::PropertySeq(offer.properties);
  } else {
//	ACE_OS::printf("New entry\n");
  	new_property_map_entry = 
			new CosTrading::PropertySeq(modify_list);
  }

  this->host_db_.bind (service_host, new_property_map_entry);

//  ACE_OS::printf("Bind done\n");

  //
  // Check all entries in the conversion database
  //
  TAO_Offer_Id_Iterator* id_iter = TAO_Offer_Database<LOCK_TYPE>::retrieve_all_offer_ids ();
  CosTrading::OfferIdSeq_var offer_ids_var;

  CORBA::Boolean next_one;
  CosTrading::Offer *orig_offer;

  next_one = 1;
  while(next_one) {
 	next_one = id_iter->next_n(1, offer_ids_var, ACE_TRY_ENV);
	ACE_TRY_CHECK;
//	ACE_OS::printf("updating %s\n", (char *)offer_ids_var[0].in());
	if (offer_ids_var->length()!= 0) {
		const char *type = parse_service_type_name(
					(char *)offer_ids_var[0].in());

		if (ACE_OS::strcmp(type, "ResourceMgmt_Lrm")==0) {
#if 0
			ACE_OS::printf("updating Lrm here? %s\n",
					(char *)offer_ids_var[0].in());
#endif
			continue;
		}

		if (!is_derived_type(type, "ResourceMgmt_base")) {
			continue;
		}

//		ACE_OS::printf("checking original databse\n");

  		TAO_String_Hash_Key key = ACE_static_cast(const char*, offer_ids_var[0].in());
		if (orig_db_.find(key, orig_offer) != -1) {
//			ACE_OS::printf("Found original entry\n");
			char *h = this->hostname(orig_offer);
			if (h && ACE_OS::strcmp(host, h)==0) {
				convert_offer((CosTrading::OfferId)offer_ids_var[0].in());
			}
		} else {
//			ACE_OS::printf("Cannot find original entry\n");
		}
	}
  }

  id_iter->destroy(ACE_TRY_ENV);
  ACE_TRY_CHECK;
  }

  ACE_CATCH (CORBA::SystemException, sysex)
  {
          ACE_UNUSED_ARG (sysex);
          ACE_TRY_ENV.print_exception ("System Exception");
          exit(1);
  }
  ACE_CATCH (CORBA::UserException, userex)
  {
          ACE_UNUSED_ARG (userex);
          ACE_TRY_ENV.print_exception ("User Exception");
          exit(1);
  }
  ACE_ENDTRY;

//  ACE_OS::printf("Conversion done\n");
}

template <class LOCK_TYPE>
int
SharedProp_Database<LOCK_TYPE>::modify (
	const char *id,
	const CosTrading::PropertyNameSeq& del_list,
	const CosTrading::PropertySeq& modify_list,
	CORBA::Environment& ACE_TRY_ENV)
   ACE_THROW_SPEC ((CORBA::SystemException,
                  CosTrading::NotImplemented,
                  CosTrading::IllegalOfferId,
                  CosTrading::UnknownOfferId,
                  CosTrading::Register::ProxyOfferId,
                  CosTrading::IllegalPropertyName,
                  CosTrading::Register::UnknownPropertyName,
                  CosTrading::PropertyTypeMismatch,
                  CosTrading::ReadonlyDynamicProperty,
                  CosTrading::Register::MandatoryProperty,
                  CosTrading::Register::ReadonlyProperty,
                  CosTrading::DuplicatePropertyName))
{
  const char *type = parse_service_type_name(id);

//  ACE_OS::printf("SharedProp_Database<>::modify(%s), type = %s\n", id, type);

  if (ACE_OS::strcmp(type, "ResourceMgmt_Grm")==0) return(0);

  if (ACE_OS::strcmp(type, "ResourceMgmt_Lrm")==0) {
//	ACE_OS::printf("Updating Lrm\n");
	CosTrading::Offer *old_offer 
		= lookup_offer ((CosTrading::OfferId)id, ACE_TRY_ENV);
	ACE_CHECK_RETURN(0);

	char *host = this->hostname(old_offer);
	if (host) update(host, del_list, modify_list);

	return (1);
  }

  ACE_WRITE_GUARD_RETURN (LOCK_TYPE, ace_mon, this->host_db_lock_, 0);

  // if type is derived from ResourceMgmt_base , update original database
  if (is_derived_type(type, "ResourceMgmt_base")) {
//	ACE_OS::printf("type = %s derived from ResourceMgmt_base\n", type);

        TAO_String_Hash_Key offer_id_key (id);
	CosTrading::Offer *offer = 0;

	if (orig_db_.find(offer_id_key, offer) != -1) {
	      // Yank our friend, the type struct.
	      CosTradingRepos::ServiceTypeRepository::TypeStruct_var type_struct = repos_->fully_describe_type (type, ACE_TRY_ENV);
      	      ACE_CHECK_RETURN(0);

              TAO_Offer_Modifier offer_mod (type, type_struct.in (), offer);

	      // Delete, add, and change properties of the offer.
	      offer_mod.delete_properties (del_list, ACE_TRY_ENV);
	      ACE_CHECK_RETURN(0);

	      offer_mod.merge_properties (modify_list, ACE_TRY_ENV);
	      ACE_CHECK_RETURN(0);

	      // Alter our reference to the offer. We do this last, since the
	      // spec says: modify either suceeds completely or fails
	      // completely.
	      offer_mod.affect_change (modify_list);

              // Update databse
	      convert_offer((CosTrading::OfferId)id);
	} else {
		ACE_OS::printf("It's weired. We should have orig_db entry\n");
	}

	return (0);
   }
   else {
//	ACE_OS::printf("type = %s not derived from ResourceMgmt_base\n", type);
   }

   // otherwise do nothing
   return (1);
}

template <class LOCK_TYPE>
void
SharedProp_Database<LOCK_TYPE>::withdraw(
	const char *host,
	CORBA::Environment& ACE_TRY_ENV)
{
//  ACE_OS::printf("SharedProp_Database::withdraw(%s)\n", host);

  TAO_String_Hash_Key service_host (host);
  CosTrading::PropertySeq *properties;

  ACE_WRITE_GUARD (LOCK_TYPE, ace_mon, this->host_db_lock_);

  if (this->host_db_.unbind (service_host, properties) != -1) {
  	delete properties;
  }

  //
  // Check all entries in the conversion database
  //
  TAO_Offer_Id_Iterator* id_iter = retrieve_all_offer_ids ();
  CosTrading::OfferIdSeq_var offer_ids_var;
  CORBA::Boolean next_one;
  CosTrading::Offer *offer_item;
  CosTrading::Offer *orig_offer_item;
  ACE_TRY {
	 next_one = 1;
	 while(next_one) {
	 	next_one = id_iter->next_n(1, offer_ids_var, ACE_TRY_ENV);
		ACE_TRY_CHECK;

		if (offer_ids_var->length()!= 0) {
#if 0
			ACE_OS::printf("checking %s\n",
					(char *)offer_ids_var[0].in());
#endif

			const char *type = parse_service_type_name(
					(char *)offer_ids_var[0].in());

			if (ACE_OS::strcmp(type, "ResourceMgmt_Lrm")==0) {
#if 0
				ACE_OS::printf("withdrawing Lrm here? %s\n",
					(char *)offer_ids_var[0].in());
#endif
				continue;
			}

			if (!is_derived_type(type, "ResourceMgmt_base")) {
				continue;
			}

			offer_item = lookup_offer (
				(CosTrading::OfferId) offer_ids_var[0].in(),
				ACE_TRY_ENV);
			ACE_TRY_CHECK;

			char *offer_host = this->hostname(offer_item);

			if (offer_host != 0
				&& ACE_OS::strcmp(offer_host, host)==0) {
#if 0
			ACE_OS::printf("recover from the original offer %s\n",
					(char *)offer_ids_var[0].in());
#endif

  				TAO_String_Hash_Key key
					=  ACE_static_cast (const char*,
							 offer_ids_var[0]);
//				orig_db_.unbind(key, orig_offer_item);
				orig_db_.find(key, orig_offer_item);
				offer_item->properties = orig_offer_item->properties;
//				delete orig_offer_item;
#if 0
			ACE_OS::printf("recovering %s done\n",
					(char *)offer_ids_var[0].in());
#endif
			}

		}
	}
	id_iter->destroy(ACE_TRY_ENV);
	ACE_TRY_CHECK;
  }
  ACE_CATCH (CORBA::SystemException, sysex)
  {
          ACE_UNUSED_ARG (sysex);
          ACE_TRY_ENV.print_exception ("System Exception");
          exit(1);
  }
  ACE_CATCH (CORBA::UserException, userex)
  {
          ACE_UNUSED_ARG (userex);
          ACE_TRY_ENV.print_exception ("User Exception");
          exit(1);
  }
  ACE_ENDTRY;
}

// If type_name is derived from base name, return 1, otherwise 0
template <class LOCK_TYPE>
int
SharedProp_Database<LOCK_TYPE>::is_derived_type(
	const char *type_name,
	const char *base_name)
{
//  ACE_OS::printf("is_derived_type(%s, %s)\n", type_name, base_name);

  CosTradingRepos::ServiceTypeRepository::TypeStruct_var type_struct;

  ACE_TRY_NEW_ENV {
	if (ACE_OS::strcmp(type_name, base_name)==0) return 1;

	type_struct = repos_->describe_type(type_name, ACE_TRY_ENV);
	ACE_TRY_CHECK;

	CORBA::ULong length = type_struct->super_types.length();

	for (CORBA::ULong i = 0 ; i < length; ++i) {
		if (is_derived_type(type_struct->super_types[i].in(), base_name)) return 1;
	}
  }
  ACE_CATCH (CORBA::SystemException, sysex)
  {
          ACE_UNUSED_ARG (sysex);
          ACE_TRY_ENV.print_exception ("System Exception");
          return 0;
  }
  ACE_CATCH (CORBA::UserException, userex)
  {
          ACE_UNUSED_ARG (userex);
          ACE_TRY_ENV.print_exception ("User Exception");
          return 0;
  }
  ACE_ENDTRY;
  return 0;
}



//
// If it contains specified hostname, return 0, otherwise return -1
//
template <class LOCK_TYPE>
char *
SharedProp_Database<LOCK_TYPE>::hostname(
	const CosTrading::Offer   *offer)
{
// int ret_value = -1;
char *host = 0;

	for (CORBA::ULong ind = 0 ; ind < offer->properties.length() ; ind++) {
		if (ACE_OS::strcmp((offer->properties[ind]).name.in(),
					"hostname")==0) {
			(offer->properties[ind]).value >>= host;
//			ACE_OS::printf("hostname property found %s\n", host);
		}
	}

	return host;
}

template <class LOCK_TYPE>
void
SharedProp_Database<LOCK_TYPE>::convert_offer(const CosTrading::OfferId offer_id) {

  ACE_OS::printf("SharedProp_Database::convert_offer(%s)\n", offer_id); // teste

  // skip Grm and Lrm
  const char *type = parse_service_type_name((char *)offer_id);

  if (ACE_OS::strcmp(type, "ResourceMgmt_Grm")==0) {
    ACE_OS::printf("\tDo not convert type %s\n", type);
    return;
  }

  if (ACE_OS::strcmp(type, "ResourceMgmt_Lrm")==0) {
    ACE_OS::printf("\tDo not convert type %s\n", type); // teste
    return;
  }

  if (!is_derived_type(type, "ResourceMgmt_base")) {
    ACE_OS::printf("\tNot deribed from ResourceMgmt_base. Do not convert type %s\n", type); // teste
    return;
  }

  CORBA::Environment ACE_TRY_ENV;

  CosTrading::Offer *offer = lookup_offer(offer_id, ACE_TRY_ENV);
  ACE_CHECK;

  //
  //	Prepare original offer
  //
  TAO_String_Hash_Key key =  CORBA::string_dup((char*)offer_id);
  CosTrading::Offer *orig_offer; 

  if (this->orig_db_.find (key, orig_offer) == -1) {
    ACE_OS::printf("\tNo original offer Make an entry\n"); // teste
    orig_offer = new CosTrading::Offer;
    orig_offer->reference = CORBA::Object::_duplicate(offer->reference);
    orig_offer->properties = offer->properties;
    orig_db_.bind(key, orig_offer);
//		return;
  }

  //
  //	hostname & host properties
  //
  const char *host = this->hostname (offer);
  if (host == 0) return;

  TAO_String_Hash_Key service_host (CORBA::string_dup(host));
  CosTrading::PropertySeq *properties;
  if (this->host_db_.find (service_host, properties) == -1) {
    ACE_OS::printf("There is no corresponding Lrm entry\n"); // teste
    offer->properties = orig_offer->properties;
    return;
  }

  offer->properties = orig_offer->properties; // back to original

  //
  //	filter shared proeprties from host properties
  //
  CosTrading::Lookup::SpecifiedProps desired_props;
  desired_props._d(CosTrading::Lookup::some);
  
  CosTrading::PropertyNameSeq name_seq;

  name_seq.length(27);
  // static resources
  name_seq[0] = CORBA::string_dup("hostname");
  name_seq[1] = CORBA::string_dup("os_name");
  name_seq[2] = CORBA::string_dup("os_version");
  name_seq[3] = CORBA::string_dup("processor_name");
  name_seq[4] = CORBA::string_dup("processors");
  name_seq[5] = CORBA::string_dup("processor_MHz");
  name_seq[6] = CORBA::string_dup("processor_util");
  name_seq[7] = CORBA::string_dup("RAM_total");
  name_seq[8] = CORBA::string_dup("VM_total");

  // dynamic resources
  name_seq[9] = CORBA::string_dup("RAM_free");
  name_seq[10] = CORBA::string_dup("VM_free");
  name_seq[11] = CORBA::string_dup("FS_total");
  name_seq[12] = CORBA::string_dup("FS_free");
  name_seq[13] = CORBA::string_dup("files_total");
  name_seq[14] = CORBA::string_dup("files_free");
  name_seq[15] = CORBA::string_dup("ComponentsList");


  // DSRT
  name_seq[16] = CORBA::string_dup("DSRT2");
  name_seq[17] = CORBA::string_dup("DSRT2_availPermUtil");
  name_seq[18] = CORBA::string_dup("DSRT2_overrunRaito");
  name_seq[19] = CORBA::string_dup("DSRT2_permRaito");
  name_seq[20] = CORBA::string_dup("DSRT2_tsRaito");
  name_seq[21] = CORBA::string_dup("DSRT2_permUsage");
  name_seq[22] = CORBA::string_dup("DSRT2_burstUsage");
  name_seq[23] = CORBA::string_dup("DSRT2_nonconformUsage");
  name_seq[24] = CORBA::string_dup("DSRT2_probeUsage");
  name_seq[25] = CORBA::string_dup("DSRT2_tsUsage");

  // Test resources
  name_seq[26] = CORBA::string_dup("Total_number");

  desired_props.prop_names(name_seq);

  TAO_Property_Filter prop_filter(desired_props, ACE_TRY_ENV);
  ACE_CHECK;

  CosTrading::Offer temp_offer, filtered_offer;

  temp_offer.properties = *properties;
  prop_filter.filter_offer(&temp_offer, filtered_offer);

  CosTrading::PropertySeq modify_list = filtered_offer.properties;

  //
  // Actual merge
  //

  // Yank our friend, the type struct.
  CosTradingRepos::ServiceTypeRepository::TypeStruct_var type_struct =
    repos_->fully_describe_type (type, ACE_TRY_ENV);
  ACE_CHECK;
  TAO_Offer_Modifier offer_mod (type, type_struct.in (), offer);

  // Delete, add, and change properties of the offer.

  offer_mod.merge_properties (modify_list, ACE_TRY_ENV);
  ACE_CHECK;

  // Alter our reference to the offer. We do this last, since the
  // spec says: modify either suceeds completely or fails
  // completely.
  offer_mod.affect_change (modify_list);
}

template <class LOCK_TYPE>
void
SharedProp_Database<LOCK_TYPE>::add_props (
  CosTradingRepos::ServiceTypeRepository::PropStructSeq& props,
  const char *name,
  CORBA::TypeCode_ptr tc)
{
  CORBA::ULong ind;
  ind = props.length();
  props.length(ind+1);

  props[ind].name = CORBA::string_dup(name);
  props[ind].value_type = CORBA::TypeCode::_duplicate(tc);
  props[ind].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;
}

//
// Register service type
//
template <class LOCK_TYPE>
void
SharedProp_Database<LOCK_TYPE>::add_ResourceMgmt_types (CosTradingRepos::ServiceTypeRepository_ptr repos) {
  ACE_OS::printf("SharedProp_Database::add_ResourceMgmt_types()\n"); // teste

  // Initialize repos_
  repos_ = CosTradingRepos::ServiceTypeRepository::_duplicate(repos);

  CosTradingRepos::ServiceTypeRepository::PropStructSeq props;
  CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq base_types;

  //
  // Define Grm Type
  //

  repos->add_type ("ResourceMgmt_Grm",
		   "IDL:twoK/ResourceMgmt/Grm:1.0",
		   props,
		   base_types);

  ACE_OS::printf("SharedProp_Database::add_ResourceMgmt_types() added Grm type\n"); // teste
  //
  // Define base Type for 2K resource
  //
  // static resources
  add_props(props, "hostname",		CORBA::_tc_string);
  add_props(props, "os_name",		CORBA::_tc_string);
  add_props(props, "os_version",	CORBA::_tc_string);
  add_props(props, "processor_name",	CORBA::_tc_string);
  add_props(props, "processors",	CORBA::_tc_ulong);
  add_props(props, "processor_MHz",	CORBA::_tc_ulong);
  add_props(props, "RAM_total",		CORBA::_tc_ulong);
  add_props(props, "VM_total",		CORBA::_tc_ulong);

  // dynamic resources

  add_props(props, "RAM_free",		CORBA::_tc_ulong);
  add_props(props, "VM_free",		CORBA::_tc_ulong);
  add_props(props, "processor_util",	CORBA::_tc_double);
  add_props(props, "FS_total",		CORBA::_tc_ulong);
  add_props(props, "FS_free",		CORBA::_tc_ulong);
  add_props(props, "files_total",	CORBA::_tc_ulong);
  add_props(props, "files_free",	CORBA::_tc_ulong);
  add_props(props, "ComponentsList",	ResourceMgmt::_tc_ComponentsList);

  // DSRT
  add_props(props, "DSRT2",		CORBA::_tc_boolean);
  add_props(props, "DSRT2_availPermUtil",	CORBA::_tc_float);
  add_props(props, "DSRT2_overrunRaito",	CORBA::_tc_float);
  add_props(props, "DSRT2_permRaito",		CORBA::_tc_float);
  add_props(props, "DSRT2_tsRaito",		CORBA::_tc_float);
  add_props(props, "DSRT2_permUsage",		CORBA::_tc_long);
  add_props(props, "DSRT2_burstUsage",		CORBA::_tc_long);
  add_props(props, "DSRT2_nonconformUsage",	CORBA::_tc_long);
  add_props(props, "DSRT2_probeUsage",		CORBA::_tc_long);
  add_props(props, "DSRT2_tsUsage",		CORBA::_tc_long);

  // Test resources

  add_props(props, "Total_number",		CORBA::_tc_long);

  ACE_OS::printf("Lrm_i::type registration properties = %d \n", props.length()); // teste

  repos->add_type ("ResourceMgmt_base",
		   "IDL:omg.org/CORBA/Object:1.0",
		   props,
		   base_types);

  ACE_OS::printf("SharedProp_Database::add_ResourceMgmt_types() added base type\n"); // teste

  //
  // Define Lrm type for 2K resource
  //
  base_types.length(1);
  base_types[0] = CORBA::string_dup("ResourceMgmt_base");

  props.length(0);
  add_props(props, "timeout",		CORBA::_tc_long);

  repos->add_type ("ResourceMgmt_Lrm",
		   "IDL:twoK/ResourceMgmt/Lrm:1.0",
		   props,
		   base_types );

  ACE_OS::printf("SharedProp_Database::add_ResourceMgmt_types() added Lrm type\n"); // teste
  ACE_OS::printf("SharedProp_Database::add_ResourceMgmt_types() props.length() = %d\n", props.length()); // teste
  ACE_OS::printf("SharedProp_Database::add_ResourceMgmt_types() base_typess.length() = %d\n", base_types.length()); // teste
}

#endif /* SHAREDPROP_DATABASE_C */
