/* -*- C++ -*- */

// ========================================================================
// $Id: SharedProp_Database.h,v 1.2 2000/09/30 18:41:08 yamane Exp $
//
// = LIBRARY
//    orbsvcs
//
// = FILENAME
//    HostProp_Database.h
//
// = AUTHOR
//    Tomonori Yamane <yamane@cs.uiuc.edu>
//
// ========================================================================

#ifndef SHAREDPROP_DATABASE_H
#define SHAREDPROP_DATABASE_H

// #include <orbsvcs/CosTradingC.h>
#include "Trader.h"
#include "Offer_Database.h"
#include "orbsvcs/ResourceMgmtC.h"

// = DESCRIPTION
//
//   The SharedProperty_Database encapsulates the mapping of hostname
//   to those properties exported with that hostname. The underlying
//   structure is a map.
//
template <class LOCK_TYPE>
class SharedProp_Database : public TAO_Offer_Database<LOCK_TYPE>
{
//  friend class TAO_Service_Offer_Iterator<LOCK_TYPE>;
public:

  SharedProp_Database (void);
  // No arg constructor.

  ~SharedProp_Database (void);

  //
  // Provide same interface as Offer_Database
  //
  CosTrading::OfferId insert_offer (const char* type,
                                    CosTrading::Offer* offer);
  // Add an offer of type <type> and generate a CosTrading::OfferId
  // for it. Returns 0 on failure.

  int remove_offer (const CosTrading::OfferId offer_id,
                    CORBA::Environment& _env)
    ACE_THROW_SPEC ((CosTrading::IllegalOfferId,
                    CosTrading::UnknownOfferId));

  //
  // Interface specific to the SharedProp_Database
  //
  void update(
		const char *host,
		const CosTrading::PropertyNameSeq & del_list,
		const CosTrading::PropertySeq & modify_list);

  int modify (const char *id,
        const CosTrading::PropertyNameSeq& del_list,
        const CosTrading::PropertySeq& modify_list,
        CORBA::Environment& ACE_TRY_ENV)
   ACE_THROW_SPEC ((CORBA::SystemException,
                  CosTrading::NotImplemented,
                  CosTrading::IllegalOfferId,
                  CosTrading::UnknownOfferId,
                  CosTrading::Register::ProxyOfferId,
                  CosTrading::IllegalPropertyName,
                  CosTrading::Register::UnknownPropertyName,
                  CosTrading::PropertyTypeMismatch,
                  CosTrading::ReadonlyDynamicProperty,
                  CosTrading::Register::MandatoryProperty,
                  CosTrading::Register::ReadonlyProperty,
                  CosTrading::DuplicatePropertyName));

  void withdraw(const char *host, CORBA::Environment& env);

  static void add_props (
	CosTradingRepos::ServiceTypeRepository::PropStructSeq& props,
	const char *name,
	CORBA::TypeCode_ptr tc);

  void add_ResourceMgmt_types (
	CosTradingRepos::ServiceTypeRepository_ptr rep);
  // Add type definition for ResourceMgmt_base

  typedef ACE_Hash_Map_Manager_Ex
  <
	TAO_String_Hash_Key,
	CosTrading::PropertySeq*,
	ACE_Hash<TAO_String_Hash_Key>,
	ACE_Equal_To<TAO_String_Hash_Key>,
	ACE_Null_Mutex
  > Property_Database;

  typedef ACE_Hash_Map_Manager_Ex
  <
	TAO_String_Hash_Key,
	CosTrading::Offer*,
	ACE_Hash<TAO_String_Hash_Key>,
	ACE_Equal_To<TAO_String_Hash_Key>,
	ACE_Null_Mutex
  > Original_Offer_Database;

private:
  int  is_derived_type(const char *type_name, const char *base_name);
  // If type_name is derived from base name, return 1, otherwise 0

  char *hostname (const CosTrading::Offer *offer);
  // Return value of hostname property or 0 if there is no "hostname" property

  void convert_offer(const CosTrading::OfferId);
  // If offer includes "hostname" property, add other properties too

  CORBA::ULong parse_offer_id(const char *id);
  // convert offer id to long integer
  // It depends on the TAO Trader implementation

  const char *parse_service_type_name(const char *id);
  // convert offer id to service_type
  // It depends on the TAO Trader implementation

  CosTradingRepos::ServiceTypeRepository_var repos_;
  // Type repository
  //
  // Set :  add_ResourceMgmt_types ()
  // Used:  is_derived_type()

  Property_Database host_db_;

  LOCK_TYPE host_db_lock_;

  // = Disallow these operations.
  ACE_UNIMPLEMENTED_FUNC (void operator= (const SharedProp_Database<LOCK_TYPE> &))
  ACE_UNIMPLEMENTED_FUNC (SharedProp_Database (const SharedProp_Database<LOCK_TYPE> &))

//  LOCK_TYPE convert_db_lock_;

  Original_Offer_Database orig_db_;
};

#if defined (ACE_TEMPLATES_REQUIRE_SOURCE)
#include "SharedProp_Database.cpp"
#endif  /* ACE_TEMPLATES_REQUIRE_SOURCE */

#endif //  SHAREDPROP_DATABASE_H

