/******************************************************************************
*                                                                             *
*  File: client_i.cpp                                                         *
*                                                                             *
*  Description: A test client for the Lrm test.                               *
*               Based on the TAO Cubit example.                               *
*                                                                             *
*  Project: 2K                                                                *
*  Author: Tomonori Yamane (yamane@uiuc.edu)                                  *
*                                                                             *
*******************************************************************************
 Revision Control Information
 $Revision: 1.1 $
 $Log: Lrm_client_i.cpp,v $
 Revision 1.1  2000/02/19 20:55:55  yamane
 Initial version

 Revision 1.1.1.1  1999/08/30 04:41:29  col
 Initial check-in

 Revision 1.1.1.1  1999/04/01 18:16:37  f-kon
 This new 2K source tree contains the most up-to-date version of ACE and TAO
 and including the dynamicTAO code as well as some 2K tests and preliminary
 versions of some of its high-level systems.


 Revision 1.1  1999/02/22 22:29:15  f-kon
 *** empty log message ***


******************************************************************************/

#include "Lrm_client_i.h"

#include "orbsvcs/CosTradingC.h"

//////////////     Specific for Lrm
#include "ace/OS.h"

////////////////////////////////////////////////////////////////////////////////////

#include "P_Timer.h"

// Constructor.
LrmClient::LrmClient (void) {
  hang_flag = 0;
}

// Parses the command line arguments and returns an error status.

int
LrmClient::parse_args (void) {
  ACE_Get_Opt get_opts (argc_, argv_, "dh");
  int c;

  while ((c = get_opts ()) != -1)
    switch (c)
      {
      case 'd':  // debug flag
        TAO_debug_level++;
        break;
      case 'h':  // hang all newComponents
	hang_flag++;
	break;
      case '?':
      default:
        ACE_ERROR_RETURN ((LM_ERROR,
                           "usage:  %s"
                           " [-d]"
                           "\n",
                           argv_[0]),
                          -1);
      }

  // Indicates successful parsing of command line.
  return 0;
}

void
LrmClient::dump_ComponentsList(CORBA::Any& a) {
  ResourceMgmt::ComponentsList *list;

  if (a >>= list) {
    ACE_OS::printf("Oh! ComponentsList type!!! length = %d\n", list->length());
    for (CORBA::ULong i = 0 ; i < list->length() ; i++) {
      ACE_OS::printf("ComponentsList[%d] = %s\n", i, (*list)[i].in());
    }	
  }
}

void
LrmClient::test_simpleQuery(const char *comp_name) {
  ACE_OS::printf("LrmClient::test_simpleQuery()\n");

  char service_constraint[1024] ;
  char preference[1024];
  preference[0] = '\0';

//	ACE_OS::sprintf(service_constraint, "(%s) and (%s) and (%s)",
//		 constr1, constr2, constr3) ;

//	ACE_OS::printf("service_constraint = %s\n", service_constraint) ;

  CosTrading::PropertySeq resource_spec;
  resource_spec.length(0);

//	CosTrading::PolicySeq policies;
//	policies.length(0);

  //
  //	Offer
  //
  CosTrading::OfferSeq_var offers;

  //
  //	Constraint
  //
  ACE_OS::printf("Please input the constraints\n");
  //ACE_OS::gets(service_constraint, 1024);
  ACE_OS::strcpy(service_constraint, "RAM_total == 186");
  ACE_OS::printf("Constraints : %s\n", service_constraint);



  //
  //	preference
  //
  ACE_OS::printf("Please input the preference\n");
  // ACE_OS::gets(preference, 1024);
  ACE_OS::printf("Constraints : %s\n", preference);

  //
  //	Do not return properties
  //
  char specified_props[1024] ;

  CosTrading::Lookup::SpecifiedProps desired_props;
  CosTrading::PropertyNameSeq name_seq;

  ACE_OS::printf("Please input the specified properties type (none/all/some)\n");
  // ACE_OS::gets(specified_props, 1024);

  if (strcmp(specified_props, "none")==0) {
    ACE_OS::printf("None property\n");
    desired_props._d(CosTrading::Lookup::none);
  }
  else if (strcmp(specified_props, "all")==0) {
    ACE_OS::printf("All properties\n");
    desired_props._d(CosTrading::Lookup::all);
  }
  else { // some
    ACE_OS::printf("Some properties\n");
    desired_props._d(CosTrading::Lookup::some);
    name_seq.length(3);
    name_seq[0] = CORBA::string_dup("os_name");
    name_seq[1] = CORBA::string_dup("RAM_total");
    name_seq[2] = CORBA::string_dup("hostname");
    desired_props.prop_names(name_seq);
  }

  //
  //	Do not return properties
  //
  char category_name[1024] ;
  char component_name[1024] ;
  category_name[0] = '\0';
  component_name[0] = '\0';

  ACE_OS::printf("Please input the category name\n");
  // ACE_OS::gets(category_name, 1024);

  if (ACE_OS::strlen(category_name)==0) {
    ACE_OS::strcpy(category_name, "SizedComponent");
  }

  ACE_OS::printf("Please input the component name)\n");
  // ACE_OS::gets(component_name, 1024);

  if (ACE_OS::strlen(component_name)==0) {
    ACE_OS::strcpy(component_name, comp_name);
  }


#if 1
  P_Timer pf;

  pf.start();
  offers = Lrm_->execute_application(category_name,
				component_name,
				"ARG=XYZ",
				resource_spec,
				service_constraint,
				preference,
				/* policies, */
				desired_props);
  pf.stop();

  ACE_OS::printf("execute_application returned\n");

  if (offers->length() != 0) {
    char *str = orb_->object_to_string(offers[0].reference);
    ACE_OS::printf("IOR[0] returned is %s\n", str);
    ACE_OS::printf("property length = %d\n", offers[0].properties.length());
    for (CORBA::ULong i = 0 ; i <  offers[0].properties.length() ; i++) {
      ACE_OS::printf("prop[%d].name = %s\n", i, offers[0].properties[i].name.in());
      CORBA::Any::dump(offers[0].properties[i].value);
      dump_ComponentsList(offers[0].properties[i].value);
    }
    for (CORBA::ULong j = 1 ; j <  offers->length() ; j++) {
      CORBA::string_free(str);
      str = orb_->object_to_string(offers[j].reference);
      ACE_OS::printf("IOR[%d] = %s\n", j, str);
    }
  } else {
    ACE_OS::printf("OfferSeq returns null length\n");
  }

  pf.print();
#endif

#if 0
  for (int i = 0 ; i < 1 ; i++) {
    CORBA::String_var sss = Lrm_->set_threshold("RAM_total > 2");
  }
#endif
//	CORBA::String_var sss3 = Lrm_->set_threshold("RAM_total < 2");
#if 0
  CORBA::Object_var obj;
  
  offers = Lrm_->execute_application ("Hello",
				 "LoadableHello1",
				 "ARG=XYZ",
				 resource_spec,
				 constr4,
				 pref,
				 policies,
				 desired_props);

  obj = offer->reference.in();
  // memory leak here
  *str = orb_->object_to_string(obj);
  ACE_OS::printf("IOR returned is %s\n", str);
#endif
}

void
LrmClient::run(const char *comp_name) {
  test_simpleQuery(comp_name);

  ACE_OS::sleep(1);

  ACE_OS::printf("End client execution.\n");
}


LrmClient::~LrmClient (void) {
  ACE_OS::printf("Destructed Client Object.\n");
}

void
LrmClient::kill_newComponents (const char *host, CosNaming::NamingContext *inc) {
  ACE_OS::printf("kill_newComponents(%s)\n", host);

  CosNaming::NamingContext_var nc;
  CORBA::Object_var nc_obj;
  CORBA::Object_var newComp_obj;
  ResourceMgmt::Lrm_var newComp;

  CosNaming::Name name;
  name.length(5);

  CORBA::Environment ACE_TRY_ENV;

  ACE_TRY {

    name[0].id = CORBA::string_dup("SRG");
    name[0].kind = CORBA::string_dup("");
    name[1].id = CORBA::string_dup("services");
    name[1].kind = CORBA::string_dup("");
    name[2].id = CORBA::string_dup("RMP");
    name[2].kind = CORBA::string_dup("");
    name[3].id = CORBA::string_dup("newComponent");
    name[3].kind = CORBA::string_dup("");
    name[4].id = CORBA::string_dup(host);
    name[4].kind = CORBA::string_dup("");

    nc_obj = inc->resolve(name, ACE_TRY_ENV);
    ACE_TRY_CHECK;
    nc = CosNaming::NamingContext::_narrow(nc_obj, ACE_TRY_ENV);
    ACE_TRY_CHECK;

    CosNaming::BindingList_var bl;
    CosNaming::BindingIterator_var blIt;

    nc->list(100, bl, blIt, ACE_TRY_ENV);
    ACE_TRY_CHECK;

    if (bl->length()==0) {
      ACE_OS::printf("No newComponents registered\n");
      exit(1);
    }

    // (1) We also need to handle continuing Iterator but I don't have
    //     enough time to do that.
    // (2) I also should delete name entry but I don't.
    // (3) I also should handle hosts other than localhost, but I don't.

    for (CORBA::ULong i = 0 ; i < bl->length() ; i++) {
      newComp_obj = nc->resolve(bl[0].binding_name, ACE_TRY_ENV);
      ACE_TRY_CHECK;

      ACE_OS::printf("newComponent %d is resolved\n", i);
      
      newComp = ResourceMgmt::Lrm::_narrow(newComp_obj, ACE_TRY_ENV);
      ACE_TRY_CHECK;

      newComp->shutdown(ACE_TRY_ENV);
      ACE_TRY_ENV.clear();

      ACE_OS::printf("sent hangup to %d\n", i);
    }


    if (!CORBA::is_nil(blIt.in()))  {
      blIt->destroy(ACE_TRY_ENV);
    }
  } ACE_CATCH (CORBA::SystemException, sysex) {
    ACE_UNUSED_ARG (sysex);
    ACE_OS::printf("send_nack() system exception\n");
    ACE_TRY_ENV.print_exception ("System Exception");
    exit(1);
  } ACE_CATCH (CORBA::UserException, userex) {
    ACE_UNUSED_ARG (userex);
    ACE_OS::printf("send_nack() user exception\n");
    ACE_TRY_ENV.print_exception ("User Exception");
    exit(1);
  }
  ACE_ENDTRY;
}

CORBA::Object_ptr
LrmClient::direct_search(const char *service_id) {
  CORBA::Object_var obj;

  ACE_TRY_NEW_ENV {
    obj = orb_->resolve_initial_references(service_id, ACE_TRY_ENV);
    ACE_TRY_CHECK;

    ACE_OS::printf("resolve_initial_reference (%s) returns %s\n", service_id, orb_->object_to_string(obj));
  } ACE_CATCHANY {
    ACE_TRY_ENV.print_exception ("LrmClient::direct_serach");
    return obj._retn();
  }
  ACE_ENDTRY;

  return obj._retn();  
}

int
LrmClient::init(int argc, char **argv) {
  argc_ = argc;
  argv_ = argv;

  ACE_TRY_NEW_ENV {
    // Retrieve the ORB.
    ACE_OS::printf("Init orb\n");
    orb_ = CORBA::ORB_init (argc_,
			    argv_,
			    "internet",
			    ACE_TRY_ENV);
    ACE_TRY_CHECK;
    ACE_OS::printf("Init orb done\n");

    // Parse command line and verify parameters.
    if (parse_args () == -1)
      return -1;

    // Find Name server

    CORBA::Object_var obj;
    CORBA::Object_var Lrm_object;

    Lrm_object = direct_search("LrmService");
    if (CORBA::is_nil(Lrm_object.in())) {
      ACE_OS::printf("LrmService is nil.\n");
      
      // Find Trader server
      //
      CORBA::Object_var trading_obj;

      char buf[512];
      ACE_OS::hostname(buf, 511);

      char constraint[512];
      //  ACE_OS::sprintf(constraint, "hostname == '%s'", buf);
      //  ACE_OS::printf("constraint = %s\n", constraint);
      constraint[0] = 0;

      char preference[512];
      ACE_OS::sprintf(preference, "with hostname == '%s'", buf);
      ACE_OS::printf("preference = %s\n", preference);

      for (int i = 0 ; i < 3 ; ++i) {
	trading_obj = orb_->resolve_initial_references("TradingService");
	if (CORBA::is_nil(trading_obj.in())) {
	  ACE_OS::printf("TradingService is nil. Try again\n");
	  ACE_OS::sleep(1);
	} else {
	  ACE_OS::printf("TradingService resolved\n");
	  break;
	}
      }

      if (CORBA::is_nil(trading_obj.in())) {
	ACE_OS::printf("TradingService is nil.\n");
	exit(0);
      }

      CosTrading::Lookup_var lookup_if_var = CosTrading::Lookup::_narrow (trading_obj.in (), ACE_TRY_ENV);
      ACE_TRY_CHECK;
      ACE_OS::printf("lookup_if narrowed\n");

      CosTrading::PolicySeq policies; // Empty sequence

      CosTrading::Lookup::SpecifiedProps desired_props;       // Don't return props
      desired_props._d(CosTrading::Lookup::all);

      CosTrading::PolicyNameSeq_var   policies_applied;       // out param
      CosTrading::OfferSeq_var        offers;         // out param
      CosTrading::OfferIterator_var   iterator;       // out param

      // Run query
      lookup_if_var->query("ResourceMgmt_Lrm",
			   constraint,
			   preference,
			   policies,
			   desired_props,
			   10,
			   offers,
			   iterator,
			   policies_applied,
			   ACE_TRY_ENV
			   );

      ACE_OS::printf("query returned\n");
      ACE_TRY_CHECK;
      ACE_OS::printf("query returned without exception\n");

      for (CORBA::ULong j = 0 ; j < offers->length() ; j++) {
	ACE_OS::printf("Lrm_object[%d] = %s\n", j, orb_->object_to_string(offers[j].reference.in(), ACE_TRY_ENV));
	ACE_OS::printf("# of properties = %d\n", offers[j].properties.length());
	for (CORBA::ULong k = 0 ; k < offers[j].properties.length() ; k++) {
	  ACE_OS::printf("prop[%d] = %s\t", k, offers[j].properties[k].name.in());
	  CORBA::Any::dump(offers[j].properties[k].value);
	  dump_ComponentsList(offers[0].properties[k].value);
	  ACE_OS::printf("\n");
	}
	ACE_TRY_CHECK;
      }

      // Clean up
      if (!CORBA::is_nil(iterator.ptr()))
	iterator->destroy();

      if (offers->length() == 0) {
	ACE_OS::printf("No matching Lrm\n");
	ACE_OS::exit(1);
	//          Lrm_object = ResourceMgmt::Lrm::_nil();
      } else {
	if (CORBA::is_nil(Lrm_object.in())) {
	  Lrm_object = CORBA::Object::_duplicate(offers[0].reference.in());
	}
	ACE_OS::printf("Lrm found\n");
      }

#if 0

      obj = orb_->resolve_initial_references("NameService");
      ACE_OS::printf("NamingService resolved\n");
      if (CORBA::is_nil(obj)) {
        ACE_OS::printf("NamingService is nil.\n");
      }

      CosNaming::NamingContext_var inc;
      inc = CosNaming::NamingContext::_narrow(obj);

      ACE_OS::printf("NamingService narrowed\n");
      //
      // Create an initial tree (SRG/services/Grm/Grm,Rup,Rrp) if it does not exist
      //
      CosNaming::Name name;
      char buf[512];
      ACE_OS::hostname(buf, 511);

      // kill registered newComponents if necessary
      if (hang_flag) {
  	kill_newComponents(buf, inc.in());
	exit(0);
      }

      name.length(5);
      name[0].id = CORBA::string_dup("SRG");
      name[0].kind = CORBA::string_dup("");
      name[1].id = CORBA::string_dup("services");
      name[1].kind = CORBA::string_dup("");
      name[2].id = CORBA::string_dup("RMP");
      name[2].kind = CORBA::string_dup("");
      name[3].id = CORBA::string_dup("Lrm");
      name[3].kind = CORBA::string_dup("");
      name[4].id = CORBA::string_dup(buf);
      name[4].kind = CORBA::string_dup("");

      Lrm_object = inc->resolve(name, ACE_TRY_ENV);
      ACE_TRY_CHECK;

#endif
  }
#if 0
    CORBA::Object_var Lrm_object =
      orb_->string_to_object (Lrm_key_, ACE_TRY_ENV);
    ACE_TRY_CHECK;
#endif

    ACE_OS::printf("We are going to narrow Lrm_object\n");
    Lrm_ = ResourceMgmt::Lrm::_narrow (Lrm_object.in(), ACE_TRY_ENV);
    ACE_TRY_CHECK;
    ACE_OS::printf("Narrowing done\n");
    
    if (CORBA::is_nil (Lrm_.in ())) {
      ACE_ERROR_RETURN ((LM_ERROR,
			 "invalid Lrm key <%s>\n",
			 Lrm_key_),
			-1);
      exit(1);
    }
  } ACE_CATCHANY {
    ACE_TRY_ENV.print_exception ("LrmClient::init");
    return -1;
  }
  ACE_ENDTRY;

  return 0;
}

void
LrmClient::fini () {
  // Shutdown local client ORB.
  orb_->shutdown ();
}

// i guess the following is not required.

// #if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)
// template class ACE_Env_Value<unsigned long>;
// template class ACE_Env_Value<unsigned short>;
// #elif defined (ACE_HAS_TEMPLATE_INSTANTIATION_PRAGMA)
// #pragma instantiate ACE_Env_Value<unsigned long>
// #pragma instantiate ACE_Env_Value<unsigned short>
// #endif /* ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION */
