/******************************************************************************
*                                                                             *
*  File: dynconfClient.cpp                                                    *
*                                                                             *
*  Description:  Measure the times for calls to the dynconfServer.            *
*                                                                             *
*  Project: 2K - dynamic configuration                                        *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.1 $
   $Log: dynconfClient.cpp,v $
   Revision 1.1  2000/01/27 17:49:31  f-kon
   *** empty log message ***


******************************************************************************/
#include "ace/Get_Opt.h"
#include "tao/TAO.h"
#include "dynamicTAO/src/DynamicConfigurator/dynamicConfigurator_i_.h"

#include "tao/Timeprobe.h"
#include "ace/Read_Buffer.h"
#include <math.h>

#include "ace/SOCK_Connector.h"
#include "ace/SOCK_Stream.h"

void test_calls (DynamicConfigurator_ptr dyn_conf, CORBA::Environment ACE_TRY_ENV);

char *
read_ior (char *filename)
{
  // Open the file for reading.
  ACE_HANDLE f_handle_ = ACE_OS::open (filename, 0);

  if (f_handle_ == ACE_INVALID_HANDLE)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "Unable to open %s for reading: %p\n",
                       filename),
                      0);
  ACE_Read_Buffer ior_buffer (f_handle_);
  char *hello_key_ = ior_buffer.read ();

  if (hello_key_ == 0)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "Unable to allocate memory to read ior: %p\n"),
                      0);
  return ACE_OS::strdup (hello_key_);
}// read_ior

// Parses the command line arguments and returns the dynConfServer ior
char *
parse_args (int argc, char **argv, int *repetitions)
{
  ACE_Get_Opt get_opts (argc, argv, "dn:f:k:xs");
  int c;
  char *ior = 0;

  while ((c = get_opts ()) != -1)
    switch (c)
      {
      case 'f': // read the IOR from the file.
        ior = read_ior (get_opts.optarg);
        if (ior == NULL)
          ACE_ERROR_RETURN ((LM_ERROR,
                             "Unable to read ior from %s : %p\n",
                             get_opts.optarg),
                            0);
        break;
      case 'k': // read the IOR from the command-line.
        ior = ACE_OS::strdup (get_opts.optarg);
        break;
      case 'n': // number of times we should run the test
         *repetitions = ACE_OS::atoi (get_opts.optarg);
        break;
      case '?':
      default:
        ACE_ERROR_RETURN ((LM_ERROR,
                           "usage:  %s"
                           " [-f hello-obj-ref-key-file]"
                           " [-k obj-ref-key]"
                           "\n",
                           argv [0]),
                          0);
      }

  // Indicates successful parsing of command line.
  return ior;
}

////////////////////////////////////////////////////////////////////////////////
// 
// We get an IOR for a remote DynamicConfigurator, start a timer and then call
// the test_calls function to issue a bunch of calls to that remote object. We
// then print the elapsed time and exit.
//
////////////////////////////////////////////////////////////////////////////////

main(int argc, char**argv)
{
  TAO_ORB_Manager orb;
  CORBA::ORB_var orb_aux;
  int repetitions = 1;
  
  char * ior = parse_args (argc, argv, &repetitions);
  
  if (ior == 0)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "usage:  %s"
                       " [-f hello-obj-ref-key-file]"
                       " [-k obj-ref-key]"
                       "\n",
                       argv [0]),
                      -1);
  ACE_TRY_NEW_ENV
    {
      if(orb.init (argc,argv, ACE_TRY_ENV)==-1)
        {
          ACE_ERROR_RETURN((LM_ERROR,"%p\n","orb.init"),-1);
        }
      ACE_TRY_CHECK;        
      
      orb_aux = orb.orb();
      
      CORBA::Object_var dynconf_obj = 
        orb_aux->string_to_object (ior, ACE_TRY_ENV);
      
      if (CORBA::is_nil (dynconf_obj.in ()))
        ACE_ERROR_RETURN ((LM_ERROR,
                           "(%P|%t) Unable to resolve the DynConf Service.\n"),-1);
      
      DynamicConfigurator_var dyn_conf = 
        DynamicConfigurator::_narrow (dynconf_obj.in (), ACE_TRY_ENV);
      
      /********************************************************************************
             Start timer and call function that issue requests to dynconfServer
      ********************************************************************************/

      ACE_Time_Value start_time, end_time, elapsed_time;
      double average = 0.0, this_iteration = 0.0, sum = 0.0, sum_of_square = 0.0,
             variance = 0.0;
      int i;
      
      for (i = 0; i < repetitions; i++)
        {
          start_time = ACE_OS::gettimeofday();
          test_calls (dyn_conf.in (), ACE_TRY_ENV);
          end_time = ACE_OS::gettimeofday();
          
          elapsed_time = end_time - start_time;
          this_iteration = (double) elapsed_time.msec();
          ACE_OS::printf("elapsed time = %f ms\n", this_iteration);
          
          sum += this_iteration;
          sum_of_square += this_iteration * this_iteration;
          
          ACE_TRY_CHECK;
        }
      
      average  = sum / repetitions;
      variance =  sum_of_square / repetitions - average*average;
      if (variance < 0 )
        ACE_ERROR_RETURN ((LM_ERROR, "negative variance!!\n"), -1);
      
      ACE_OS::printf("--------------------------------------------------------\n"
                     "Average elapsed time = %.2f ms  Variance = %.2f "
                     "Standard deviation = %.2f \n", 
                     average, variance, sqrt(variance) );
    }
  ACE_CATCHANY
    {
      ACE_ANY_EXCEPTION._tao_print_exception ("dynconfClient:main");
      return -1;
    }
  ACE_ENDTRY;

}// main ()


////////////////////////////////////////////////////////////////////////////////
//
// The function that contains the all the CORBA calls to the remote
// DynamicConfigurator.
//
////////////////////////////////////////////////////////////////////////////////
void
test_calls (DynamicConfigurator_ptr dyn_conf, CORBA::Environment ACE_TRY_ENV)
{
  //Load an implementation
//   Configuration::ComponentConfigurator_var compConfig;
//   dyn_conf->load_implementation("Concurrency", 
//                                 "Thread_Strategy", "", 
//                                 Configuration::Factory::_nil(),
//                                 compConfig,
//                                 ACE_TRY_ENV);
  
  
//   //Now hook the service
//   dyn_conf->hook_implementation("Concurrency:1", "TAO", "Concurrency_Strategy",
//                                 ACE_TRY_ENV);
//   ACE_CHECK;
  
  int j;
  
  //List Categories///////////////////////////////////////////////////////////////
//   DynamicConfigurator::stringList *list;
  
//   list = dyn_conf->list_categories(ACE_TRY_ENV);
//   ACE_CHECK;
  
//   ACE_OS::printf("\nNumber of categories:%d\n",list->length());
  
//   for(j=0;j<list->length();j++)
//     {
//       ACE_OS::printf("%s\n",((*list)[j]).in());
//     }
  ////////////////////////////////////////////////////////////////////////////////

  ACE_INET_Addr remote_addr(10002,"nautilus.escet.urjc.es");
  ACE_SOCK_Connector connector;
  ACE_SOCK_Stream stream;
  int retv = 0;
  char buf[512];
  
  if (connector.connect(stream, remote_addr) < 0)
    {
      ACE_OS::printf("Failed to connect\n");
    }
  else 
    {
      ACE_OS::printf("Connected stream\n");
       stream.send("list_categories\r\n", ACE_OS::strlen ("list_categories\r\n"));
       retv = stream.recv(buf, 512);
       ACE_OS::printf("received %d bytes: <%s>\n", retv, buf);
    }

  stream.close();
  ACE_OS::printf("Closed stream\n");

  //List Implementations//////////////////////////////////////////////////////////
  // list=dyn_conf->list_implementations("Concurrency",ACE_TRY_ENV);
//   ACE_CHECK;
  
//   ACE_OS::printf("\nNumber of Concurrency implementations:%d\n",list->length());
  
//   for(j=0;j<list->length();j++)
//     {
//       ACE_OS::printf("%s\n",((*list)[j]).in());
//     }
  
  //List Loaded Implementations/////////////////////////////////////////////////////
  
  // list=dyn_conf->list_loaded_implementations(ACE_TRY_ENV);
//   ACE_CHECK;
  
//   ACE_OS::printf("\nNumber of loaded implementations:%d\n",list->length());
  
//   for(j=0;j<list->length();j++)
//     {
//       ACE_OS::printf("%s\n",((*list)[j]).in());
  //  }
  ////////////////////////////////////////////////////////////////////////////////
  
  
  //Get Implementation Info/////////////////////////////////////////////////////
  //   char *ret;
  
//   ret=dyn_conf->get_impl_info("Concurrency:1",ACE_TRY_ENV);
//   ACE_CHECK;
  
//   ACE_OS::printf("\nConcurrency:1 info: %s\n",ret);
  ////////////////////////////////////////////////////////////////////////////////
  
  
  //List Domain Components/////////////////////////////////////////////////////
//   list=dyn_conf->list_domain_components(ACE_TRY_ENV);
//   ACE_CHECK;
  
//   ACE_OS::printf("\nNumber of domain components:%d\n",list->length());
  
//   for(j=0;j<list->length();j++)
//     {
//       ACE_OS::printf("%s\n",((*list)[j]).in());
//     }
  ////////////////////////////////////////////////////////////////////////////////
  
  
  //List Hooks/////////////////////////////////////////////////////
//   list=dyn_conf->list_hooks("TAO",ACE_TRY_ENV);
//   ACE_CHECK;
  
//   ACE_OS::printf("\nNumber of hooks in TAO:%d\n",list->length());
  
//   for(j=0;j<list->length();j++)
//     {
//       ACE_OS::printf("%s\n",((*list)[j]).in());
//     }
  ////////////////////////////////////////////////////////////////////////////////
  
  
  //Get Component Info/////////////////////////////////////////////////////
//   ret=dyn_conf->get_comp_info("TAO",ACE_TRY_ENV);
//   ACE_CHECK;
  
//   ACE_OS::printf("\nTAO info: %s\n",ret);
  ////////////////////////////////////////////////////////////////////////////////
  
  
  //Get Component Info/////////////////////////////////////////////////////
//   ret=dyn_conf->get_hooked_comp ("TAO", "Concurrency_Strategy",ACE_TRY_ENV);
//   ACE_CHECK;
  
//   ACE_OS::printf("TAO/Concurrency_Strategy = %s\n",ret);
  ////////////////////////////////////////////////////////////////////////////////
  
}// test_calls ()
