/******************************************************************************
*                                                                             *
*  File: dynconfServer.cpp                                                    *
*                                                                             *
*  Description:  A test for the dynamicTAO DynamicConfigurator                *
*                                                                             *
*                                                                             *
*  Project: 2K - dynamic configuration                                        *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.1 $
   $Log: dynconfServer.cpp,v $
   Revision 1.1  2000/01/27 17:49:31  f-kon
   *** empty log message ***

******************************************************************************/
#include "ace/Get_Opt.h"
#include "tao/TAO.h"
#include "dynamicTAO/src/DynamicConfigurator/dynamicConfigurator_i_.h"


FILE* parse_args (int argc, char **argv)
{
  ACE_Get_Opt get_opts (argc, argv, "dn:o:s");
  int c;

  while ((c = get_opts ()) != -1)
    switch (c)
      {        
      case 'o': // output the IOR to a file.
        {
          FILE *ior_output_file_ = ACE_OS::fopen (get_opts.optarg, "w");
          
          if (ior_output_file_ == 0)
            ACE_ERROR_RETURN ((LM_ERROR,
                               "Unable to open %s for writing: %p\n",
                               get_opts.optarg), 0);
          else
            return (ior_output_file_);
        }
        break;

      case '?':
      default: 
        ACE_ERROR_RETURN((LM_ERROR,"usage: %s [-o ior-file-name]\n", argv[0]),0);
      }
  return (0);
}// parse_args

	
main(int argc, char**argv)
{
    TAO_ORB_Manager orb;
    CORBA::ORB_var orb_aux;

    FILE* ior_output_file_ = parse_args (argc, argv);
    
    ACE_TRY_NEW_ENV
      {
	if(orb.init (argc,argv, "dynconfServer", ACE_TRY_ENV)==-1)
          {
	    ACE_ERROR_RETURN((LM_ERROR,"%p\n","orb.init"),-1);
	  }
	ACE_TRY_CHECK;

        // The following call is required for clients,
        //  which are not going to enter "orb->run" loop, yet may want to
        //  service incoming requests.
        orb.activate_poa_manager (ACE_TRY_ENV);
        ACE_TRY_CHECK;

	orb_aux = orb.orb();
	
	CORBA::Object_var dynconf_obj = 
	  orb_aux->resolve_initial_references ("DynamicConfigurator");
	
	if (CORBA::is_nil (dynconf_obj.in ()))
	  ACE_ERROR_RETURN ((LM_ERROR,
			     "(%P|%t) Unable to resolve the DynConf Service.\n"),-1);
	
        DynamicConfigurator_var dyn_conf = 
            DynamicConfigurator::_narrow (dynconf_obj.in (), ACE_TRY_ENV);

        CORBA::String_var str  = orb_aux->object_to_string(dyn_conf.in());

        ACE_DEBUG ((LM_DEBUG, "The IOR is: <%s>\n", str.in ()));

        if (ior_output_file_)
          {
            ACE_OS::fprintf (ior_output_file_,
                             "%s",
                             str.in ());
            ACE_OS::fclose (ior_output_file_);
          }

        if (orb.run (ACE_TRY_ENV) == -1)
          ACE_ERROR_RETURN ((LM_ERROR,
                             "orb_manager::run"),
                            -1);
      }
    ACE_CATCHANY
        {
          ACE_ANY_EXCEPTION._tao_print_exception ("dynconfServer:main ()");
          return -1;
        }
    ACE_ENDTRY;

}// main ()
