/******************************************************************************
*                                                                             *
*  File: local.cpp                                                            *
*                                                                             *
*  Description:  A test for the dynamicTAO DynamicConfigurator                *
*                                                                             *
*                                                                             *
*  Project: 2K - dynamic configuration                                        *
*  Author: Manuel Roman (mroman@cs.uiuc.edu)                                  *
*          Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.3 $
   $Log: local.cpp,v $
   Revision 1.3  2000/10/09 21:02:44  f-kon
   Changes to use new TwoK:: namespace for 2K services integrated with Tomo's
   changes for Windows compilation.

   Revision 1.2  2000/03/02 21:40:28  f-kon
   *** empty log message ***

   Revision 1.1  2000/01/27 17:49:32  f-kon
   *** empty log message ***

   Revision 1.1.1.1  1999/10/12 00:42:47  f-kon
   Version compatible with TAO 1.0

   Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
   This new 2K source tree contains the most up-to-date version of ACE and TAO
   and including the dynamicTAO code as well as some 2K tests and preliminary
   versions of some of its high-level systems.


******************************************************************************/

#include "tao/TAO.h"
#include "dynamicTAO/src/DynamicConfigurator/dynamicConfigurator_i_.h"

main(int argc, char**argv)
{
    TAO_ORB_Manager orb;
    CORBA::ORB_var orb_aux;

    TAO_TRY
      {
	if(orb.init (argc,argv, TAO_TRY_ENV)==-1)
	  {
	    ACE_ERROR_RETURN((LM_ERROR,"%p\n","orb.init"),-1);
	  }
	TAO_CHECK_ENV;
	
	orb_aux = orb.orb();
	
	CORBA::Object_var dynconf_obj = 
	  orb_aux->resolve_initial_references ("DynamicConfigurator");
	
	if (CORBA::is_nil (dynconf_obj.in ()))
	  ACE_ERROR_RETURN ((LM_ERROR,
			     "(%P|%t) Unable to resolve the DynConf Service.\n"),-1);
	
        DynamicConfigurator_var dyn_conf = 
            DynamicConfigurator::_narrow (dynconf_obj.in (),TAO_TRY_ENV);

        //Load the service
        TwoK::Configuration::ComponentConfigurator_var compConfig;
        dyn_conf->load_implementation("Concurrency", 
                                      "Thread_Strategy", "", 
                                      TwoK::Configuration::CCFactory::_nil(),
                                      compConfig,
                                      TAO_TRY_ENV);
        TAO_CHECK_ENV;
        
        //Now hook the service
        dyn_conf->hook_implementation("Concurrency:1", "TAO", "Concurrency_Strategy",
				      TAO_TRY_ENV);
        TAO_CHECK_ENV;
        
        /*TAO_ORB_Core_instance()->interpreter()->interpret("servertest","ping Group1 Hello");*/
        
        //Try to export the reference of the dynamicConfigurator
        ACE_OS::printf ("DynamicConfigurator %s\n",
		       orb_aux->object_to_string (dyn_conf.in(),TAO_TRY_ENV));
        TAO_CHECK_ENV;
        
        int j;
            
        //List Categories///////////////////////////////////////////////////////////////
        DynamicConfigurator::stringList *list;
            
        list = dyn_conf->list_categories(TAO_TRY_ENV);
        TAO_CHECK_ENV;
            
        ACE_OS::printf("\nNumber of categories:%d\n",list->length());
            
        for(j=0;j<list->length();j++)
            {
                ACE_OS::printf("%s\n",((*list)[j]).in());
            }
        ////////////////////////////////////////////////////////////////////////////////
            
            
        //List Implementations//////////////////////////////////////////////////////////
        list=dyn_conf->list_implementations("Concurrency",TAO_TRY_ENV);
        TAO_CHECK_ENV;
            
        ACE_OS::printf("\nNumber of Concurrency implementations:%d\n",list->length());
            
        for(j=0;j<list->length();j++)
            {
                ACE_OS::printf("%s\n",((*list)[j]).in());
            }
        ////////////////////////////////////////////////////////////////////////////////
#if 0 
        //Load the service
        dyn_conf->load_implementation("Security", "Null_Security_Strategy", "", 
				      TAO_TRY_ENV);
        TAO_CHECK_ENV;
  
        //Now hook the service
        dyn_conf->hook_implementation("Security:1", "TAO", "Security_Strategy",
				      TAO_TRY_ENV);
        TAO_CHECK_ENV;
#endif
        //List Loaded Implementations/////////////////////////////////////////////////////
  
        list=dyn_conf->list_loaded_implementations(TAO_TRY_ENV);
        TAO_CHECK_ENV;
  
        ACE_OS::printf("\nNumber of loaded implementations:%d\n",list->length());
  
        for(j=0;j<list->length();j++)
	  {
	    ACE_OS::printf("%s\n",((*list)[j]).in());
	  }
        ////////////////////////////////////////////////////////////////////////////////
  
  
        //Get Implementation Info/////////////////////////////////////////////////////
        char *ret;
  
        ret=dyn_conf->get_impl_info("Concurrency:1",TAO_TRY_ENV);
        TAO_CHECK_ENV;
  
        ACE_OS::printf("\nConcurrency:1 info: %s\n",ret);
        ////////////////////////////////////////////////////////////////////////////////
  
  
        //List Domain Components/////////////////////////////////////////////////////
        list=dyn_conf->list_domain_components(TAO_TRY_ENV);
        TAO_CHECK_ENV;
  
        ACE_OS::printf("\nNumber of domain components:%d\n",list->length());
  
        for(j=0;j<list->length();j++)
            {
                ACE_OS::printf("%s\n",((*list)[j]).in());
            }
        ////////////////////////////////////////////////////////////////////////////////
  
  
        //List Hooks/////////////////////////////////////////////////////
        list=dyn_conf->list_hooks("TAO",TAO_TRY_ENV);
        TAO_CHECK_ENV;
  
        ACE_OS::printf("\nNumber of hooks in TAO:%d\n",list->length());
  
        for(j=0;j<list->length();j++)
            {
                ACE_OS::printf("%s\n",((*list)[j]).in());
            }
        ////////////////////////////////////////////////////////////////////////////////
  
  
        //Get Component Info/////////////////////////////////////////////////////
        ret=dyn_conf->get_comp_info("TAO",TAO_TRY_ENV);
        TAO_CHECK_ENV;
  
        ACE_OS::printf("\nTAO info: %s\n",ret);
        ////////////////////////////////////////////////////////////////////////////////
  
  
        //Get Component Info/////////////////////////////////////////////////////
        ret=dyn_conf->get_hooked_comp ("TAO", "Concurrency_Strategy",TAO_TRY_ENV);
        TAO_CHECK_ENV;
    
        ACE_OS::printf("TAO/Concurrency_Strategy = %s\n",ret);
        ////////////////////////////////////////////////////////////////////////////////
      }
    TAO_CATCHANY
        {
            TAO_TRY_ENV.print_exception("");
            return -1;
        }
    TAO_ENDTRY;
    
}// main ()
