/******************************************************************************
*                                                                             *
*  File: LoadableHelloComp1.cpp                                               *
*                                                                             *
*  Description: An implementation of hello that can be dynamically loaded.    *
*               Creates a local ComponentConfigurator, but not a CORBA one.   *
*                                                                             *
*  Project: 2K - dynamic reconfiguration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.4 $
   $Log: LoadableHelloComp1.cpp,v $
   Revision 1.4  2000/10/09 21:02:48  f-kon
   Changes to use new TwoK:: namespace for 2K services integrated with Tomo's
   changes for Windows compilation.

   Revision 1.3  2000/09/30 18:48:31  yamane
   Patches for Windows

   Revision 1.1.1.2  1999/10/17 18:36:17  yamane
   Cleanup for native exception and linux support

   Revision 1.1.1.1  1999/10/12 00:42:47  f-kon
   Version compatible with TAO 1.0

******************************************************************************/

#include "LoadableHelloComp1.h"

int
LoadableHelloComp1_Factory::init (int argc, char *argv[])
{
  // The following init creates a local ComponentConfigurator but not a CORBA one.
  Simple_Component_Factory::init (argc, argv);
  localConfigurator ()->name ("The componentized hello 1.0");
  localConfigurator ()->info ("Returns a componentized 1.0 hello world string");
  return 0;
}


const char *
LoadableHelloComp1_Factory::getHello ()
{
  return "Hello again, my dearest Component World!";
}

int
LoadableHelloComp1_Factory::configure (char *messageString)
{
  ACE_OS::printf("The componentized Hello implementation <%s> received the following "
                 "configuration message: <%s>\n", 
                 localConfigurator ()->name (), 
                 messageString);
  return 0;
}



ACE_FACTORY_DEFINE (LOADABLE_HELLO, LoadableHelloComp1_Factory)
