/******************************************************************************
*                                                                             *
*  File: client_i.cpp                                                         *
*                                                                             *
*  Description: A test client for the Hello World test.                       *
*               Based on the TAO Cubit example.                               *
*                                                                             *
*  Project: 2K                                                                *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
 Revision Control Information
 $Revision: 1.1.1.3 $
 $Log: client_i.cpp,v $
 Revision 1.1.1.3  2000/09/30 18:50:48  yamane
 Patches for Windows

 Revision 1.1.1.2  1999/10/15 22:21:22  yamane
 Clean up for native exception support

 Revision 1.1.1.1  1999/10/12 00:42:47  f-kon
 Version compatible with TAO 1.0

 Revision 1.1.1.1  1999/04/01 18:16:37  f-kon
 This new 2K source tree contains the most up-to-date version of ACE and TAO
 and including the dynamicTAO code as well as some 2K tests and preliminary
 versions of some of its high-level systems.


 Revision 1.1  1999/02/22 22:29:15  f-kon
 *** empty log message ***


******************************************************************************/

#include "ace/Env_Value_T.h"
#include "ace/Read_Buffer.h"
#include "client_i.h"

#include "tao/Timeprobe.h"

// Constructor.
HelloClient::HelloClient (void)
  : f_handle_ (ACE_INVALID_HANDLE)
{
}

int
HelloClient::read_ior (char *filename)
{
  // Open the file for reading.
  f_handle_ = ACE_OS::open (filename,0);

  if (f_handle_ == ACE_INVALID_HANDLE)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "Unable to open %s for writing: %p\n",
                       filename),
                      -1);
  ACE_Read_Buffer ior_buffer (f_handle_);
  hello_key_ = ior_buffer.read ();

  if (hello_key_ == 0)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "Unable to allocate memory to read ior: %p\n"),
                      -1);
  return 0;
}

// Parses the command line arguments and returns an error status.

int
HelloClient::parse_args (void)
{
  ACE_Get_Opt get_opts (argc_, argv_, "dn:f:k:xs");
  int c;
  int result;

  while ((c = get_opts ()) != -1)
    switch (c)
      {
      case 'd':  // debug flag
        TAO_debug_level++;
        break;
      case 'f': // read the IOR from the file.
        result = read_ior (get_opts.optarg);
        if (result < 0)
          ACE_ERROR_RETURN ((LM_ERROR,
                             "Unable to read ior from %s : %p\n",
                             get_opts.optarg),
                            -1);
        break;
      case 'k': // read the IOR from the command-line.
        hello_key_ =
          ACE_OS::strdup (get_opts.optarg);
        break;
      case '?':
      default:
        ACE_ERROR_RETURN ((LM_ERROR,
                           "usage:  %s"
                           " [-d]"
                           " [-f hello-obj-ref-key-file]"
                           " [-k obj-ref-key]"
                           "\n",
                           argv_ [0]),
                          -1);
      }

  // Indicates successful parsing of command line.
  return 0;
}



void
HelloClient::run ()
{
  u_int i;
  char *str;

  // Make the calls in a loop.
  for (i = 0; i < 100; i++)
    {
      ACE_TRY_NEW_ENV {
        str = hello_->getHello (ACE_TRY_ENV);
        ACE_TRY_CHECK;
        ACE_OS::printf ("Hello server returned <%s>\n", str);
      }
      ACE_CATCHANY
      {
        ACE_TRY_ENV.print_exception ("on getHello() call");
        ACE_TRY_ENV.clear ();
      }
      ACE_ENDTRY;

      ACE_OS::sleep(2);
    }

  ACE_OS::printf("End client execution.\n");
}// run ()



HelloClient::~HelloClient (void)
{
  if (f_handle_ != ACE_INVALID_HANDLE)
    ACE_OS::close (f_handle_);

  ACE_OS::printf("Destructed Client Object.\n");
}


int
HelloClient::init (int argc, char **argv)
{
  argc_ = argc;
  argv_ = argv;

  ACE_TRY_NEW_ENV
    {
      // Retrieve the ORB.
      orb_ = CORBA::ORB_init (argc_,
                              argv_,
                              "internet",
                              ACE_TRY_ENV);
      ACE_TRY_CHECK;

      // Parse command line and verify parameters.
      if (parse_args () == -1)
        return -1;
      
      CORBA::Object_var hello_object =
        orb_->string_to_object (hello_key_,
                                ACE_TRY_ENV);

      ACE_TRY_CHECK;
      hello_ =
        hello::_narrow (hello_object.in(), ACE_TRY_ENV);
      ACE_TRY_CHECK;
      
      if (CORBA::is_nil (hello_.in ()))
        ACE_ERROR_RETURN ((LM_ERROR,
                           "invalid hello key <%s>\n",
                           hello_key_),
                          -1);

    }
  ACE_CATCHANY
    {
      ACE_ANY_EXCEPTION._tao_print_exception ("HelloClient::init");
      return -1;
    }
  ACE_ENDTRY;

  return 0;
}

void
HelloClient::fini ()
{
  // Shutdown local client ORB.
  orb_->shutdown ();
}

// i guess the following is not required.

// #if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)
// template class ACE_Env_Value<unsigned long>;
// template class ACE_Env_Value<unsigned short>;
// #elif defined (ACE_HAS_TEMPLATE_INSTANTIATION_PRAGMA)
// #pragma instantiate ACE_Env_Value<unsigned long>
// #pragma instantiate ACE_Env_Value<unsigned short>
// #endif /* ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION */
