/******************************************************************************
*                                                                             *
*  File: hello_i.cpp                                                          *
*                                                                             *
*  Description: Implementation of the hello servant.                          *
*                                                                             *
*                                                                             *
*  Project: 2K - dynamic reconfiguration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************

   Revision Control Information

   $Revision: 1.1.1.2 $

   $Log: hello_i.cpp,v $
   Revision 1.1.1.2  1999/10/17 18:36:29  yamane
   Cleanup for native exception and linux support

   Revision 1.1.1.1  1999/10/12 00:42:47  f-kon
   Version compatible with TAO 1.0

   Revision 1.1.1.1  1999/04/01 18:16:37  f-kon
   This new 2K source tree contains the most up-to-date version of ACE and TAO
   and including the dynamicTAO code as well as some 2K tests and preliminary
   versions of some of its high-level systems.


   Revision 1.1  1999/02/22 22:29:15  f-kon
   *** empty log message ***

******************************************************************************/

#include "hello_i.h"
#include "LoadableHello.h"
#include "LoadableHelloComp.h"
#include "dynamicTAO/src/ace/Dynamic_Service_Config.h"

hello_i::hello_i () 
  : myConfigurator_ (new LockingConfigurator("Hello servant", this))
{
  myConfigurator_->addHook (HOOK_NAME);
  myConfigurator_->info ("A hello server that can be dynamically reconfigured");
  
  // Register ourselves in the ORB domain configurator. This makes us
  // accessible through the Dynamic Configuration service.
  Dynamic_Service_Config::instance ()->domainConfigurator ()-> addHook("HelloServer");
  Dynamic_Service_Config::instance ()->domainConfigurator ()-> hook("HelloServer", myConfigurator_);
}// constructor

hello_i::~hello_i ()
{
  Dynamic_Service_Config::instance ()->domainConfigurator ()-> unhook("HelloServer");
  Dynamic_Service_Config::instance ()->domainConfigurator ()-> deleteHook("HelloServer");
  if (myConfigurator_)
    {
      delete myConfigurator_;
      myConfigurator_ = 0;
    }
}// destructor

  
void 
hello_i::echo1 (const char *arg, CORBA::Environment &_tao_environment)
{
  ACE_UNUSED_ARG(_tao_environment);
  ACE_OS::printf("client sent string \"%s\"\n", arg);
}

void 
hello_i::echo2 (const char *arg, CORBA::Environment &_tao_environment)
{
  ACE_UNUSED_ARG(_tao_environment);
  ACE_OS::printf("client sent string \"%s\"\n", arg);
}

  
char *
hello_i::getHello (CORBA::Environment &_tao_environment)
{
  if(myConfigurator_ == 0)
    // raise exception
    return 0;
  
  ComponentConfigurator *helloConfig = myConfigurator_->getHookedComponent (HOOK_NAME);

  if (helloConfig == 0)
    {
      // No implementation is available, raise exception.
      _tao_environment.clear ();
      _tao_environment.exception 
        (new hello::NotReady ("No Hello for you. "
                       "A hello implementation must be attached to the servant first."));
      return 0;
    }

  const char *str;
  // Check the type of the hello implementation we got.
  // The following code shows that our current treatment of types is not neat at all.
  // Later, we must use RTTI to do it in an elegant way.
  Simple_Strategy_Factory *ssf = (Simple_Strategy_Factory *) helloConfig->implementation();

  myConfigurator_->lock ()->acquire_read (); // This prevents anyone from changing the 
                                             // implementation while I'm using it.
  
  if (!ssf->typeHasCorbaConfigurator ())
    {
      LoadableHello_Factory *lh = (LoadableHello_Factory *) ssf;
      str = lh->getHello ();
    }
  else
    {
      LoadableHelloComp_Factory *lhc = (LoadableHelloComp_Factory *) ssf;
      str = lhc->getHello ();
    }

  myConfigurator_->lock ()->release ();
  
  return CORBA::string_dup(str);
}// getHello ()

