/******************************************************************************
*                                                                             *
*  File: server_i.cpp                                                         *
*                                                                             *
*  Description: A test server for the Hello World test.                       *
*               Based on the TAO Cubit example.                               *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
 Revision Control Information
 $Revision: 1.1.1.1 $
 $Log: server_i.cpp,v $
 Revision 1.1.1.1  1999/10/12 00:42:47  f-kon
 Version compatible with TAO 1.0

 Revision 1.1.1.1  1999/04/01 18:16:37  f-kon
 This new 2K source tree contains the most up-to-date version of ACE and TAO
 and including the dynamicTAO code as well as some 2K tests and preliminary
 versions of some of its high-level systems.


 Revision 1.1  1999/02/22 22:29:16  f-kon
 *** empty log message ***

 Revision 1.1  1999/02/15 18:03:19  f-kon
 *** empty log message ***


******************************************************************************/

#include "server_i.h"

HelloServer::HelloServer (void)
    : 
    ior_output_file_ (0)
{
}

int
HelloServer::parse_args (void)
{
  ACE_Get_Opt get_opts (argc_, argv_, "dn:o:s");
  int c;

  while ((c = get_opts ()) != -1)
    switch (c)
      {
      case 'd':  // debug flag.
        TAO_debug_level++;
        break;
      case 'o': // output the IOR to a file.
        ior_output_file_ = ACE_OS::fopen (get_opts.optarg, "w");
        if (ior_output_file_ == 0)
          ACE_ERROR_RETURN ((LM_ERROR,
                             "Unable to open %s for writing: %p\n",
                             get_opts.optarg), -1);
        break;
      case '?':
      default:
        ACE_ERROR_RETURN ((LM_ERROR,
                           "usage:  %s"
                           " [-d]"
                           " [-o] <ior_output_file>"
                           "\n",
                           argv_ [0]),
                          1);
      }

  // Indicates successful parsing of command line.
  return 0;
}

int
HelloServer::init (int argc,
                   char** argv,
                   CORBA::Environment& ACE_TRY_ENV )
{
  int ret;

  // Call the init of <TAO_ORB_Manager> to initialize the ORB and
  // create a child POA under the root POA.
  if (orb_manager_.init_child_poa (argc,
                                   argv,
                                   "child_poa",
                                   ACE_TRY_ENV) == -1)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "%p\n",
                       "init_child_poa"),
                      -1);

  ACE_CHECK_RETURN (-1);
  argc_ = argc;
  argv_ = argv;

  ret = parse_args ();
  if (ret != 0)
    return ret;

  // Create a ComponentConfigurator for ourselves and add it to the domain
  // through the DynamicConfigurator interface.

  hello_impl_ = new hello_i ();

  CORBA::String_var str  =
    orb_manager_.activate_under_child_poa ("HelloWorld",
                                           hello_impl_,
                                           ACE_TRY_ENV);
  ACE_DEBUG ((LM_DEBUG,
              "The IOR is: <%s>\n",
              str.in ()));

  if (ior_output_file_)
    {
      ACE_OS::fprintf (ior_output_file_,
                       "%s",
                       str.in ());
      ACE_OS::fclose (ior_output_file_);
    }

  return 0;
}

int
HelloServer::run (CORBA::Environment& env)
{
  if (orb_manager_.run (env) == -1)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "HelloServer::run"),
                      -1);
  return 0;
}

HelloServer::~HelloServer (void)
{
}
