/******************************************************************************
*                                                                             *
*  File: ComponentConfiguratorTest.cpp                                        *
*                                                                             *
*  Description: A test for the ComponentConfigurator framework.               *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
 Revision Control Information
 $Revision: 1.1.1.1 $
 $Log: ComponentConfiguratorTest.cpp,v $
 Revision 1.1.1.1  1999/10/12 00:43:29  f-kon
 Version compatible with TAO 1.0

 Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
 This new 2K source tree contains the most up-to-date version of ACE and TAO
 and including the dynamicTAO code as well as some 2K tests and preliminary
 versions of some of its high-level systems.


 Revision 1.3  1999/02/22 22:20:40  f-kon
 Introduced NiceConfigurator and LockingConfigurator and performed several
 fixes on dynamic reconfiguration.

 Revision 1.2  1999/02/02 22:42:39  f-kon
 fixed errors with #includes


******************************************************************************/
#include "SimpleComponent.h"
#include "SimpleEvent.h"

#include "LockingConfigurator.h"

int main()
{ 
    printf ("starting test.\n");
    
    ComponentConfigurator *cc1 = new SimpleConfigurator ("primeiro", 0);
    ComponentConfigurator *cc2 = new LockingConfigurator ("segundo", 0);
    
    delete cc1;
    delete cc2;
    
    
    SimpleComponent *a = new SimpleComponent ("Fabio");
    SimpleComponent *b = new SimpleComponent ("Paula");
    SimpleComponent *c = new SimpleComponent (0);
    printf ("created simple components.\n");

    // normal cases    
    ACE_ASSERT(a->configurator->addHook ("LeftHand")  == 0);
    ACE_ASSERT(a->configurator->addHook ("RightHand") == 0);
    ACE_ASSERT(b->configurator->addHook ("LeftHand")  == 0);
    ACE_ASSERT(b->configurator->addHook ("RightHand") == 0);
    
    // erroneous cases
    ACE_ASSERT(a->configurator->addHook ("LeftHand") == -1); // repeating same hook
    ACE_ASSERT(a->configurator->addHook (0) == -1);          // null hook

    // normal cases
    ACE_ASSERT(a->configurator->deleteHook("LeftHand") == 0);
    ACE_ASSERT(a->configurator->addHook ("LeftHand") == 0);

    printf ("added hooks.\n");
    printf ("number of clients in %s: %d\n", a->configurator->name (), 
            a->configurator->numberOfClients ());
    printf ("number of clients in %s: %d\n", b->configurator->name (), 
            b->configurator->numberOfClients ());
    
    ACE_ASSERT(a->configurator->hook ("LeftHand", b->configurator) == 0);
    ACE_ASSERT(b->configurator->registerClient (a->configurator, "LeftHand") == 0);
    
    ACE_ASSERT(b->configurator->hook ("RightHand", a->configurator) == 0);
    ACE_ASSERT(a->configurator->registerClient (b->configurator, "RightHand") == 0);

    // erroneous cases (repetition)
    ACE_ASSERT(a->configurator->hook ("LeftHand", b->configurator) == -1);
    ACE_ASSERT(b->configurator->registerClient (a->configurator, "LeftHand") == -1);

    printf ("added hooks, hooked components, and registered clients.\n");
    printf ("number of clients in %s: %d\n", a->configurator->name (), 
            a->configurator->numberOfClients ());
    printf ("number of clients in %s: %d\n", b->configurator->name (), 
            b->configurator->numberOfClients ());
    
    a->configurator->
      eventOnHookedComponent(b->configurator, 
                             new SimpleEvent (ComponentEvent::FAILED, "help"));
    b->configurator->
      eventOnHookedComponent(a->configurator, 
                             new SimpleEvent (ComponentEvent::RECONFIGURED));
        
    ACE_ASSERT(b->configurator->unregisterClient (a->configurator, "LeftHand") == 0);
    ACE_ASSERT(a->configurator->unregisterClient (b->configurator, "RightHand") == 0);

    printf ("removed clients.\n");
    printf ("number of clients in %s: %d\n", a->configurator->name (), 
            a->configurator->numberOfClients ());
    printf ("number of clients in %s: %d\n", b->configurator->name (), 
            b->configurator->numberOfClients ());

    
    printf ("done.\n");
    
}// main ()
