/******************************************************************************
*                                                                             *
*  File: ComponentEvent.h                                                     *
*                                                                             *
*  Description: An abstract class for representing events related to          *
*               ComponentConfigurators.                                       *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************

   Revision Control Information

   $Revision: 1.1.1.2 $

   $Log: ComponentEvent.h,v $
   Revision 1.1.1.2  2000/09/30 15:55:07  yamane
   Patches for Windows

   Revision 1.1.1.1  1999/10/12 00:43:29  f-kon
   Version compatible with TAO 1.0

   Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
   This new 2K source tree contains the most up-to-date version of ACE and TAO
   and including the dynamicTAO code as well as some 2K tests and preliminary
   versions of some of its high-level systems.


   Revision 1.5  1999/02/22 22:20:41  f-kon
   Introduced NiceConfigurator and LockingConfigurator and performed several
   fixes on dynamic reconfiguration.

   Revision 1.4  1999/02/02 22:42:40  f-kon
   fixed errors with #includes

   Revision 1.3  1999/02/01 18:13:13  ckhess
   1) Fixed #include "dynamicTAO/src/ace/ACE_2K_OS.h" fo be correct case.

   Revision 1.2  1999/01/26 18:22:13  mroman
   DomainConfigurator running on Windows NT

   Revision 1.1  1999/01/07 22:43:17  f-kon
   Added TAOConfigurator and new DCP commands

******************************************************************************/

#if !defined (COMPONENT_EVENT_H)
#define       COMPONENT_EVENT_H

#include "dynamicTAO/src/ace/ACE_2K_OS.h"
#include "SimpleConfigurator_export.h"

class SIMPLECONF_2K_Export ComponentEvent {
  protected:
    /* unknown event String. */
    static const char * const UNKNOWN;

  public:
    enum EVENT {
        DELETED      = 1,
        FAILED       = 2,
        RECONFIGURED = 4,
        REPLACED     = 8,
        MIGRATED     = 16
    };
    
    virtual int type () const = 0;
  
/* Converts the event into a String.
 * @return a string describing the event.
 */
    virtual const char * toString() const = 0;
};

#endif
