/* -*- C++ -*- */
/******************************************************************************
*                                                                             *
*  File: DependencySpecification.h                                            *
*                                                                             *
*  Description: Defines the DependencySpecification type,                     *
*               a pair (hookName, component).                                 *
*               It is used in two situations:                                 *
*                  1) to represent components attached to local hooks and     *
*                  2) to represent client hooks to which this component       *
*                     is attached.                                            *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************

   Revision Control Information

   $Revision: 1.1.1.2 $

   $Log: DependencySpecification.h,v $
   Revision 1.1.1.2  2000/09/30 15:56:09  yamane
   Patches for Windows

   Revision 1.1.1.1  1999/10/12 00:43:29  f-kon
   Version compatible with TAO 1.0

   Revision 1.1.1.2  1999/09/22 19:21:48  yamane
   dynamicTAO for TAO1.0

   Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
   This new 2K source tree contains the most up-to-date version of ACE and TAO
   and including the dynamicTAO code as well as some 2K tests and preliminary
   versions of some of its high-level systems.


   Revision 1.4  1999/02/02 22:42:41  f-kon
   fixed errors with #includes

   Revision 1.3  1999/02/01 18:11:13  ckhess
   1) Fixed #include "dynamicTAO/src/ace/ACE_2K_OS.h" to be correct case.

   Revision 1.2  1999/01/26 18:22:14  mroman
   DomainConfigurator running on Windows NT

   Revision 1.1  1999/01/07 22:43:17  f-kon
   Added TAOConfigurator and new DCP commands


******************************************************************************/

#if !defined (HOOK_SPECIFICATION_H)
#define HOOK_SPECIFICATION_H

#include "ace/Containers.h"
#include "dynamicTAO/src/ace/ACE_2K_OS.h"
#include "SimpleConfigurator_export.h"

class ComponentConfigurator;

class SIMPLECONF_2K_Export DependencySpecification
{
public:
  inline DependencySpecification (const char *name = 0, 
                            ComponentConfigurator *compConfig = 0)
    {
      if (name)
        name_ = ACE_OS::strdup(name);
      else
        name_ = 0;
      componentConfig_ = compConfig;
    }

  inline ~DependencySpecification ()
    {
      if(name_)
        ACE_OS::free (name_);
    }

  // accessor methods
  inline const char *name () { return name_;}
  inline ComponentConfigurator *componentConfig () { return componentConfig_; }
  inline void componentConfig (ComponentConfigurator *cc) 
    { componentConfig_ = cc; }

protected:
  // This friend declarations allow us to use this type in linked lists.
  friend class ACE_Double_Linked_List<DependencySpecification>;
  friend ACE_Double_Linked_List_Iterator<DependencySpecification>;
//  Add friend declaration to reflect the ACE change
  friend ACE_Double_Linked_List_Iterator_Base<DependencySpecification>;

  DependencySpecification *next_;
  // Pointer to next element in the list of <DependencySpecification>s.

  DependencySpecification *prev_;
  // Pointer to previous element in the list of <DependencySpecificarion>s.

private:
  char *name_;
  ComponentConfigurator *componentConfig_;
};

typedef ACE_Double_Linked_List<DependencySpecification> DependencyList;
typedef ACE_Double_Linked_List_Iterator<DependencySpecification> DependencySpecIterator;
    
#endif
