/* -*- C++ -*- */
/******************************************************************************
*                                                                             *
*  File: HookSpecification.h                                                  *
*                                                                             *
*  Description: Defines the HookSpecification type,                           *
*               a pair (hookName, component).                                 *
*               It is used in two situations:                                 *
*                  1) to represent components attached to local hooks and     *
*                  2) to represent client hooks to which this component       *
*                     is attached.                                            *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************

   Revision Control Information

   $Revision: 1.1.1.2 $

   $Log: HookSpecification.h,v $
   Revision 1.1.1.2  2000/09/30 15:56:49  yamane
   Patches for Windows

   Revision 1.1.1.1  1999/10/12 00:43:29  f-kon
   Version compatible with TAO 1.0

   Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
   This new 2K source tree contains the most up-to-date version of ACE and TAO
   and including the dynamicTAO code as well as some 2K tests and preliminary
   versions of some of its high-level systems.


   Revision 1.2  1999/01/26 18:22:15  mroman
   DomainConfigurator running on Windows NT

   Revision 1.1  1999/01/07 22:43:17  f-kon
   Added TAOConfigurator and new DCP commands


******************************************************************************/

#if !defined (HOOK_SPECIFICATION_H)
#define HOOK_SPECIFICATION_H

#include "ace/Containers.h"
#include "SimpleConfigurator_export.h"

class ComponentConfigurator;

class SIMPLECONF_2K_Export HookSpecification
{
  // This friend declarations allow us to use this type in linked lists.
  friend class ACE_Double_Linked_List<HookSpecification>;
  friend ACE_Double_Linked_List_Iterator<HookSpecification>;

public:
  inline HookSpecification (const char *name = 0, 
                            ComponentConfigurator *compConfig = 0)
    {
      if (name)
        name_ = ACE_OS::strdup(name);
      else
        name_ = 0;
      componentConfig_ = compConfig;
    }

  inline ~HookSpecification ()
    {
      if(name_)
        ACE_OS::free (name_);
    }

  // accessor methods
  inline const char *name () { return name_;}
  inline ComponentConfigurator *componentConfig () { return componentConfig_; }
  inline void componentConfig (ComponentConfigurator *cc) 
    { componentConfig_ = cc; }

protected:
  HookSpecification *next_;
  // Pointer to next element in the list of <HookSpecification>s.

  HookSpecification *prev_;
  // Pointer to previous element in the list of <HookSpecificarion>s.

private:
  char *name_;
  ComponentConfigurator *componentConfig_;
};

typedef ACE_Double_Linked_List<HookSpecification> HookList;
typedef ACE_Double_Linked_List_Iterator<HookSpecification> HookSpecIterator;
    
#endif
