/******************************************************************************
*                                                                             *
*  File: NiceConfigurator.cpp                                                 *
*                                                                             *
*  Description: A concrete class that reifies dependencies between components.*
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.1.1.1 $
   $Log: NiceConfigurator.cpp,v $
   Revision 1.1.1.1  1999/10/12 00:43:29  f-kon
   Version compatible with TAO 1.0

   Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
   This new 2K source tree contains the most up-to-date version of ACE and TAO
   and including the dynamicTAO code as well as some 2K tests and preliminary
   versions of some of its high-level systems.


   Revision 1.2  1999/03/03 16:11:56  f-kon
   Implemented DCP operation configure_implementation <loaded_implementation> <configuration_string>

   Revision 1.1  1999/02/22 22:22:43  f-kon
   Introducing NiceConfigurator and LockingConfigurator.

******************************************************************************/

#include "NiceConfigurator.h"

/* Creates a new configurator for a given object.
 *
 * @param     name             the component name.
 * @param     implementation   the object implementation.
 */
NiceConfigurator::NiceConfigurator (const char *name, 
                                        Object *implementation)
  : SimpleConfigurator (name, implementation)
{
}

/* Attaches a ComponentConfigurator to the given hook. Note that one can only 
 * attach a component to a hook that is vacant.
 * @param  hookName  the name of the hook.
 * @param  cc        the ComponentConfigurator to be attached to the hook.
 */
int NiceConfigurator::hook (const char *hookName, 
                              ComponentConfigurator *cc) 
{
  DependencySpecification *hs = findHook (hookName);
    
  if (hs == 0)
    return -1; // The hook does not exist.
 
 // It's called NiceConfigurator because it allows us to hook on top of
 // another component without complaining. Note that it does NOT inform the
 // old hooked component.
 //  if (hs->componentConfig () != 0)
 //     {
 //       // The hook is not vacant, let's inform the current hooked component
 //       // that we are not going to be its client anymore.
 //       hs->componentConfig ()-> unregisterClient (this, hookName);
 //     }
  
  hs->componentConfig(cc);
  return 0;
  
}// hook ()


/* If the event is of type ComponentEvent::DELETED, then look for the hooks to which the 
 * deleted component is attached and sets it to null.
 * @param  hookedComponent the ComponentConfigurator that generated the event.
 * @param  e               the event that was generated. 
 */
void 
NiceConfigurator::eventOnHookedComponent (const ComponentConfigurator *hookedComponent,
                                            const ComponentEvent *e)
{
  if (e->type () == ComponentEvent::DELETED)
    {
      // finds the hooks to which this hookedComponent is attached.
      DependencySpecIterator hi(hookList_);
    
      for (DependencySpecification *hs = hi.next ();
           hs != 0;
           hi.advance (), hs = hi.next ())
        {
          if (hs->componentConfig () == hookedComponent)
            {
              hs->componentConfig(0);
              ACE_OS::printf ("Component %s "
                              "detaching %s from hook %s\n",
                              name (), hookedComponent->name (),
                              hs->name());
            }
        }
    }
}// eventOnHookedComponent ()
