/******************************************************************************
*                                                                             *
*  File: NiceConfigurator.h                                                 *
*                                                                             *
*  Description: A concrete class that reifies dependencies between components.*
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.1.1.2 $
   $Log: NiceConfigurator.h,v $
   Revision 1.1.1.2  2000/09/30 15:58:55  yamane
   Patches for Windows

   Revision 1.1.1.1  1999/10/12 00:43:29  f-kon
   Version compatible with TAO 1.0

   Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
   This new 2K source tree contains the most up-to-date version of ACE and TAO
   and including the dynamicTAO code as well as some 2K tests and preliminary
   versions of some of its high-level systems.


   Revision 1.2  1999/03/03 16:11:57  f-kon
   Implemented DCP operation configure_implementation <loaded_implementation> <configuration_string>

   Revision 1.1  1999/02/22 22:22:43  f-kon
   Introducing NiceConfigurator and LockingConfigurator.

******************************************************************************/

#if !defined (NICE_CONFIGURATOR_H)
#define       NICE_CONFIGURATOR_H

#include "SimpleConfigurator.h"

class SIMPLECONF_2K_Export NiceConfigurator : public SimpleConfigurator {
  public:
    /* Creates a new configurator for a given object.
     *
     * @param     name             the component name.
     * @param     implementation   the object implementation.
     */
    NiceConfigurator (const char *name, Object *implementation);
    
    /* Attaches a ComponentConfigurator to the given hook.
     * @param  hookName  the name of the hook.
     * @param  cc        the ComponentConfigurator to be attached to the hook.
     */
    int hook (const char *hookName, ComponentConfigurator *cc);

  /* If the event is of type ComponentEvent::DELETED, then look for the hooks to which the 
   * deleted component is attached and sets it to null.
   * @param  hookedComponent the ComponentConfigurator that generated the event.
   * @param  e               the event that was generated. 
   */
    void eventOnHookedComponent (const ComponentConfigurator *hookedComponent,
                                 const ComponentEvent *e);
};

#endif
