/******************************************************************************
*                                                                             *
*  File: SimpleEvent.h                                                        *
*                                                                             *
*  Description: An concrete class for representing simple events.             *
*                                                                             *
*  Project: 2K - Automatic Configuration                                      *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************

   Revision Control Information

   $Revision: 1.1.1.1 $

   $Log: SimpleEvent.cpp,v $
   Revision 1.1.1.1  1999/10/12 00:43:29  f-kon
   Version compatible with TAO 1.0

   Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
   This new 2K source tree contains the most up-to-date version of ACE and TAO
   and including the dynamicTAO code as well as some 2K tests and preliminary
   versions of some of its high-level systems.


   Revision 1.4  1999/02/19 20:03:25  f-kon
   *** empty log message ***

   Revision 1.3  1999/02/02 22:42:44  f-kon
   fixed errors with #includes

   Revision 1.2  1999/01/26 18:22:19  mroman
   DomainConfigurator running on Windows NT

   Revision 1.1  1999/01/07 22:43:18  f-kon
   Added TAOConfigurator and new DCP commands

******************************************************************************/

#include "ace/OS.h"
#include "SimpleEvent.h"

const char * const ComponentEvent::UNKNOWN = "UNKNOWN";

SimpleEvent::SimpleEvent (int type, const char *description)
{
    type_ = type;
    // FIX: modify the following line to make sure that description is <= MAXNAMLEN
    if(description)
      description_ = ACE_OS::strdup (description);
    else
      description_ = 0;
}

SimpleEvent::~SimpleEvent ()
{
    if (description_ != 0)
        ACE_OS::free (description_);
}


/* Converts the event into a String.
 * @return a string describing the event.
 */
const char *SimpleEvent::toString() const
{
    char *typeString;
    
    switch (type_) {
    case DELETED:
        typeString = "DELETED"; break;
    case FAILED:
        typeString = "FAILED"; break;
    case RECONFIGURED:
        typeString = "RECONFIGURED"; break;
    case REPLACED:
        typeString = "REPLACED"; break;
    case MIGRATED:
        typeString = "MIGRATED"; break;
    default:    
        return UNKNOWN;
    }


    char *buffer = new char[ACE_OS::strlen(typeString) + 
                           ((description_)? ACE_OS::strlen(description_):0) + 
                           sizeof("type = %s, description = %s")];

    if (description_ != 0)
        ACE_OS::sprintf(buffer, "type = %s, description = %s", 
                        typeString, description_);
    else
        ACE_OS::sprintf(buffer, "type = %s", typeString);        
    
    return buffer;
}
// Note that subclasses of this class can test for different kinds of
// events that were not defined in the ComponentEvent interface. So,
// subclasses can define their own application-specific events.


