/******************************************************************************
*                                                                             *
*  File: Component_Factory.cpp                                                *
*                                                                             *
*  Description: A wrapper for 2K components that can be dynamically loaded.   *
*               This components are implemented as DLLs and they contain two  *
*               ComponentConfigurators (a local one that represents the       *
*               dependencies inside the process and a CORBA-based distributed *
*               one.                                                          *
*  Note: In addition to what the Simple_Component_Factory does,               *
*        this class creates a CORBA ComponentConfigurator.                    *
*                                                                             *
*  Project: 2K - Automatic Configuration.                                     *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.1.1.1 $
   $Log: Component_Factory.cpp,v $
   Revision 1.1.1.1  1999/10/12 00:43:30  f-kon
   Version compatible with TAO 1.0

******************************************************************************/

#include "Component_Factory.h"
#include "2k/ComponentConfigurator/Configuration_i.h"

////////////////////////////////////////////////////////////////////////////////
// It is the first thing called after the component is dynamically linked and
// before the prerequisites for the component are resolved.
////////////////////////////////////////////////////////////////////////////////
int
Component_Factory::init (int argc, char *argv[])
{
  ACE_TRACE ("Component_Factory::init");
  char *name = "<no name>";
  char *info = "<no info>";
  
  if (argc > 0)
  {
    name = CORBA::string_dup (argv[0]);
    if (argc > 1)
      info = CORBA::string_dup (argv[1]);
  }

  localConfigurator_ = new SimpleConfigurator (name, this);
  localConfigurator ()->info (info);
  thisTypeHasCorbaConfigurator_ = 1;
  
  ComponentConfigurator_i *ccptr = 
    new ComponentConfigurator_i (TAO_ORB_Core_instance ()->root_poa_reference (), name, info,
                                 CORBA::Object::_nil ());
  // If you are going to subclass from this, take a look at the definition of
  // the other constructor for ComponentConfigurator_i.

  corbaConfigurator_ = ccptr->_this (); // Registers the new object with the POA.

  // Subclasses can add references from the configurator to its implementation
  // and from the implementation to its configurator.

  return 0;
}// init

