/******************************************************************************
*                                                                             *
*  File: Component_Factory.h                                                  *
*                                                                             *
*  Description: A wrapper for 2K components that can be dynamically loaded.   *
*               This components are implemented as DLLs and they contain two  *
*               ComponentConfigurators (a local one that represents the       *
*               in-process dependencies and a CORBA-based distributed one.    *                                                   *
*                                                                             *
*  Project: 2K - Automatic Configuration.                                     *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.1.1.2 $
   $Log: Component_Factory.h,v $
   Revision 1.1.1.2  2000/09/30 17:23:29  yamane
   Patches for Windows

   Revision 1.1.1.1  1999/10/12 00:43:30  f-kon
   Version compatible with TAO 1.0

******************************************************************************/

#if !defined (COMPONENT_FACTORY_H)
#  define     COMPONENT_FACTORY_H

#include "Simple_Component_Factory.h"
#include "2k/ComponentConfigurator/ConfigurationS.h" // the CORBA one.

class COMPONENTS_2K_Export Component_Factory : public Simple_Component_Factory 
  // = TITLE 
  // This is a wrapper that can be added to Components so they become 
  // accessible from the Automatic Configuration infrastructure.
  // In addition to what the Simple_Component_Factory does, 
  // it creates a CORBA ComponentConfigurator.
{
public:
  // init is the first thing called after the component is dynamically linked
  // and BEFORE the prerequisites for the component are resolved. The method
  // creates a CORBA ComponentConfigurator for this component.
  virtual int init (int, char *[]);
};



#endif 


