/******************************************************************************
*                                                                             *
*  File: Simple_Component_Factory.h                                           *
*                                                                             *
*  Description: A wrapper for 2K components that can be dynamically loaded.   *
*               This components are implemented as DLLs and they contain two  *
*               ComponentConfigurators (a local one that represents the       *
*               inside-the-process dependencies and a CORBA-based distributed *
*               one.                                                          *
*                                                                             *
*  Note: This class does not initialize the CORBA ComponentConfigurator, if   *
*        you need this initialization, use Component_Factory instead.         *
*                                                                             *
*  Project: 2K - Automatic Configuration.                                     *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.2 $
   $Log: Simple_Component_Factory.h,v $
   Revision 1.2  2000/10/09 21:03:07  f-kon
   Changes to use new TwoK:: namespace for 2K services integrated with Tomo's
   changes for Windows compilation.

   Revision 1.1.1.2  2000/09/30 17:24:04  yamane
   Patches for Windows

   Revision 1.1.1.1  1999/10/12 00:43:30  f-kon
   Version compatible with TAO 1.0

******************************************************************************/

#if !defined (SIMPLE_COMPONENT_FACTORY_H)
#  define     SIMPLE_COMPONENT_FACTORY_H

#include "Components_export.h"
#include "dynamicTAO/src/ace/Simple_Strategy_Factory.h"
#include "2k/ComponentConfigurator/ConfigurationC.h" // the CORBA one.

class COMPONENTS_2K_Export Simple_Component_Factory : public Simple_Strategy_Factory 
  // = TITLE 
  // This is a wrapper that can be added to Components so they become 
  // accessible from the Automatic Configuration infrastructure.
{
public:
  Simple_Component_Factory ();

  // = Initialization and termination methods.

  virtual int init (int, char *[]);
  virtual int fini (void);

  // start is called AFTER the prerequisites of the component are resolved. At
  // this time, its dependencies, if any, were attached to its CORBA
  // ComponentConfigurator hooks.
  virtual int start (const TwoK::Configuration::ComponentConfigurator_var &compConfig);

  virtual void *component_implementation (void);

  ComponentConfigurator *localConfigurator (); 
  void localConfigurator (ComponentConfigurator *c); 
  // accessor method for the local component configurator.

  TwoK::Configuration::ComponentConfigurator_var corbaConfigurator (); 
  void corbaConfigurator (const TwoK::Configuration::ComponentConfigurator_var &c);
  // accessor methods for the corbacomponent configurator.

protected:
  TwoK::Configuration::ComponentConfigurator_var corbaConfigurator_;

  void *strategy (void);
};

#endif









