/******************************************************************************
*                                                                             *
*  File: dynamicConfigurator_i_.cpp                                           *
*                                                                             *
*  Description: The implementation of the dynamicConfigurator interface.      *
*                                                                             *
*  Project: 2K - Dynamic Configuration                                        *
*  Author: Manuel Roman (mroman1@cs.uiuc.edu)                                 *
*  Modified by: Fabio Kon (f-kon@cs.uiuc.edu)                                 *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.4 $
   $Log: dynamicConfigurator_i_.cpp,v $
   Revision 1.4  2000/10/09 21:03:12  f-kon
   Changes to use new TwoK:: namespace for 2K services integrated with Tomo's
   changes for Windows compilation.

   Revision 1.3  2000/09/30 17:27:33  yamane
   Patches for Windows

   Revision 1.2  2000/01/18 22:12:32  f-kon
   various improvements to better serve the AutoConfig service

   Revision 1.1.1.3  1999/10/27 16:55:36  yamane
   Cleanup for SunC++5.0

   Revision 1.1.1.2  1999/10/15 22:15:09  yamane
   Clean up for native exception support

   Revision 1.1.1.1  1999/10/12 00:43:29  f-kon
   Version compatible with TAO 1.0

   Revision 1.2  1999/06/30 20:09:21  f-kon
   minor modifications on indentation.

******************************************************************************/

#include "ace/Service_Repository.h"
#include "dynamicConfigurator_i_.h"
#include "dynamicTAO/src/ace/Dynamic_Service_Config.h"
#include "dynamicTAO/src/Components/Simple_Component_Factory.h"
#include "dynamicTAO/src/ace/Simple_Strategy_Factory.h"
#include "2k/ComponentConfigurator/ConfigurationC.h"

#include "ace/Dynamic_Service.h"


const int DynamicConfigurator_i::MAX_STRINGS=50; // This is *very* poblematic hack.
                                                 // Isn't there a neat solution?

const int DynamicConfigurator_i::MAX_BUFFER=1024;// This is just a little ugly.


DynamicConfigurator_i::DynamicConfigurator_i()
{										 
  dc_pers_rep_ = Dynamic_Service_Config::instance ()->pers_rep();
}



DynamicConfigurator_i::~DynamicConfigurator_i()
{
}


DynamicConfigurator::stringList *
DynamicConfigurator_i::list_categories (CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      DynamicConfigurator::SERVICE_ERROR
    ))
{
	char **dyn_array=DynamicConfigurator::stringList::allocbuf(DynamicConfigurator_i::MAX_STRINGS);
	int numberStrings;

	if (dc_pers_rep_->get_contents (dyn_array, numberStrings, 
                                        DynamicConfigurator_i::MAX_STRINGS) == -1)
	{
		CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
		ACE_TRY_ENV.exception (exception);

		DynamicConfigurator::stringList *ret = 
                  new DynamicConfigurator::stringList(0,0,0,FALSE);
		return ret;		
	}
	else if (numberStrings==0)
	{
		DynamicConfigurator::stringList *ret = 
                  new DynamicConfigurator::stringList(0,0,0,FALSE);
		return ret;
	}
	else
	{
		DynamicConfigurator::stringList *ret = 
                  new DynamicConfigurator::stringList(numberStrings,numberStrings,dyn_array,TRUE);
		return ret;
	}
	DynamicConfigurator::stringList *ret = 
          new DynamicConfigurator::stringList(0,0,0,FALSE);
	return ret;
}


DynamicConfigurator::stringList *
DynamicConfigurator_i::list_implementations (const char * serviceName,CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      DynamicConfigurator::SERVICE_ERROR
    ))
{
	char **dyn_array=DynamicConfigurator::stringList::allocbuf(DynamicConfigurator_i::MAX_STRINGS);
	int numberStrings;

	if (dc_pers_rep_->get_contents (dyn_array, numberStrings, DynamicConfigurator_i::MAX_STRINGS, 
                                        serviceName) == -1)
	{
		DynamicConfigurator::stringList *ret = 
                  new DynamicConfigurator::stringList(0,0,0,FALSE);
		
		CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
		ACE_TRY_ENV.exception (exception);
		return ret;
	}
	else if (numberStrings==0)
	{
		DynamicConfigurator::stringList *ret = 
                  new DynamicConfigurator::stringList(0,0,0,FALSE);
		return ret;
	}
	else
	{
		DynamicConfigurator::stringList *ret = 
                  new DynamicConfigurator::stringList(numberStrings,numberStrings,dyn_array,TRUE);
		return ret;
	}
	DynamicConfigurator::stringList *ret = new DynamicConfigurator::stringList(0,0,0,FALSE);
	return ret;
}

CORBA::Boolean 
DynamicConfigurator_i::have_implementation (const char * categoryName, const char * impName,
                                            CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      DynamicConfigurator::SERVICE_ERROR
      ))
{
  char pathname[MAXPATHLEN]; 
  struct stat st;
  dc_pers_rep_->build_pathname (pathname, categoryName, impName);
 
  if (ACE_OS::stat(pathname, &st) < 0)
    {
      // Could not access the file. Let's try to add the DLL prefix and suffixes to see if 
      // that will help locating it.
      char newName[MAXNAMELEN];
      ACE_OS::sprintf(newName, "%s%s%s", ACE_DLL_PREFIX, impName, ACE_DLL_SUFFIX);
      // try again
      dc_pers_rep_->build_pathname (pathname, categoryName, newName);
      if (ACE_OS::stat(pathname, &st) < 0)
        {
          // it does't exist, return false
         return (CORBA::Boolean) 0;
        }
    }
  
  // the file exists, return true.
  return (CORBA::Boolean) 1;

}// have_implementation ()

DynamicConfigurator::stringList *
DynamicConfigurator_i::list_loaded_implementations(CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
      CORBA::SystemException
    ))
{
	int elements = 0;
	int pos=0;

	ACE_TRACE ("DCP_Broker::list_loaded_services");
	ACE_Service_Repository_Iterator sri (*ACE_Service_Repository::instance ());
	
	elements=ACE_Service_Repository::instance ()->current_size();
	
	if(elements==0)
	{
		DynamicConfigurator::stringList *ret = 
                  new DynamicConfigurator::stringList(0,0,0,FALSE);
		return ret;
	}
	else
	{
		char **dyn_array=DynamicConfigurator::stringList::allocbuf(elements);

		for (const ACE_Service_Type *sr; sri.next (sr) != 0; sri.advance ())
		{	
			dyn_array[pos++]=ACE_OS::strdup(sr->name());
		}
		
		DynamicConfigurator::stringList *ret = 
                  new DynamicConfigurator::stringList(pos,pos,dyn_array,TRUE);
		
		return ret;
	}
}


char * 
DynamicConfigurator_i::get_impl_info (const char * implName, CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      DynamicConfigurator::SERVICE_ERROR
    ))
{
	int ret = 0;
	const ACE_Service_Type *srvc = 0;
	char buf[BUFSIZ];
	char *bufP = buf;
 
	// Try to find service in the repository
	ret = ACE_Service_Repository::instance ()->find(implName, &srvc, 1);

	if (ret == -1 || srvc == 0) // Didn't find service in the repository.
	{
		CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
		ACE_TRY_ENV.exception (exception);
		return NULL;
	}	
 
	// The service was found, get its info.
	int len = srvc->type ()->info (&bufP, BUFSIZ - 4);

	return bufP;
}


DynamicConfigurator::stringList *
DynamicConfigurator_i::list_domain_components (CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
      CORBA::SystemException
    ))
{
  char **dyn_array = DynamicConfigurator::stringList::allocbuf(DynamicConfigurator_i::MAX_STRINGS);
  int size;
	
  Dynamic_Service_Config::instance ()->get_components(dyn_array, size, 
                                                      DynamicConfigurator_i::MAX_STRINGS);
  
  if (size == 0)
  {
	DynamicConfigurator::stringList *ret = new DynamicConfigurator::stringList(0,0,0,FALSE);
	return ret;
  }
  else
  {
	DynamicConfigurator::stringList *ret = 
          new DynamicConfigurator::stringList(size,size,dyn_array,TRUE);
	return ret;
  }
}


DynamicConfigurator::stringList *
DynamicConfigurator_i::list_hooks(const char * appName,CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      DynamicConfigurator::SERVICE_ERROR
    ))
{
  char **dyn_array=DynamicConfigurator::stringList::allocbuf(DynamicConfigurator_i::MAX_STRINGS);
  int numStrings;
  
  if (Dynamic_Service_Config::instance ()->get_hooks((char*)appName,
                                                     dyn_array, 
                                                     numStrings, 
                                                     DynamicConfigurator_i::MAX_STRINGS)
      == -1)
    {			
      DynamicConfigurator::stringList *ret = 
        new DynamicConfigurator::stringList(0,0,0,FALSE);
      CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
      ACE_TRY_ENV.exception (exception);
      return ret;
    }
  else
    {
      if (numStrings == 0)
        {
          DynamicConfigurator::stringList *ret = 
            new DynamicConfigurator::stringList(0,0,0,FALSE);
          return ret;
        }
      else	
        {	
          DynamicConfigurator::stringList *ret = 
            new DynamicConfigurator::stringList(numStrings,numStrings,dyn_array,TRUE);
          return ret;
        }
    }
}


char *
DynamicConfigurator_i::get_comp_info(const char *componentName,CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      DynamicConfigurator::SERVICE_ERROR
    ))
{
  char *buf=new char[DynamicConfigurator_i::MAX_BUFFER];
	
  if (Dynamic_Service_Config::instance ()->get_comp_info ((char*)componentName, 
                                                          (char*)buf, 
                                                          DynamicConfigurator_i::MAX_BUFFER) 
      == -1)
    {
      CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
      ACE_TRY_ENV.exception (exception);
      return NULL;
    }
  else
    {
      return buf;
    }
}


////////////////////////////////////////////////////////////////////////
// Returns the version number if successful and -1 otherwise.
////////////////////////////////////////////////////////////////////////
int DynamicConfigurator_i::load_implementation (const char *categoryName,
                                                const char *impName,
                                                const char *params,
                                                TwoK::Configuration::CCFactory_ptr factory,
                                                TwoK::Configuration::ComponentConfigurator_out cc, 
                                                CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      DynamicConfigurator::SERVICE_ERROR
    ))
{
  char pathname[MAXPATHLEN]; 
  char paramCopy[MAXNAMELEN]; 
  
  dc_pers_rep_->build_pathname (pathname, categoryName);
  
  ACE_OS::strcpy(paramCopy, params);
  
  int version = Dynamic_Service_Config::instance ()->install (pathname, 
                                                              impName, 
                                                              categoryName, 
                                                              paramCopy);
  
  if (version > 0)
    {
      char newServName[MAXPATHLEN];
      ACE_OS::sprintf (newServName, "%s:%d", categoryName, version);
      
      Simple_Strategy_Factory *ssf =
        ACE_Dynamic_Service<Simple_Strategy_Factory>::instance (newServName);

      if (ssf == 0)
        {
          ACE_DEBUG ((LM_WARNING, "Internal error in DynamicConfigurator, could not"
                      "locate <%s>\n", newServName));
          return version;
        }

      if (ssf->typeHasCorbaConfigurator ())
        {
          Simple_Component_Factory *cf = (Simple_Component_Factory *) ssf;
          // The following will only work if init() was called on the Simple_Component_Factory,
          // which is done during the call to Dynamic_Service_Config::install above.
          cc = cf->corbaConfigurator (); 
          
          if (CORBA::is_nil (& (*cc)))
            {
              // The component doesn't have its own CORBA ComponentConfiguratorv yet.
              // If we received a factory as a parameter, we use it to create the configurator.
              if (!CORBA::is_nil (factory))
                {
                  ACE_DECLARE_NEW_CORBA_ENV;
                  // Now, create a CORBA ComponentConfigurator for it using the given factory.
                  cc = factory->newConfigurator (newServName, CORBA::Object::_nil (), ACE_TRY_ENV);
                  ACE_CHECK_RETURN (-1);
                  
                  cf->corbaConfigurator(TwoK::Configuration::ComponentConfigurator::_duplicate (cc));
                }
              else
                {
                  // no luck, the object will not have a ComponentConfigurator.
                  return version;
                }
            }
          // cc contains the CORBA ComponentConfigurator
          // anything to be done here?
        }
      
      return version;
    }
  else 
    {
      ACE_THROW_RETURN (DynamicConfigurator::SERVICE_ERROR (), -1);
    }
  return -1;
  
}// load_implementation ()


//CHANGE:It should return something
void 
DynamicConfigurator_i::hook_implementation(const char *impName,
                                           const char *componentName,
                                           const char * hookName,
                                           CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      DynamicConfigurator::SERVICE_ERROR
    ))
{
  if(Dynamic_Service_Config::instance ()->hook (impName, 
                                                hookName, (char*) componentName)
     ==-1)
    {
      CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
      ACE_TRY_ENV.exception (exception);
    }
}


void 
DynamicConfigurator_i::suspend_implementation(const char *impName,
                                              CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      DynamicConfigurator::SERVICE_ERROR
    ))
{
	if(Dynamic_Service_Config::instance ()->suspend (impName)==-1)
	{
		CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
		ACE_TRY_ENV.exception (exception);
	}
}


void 
DynamicConfigurator_i::resume_implementation(const char * impName,
                                             CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      DynamicConfigurator::SERVICE_ERROR
    ))
{
	if(Dynamic_Service_Config::instance ()->resume (impName)==-1)
	{
		CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
		ACE_TRY_ENV.exception (exception);
	}
}


void 
DynamicConfigurator_i::remove_implementation(const char * impName,
                                             CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      DynamicConfigurator::SERVICE_ERROR
    ))
{
	if(Dynamic_Service_Config::instance ()->remove (impName)==-1)
	{
		CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
		ACE_TRY_ENV.exception (exception);
	}
}	




void DynamicConfigurator_i::configure_implementation(const char *impName, 
                                                     const char * message,
                                                     CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      DynamicConfigurator::SERVICE_ERROR
    ))
{
  if (impName == 0 || message == 0)
    {
      return;
    }
  
  // Now, an ugly hack to recognize ACE Services that are not 2K components: 2K
  // components must have the ":" to separate the category name from the version
  // number. If it doesn't have the ":", we assume it's not a 2K component.
  if (!strchr(impName, ':')) 
    {
      // Not a 2K component.
      CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
      ACE_TRY_ENV.exception (exception);
      return;
    }
  else 
    {
      // It is a 2K component.
      // Try to find implementationName in the repository
      Simple_Strategy_Factory *ssf = 
        ACE_Dynamic_Service<Simple_Strategy_Factory>::instance (impName);
      
      // IMPORTANT: The previous call should return 0 if implementationName is not
      // of type Simple_Strategy_Factory but the implementation of
      // ACE_Dynamic_Service<>::instance does NOT use RTTI yet. While this is not
      // fixed, this won't be completely safe.
      if (ssf == 0) // Didn't find a valid implementation in the repository.
        {
          CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
          ACE_TRY_ENV.exception (exception);
          return;
        }
      
      // The implementationName was found, sends it the messageString
      char *non_const_mssg=ACE_OS::strdup(message);
      
      if(ssf->configure (non_const_mssg)==-1)
        {
          CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
          ACE_TRY_ENV.exception (exception);
        }
    }
}// configure_implementation ()


char *DynamicConfigurator_i::get_hooked_comp (const char * componentName,
                                              const char * hookName,
                                              CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      DynamicConfigurator::SERVICE_ERROR
    ))
{
  char *msg = (char *)Dynamic_Service_Config::instance ()->get_hooked_comp (hookName, 
                                                                            (char *)componentName);
 
  if (msg == 0)
  {
	CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
	ACE_TRY_ENV.exception (exception);
	return 0;
  }

  return msg;
}


void DynamicConfigurator_i::upload_implementation (const char * categoryName, 
                                                   const char * impName, 
                                                   const DynamicConfigurator::implCode & binCode, 
                                                   CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      DynamicConfigurator::SERVICE_ERROR
    ))
{
  struct stat st;
  char pathname[MAXPATHLEN];
  dc_pers_rep_->build_pathname (pathname, categoryName, NULL);

  // First, let's check if the directory corresponding to categoryName exists.
  if (ACE_OS::stat(pathname, &st) < 0)
    {
      // It doesn't exist, let's try to create it. To do that, we iterate over
      // the components of the categoryName and create one directory at a
      // time.
      char *dir_component;
      char *p = 0;
      char *catName = ACE_OS::strdup (categoryName);
      dc_pers_rep_->build_pathname (pathname, NULL, NULL);

      
      // extract first component of the categoryName
      dir_component = ACE_OS::strtok_r (catName, "/\\", &p);
      while (dir_component != NULL && *dir_component != '\0')
        {
          ACE_OS::strcat (pathname, ACE_DIRECTORY_SEPARATOR_STR_A);
          ACE_OS::strcat (pathname, dir_component);

          // First check if this subdirectory already exists and skips it if it exists.
          if (ACE_OS::stat(pathname, &st) < 0)
            {
              // This directory does not exist, let's try to create it.
              // Create directory setting mode to user(RWX) and group(RX).
              if (ACE_OS::mkdir(pathname, S_IRWXU | S_IRGRP | S_IXGRP)  < 0)
                {
                  ACE_DEBUG ((LM_ERROR, "Could not create directory %s\n", pathname));
                  ACE_TRY_ENV.exception (new DynamicConfigurator::SERVICE_ERROR);
                  return;
                }
            }
          
          // Now, extract the next directory component
          dir_component = ACE_OS::strtok_r (NULL, "/\\", &p);
        }
    }
  
  // Now, let's create a file containing binCode
  const unsigned char *buf = binCode.get_buffer ();
  unsigned long size = binCode.length ();

  // Now, set the full pathname of the file.
  dc_pers_rep_->build_pathname (pathname, categoryName, impName);

  FILE *f = ACE_OS::fopen(pathname, "w");
  if (f == 0 || 
      ACE_OS::fwrite (buf, 1, size, f) < size ||    // write
      ACE_OS::fclose (f) < 0                        // close
     )     
    {
      ACE_DEBUG ((LM_ERROR, "Could not write to file %s\n", pathname));
      ACE_TRY_ENV.exception (new DynamicConfigurator::SERVICE_ERROR);
    }
}// upload_implementation ()
 
 
void DynamicConfigurator_i::delete_implementation (const char * categoryName, 
                                                   const char * impName, 
                                                   CORBA::Environment &ACE_TRY_ENV)
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      DynamicConfigurator::SERVICE_ERROR
    ))
{
  // NEED TO ADD A CALL TO: 
  dc_pers_rep_-> delete_entry(categoryName, impName);
  // THROW EXCEPTION:
}


////////////////////////////////////////////////////////////////////////////////
// Note: downloading a file using this method will not work if the file is too
// big.  To solve that, we should use something like an iterator. Henning's
// book suggest looking at its section 18.7.
////////////////////////////////////////////////////////////////////////////////
void 
DynamicConfigurator_i::download_implementation (const char * categoryName, 
                                                char *& impName, 
                                                DynamicConfigurator::implCode_out binCode, 
                                                CORBA::Environment &ACE_TRY_ENV )
    ACE_THROW_SPEC ((
      CORBA::SystemException,
      DynamicConfigurator::SERVICE_ERROR
    ))
{
  // open file
  char pathname[MAXPATHLEN]; 
  struct stat st;
  
  // first set the out parameter to nil in case we fail to return it.
  binCode = 0;//DynamicConfigurator::implCode::_nil ();
  
  dc_pers_rep_->build_pathname (pathname, categoryName, impName);
  
  FILE *f = ACE_OS::fopen(pathname, "r");
  if (f == 0 || ACE_OS::stat(pathname, &st) < 0)
    {
      // Could not access the file. Let's try to add the DLL prefix and suffixes to see if 
      // that will help locating it.
      if (f) ACE_OS::fclose (f);

      char newName[MAXNAMELEN];
      ACE_OS::sprintf(newName, "%s%s%s", ACE_DLL_PREFIX, impName, ACE_DLL_SUFFIX);
      // try again
      dc_pers_rep_->build_pathname (pathname, categoryName, newName);
      f = ACE_OS::fopen(pathname, "r");
      if (f == 0 || ACE_OS::stat(pathname, &st) < 0)
        {
          //          ACE_TRY_ENV.exception (new DynamicConfigurator::SERVICE_ERROR);
          //return;
          ACE_THROW (DynamicConfigurator::SERVICE_ERROR ());
        }
      // at this point, we located the file with the new name, let's fix impName out value.
      CORBA::string_free (impName);
      impName = CORBA::string_dup (newName);
    }

  CORBA::Octet *buf = DynamicConfigurator::implCode::allocbuf (st.st_size);
  if (buf != 0)
    {
      ACE_OS::fread (buf, st.st_size, 1, f);
      binCode = new DynamicConfigurator::implCode (st.st_size, st.st_size, buf, 1);
    }
  else
    {
      ACE_TRY_ENV.exception (new CORBA::NO_MEMORY ());
      ((CORBA_SystemException *)ACE_TRY_ENV.exception())->completed (CORBA::COMPLETED_NO);
    }
  
  ACE_OS::fclose (f);


}// download_implementation ()

