/******************************************************************************
*                                                                             *
*  File: dynamicConfigurator_i_.cpp                                           *
*                                                                             *
*  Description: The implementation of the dynamicConfigurator interface.      *
*                                                                             *
*                                                                             *
*  Project: 2K - Dynamic Configuration                                        *
*  Author: Manuel Roman (mroman1@cs.uiuc.edu)                                 *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.1.1.1 $
   $Log: dynamicConfigurator_i_.cpp,v $
   Revision 1.1.1.1  1999/10/12 00:43:29  f-kon
   Version compatible with TAO 1.0

   Revision 1.2.1.1  1999/09/22 23:00:34  yamane
   dynamicTAO for TAO1.0

   Revision 1.2  1999/06/30 20:09:21  f-kon
   minor modifications on indentation.

******************************************************************************/

#include "ace/Service_Repository.h"
#include "dynamicConfigurator_i_.h"
#include "dynamicTAO/src/ace/Dynamic_Service_Config.h"
#include "dynamicTAO/src/ace/Simple_Strategy_Factory.h"
#include "ace/Dynamic_Service.h"

const int DynamicConfigurator_i::MAX_STRINGS=50; // This is *very* poblematic hack.
                                                 // There's no neat solution?

const int DynamicConfigurator_i::MAX_BUFFER=1024;// This is just a little ugly.


DynamicConfigurator_i::DynamicConfigurator_i()
{										 
	dc_pers_rep_=Dynamic_Service_Config::instance()->pers_rep();
}



DynamicConfigurator_i::~DynamicConfigurator_i()
{
}


DynamicConfigurator::stringList *DynamicConfigurator_i::list_categories (CORBA::Environment &_tao_environment)
{
	char **dyn_array=DynamicConfigurator::stringList::allocbuf(DynamicConfigurator_i::MAX_STRINGS);
	int numberStrings;

	if (dc_pers_rep_->get_contents (dyn_array, numberStrings, DynamicConfigurator_i::MAX_STRINGS)
            == -1)
	{
		CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
		_tao_environment.exception (exception);

		DynamicConfigurator::stringList *ret = 
                  new DynamicConfigurator::stringList(0,0,0,FALSE);
		return ret;		
	}
	else if (numberStrings==0)
	{
		DynamicConfigurator::stringList *ret = 
                  new DynamicConfigurator::stringList(0,0,0,FALSE);
		return ret;
	}
	else
	{
		DynamicConfigurator::stringList *ret = 
                  new DynamicConfigurator::stringList(numberStrings,numberStrings,dyn_array,TRUE);
		return ret;
	}
	DynamicConfigurator::stringList *ret = new DynamicConfigurator::stringList(0,0,0,FALSE);
	return ret;
}


DynamicConfigurator::stringList *DynamicConfigurator_i::list_implementations (const char * serviceName,CORBA::Environment &_tao_environment)
{
	char **dyn_array=DynamicConfigurator::stringList::allocbuf(DynamicConfigurator_i::MAX_STRINGS);
	int numberStrings;

	if (dc_pers_rep_->get_contents (dyn_array, numberStrings, 
                                        DynamicConfigurator_i::MAX_STRINGS, serviceName) == -1)
	{
		DynamicConfigurator::stringList *ret = 
                  new DynamicConfigurator::stringList(0,0,0,FALSE);
		
		CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
		_tao_environment.exception (exception);
		return ret;
	}
	else if (numberStrings==0)
	{
		DynamicConfigurator::stringList *ret = 
                  new DynamicConfigurator::stringList(0,0,0,FALSE);
		return ret;
	}
	else
	{
		DynamicConfigurator::stringList *ret = 
                  new DynamicConfigurator::stringList(numberStrings,numberStrings,dyn_array,TRUE);
		return ret;
	}
	DynamicConfigurator::stringList *ret = new DynamicConfigurator::stringList(0,0,0,FALSE);
	return ret;
}


DynamicConfigurator::stringList *
DynamicConfigurator_i::list_loaded_implementations(CORBA::Environment &_tao_environment)
{
	int elements = 0;
	int pos=0;

	ACE_TRACE ("DCP_Broker::list_loaded_services");
	ACE_Service_Repository_Iterator sri (*ACE_Service_Repository::instance ());
	
	elements=ACE_Service_Repository::instance ()->current_size();
	
	if(elements==0)
	{
		DynamicConfigurator::stringList *ret = new DynamicConfigurator::stringList(0,0,0,FALSE);
		return ret;
	}
	else
	{
		char **dyn_array=DynamicConfigurator::stringList::allocbuf(elements);

		for (const ACE_Service_Type *sr; sri.next (sr) != 0; sri.advance ())
		{	
			dyn_array[pos++]=ACE_OS::strdup(sr->name());
		}
		
		DynamicConfigurator::stringList *ret = new DynamicConfigurator::stringList(pos,pos,dyn_array,TRUE);
		
		return ret;
	}
}


char * DynamicConfigurator_i::get_impl_info (const char * implName, CORBA::Environment &_tao_environment)
{
	int ret = 0;
	const ACE_Service_Type *srvc = 0;
	char buf[BUFSIZ];
	char *bufP = buf;
 
	// Try to find service in the repository
	ret = ACE_Service_Repository::instance ()->find(implName, &srvc, 1);

	if (ret == -1 || srvc == 0) // Didn't find service in the repository.
	{
		CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
		_tao_environment.exception (exception);
		return NULL;
	}	
 
	// The service was found, get its info.
	int len = srvc->type ()->info (&bufP, BUFSIZ - 4);

	return bufP;
}


DynamicConfigurator::stringList *DynamicConfigurator_i::list_domain_components (CORBA::Environment &_tao_environment)
{
  char **dyn_array=	DynamicConfigurator::stringList::allocbuf(DynamicConfigurator_i::MAX_STRINGS);
  int size;
	
  Dynamic_Service_Config::instance()->get_components(dyn_array, size, DynamicConfigurator_i::MAX_STRINGS);
  
  if (size == 0)
  {
	DynamicConfigurator::stringList *ret = new DynamicConfigurator::stringList(0,0,0,FALSE);
	return ret;
  }
  else
  {
	DynamicConfigurator::stringList *ret = new DynamicConfigurator::stringList(size,size,dyn_array,TRUE);
	return ret;
  }
}


DynamicConfigurator::stringList *DynamicConfigurator_i::list_hooks(const char * appName,CORBA::Environment &_tao_environment)
{
	char **dyn_array=DynamicConfigurator::stringList::allocbuf(DynamicConfigurator_i::MAX_STRINGS);
	int numStrings;
	
	if(Dynamic_Service_Config::instance()->get_hooks((char*)appName,dyn_array, numStrings, DynamicConfigurator_i::MAX_STRINGS)==-1)
	{			
		DynamicConfigurator::stringList *ret = new DynamicConfigurator::stringList(0,0,0,FALSE);
		CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
		_tao_environment.exception (exception);
		return ret;
	}
	else
	{
		if (numStrings == 0)
		{
			DynamicConfigurator::stringList *ret = new DynamicConfigurator::stringList(0,0,0,FALSE);
			return ret;
		}
		else	
		{	
			DynamicConfigurator::stringList *ret = new DynamicConfigurator::stringList(numStrings,numStrings,dyn_array,TRUE);
			return ret;
		}
	}
}


char *DynamicConfigurator_i::get_comp_info(const char *componentName,CORBA::Environment &_tao_environment)
{
  char *buf=new char[DynamicConfigurator_i::MAX_BUFFER];
	
  if (Dynamic_Service_Config::instance()->get_comp_info ((char*)componentName, (char*)buf, DynamicConfigurator_i::MAX_BUFFER) == -1)
  {
	CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
	_tao_environment.exception (exception);
	return NULL;
  }
  else
  {
	return buf;
  }
}


//CHANGE: It should return something
void DynamicConfigurator_i::load_implementation (const char *categoryName,
                                                 const char *impName,
                                                 const char *params,
                                                 CORBA::Environment &_tao_environment)
{
	char pathname[MAXPATHLEN]; 
	char *final;

	dc_pers_rep_->build_pathname (pathname, categoryName);

	final=ACE_OS::strdup(params);

	if (Dynamic_Service_Config::instance()->install (pathname, impName, categoryName, final)==-1)
	{
		CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
		_tao_environment.exception (exception);
	}
}



//CHANGE:It should return something
void DynamicConfigurator_i::hook_implementation(const char *impName,
                                                const char *componentName,
                                                const char * hookName,
                                                CORBA::Environment &_tao_environment)
{
	if(Dynamic_Service_Config::instance()->hook (impName, hookName, (char*) componentName)==-1)
	{
		CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
		_tao_environment.exception (exception);
	}
}


void DynamicConfigurator_i::suspend_implementation(const char *impName,
                                                   CORBA::Environment &_tao_environment)
{
	if(Dynamic_Service_Config::instance()->suspend (impName)==-1)
	{
		CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
		_tao_environment.exception (exception);
	}
}


void DynamicConfigurator_i::resume_implementation(const char * impName,
                                                  CORBA::Environment &_tao_environment)
{
	if(Dynamic_Service_Config::instance()->resume (impName)==-1)
	{
		CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
		_tao_environment.exception (exception);
	}
}


void DynamicConfigurator_i::remove_implementation(const char * impName,
                                                  CORBA::Environment &_tao_environment)
{
	if(Dynamic_Service_Config::instance()->remove (impName)==-1)
	{
		CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
		_tao_environment.exception (exception);
	}
}	




void DynamicConfigurator_i::configure_implementation(const char *impName, 
                                                     const char * message,
                                                     CORBA::Environment &_tao_environment)
{
	if (impName == 0 || message == 0)
    {
      return;
    }
 
	// Now, an ugly hack to recognize ACE Services that are not 2K components: 2K
	// components must have the ":" to separate the category name from the version
	// number. If it doesn't have the ":", we assume it's not a 2K component.
	if (!strchr(impName, ':')) 
    {
		// Not a 2K component.
		CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
		_tao_environment.exception (exception);
		return;
	}
	else 
    {
		// It is a 2K component.
		// Try to find implementationName in the repository
		Simple_Strategy_Factory *ssf = 
			ACE_Dynamic_Service<Simple_Strategy_Factory>::instance (impName);
     
		// IMPORTANT: The previous call should return 0 if implementationName is not
		// of type Simple_Strategy_Factory but the implementation of
		// ACE_Dynamic_Service<>::instance does NOT use RTTI yet. While this is not
		// fixed, this won't be completely safe.
		if (ssf == 0) // Didn't find a valid implementation in the repository.
		{
			CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
			_tao_environment.exception (exception);
			 return;
        }
 
		// The implementationName was found, sends it the messageString
		char *non_const_mssg=ACE_OS::strdup(message);

		if(ssf->configure (non_const_mssg)==-1)
		{
			CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
			_tao_environment.exception (exception);
		}
	}
}


char *
DynamicConfigurator_i::get_hooked_comp (const char * componentName,
                                        const char * hookName,
                                        CORBA::Environment &_tao_environment)
{
  char *msg = (char *)Dynamic_Service_Config::instance()->get_hooked_comp (hookName, (char *)componentName);
 
  if (msg == 0)
  {
	CORBA::Exception *exception = new DynamicConfigurator::SERVICE_ERROR;
	_tao_environment.exception (exception);
	return 0;
  }

  return msg;
}


// this can be removed. It's no longer supported.
DynamicConfigurator::stringList *
DynamicConfigurator_i::get_usage (const char * request, CORBA::Environment &_tao_environment)
{
	return NULL;
}


// Why isn't this implemented?
void DynamicConfigurator_i::upload_implementation (const char * categoryName, 
                                                   const char * impName, 
                                                   const DynamicConfigurator::implCode & binCode, 
                                                   CORBA::Environment &_tao_environment)
{
}
 
// Why isn't this implemented? 
void DynamicConfigurator_i::delete_implementation (const char * categoryName, 
                                                   const char * impName, 
                                                   CORBA::Environment &_tao_environment)
{
}
