/******************************************************************************
*                                                                             *
*  File: client_i.cpp                                                         *
*                                                                             *
*  Description: A test client for the Hello World test.                       *
*               Based on the TAO Cubit example.                               *
*                                                                             *
*  Project: 2K                                                                *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************
 Revision Control Information
 $Revision: 1.1.1.3 $
 $Log: client_i.cpp,v $
 Revision 1.1.1.3  2000/09/30 17:31:23  yamane
 Patches for Windows

 Revision 1.1.1.2  1999/10/17 18:41:09  yamane
 Cleanup for native exception and linux support

 Revision 1.1.1.1  1999/10/12 00:43:29  f-kon
 Version compatible with TAO 1.0

 Revision 1.1.1.1  1999/09/22 04:51:47  yamane
 Adding MonitoringService

 Revision 1.1  1999/02/22 22:29:15  f-kon
 *** empty log message ***


******************************************************************************/

#include "ace/Env_Value_T.h"
#include "ace/Read_Buffer.h"
#include "client_i.h"
// #include <sys/wait.h>
#include "tao/Timeprobe.h"
#include <stdlib.h>
#include <math.h>

#include "results.h"

// Constructor.
HelloClient::HelloClient (void)
  : f_handle_ (ACE_INVALID_HANDLE)
{
}

int
HelloClient::read_ior (char *filename)
{
  // Open the file for reading.
  f_handle_ = ACE_OS::open (filename,0);

  if (f_handle_ == ACE_INVALID_HANDLE)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "Unable to open %s for writing: %p\n",
                       filename),
                      -1);
  ACE_Read_Buffer ior_buffer (f_handle_);
  hello_key_ = ior_buffer.read ();

  if (hello_key_ == 0)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "Unable to allocate memory to read ior: %p\n"),
                      -1);
  return 0;
}

// Parses the command line arguments and returns an error status.

int
HelloClient::parse_args (void)
{
  ACE_Get_Opt get_opts (argc_, argv_, "dn:f:k:xs");
  int c;
  int result;

  while ((c = get_opts ()) != -1)
    switch (c)
      {
      case 'd':  // debug flag
        TAO_debug_level++;
        break;
      case 'f': // read the IOR from the file.
        result = read_ior (get_opts.optarg);
        if (result < 0)
          ACE_ERROR_RETURN ((LM_ERROR,
                             "Unable to read ior from %s : %p\n",
                             get_opts.optarg),
                            -1);
        break;
      case 'k': // read the IOR from the command-line.
        hello_key_ =
          ACE_OS::strdup (get_opts.optarg);
        break;
      case '?':
      default:
        ACE_ERROR_RETURN ((LM_ERROR,
                           "usage:  %s"
                           " [-d]"
                           " [-f hello-obj-ref-key-file]"
                           " [-k obj-ref-key]"
                           "\n",
                           argv_ [0]),
                          -1);
      }

  // Indicates successful parsing of command line.
  return 0;
}



void
HelloClient::run ()
{
  int i;
  char *str;
//  struct timeval starttime, endtime;
  double temp1=0.0; 
  double temp2=0.0; 
  double temp3=0.0;
  double va1=0.0;
  double va2=0.0;
  double va3=0.0;
  double x=0.0, y=0.0, z=0.0;

//  double start, end;
  
  hello_->null1 (env_);
  hello_->null2 (env_);
  hello_->echo1 ("my test  string", env_);
  hello_->echo2 ("my other string", env_);
  str = hello_->getHello (env_);

  int N=50;
  results_.call_count (0);
  results_.error_count (0);
  results_.iterations (N);

  for (i = 0; i < N; i++)
    {
      results_.call_count (results_.call_count () + 1);
      results_.start_timer ();
      hello_->null1(env_);
      results_.stop_timer ();
      
      if (env_.exception ()!= 0)
        env_.print_exception ("null1");
    }
  ACE_OS::printf ("\nnull1\n");
  results_.print_stats ();
  
  results_.call_count (0);
  for (i = 0; i < N; i++)
    {
      results_.call_count (results_.call_count () + 1);
      results_.start_timer ();
      hello_->null2(env_);
      results_.stop_timer ();

      if (env_.exception ()!= 0)
        env_.print_exception ("null2");
    }
  ACE_OS::printf ("\nnull2\n");
  results_.print_stats ();

  results_.call_count (0);
  for (i = 0; i < N; i++)
    {
      results_.call_count (results_.call_count () + 1);
      results_.start_timer ();
      hello_->echo1("my test  string",env_);
      results_.stop_timer ();

   if (env_.exception ()!= 0)
        env_.print_exception ("echo1");
    }
  ACE_OS::printf ("\necho1\n");
  results_.print_stats ();
  
  results_.call_count (0);
  for (i = 0; i < N; i++)
    {
      results_.call_count (results_.call_count () + 1);
      results_.start_timer ();
      hello_->echo2("my other string", env_);
      results_.stop_timer ();

      if (env_.exception ()!= 0)
        env_.print_exception ("echo2");
    }
  ACE_OS::printf ("\necho2\n");
  results_.print_stats ();
  
  results_.call_count (0);
  for (i = 0; i < N; i++)
    {
      results_.call_count (results_.call_count () + 1);
      results_.start_timer ();
      str = hello_->getHello(env_);
      results_.stop_timer ();

      if (env_.exception ()!= 0)
        env_.print_exception ("echo2");
    }
  ACE_OS::printf ("\ngetHello\n");
  results_.print_stats ();
  
//   for (i = 0; i < N; i++)
//     {
//       gettimeofday(&starttime,NULL);
//       hello_->null1(env_);
//       gettimeofday(&endtime,NULL);
//       x=(double)((double)endtime.tv_sec*1000.0+(double)endtime.tv_usec/1000.0) - 
// 	((double)starttime.tv_sec*1000.0+(double)starttime.tv_usec/1000.0);
//       temp1= temp1 + ((double)x)/(double)N;
//     }
//   ACE_OS::printf ("my average = %f\n\n", temp1);

//   temp1= 0.0;  
//   va1 = 0.0;

//  for (i = 0; i < N; i++)
//     {
//       gettimeofday(&starttime,NULL);
//       hello_->null1(env_);
//       gettimeofday(&endtime,NULL);
//       x=(double)((double)endtime.tv_sec*1000.0+(double)endtime.tv_usec/1000.0) - 
// 	((double)starttime.tv_sec*1000.0+(double)starttime.tv_usec/1000.0);
//       temp1= temp1 + ((double)x)/(double)N;
//       //      va1= va1 + ((double)x*x)/(double)N;
//     }

//   ACE_OS::printf ("my average = %f\n\n", temp1);
 

//   temp1= 0.0;  
//   va1 = 0.0;
//   ACE_OS::sleep(1);    


//   for (i = 0; i < N; i++)
//     {
//       gettimeofday(&starttime,NULL);
//       hello_->null1(env_);
//       gettimeofday(&endtime,NULL);
//       x=(double)((double)endtime.tv_sec*1000.0+(double)endtime.tv_usec/1000.0) - 
// 	((double)starttime.tv_sec*1000.0+(double)starttime.tv_usec/1000.0);
//       temp1= temp1 + ((double)x)/(double)N;
//       //      va1= va1 + ((double)x*x)/N;
//     }

//   // end of part added by Fabio.  
//   ACE_OS::sleep(1);    


//   for (i = 0; i < N; i++)
//     { 
//       gettimeofday(&starttime,NULL);
//       str = hello_->getHello (env_);
//       gettimeofday(&endtime,NULL);
//       y=(endtime.tv_sec*1000.0+endtime.tv_usec/1000.0) - 
// 	(starttime.tv_sec*1000.0+starttime.tv_usec/1000.0);
//       temp2= temp2 + ((double)y)/N;
//       //      va2= va2 + ((double)y*y)/N;
//     }

//   ACE_OS::sleep(1);  
//   for (i = 0; i < N; i++)
//     {
//       gettimeofday(&starttime,NULL);
//       hello_->echo2("hello.....", env_);
//       gettimeofday(&endtime,NULL);
//       z=(endtime.tv_sec*1000.0+endtime.tv_usec/1000.0) - 
// 	(starttime.tv_sec*1000.0+starttime.tv_usec/1000.0);
//       temp3= temp3 + ((double)z)/N;
//       //      va3= va3 + ((double)z*z)/N;
//     }    
        
 
//   ACE_OS::printf("******Performance Measurements*********\n\n");
//   ACE_OS::printf(" Methods       Average Delay  \n\n");
//   ACE_OS::printf(" null1()       %.3f           \n", temp1 /*, va1-temp1*temp1*/);
//   ACE_OS::printf(" getHello()    %.3f           \n", temp2 /*, va2-temp2*temp2*/);
//   ACE_OS::printf(" echo1()       %.3f           \n", temp3 /*, va3-temp3*temp3*/);
//   ACE_OS::printf("\n\n");
  
}// run ()



HelloClient::~HelloClient (void)
{
  if (f_handle_ != ACE_INVALID_HANDLE)
    ACE_OS::close (f_handle_);

  ACE_OS::printf("Destructed Client Object.\n");
}


int
HelloClient::init (int argc, char **argv)
{
  argc_ = argc;
  argv_ = argv;

  TAO_TRY
    {
      // Retrieve the ORB.
      orb_ = CORBA::ORB_init (argc_,
                              argv_,
                              "internet",
                              TAO_TRY_ENV);
      TAO_CHECK_ENV;

      // Parse command line and verify parameters.
      if (parse_args () == -1)
        return -1;
      
      CORBA::Object_var hello_object =
        orb_->string_to_object (hello_key_,
                                TAO_TRY_ENV);
      TAO_CHECK_ENV;

      hello_ =
        hello::_narrow (hello_object.in(), TAO_TRY_ENV);
      TAO_CHECK_ENV;
      
      if (CORBA::is_nil (hello_.in ()))
        ACE_ERROR_RETURN ((LM_ERROR,
                           "invalid hello key <%s>\n",
                           hello_key_),
                          -1);

    }
  TAO_CATCHANY
    {
      TAO_TRY_ENV.print_exception ("HelloClient::init");
      return -1;
    }
  TAO_ENDTRY;

  return 0;
}

void
HelloClient::fini ()
{
  // Shutdown local client ORB.
  orb_->shutdown ();
}

// i guess the following is not required.

// #if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)
// template class ACE_Env_Value<unsigned long>;
// template class ACE_Env_Value<unsigned short>;
// #elif defined (ACE_HAS_TEMPLATE_INSTANTIATION_PRAGMA)
// #pragma instantiate ACE_Env_Value<unsigned long>
// #pragma instantiate ACE_Env_Value<unsigned short>
// #endif /* ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION */
