//**************************************************************************
// AUTHOR: 
//      Jina Mao      
// 
// DATE:
//      May 3, 1999
//
// FILE:
//      Date_EntryList.cpp
//
// DESCRIPTION:
//      This class implements a list of entries which contain information about
//      method calls on a particular object.
// 
//***************************************************************************


#include "Data_EntryList.h"

// Constructor
Data_EntryList::Data_EntryList()
{
  this->entries = new Data_Entry[120];
  this->numofEntry = 0;
  this->sizeoflist = 120;
}
// Constructor
Data_EntryList::Data_EntryList(int size)
{
  this->entries = new Data_Entry[size];
  this->numofEntry = 0;
  this->sizeoflist = size;
}
// Constructor
Data_EntryList::Data_EntryList (char *name)
{
  //  ACE_OS::printf("in Data_EntryList constructor 1\n");
  this->ob_name = name;
  entries = new Data_Entry[20];
  this->numofEntry = 0;
  this->sizeoflist = 20;  
  //ACE_OS::printf("numofEntry is %d, sizeoflist is %d\n", numofEntry, sizeoflist);
}

// Constructor
Data_EntryList::Data_EntryList (char *name, int size)
{  
  //ACE_OS::printf("in Data_EntryList constructor 2\n");
  this->ob_name = name;
  this->entries = new Data_Entry[size];
  this->numofEntry = 0;
  this->sizeoflist = size;
}


Data_EntryList::~Data_EntryList()
{
  //ACE_OS::printf("Data_EntryList::~Data_EntryList()\n");
  //for (int i=0; i<sizeoflist; i++)
  //  delete entries[i];
  delete [] entries;
  delete ob_name;
}

// clean the list after the entries have been dumped to a file.
// It involves resetting the object name.
void Data_EntryList::reset(char* name)
{
  //ACE_OS::printf("In Data_EntryList::reset().\n");
  CORBA::string_free(this->ob_name);
  this->ob_name = ACE_OS::strdup(name);
  this->numofEntry = 0;
}

// Insert an entry into the list.
int Data_EntryList::insertEntry(Data_Entry* d)
{
  //ACE_OS::printf("In Data_EntryList::insertEntry().\n");
  if (numofEntry < sizeoflist)
    {
      //ACE_OS::printf("before inserting... numofEntry is %d\n", numofEntry);
      entries[numofEntry].copy(d);
      //ACE_OS::printf("after inserting...\n");
      this->numofEntry += 1;
      //ACE_OS::printf("after incrementint... numofEntry is %d\n", numofEntry);
      return 1;
    }
  else
    {
      ACE_OS::printf("Shouldn't happen\n");
      return 0;
    }
}

// Dump all the entries in the list to a file fd.
void Data_EntryList::dump(FILE* fd)
{
  //ACE_OS::printf("In Data_EntryList::dump(). numofEntry is %d\n", numofEntry);
  if (fd) {
    ACE_OS::fputs("\n", fd);
    for (int i=0; i<numofEntry; i++)
      {
	//ACE_OS::printf("tostring gives %s\n", entries[i].toString());
	ACE_OS::fputs(entries[i].toString(), fd);
	ACE_OS::fputs("\n", fd);
      }
    this->numofEntry = 0;
  }
  else {
    ACE_OS::printf("file not open\n");
  }
}

// Check if the list is over the water_mark being passed in.
int Data_EntryList::over_water_mark(int watermark)
{
  //ACE_OS::printf("In Data_EntryList::over_water_mark(). %d %d\n", this->numofEntry, watermark);
  if ((numofEntry >= watermark) || (numofEntry >= sizeoflist-1))
    {
      ACE_OS::printf("<STORAGE> Over watermark.\n");
      return 1;
    }
  else
    return 0;
}

// Get the number of entries in this list.
int Data_EntryList::getCount()
{
  return this->numofEntry;
}

// Get the maximum size of the list.
int Data_EntryList::getSize()
{
  return this->sizeoflist;
}

// Get the object name of the list.
char* Data_EntryList::getObjName()
{
  return this->ob_name;
}
