//**************************************************************************
// AUTHOR: 
//      Jina Mao      
// 
// DATE:
//      May 3, 1999
//
// FILE:
//      QueryInfo.cpp
//
// DESCRIPTION:
//      This class implements the QueryInfo interface server skeleton in
//      Storage.idl. It provides operations for the users to query the 
//      information stored in the log files about object execution histories.
// 
//***************************************************************************

#include "QueryInfo_i.h"
#include <limits.h>  

QueryInfo_i::QueryInfo_i ()
{
  filename = "default.log";
  holder1[0]='\0';
}

QueryInfo_i::~QueryInfo_i ()
{
}
  
M_STORAGE::ReturnInfo* 
QueryInfo_i::query_num_calls_1(const char *ob_name, CORBA::ULong start, CORBA::ULong end,
			     CORBA::Environment &_tao_environment)
{
  ACE_UNUSED_ARG(_tao_environment);
  ACE_OS::printf("<QUERY> Called QueryInfo_i::query_num_calls_1()\n");
  int i=0;
  char buf[100];

  CORBA::ULong first, second, temp;

  first = start;

  if (end==0)
    second=ULONG_MAX;
  else
    second=end;
  
  M_STORAGE::ReturnInfo *ret;
  ret = new M_STORAGE::ReturnInfo();
  filename[0]='\0';
  strcpy(filename, "OBJ/");
  strcat(filename, ob_name);
  strcat(filename, ".log");
  fdlog=fopen(filename, "r");
  ACE_OS::printf("Getting the number of calls.\n");
  if (fdlog==NULL)
    {
      ret->num1(0);
      return ret;
    }
  else 
    {
      while (!feof(fdlog))
	{
	  fgets(buf, 100, fdlog);
	  if (strlen(buf) > 22)
	    { 
	      strtok(buf, " ");
	      strtok(NULL, " ");
	      temp = atol(strtok(NULL, " "));
	      if ((temp >= first) && (temp <= second))
		i++;
	    }
	}
      fclose(fdlog);
      ret->num1(i);
      return ret;
    }
}


M_STORAGE::ReturnInfo* 
QueryInfo_i::query_num_calls_2(const char* ob_name, 
			       const char* op_name, 
			       CORBA::ULong start, CORBA::ULong end,
			       CORBA::Environment &_tao_environment)
{
  ACE_UNUSED_ARG(_tao_environment);

  ACE_OS::printf("<QUERY> Called QueryInfo_i::query_num_calls_2()\n");

  CORBA::ULong first, second, temp;

  first = start;

  if (end==0)
    second=ULONG_MAX;
  else
    second=end;

  M_STORAGE::ReturnInfo *ret;
  ret = new M_STORAGE::ReturnInfo();
  filename[0]='\0';
  strcpy(filename, "OBJ/");
  strcat(filename, ob_name);
  strcat(filename, ".log");
  fdlog=fopen(filename, "r");
  if (fdlog==NULL)
    {
      ret->num2(0);
      return ret;
    }
  else
    {
      int i=0;
      char buf[100];
      while (!feof(fdlog))
	{
	  fgets(buf, 100, fdlog);
	  if (strlen(buf) > 22)
	    if (!strcmp(op_name, strtok(buf, " ")))
	      {
		strtok(NULL, " ");
		temp = atol(strtok(NULL, " "));
		if ((temp>=first) && (temp <=second))
		  i++;
	      }
	}
      fclose(fdlog);
      ret->num2(i);
      return ret;
    }
}

M_STORAGE::ReturnInfo* 
QueryInfo_i::query_last_delay(const char* ob_name, 
			    const char* op_name,
			    CORBA::Environment &_tao_environment)
{
  ACE_UNUSED_ARG(_tao_environment);

  ACE_OS::printf("<QUERY>Called QueryInfo_i::query_last_delay()\n");
  M_STORAGE::ReturnInfo *ret;
  ret = new M_STORAGE::ReturnInfo();
  filename[0]='\0';
  strcpy(filename, "OBJ/");
  strcat(filename, ob_name);
  strcat(filename, ".log");
  fdlog=fopen(filename, "r");
  if (fdlog==NULL)
    {
      ret->last_delay(0);
      return ret;
    }
  else
    {
      int d=-1;
      char buf[100];
      while (!feof(fdlog))
	{
	  fgets(buf, 100, fdlog);
	  if (strlen(buf) > 22)
	    {	
	      if (!strcmp(op_name, strtok(buf, " ")))
		{
		  d = atol(strtok(NULL, " "));
		}
	    }
	}
      fclose(fdlog);
      
      ret->last_delay(d);
      return ret;
    }
}

M_STORAGE::ReturnInfo* 
QueryInfo_i::query_last_delay_from(const char* ob_name, 
		      const char* op_name,
		      const char* ip,
		      CORBA::Environment &_tao_environment)
{
  ACE_UNUSED_ARG(_tao_environment);

  ACE_OS::printf("<QUERY> Called QueryInfo_i::query_last_delay_from()\n");
  M_STORAGE::ReturnInfo *ret;
  ret = new M_STORAGE::ReturnInfo();
  filename[0]='\0';
  strcpy(filename, "OBJ/");
  strcat(filename, ob_name);
  strcat(filename, ".log");
  fdlog=fopen(filename, "r");
  if (fdlog==NULL)
    {
      ret->last_delay(0);
      return ret;
    }
  else
    {
      int d=-1;
      char buf[100];
      while (!feof(fdlog))
	{
	  fgets(buf, 100, fdlog);
	  if (strlen(buf) > 22)
	    {
	      if (!strcmp(op_name, strtok(buf, " ")))
		{
		  if (!strcmp(ip, strtok(NULL, " ")))
		    {
		      strtok(NULL, " ");
		      d = atol(strtok(NULL, " "));
		    }
		}
	    }
	}
      fclose(fdlog);
      
      ret->last_delay(d);
      return ret;
    }
}

M_STORAGE::ReturnInfo* 
QueryInfo_i::query_average_delay(const char* ob_name, 
				 const char* op_name,
				 CORBA::ULong start, CORBA::ULong end,
				 CORBA::Environment &_tao_environment)
{
  ACE_UNUSED_ARG(_tao_environment);

  ACE_OS::printf("<QUERY> Called QueryInfo_i::query_average_delay()\n");

  CORBA::ULong first, second, temp;
  
  first = start;
  
  if (end==0)
    second=ULONG_MAX;
  else
    second=end;

  M_STORAGE::ReturnInfo *ret;
  ret = new M_STORAGE::ReturnInfo();
  filename[0]='\0';
  strcpy(filename, "OBJ/");
  strcat(filename, ob_name);
  strcat(filename, ".log");
  fdlog=fopen(filename, "r");
  if (fdlog==NULL)
    {
      ret->average_delay(-1);
      return ret;
    }
  else 
    {
      int sum=0;
      int times=0;
      char buf[100];
      while (!feof(fdlog))
	{
	  fgets(buf, 100, fdlog);
	  if (strlen(buf)>22)
	    {
	      if (!strcmp(op_name, strtok(buf, " ")))
		{
		  strtok(NULL, " ");
		  temp = atol(strtok(NULL, " "));
		  if ((temp>=first)&&(temp<=second))
		    {
		      sum += atol(strtok(NULL, " "));
		      times++;
		    }
		}
	    }
	}
      fclose(fdlog);
      
      if (times==0)
	ret->average_delay(-1);
      else
	ret->average_delay((int)sum/times);
      return ret;
    }
}
 
M_STORAGE::ReturnInfo* 
QueryInfo_i::query_mr_client_machine_1(const char* ob_name,
				     CORBA::Environment &_tao_environment)
{
  ACE_UNUSED_ARG(_tao_environment);

  ACE_OS::printf("<QUERY> Called QueryInfo_i::query_mr_client_machine_1()\n");
  M_STORAGE::ReturnInfo *ret;
  ret = new M_STORAGE::ReturnInfo();
  filename[0]='\0';
  strcpy(filename, "OBJ/");
  strcat(filename, ob_name);
  strcat(filename, ".log");
  fdlog=fopen(filename, "r");
   if (fdlog==NULL)
    {
      ret->client_machine("Object is not being monitored.");
    }
   else
     {
       char buf[100];
       while (!feof(fdlog))
	 {
	   fgets(buf, 100, fdlog);
	   if (strlen(buf)>22)
	     {
	       strtok(buf, " ");
	       holder1[0]='\0';
	       ACE_OS::strcpy(holder1, strtok(NULL, " "));
	     }
	 }
       fclose(fdlog);
       if (strlen(holder1)< 2)
	 ret->client_machine("No client has called this operation.");
       else
	 ret->client_machine(holder1);
     }
   return ret;
}

M_STORAGE::ReturnInfo* 
QueryInfo_i::query_mr_client_machine_2(const char* ob_name, 
				     const char* op_name,
				     CORBA::Environment &_tao_environment)
{
  ACE_UNUSED_ARG(_tao_environment);

  ACE_OS::printf("<QUERY> Called QueryInfo_i::query_mr_client_machine_2()\n");
  M_STORAGE::ReturnInfo *ret;
  ret = new M_STORAGE::ReturnInfo();
  filename[0]='\0';
  strcpy(filename, "OBJ/");
  strcat(filename, ob_name);
  strcat(filename, ".log");
  fdlog=fopen(filename, "r");
  if (fdlog==NULL)
    {
      ret->client_machine("object is not being monitored.");
      return ret;
    }
  else
    {
      char buf[100];
      while (!feof(fdlog))
	{
	  fgets(buf, 100, fdlog);
	  if (strlen(buf)>22)
	    {
	      if (!strcmp(op_name, strtok(buf, " ")))
		{
		  holder1[0]='\0';
		  strcpy(holder1, strtok(NULL, " "));
		}
	    }
	}
      fclose(fdlog);
      if (strlen(holder1)< 2)
	ret->client_machine("No client has called this operation.");
      else
	ret->client_machine(holder1);
      return ret;
    }
}

M_STORAGE::ReturnInfo* 
QueryInfo_i::query_all_client_machines(const char* ob_name, 
					 const char* op_name,
					 CORBA::ULong start, CORBA::ULong end,
					 CORBA::Environment &_tao_environment)
{
  ACE_OS::printf("Called QueryInfo_i::query_all_client_machines()\n");
  char temp[50];
  M_STORAGE::ReturnInfo *ret;
  ret = new M_STORAGE::ReturnInfo();
  M_STORAGE::clients* m;
  m = new M_STORAGE::clients();
  int i;
  int count = 0;
  char machines[20][20];
  for (i=0; i<20; i++) 
    machines[i][0]='\0'; 
  
  filename[0]='\0'; 
  strcpy(filename, "OBJ/"); 
  strcat(filename, ob_name);
  strcat(filename, ".log");
  fdlog=fopen(filename, "r");

  if (fdlog==NULL)
    {
      m->length(1);
      (*m)[0] = "Object is not being monitored";
//
//	Fix for Linux by Tomonori Yamane yamane@uiuc.edu
//      (ret->client_machines)((const M_STORAGE::clients) *m);
//
      ret->client_machines() = *m;
    }
  else
    {
      char buf[100];
      while (!feof(fdlog))
	{
	  fgets(buf, 100, fdlog);
	  if (strlen(buf)>22)
	    {
	      strtok(buf, " ");
	      holder1[0]='\0';
	      ACE_OS::strcpy(holder1, strtok(NULL, " "));
	      for (i=0; i<count; i++)
		{
		  if (!ACE_OS::strcmp(holder1, machines[i]))
		    break;
		} 
	      if ( i==count)
		{ 
		  ACE_OS::strcpy(machines[count], holder1);
		  count++;
		}
	    }
	}
      fclose(fdlog);
     
      m->length(count);
      for (i=0; i<count; i++)
	{
	  temp[0]='\0';
	  ACE_OS::strcpy(temp, machines[i]);
	  (*m)[i]=(const char* )temp;

	}
    }
//
//	Fix for Linux by Tomonori Yamane yamane@uiuc.edu
//      (ret->client_machines)((const M_STORAGE::clients) *m);
//
  ret->client_machines() = *m;
  return ret;
}

M_STORAGE::ReturnInfo* 
QueryInfo_i::query_last_timestamp_1(const char* ob_name,
				  CORBA::Environment &_tao_environment)
{
  ACE_UNUSED_ARG(_tao_environment);

  ACE_OS::printf("<QUERY> Called QueryInfo_i::query_last_timestamp_1()\n");
  M_STORAGE::ReturnInfo *ret;
  ret = new M_STORAGE::ReturnInfo();
  filename[0]='\0';
  strcpy(filename, "OBJ/");
  strcat(filename, ob_name);
  strcat(filename, ".log");
  int stamp = -1;
  fdlog=fopen(filename, "r");
   if (fdlog==NULL)
    {
      ret->time_stamp(-1);
      return ret;
    }
   else
     {
       char buf[100];
       while (!feof(fdlog))
	 {
	   fgets(buf, 100, fdlog);
	   if (strlen(buf)>22)
	     {
	       strtok(buf, " ");
	       strtok(NULL, " ");
	       stamp = atol(strtok(NULL, " "));
	     }
	 }
       fclose(fdlog);
       ret->time_stamp(stamp);
       return ret;
     }

}

M_STORAGE::ReturnInfo* 
QueryInfo_i::query_last_timestamp_2(const char* ob_name, 
				  const char* op_name,
				  CORBA::Environment &_tao_environment)
{
  ACE_UNUSED_ARG(_tao_environment);

  ACE_OS::printf("<QUERY> Called QueryInfo_i::query_last_timestamp_2()\n");
  M_STORAGE::ReturnInfo *ret;
  ret = new M_STORAGE::ReturnInfo();
  filename[0]='\0';
  strcpy(filename, "OBJ/");
  strcat(filename, ob_name);
  strcat(filename, ".log");
  int stamp = -1;
  fdlog=fopen(filename, "r");
   if (fdlog==NULL)
    {
      ret->time_stamp(-1);
      return ret;
    }
   else
     {
       char buf[100];
       while (!feof(fdlog))
	 {
	   fgets(buf, 100, fdlog);
	   if (strlen(buf)>22)
	     {
	       if (!strcmp(op_name, strtok(buf, " ")))
		 {
		   strtok(NULL, " ");
		   stamp = atol(strtok(NULL, " "));
		 }
	     }
	 }
       fclose(fdlog);
       ret->time_stamp(stamp);
       return ret;
     }
}

M_STORAGE::ReturnInfo* 
QueryInfo_i::query_average_delay_from_1(const char* ob_name, 
					const char* ip,
					CORBA::ULong start, CORBA::ULong end,
					CORBA::Environment &_tao_environment)
{
  ACE_UNUSED_ARG(_tao_environment);

  ACE_OS::printf("<QUERY> Called QueryInfo_i::query_average_delay_from_1()\n");

  CORBA::ULong first, second, temp;

  first = start;
  
  if (end==0)
    second=ULONG_MAX;
  else
    second=end;


  M_STORAGE::ReturnInfo *ret;
  ret = new M_STORAGE::ReturnInfo();
  filename[0]='\0';
  strcpy(filename, "OBJ/");
  strcat(filename, ob_name);
  strcat(filename, ".log");
  fdlog=fopen(filename, "r");
  if (fdlog==NULL)
    {
      ret->average_delay(-1);
      return ret;
    }
  else
    {
      int sum=0;
      int times=0;
      char buf[100];
      while (!feof(fdlog))
	{
	  fgets(buf, 100, fdlog);
	  if (strlen(buf)>22)
	    {
	      strtok(buf, " ");
	      if (!strcmp(ip, strtok(NULL, " ")))
		{
		  temp=atol(strtok(NULL, " "));
		  if ((temp>=first)&&(second<=second))
		    {
		      sum += atol(strtok(NULL, " "));
		      times++;
		    }
		}
	    }
	}
      fclose(fdlog);
      
      if (times==0)
	ret->average_delay(-1);
      else
	ret->average_delay((int)sum/times);
      return ret;
    }
}
 
M_STORAGE::ReturnInfo* 
QueryInfo_i::query_average_delay_from_2(const char* ob_name,
					const char* op_name, 
					const char* ip,
					CORBA::ULong start, 
					CORBA::ULong end,
					CORBA::Environment &_tao_environment)
{
  ACE_UNUSED_ARG(_tao_environment);

  ACE_OS::printf("<QUERY> Called QueryInfo_i::query_average_delay_from_1()\n");
  
  CORBA::ULong first, second, temp;

  first = start;

  if (end==0)
    second=ULONG_MAX;
  else
    second=end;

  M_STORAGE::ReturnInfo *ret;
  ret = new M_STORAGE::ReturnInfo();
  filename[0]='\0';
  strcpy(filename, "OBJ/");
  strcat(filename, ob_name);
  strcat(filename, ".log");
  fdlog=fopen(filename, "r");
  if (fdlog==NULL)
    {
      ret->average_delay(-1);
      return ret;
    }
  else
    {
      int sum=0;
      int times=0;
      char buf[100];
      while (!feof(fdlog))
	{
	  fgets(buf, 100, fdlog);
	  if (strlen(buf)>22)
	    {
	      if (!strcmp(op_name, strtok(buf, " ")))
		{
		  if (!strcmp(ip, strtok(NULL, " ")))
		    {
		      temp = atol(strtok(NULL, " "));
		      if ((temp >= first)&&(temp<=second))
			{
			  sum += atol(strtok(NULL, " "));
			  times++;
			}
		    }
		}
	    }
	}
      fclose(fdlog);
      
      if (times==0)
	ret->average_delay(-1);
      else
	ret->average_delay((int)sum/times);
      return ret;
    }
}

