//**************************************************************************
// AUTHOR: 
//      Jina Mao      
// 
// DATE:
//      May 3, 1999
//
// FILE:
//      QueryInfo.h
//
// DESCRIPTION:
//      This class implements the QueryInfo interface server skeleton in
//      Storage.idl. It provides operations for the users to query the 
//      information stored in the log files about object execution histories.
// 
//***************************************************************************

#ifndef QUERY_I_H
#define QUERY_I_H

#include "StorageS.h"

class QueryInfo_i : public POA_M_STORAGE::QueryInfo
{
public:
  QueryInfo_i ();
  ~QueryInfo_i ();
  // Return the number of calls for object ob_name between time start and end.
  M_STORAGE::ReturnInfo* query_num_calls_1(const char *ob_name, 
					   CORBA::ULong start, 
					   CORBA::ULong end,
					   CORBA::Environment &_tao_environment);

  // Return the number of calls for object ob_name operation op_name between time start and end.
  M_STORAGE::ReturnInfo* query_num_calls_2(const char* ob_name, 
					   const char* op_name,
					   CORBA::ULong start, 
					   CORBA::ULong end,
					   CORBA::Environment &_tao_environment);
  
  // Return the last delay of the call to object ob_name operation op_name.
  M_STORAGE::ReturnInfo* query_last_delay(const char* ob_name, 
					  const char* op_name,
					  CORBA::Environment &_tao_environment);
  
  // Return the last delay of the call to object ob_name operation op_name from client host ip.
  M_STORAGE::ReturnInfo* query_last_delay_from(const char* ob_name, 
					      const char* op_name,
					      const char* ip,
					      CORBA::Environment &_tao_environment);
  
  // Return the average delay of the calls between time start to end
  // on object ob_name operation op_name.
  M_STORAGE::ReturnInfo* query_average_delay(const char* ob_name, 
					     const char* op_name,
					     CORBA::ULong start, 
					     CORBA::ULong end,
					     CORBA::Environment &_tao_environment);
  
  // Return the client machine ip that called on object ob_name most recently.
  M_STORAGE::ReturnInfo* query_mr_client_machine_1(const char* ob_name,
						  CORBA::Environment &_tao_environment);
  
  // Return the client machine ip that called on object ob_name operation op_name most recently.
  M_STORAGE::ReturnInfo* query_mr_client_machine_2(const char* ob_name, 
						  const char* op_name,
						  CORBA::Environment &_tao_environment);
  
  // Return all the client machines that have called object ob_name operation op_name 
  // between time start and end.
  M_STORAGE::ReturnInfo* query_all_client_machines(const char* ob_name, 
						   const char* op_name,
						   CORBA::ULong start, 
						   CORBA::ULong end,
						   CORBA::Environment &_tao_environment);

  // Return the last timestamp of the call to object ob_name.
  M_STORAGE::ReturnInfo* query_last_timestamp_1(const char* ob_name,
					       CORBA::Environment &_tao_environment);
  
   // Return the last timestamp of the call to object ob_name operation op_name.
  M_STORAGE::ReturnInfo* query_last_timestamp_2(const char* ob_name, 
					       const char* op_name,
					       CORBA::Environment &_tao_environment);
  
  // Return the average delay of calls on object ob_name from client machine ip between
  // time start to end.
  M_STORAGE::ReturnInfo* query_average_delay_from_1(const char* ob_name, 
						    const char* ip,
						    CORBA::ULong start, 
						    CORBA::ULong end,
						    CORBA::Environment &_tao_environment);
  
  // Return the average delay of calls on object ob_name operation op_name  
  // from client machine ip between time start to end.
  M_STORAGE::ReturnInfo* query_average_delay_from_2(const char* ob_name,
						    const char* op_name, 
						    const char* ip,
						    CORBA::ULong start, 
						    CORBA::ULong end,
						   CORBA::Environment &_tao_environment);

private:

  char* filename;
  FILE* fdlog;
  char holder1[100];
};

#endif

