//**************************************************************************
// AUTHOR: 
//      Jina Mao      
// 
// DATE:
//      May 3, 1999
//
// FILE:
//      Storage_i.cpp
//
// DESCRIPTION:
//      This class implements the Storage interface server skeleton in
//      Storage.idl. It receives entries from the interceptor and
//      delegate the task of managing to the data manager.
// 
//***************************************************************************

#include "Storage_i.h"

Storage_i::Storage_i()
{
  manager = new Data_Manager(10);
  watermark = 50;
}

Storage_i::~Storage_i()
{
  if (manager)
    ACE_OS::free(manager);
}  
  
void 
Storage_i::add_single_entry(const M_STORAGE::entry &_e, 
			    CORBA::Environment &_tao_environment)
{ 
  char objname[50]; 
  char opname[50];
  char client[50];
  CORBA::ULong timestamp;
  CORBA::ULong delay;
  Data_Entry* data;
 
  objname[0]='\0';
  opname[0]='\0';
  client[0]='\0';
  ACE_OS::printf("<STORAGE>Called Storage::add_single_entry()\n");
  M_STORAGE::entry *newe = new M_STORAGE::entry(_e);
  
  ACE_OS::strcpy(objname, (const char* )newe->ob_name._retn());
  ACE_OS::strcpy(opname, (const char* )newe->op_name._retn());
  ACE_OS::strcpy(client, (const char* )newe->client_machine._retn());

  timestamp = newe->timestamp;
  delay = newe->delay;
  data = new Data_Entry(opname, timestamp, delay, client);
  
  manager->insertNewEntry(objname, data);
  check_water_mark(objname, _tao_environment);
}

void 
Storage_i::add_multiple_entries(const M_STORAGE::entrySeq &_es, 
				CORBA::Environment &_tao_environment)
{
  //ACE_OS::printf("<STORAGE> Called Storage::add_multiple_entries 1\n");
 
  char objname[50]; 
  char opname[50];
  char client[50];  
  
  CORBA::ULong timestamp;
  CORBA::ULong delay;
  Data_Entry* data;
  int size;
   
  ACE_OS::printf("<STORAGE> Called Storage::add_multiple_entries().\n");

  objname[0]='\0';
  opname[0]='\0';
  client[0]='\0';
  
  size=_es.length();

  for (int i=0; i<size; i++)
    {
      //ACE_OS::printf("adding entry %d...\n", i);
      M_STORAGE::entry *newe = new M_STORAGE::entry(_es[i]);
      ACE_OS::strcpy(objname, (const char* )newe->ob_name._retn());
      ACE_OS::strcpy(opname, (const char* )newe->op_name._retn());
      ACE_OS::strcpy(client, (const char* )newe->client_machine._retn());
      timestamp = newe->timestamp;
      delay = newe->delay;
      
      data = new Data_Entry(opname, timestamp, delay, client);
      manager->insertNewEntry(objname, data);
      check_water_mark(objname,_tao_environment);
    }
}

void Storage_i::set_water_mark(CORBA::UShort m, 
			       CORBA::Environment &_tao_environment)
{
  watermark = m;
}

void Storage_i::check_water_mark(char* objname, CORBA::Environment &_tao_environment)
{
  if (manager->getObjCount() >= manager->getSize())
    manager->dumpToFile();
  if (manager->over_water_mark(objname, watermark)>0)
    manager->dumpToFile();
}
      
