//**************************************************************************
// AUTHOR: 
//      Jina Mao      
// 
// DATE:
//      May 3, 1999
//
// FILE:
//      client_i.cpp
//
// DESCRIPTION:
//      This class gives an user interface for data query. It also shows
//      how to call the operations provided by the storage server QueryInfo
//      interface.
// 
//***************************************************************************

#include "ace/Env_Value_T.h"
#include "ace/Read_Buffer.h"
#include "client_i.h"
#include "orbsvcs/orbsvcs/CosNamingC.h"
#include "tao/Timeprobe.h"

// Constructor.
StorageClient::StorageClient (void)
  : f_handle_ (ACE_INVALID_HANDLE)
{
}

int
StorageClient::read_ior (char *filename)
{
  // Open the file for reading.
  f_handle_ = ACE_OS::open (filename,0);

  if (f_handle_ == ACE_INVALID_HANDLE)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "Unable to open %s for writing: %p\n",
                       filename),
                      -1);
  ACE_Read_Buffer ior_buffer (f_handle_);
  Storage_key_  = ior_buffer.read ();

  if (Storage_key_ == 0)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "Unable to allocate memory to read ior: %p\n"),
                      -1);
  return 0;
}

// Parses the command line arguments and returns an error status.

int
StorageClient::parse_args (void)
{
  ACE_Get_Opt get_opts (argc_, argv_, "dn:f:k:xs");
  int c;
  int result;

  while ((c = get_opts ()) != -1)
    switch (c)
      {
      case 'd':  // debug flag
        TAO_debug_level++;
        break;
      case 'f': // read the IOR from the file.
        result = read_ior (get_opts.optarg);
        if (result < 0)
          ACE_ERROR_RETURN ((LM_ERROR,
                             "Unable to read ior from %s : %p\n",
                             get_opts.optarg),
                            -1);
        break;
      case 'k': // read the IOR from the command-line.
        Storage_key_ =
          ACE_OS::strdup (get_opts.optarg);
        break;
      case '?':
      default:
        ACE_ERROR_RETURN ((LM_ERROR,
                           "usage:  %s"
                           " [-d]"
                           " [-f Storage-obj-ref-key-file]"
                           " [-k obj-ref-key]"
                           "\n",
                           argv_ [0]),
                          -1);
      }

  // Indicates successful parsing of command line.
  return 0;
}


void StorageClient::displayMenu()
{
   // display menu
  ACE_OS::printf("******************************\n");
  ACE_OS::printf("*      INFORMATION QUERY     *\n");
  ACE_OS::printf("******************************\n\n");
  ACE_OS::printf("        get_num_of_calls_1 <object>\n");
  ACE_OS::printf("        get_num_of_calls_2 <object> <operation>\n");
  ACE_OS::printf("        get_last_delay <object> <operation>\n");
  ACE_OS::printf("        get_last_delay_from <object> <operation> <client machine>\n");
  ACE_OS::printf("        get_average_delay <object> <operation>\n");
  ACE_OS::printf("        get_average_delay_from <object> <operation> <client machine>\n");
  ACE_OS::printf("        get_last_timestamp_1 <object>\n");
  ACE_OS::printf("        get_last_timestamp_2 <object> <operation>\n");
  ACE_OS::printf("        get_most_recent_client_1 <object>\n");
  ACE_OS::printf("        get_most_recent_client_2 <object> <operation>\n");
  ACE_OS::printf("        \n");
  ACE_OS::printf("        get_num_of_calls_1_within <object> <start time> <end time>\n");
  ACE_OS::printf("        get_num_of_calls_2_within <object> <operation> <start time> <end time>\n"); 
  ACE_OS::printf("        get_average_delay_within <object> <operation> <start time> <end time>\n");
  ACE_OS::printf("        get_average_delay_from_within <object> <operation> <client machine> <start time> <end time>\n");
  ACE_OS::printf("        get_all_clients_within <object> <operation> <start time> <end time>\n");
  ACE_OS::printf("        \n");
  ACE_OS::printf("        help\n");
  ACE_OS::printf("        quit\n\n");
}

void
StorageClient::run ()
{
  int contin=1;
  char command[100];
  char temp[100];
  char obname[50];
  char oper[50];
  char cli[50];
  char ip[50];
  
  displayMenu();
  while (contin)
    {
      gets(command);
      //ACE_OS::printf("command is %s\n",command);
      if (strlen(command) < 4)
	{
	  //ACE_OS::printf("Invalid command!\n\n\n");
	}
      else {
	if (strcmp(command, "quit")==0)
	  break;
	else if (strcmp(command, "help")==0)
	  displayMenu();
	else {
	  strcpy(temp, strtok(command," "));
	  if (strcmp(temp, "get_num_of_calls_1")==0)
	    {  
	      //CORBA::string_free(obname);
	      obname[0]='\0';
	      //ACE_OS::printf("next token is %s\n", strtok(NULL, " "));
	      ACE_OS::strcpy(obname, strtok(NULL, " "));
	      //ACE_OS::printf("object name is %s\n", obname);
	      ACE_OS::printf("result: %d\n\n\n",
			     (query_info_->query_num_calls_1(obname, 0, 0, env_))->num1());
	    }
	  else  if (strcmp(temp, "get_num_of_calls_1_within")==0)
	    {  
	      //CORBA::string_free(obname);
	      obname[0]='\0';
	      //ACE_OS::printf("next token is %s\n", strtok(NULL, " "));
	      ACE_OS::strcpy(obname, strtok(NULL, " "));
	      //ACE_OS::printf("object name is %s\n", obname);
	      ACE_OS::printf("result: %d\n\n\n",
			     (query_info_->query_num_calls_1(obname, 
							     atol(strtok(NULL, " ")), 
							     atol(strtok(NULL, " ")), 
							     env_))->num1());
	    }

	  else if (strcmp(temp, "get_num_of_calls_2")==0)
	    {
	      obname[0]='\0';
	      oper[0]='\0';
	      ACE_OS::strcpy(obname, strtok(NULL, " "));
	      ACE_OS::strcpy(oper, strtok(NULL, " "));
	      ACE_OS::printf("result: %d\n\n\n",
			     (query_info_->query_num_calls_2(obname, oper, 0, 0, env_))->num2());
	    }
	  else if (strcmp(temp, "get_num_of_calls_2_within")==0)
	    {
	      obname[0]='\0';
	      oper[0]='\0';
	      ACE_OS::strcpy(obname, strtok(NULL, " "));
	      ACE_OS::strcpy(oper, strtok(NULL, " "));
	      ACE_OS::printf("result: %d\n\n\n",
			     (query_info_->query_num_calls_2(obname, oper, 
							     atol(strtok(NULL, " ")), 
							     atol(strtok(NULL, " ")), 
							     env_))->num2());
	    }
	  
	  else if (strcmp(temp, "get_last_delay")==0)
	    {
	      obname[0]='\0';
	      oper[0]='\0';
	      ACE_OS::strcpy(obname, strtok(NULL, " "));
	      ACE_OS::strcpy(oper, strtok(NULL, " "));
	      ACE_OS::printf("result: %d\n\n\n",
			     (query_info_->query_last_delay(obname, oper, env_))->last_delay());
	    }
	  else if (strcmp(temp, "get_last_delay_from")==0)
	    {
	      obname[0]='\0';
	      oper[0]='\0';
	      cli[0]='\0';
	      ip[0]='\0';
	      ACE_OS::strcpy(obname, strtok(NULL, " "));
	      ACE_OS::strcpy(oper, strtok(NULL, " "));
	      ACE_OS::strcpy(cli, strtok(NULL, " "));
	      
	      ACE_OS::printf("result: %d\n\n\n",
		    (query_info_->query_last_delay_from(obname, oper, cli, env_))->last_delay());
	    }
	  else if (strcmp(temp, "get_average_delay")==0)
	    {
	      obname[0]='\0';
	      oper[0]='\0';
	      ACE_OS::strcpy(obname, strtok(NULL, " "));
	      ACE_OS::strcpy(oper, strtok(NULL, " "));
	      ACE_OS::printf("result: %d\n\n\n",
	       	     (query_info_->query_average_delay(obname, oper, 0, 0, env_))->average_delay());
	    }
	  else if (strcmp(temp, "get_average_delay_within")==0)
	    {
	      obname[0]='\0';
	      oper[0]='\0';
	      ACE_OS::strcpy(obname, strtok(NULL, " "));
	      ACE_OS::strcpy(oper, strtok(NULL, " "));
	      ACE_OS::printf("result: %d\n\n\n",
			     (query_info_->query_average_delay(obname, oper, 
							       atol(strtok(NULL, " ")), 
							       atol(strtok(NULL, " ")), 
							       env_))->average_delay());
	    }
	  
	  else if (strcmp(temp, "get_average_delay_from")==0)
	    {
	      obname[0]='\0';
	      oper[0]='\0';
	      cli[0]='\0';
	      ACE_OS::strcpy(obname, strtok(NULL, " "));
	      ACE_OS::strcpy(oper, strtok(NULL, " "));
	      ACE_OS::strcpy(cli, strtok(NULL, " "));
	      ACE_OS::printf("result: %d\n\n\n",
         	    (query_info_->query_average_delay_from_2(obname, oper, 
							     cli, 0, 0, env_))->average_delay());
	    }
	   else if (strcmp(temp, "get_average_delay_from_within")==0)
	    {
	      obname[0]='\0';
	      oper[0]='\0';
	      cli[0]='\0';
	      ACE_OS::strcpy(obname, strtok(NULL, " "));
	      ACE_OS::strcpy(oper, strtok(NULL, " "));
	      ACE_OS::strcpy(cli, strtok(NULL, " "));
	      ACE_OS::printf("result: %d\n\n\n",
         	 (query_info_->query_average_delay_from_2(obname, oper, cli, 
							  atol(strtok(NULL, " ")), 
							  atol(strtok(NULL, " ")),
							  env_))->average_delay());
	    }

	  else if (strcmp(temp, "get_last_timestamp_1")==0)
	    {
	      obname[0]='\0';
	      ACE_OS::printf("Getting last timestamp.\n");
	      ACE_OS::strcpy(obname, strtok(NULL, " "));
	      ACE_OS::printf("result: %d\n\n\n",
			     (query_info_->query_last_timestamp_1(obname, env_))->time_stamp());
	    }
	  else if (strcmp(temp, "get_last_timestamp_2")==0)
	    {
	      obname[0]='\0';
	      oper[0]='\0';
	      ACE_OS::strcpy(obname, strtok(NULL, " "));
	      ACE_OS::strcpy(oper, strtok(NULL, " "));
	      ACE_OS::printf("result: %d\n\n\n",
			     (query_info_->query_last_timestamp_2(obname, oper, env_))->time_stamp());
	    }
	  else if (strcmp(temp, "get_most_recent_client_1")==0)
	    {
	       obname[0]='\0';
	       ACE_OS::strcpy(obname, strtok(NULL, " "));
	       ACE_OS::printf("result: %s\n\n\n",
		      (query_info_->query_mr_client_machine_1(obname, env_))->client_machine());
	    }
	   else if (strcmp(temp, "get_most_recent_client_2")==0)
	    {
	       obname[0]='\0';
	       oper[0]='\0';
	       //ACE_OS::printf("Getting most recent client machine.\n");
	       ACE_OS::strcpy(obname, strtok(NULL, " "));
	       ACE_OS::strcpy(oper, strtok(NULL, " "));
	       ACE_OS::printf("result: %s\n\n\n",
       		  (query_info_->query_mr_client_machine_2(obname, oper, env_))->client_machine());
	    }
	  else if (strcmp(temp, "get_all_clients_within")==0)
	    {
	      M_STORAGE::clients* t;
	      
	      obname[0]='\0';
	      oper[0]='\0';
	      //ACE_OS::printf("Getting most recent client machine.\n");
	      ACE_OS::strcpy(obname, strtok(NULL, " "));
	      ACE_OS::strcpy(oper, strtok(NULL, " "));
	      t = new M_STORAGE::clients((query_info_->
					     query_all_client_machines(obname, 
								       oper, 
								       atol(strtok(NULL, " ")),
								       atol(strtok(NULL, " ")), 
								       env_))->client_machines());
	      for (unsigned int i=0; i<t->length(); i++)
		ACE_OS::printf("result: %s\n", (*t)[i]._retn());
	      ACE_OS::printf("\n\n");
	    }
	  else
	    {
	      ACE_OS::printf("\n");
	    }
	}
      }
    }

}



StorageClient::~StorageClient (void)
{
  if (f_handle_ != ACE_INVALID_HANDLE)
    ACE_OS::close (f_handle_);

  ACE_OS::printf("Destructed Client Object.\n");
}


int
StorageClient::init (int argc, char **argv)
{
  argc_ = argc;
  argv_ = argv;
  int naming_result;

  TAO_TRY
    {
      // Retrieve the ORB.
      orb_ = CORBA::ORB_init (argc_,
                              argv_,
                              "internet",
                              TAO_TRY_ENV);
      TAO_CHECK_ENV;

      // Parse command line and verify parameters.
      if (parse_args () == -1)
        return -1;

      naming_result = this->init_naming_service ();
      if (naming_result < 0)
	return naming_result;

    }
  TAO_CATCHANY
    {
      TAO_TRY_ENV.print_exception ("StorageClient::init");
      return -1;
    }
  TAO_ENDTRY;

  return 0;
}

void
StorageClient::fini ()
{
  // Shutdown local client ORB.
  orb_->shutdown ();
}


int
StorageClient::init_naming_service (void)
{
  TAO_TRY
    {
      // Contact the name service
      CORBA::Object_var naming_obj =
        this->orb_->resolve_initial_references ("NameService");

      if (CORBA::is_nil (naming_obj.in ()))
        ACE_ERROR_RETURN ((LM_ERROR,
                           " (%P|%t) Unable to resolve the Name Service.\n"),
                          -1);

      CosNaming::NamingContext_var naming_context =
        CosNaming::NamingContext::_narrow (naming_obj.in (),
                                           TAO_TRY_ENV);

      TAO_CHECK_ENV;

      ACE_OS::printf("resolve name service done!");

      // Get the address of the storage server
      CosNaming::Name storage_name(2);
      storage_name.length (2);
      storage_name[0].id = CORBA::string_dup ("Storage");
      storage_name[1].id = CORBA::string_dup ("StorageServer");

      CORBA::Object_var factory_obj =
        naming_context->resolve (storage_name,TAO_TRY_ENV);
      TAO_CHECK_ENV;


      this->storage_ = M_STORAGE::Storage::_narrow (factory_obj.in (),TAO_TRY_ENV);
      TAO_CHECK_ENV;

      if (CORBA::is_nil (this->storage_.in ()))
	ACE_ERROR_RETURN ((LM_ERROR,
		 " could not resolve StorageServer in Naming service <%s>\n"), -1);

      
      // Get the address of the query server.
      CosNaming::Name query_name(2);
      query_name.length (2);
      query_name[0].id = CORBA::string_dup ("QueryInfo");
      query_name[1].id = CORBA::string_dup ("QueryServer");


      CORBA::Object_var factory_query =
        naming_context->resolve (query_name,TAO_TRY_ENV);
      TAO_CHECK_ENV;


      this->query_info_ = M_STORAGE::QueryInfo::_narrow (factory_query.in (),TAO_TRY_ENV);
      TAO_CHECK_ENV;


      if (CORBA::is_nil (this->query_info_.in ()))
	ACE_ERROR_RETURN ((LM_ERROR,
		 " could not resolve QueryServer in Naming service <%s>\n"), -1);

      ACE_OS::printf("get binding done!\n");
      //*******************************************************************
   }
  TAO_CATCHANY
    {
      TAO_TRY_ENV.print_exception ("StorageClient::init_naming_service");
      return -1;
    }
  TAO_ENDTRY;

  return 0;
}

// i guess the following is not required.

// #if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)
// template class ACE_Env_Value<unsigned long>;
// template class ACE_Env_Value<unsigned short>;
// #elif defined (ACE_HAS_TEMPLATE_INSTANTIATION_PRAGMA)
// #pragma instantiate ACE_Env_Value<unsigned long>
// #pragma instantiate ACE_Env_Value<unsigned short>
// #endif /* ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION */
