//**************************************************************************
// AUTHOR: 
//      Jina Mao      
// 
// DATE:
//      May 3, 1999
//
// FILE:
//      server_i.cpp
//
// DESCRIPTION:
//      This class is the server program for the storage server.
// 
//***************************************************************************

#include "server_i.h"

StorageServer::StorageServer (void)
    : 
    ior_output_file_ (0)
{
}

int
StorageServer::parse_args (void)
{
  ACE_Get_Opt get_opts (argc_, argv_, "dn:o:s");
  int c;

  while ((c = get_opts ()) != -1)
    switch (c)
      {
      case 'd':  // debug flag.
        TAO_debug_level++;
        break;
      case 'o': // output the IOR to a file.
        ior_output_file_ = ACE_OS::fopen (get_opts.optarg, "w");
        if (ior_output_file_ == 0)
          ACE_ERROR_RETURN ((LM_ERROR,
                             "Unable to open %s for writing: %p\n",
                             get_opts.optarg), -1);
        break;
      case '?':
      default:
        ACE_ERROR_RETURN ((LM_ERROR,
                           "usage:  %s"
                           " [-d]"
                           " [-o] <ior_output_file>"
                           "\n",
                           argv_ [0]),
                          1);
      }

  // Indicates successful parsing of command line.
  return 0;
}

int
StorageServer::init (int argc,char** argv,CORBA::Environment& env)
{
  int ret;

  // Call the init of <TAO_ORB_Manager> to initialize the ORB and
  // create a child POA under the root POA.
  if (orb_manager_.init_child_poa (argc, argv, "child_poa", env) == -1)
    ACE_ERROR_RETURN ((LM_ERROR,"%p\n","init_child_poa"),-1);

  TAO_CHECK_ENV_RETURN (env,-1);
  ACE_OS::printf("ORB initialized!\n");

  argc_ = argc;
  argv_ = argv;

  ret = parse_args ();
  if (ret != 0)
    return ret;

  // Create a ComponentConfigurator for ourselves and add it to the domain
  // through the DynamicConfigurator interface.

  //storage_impl_ = new Storage_i ();
 
  //**************************added*************************************
  CORBA::String_var str2  =
    orb_manager_.activate_under_child_poa ("QueryServer",
                                           &query_impl_,
                                           env);
  ACE_DEBUG ((LM_DEBUG,
              "QueryInfo IOR is: <%s>\n",
              str2.in ()));
  //********************************************************************

  //ACE_OS::printf("I'm here...\n");
  CORBA::String_var str  =
    orb_manager_.activate_under_child_poa ("StorageServer",
                                           &storage_impl_,
                                           env);
  ACE_DEBUG ((LM_DEBUG,
              "Storage IOR is: <%s>\n",
              str.in ()));

  ACE_OS::printf("Loading servants done!\n");
  //ACE_OS::printf("And here...\n");


  //if (ior_output_file_)
  //  {
  //    ACE_OS::fprintf (ior_output_file_,
  //                     "%s",
  //                     str.in ());
  //    ACE_OS::fclose (ior_output_file_);
  //  }
  
  //if (this->use_naming_service_)
  return this->init_naming_service (env);
 
  //return 0;
}

int
StorageServer::run (CORBA::Environment& env)
{
  if (orb_manager_.run (env) == -1)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "StorageServer::run"),
                      -1);
  
  return 0;
}

StorageServer::~StorageServer (void)
{
}


// init name server
int
StorageServer::init_naming_service (CORBA::Environment& env)
{
  int result;
  CORBA::ORB_var orb;
  PortableServer::POA_var child_poa;

  // Get the ORB
  orb = this->orb_manager_.orb ();
  child_poa = this->orb_manager_.child_poa ();

  // Starts an name server. After we have a global name server, 
  // this is not needed.
  result = this->my_name_server_.init (orb.in (),
                                       child_poa.in ());
  ACE_DEBUG ((LM_DEBUG,
              "Naming Server IOR is: <%s>\n",
              this->my_name_server_.naming_service_ior()));
  if (result < 0)
    return result;
  
  ACE_OS::printf("Naming service initialized\n");

  factory_query = this->query_impl_._this (env);
  TAO_CHECK_ENV_RETURN (env,-1);

  // Bind the address of the query server to the name service.
  CosNaming::Name query_context_name (1);
  query_context_name.length (1);
  query_context_name[0].id = CORBA::string_dup ("QueryInfo"); 



  this->QueryInfo_context_ =
    this->my_name_server_->bind_new_context (query_context_name,
                                             env);
  TAO_CHECK_ENV_RETURN (env,-1);

  CosNaming::Name factory_name_query (1);
  factory_name_query.length (1);
  factory_name_query[0].id = CORBA::string_dup ("QueryServer");
  this->QueryInfo_context_->bind (factory_name_query,factory_query.in (),env);

  TAO_CHECK_ENV_RETURN (env,-1);

  factory = this->storage_impl_._this (env);
  TAO_CHECK_ENV_RETURN (env,-1);

  // Bind the address of the storage server to the name service.
  CosNaming::Name storage_context_name (1);
  storage_context_name.length (1);
  storage_context_name[0].id = CORBA::string_dup ("Storage");
  this->Storage_context_ =
    this->my_name_server_->bind_new_context (storage_context_name,
                                             env);
  TAO_CHECK_ENV_RETURN (env,-1);

  CosNaming::Name factory_name (1);
  factory_name.length (1);
  factory_name[0].id = CORBA::string_dup ("StorageServer");
  this->Storage_context_->bind (factory_name,factory.in (),env);

  TAO_CHECK_ENV_RETURN (env,-1);

  ACE_OS::printf("Binding done!\n");

  return 0;
}
