// ============================================================================
//
// = LIBRARY
//     TAO2k
// 
// = FILENAME
//     Thread_Pool_Strategy_Factory.cpp
//
// = AUTHOR
//     Manuel Roman (mroman@cs.uiuc.edu)
//

#include "tao/corba.h"
#include "tao/ORB_Core.h"
#include "Thread_Pool_Strategy_Factory.h"

const char * const Thread_Pool_Strategy_Factory::infoStr  = "Thread-Pool Strategy";

Thread_Pool_Strategy_Factory::Thread_Pool_Strategy_Factory (void) 
  : thread_flags_ (0)
{
}

Thread_Pool_Strategy_Factory::~Thread_Pool_Strategy_Factory(void)
{
}


int Thread_Pool_Strategy_Factory::init (int argc, char *argv[])
{
  configurator (new SimpleConfigurator ("Thread Pool Strategy", this));
  configurator ()->info (infoStr);

  //Let's assign a value to Pool_Size, Low_Watermark and High_Watermark. If
  //those have not been provided, give a default value to all of them.

	if(argv==0)
	{
		pool_size_=10;
		low_wm_=2;
		high_wm_=5;
	}
	else
	{
		if(argv[0]==0)
			pool_size_=10;
		else
			pool_size_=ACE_OS::atoi(argv[0]);

		if(argv[1]==0)
			low_wm_=2;
		else
			low_wm_=ACE_OS::atoi(argv[1]);

		if(argv[2]==0)
			high_wm_=5;
		else
			high_wm_=ACE_OS::atoi(argv[2]);
	}
	
	return 1;
}


int
Thread_Pool_Strategy_Factory::open (void)
{
  TAO_ORB_Core *orb_core = TAO_ORB_Core_instance ();

  return thread_pool_strategy_.open (orb_core->thr_mgr (), this->thread_flags_,pool_size_,low_wm_,high_wm_);
}



int Thread_Pool_Strategy_Factory::info (char **info_string, size_t length) const
{
  if (*info_string == 0)
    return -1;

  ACE_OS::strncpy(*info_string, infoStr, length);
  return ACE_OS::strlen (*info_string);
}



void *
Thread_Pool_Strategy_Factory::strategy (void)
{
  return &thread_pool_strategy_;
}


int
Thread_Pool_Strategy_Factory::fini(void)
{
	int i;

	//Release all threads and let them kill themselves, before a this strategy is replaced.
	//Since there won't be requests in the list, they will just kill themselves.
	for(i=0;i<ACE_Thread_Pool_Strategy<TAO_Server_Connection_Handler>::available_threads_;i++)
		ACE_Thread_Pool_Strategy<TAO_Server_Connection_Handler>::sem_->release();
	
	ACE_OS::printf("Here finishes the execution of Thread_Pool_Strategy\n");

	return 1;
}



ACE_FACTORY_DEFINE (ACE, Thread_Pool_Strategy_Factory)


#if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)
template class ACE_Concurrency_Strategy<TAO_Server_Connection_Handler>;
#elif defined (ACE_HAS_TEMPLATE_INSTANTIATION_PRAGMA)
#pragma instantiate ACE_Concurrency_Strategy<TAO_Server_Connection_Handler>
#endif /* ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION */


