/******************************************************************************
*                                                                             *
*  File: JVM_Strategy.cpp                                                     *
*                                                                             *
*  Description: A component encapsulating a Java Virtual Machine              *
*                                                                             *
*                                                                             *
*  Project: 2K - dynamicTAO                                                   *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************

   Revision Control Information

   $Revision: 1.1.1.1 $

   $Log: JVM_Strategy.cpp,v $
   Revision 1.1.1.1  1999/10/12 00:43:30  f-kon
   Version compatible with TAO 1.0

   Revision 1.2  1999/05/12 19:09:15  f-kon
   *** empty log message ***

   Revision 1.1  1999/04/23 22:33:58  f-kon
   A new, more flexible, component encapsulating the JVM

******************************************************************************/

#include "JVM_Strategy.h"
#include "jni.h"

int JVM_Strategy::open()
{
    ACE_OS::printf("Opening JVM\n");
    
    JNI_GetDefaultJavaVMInitArgs(&vm_args);
    
    vm_args.classpath = CLASSPATH;
    
    /* Create the Java VM */
    res = JNI_CreateJavaVM(&jvm, &env, &vm_args);
    
    if (res < 0) 
	{
            ACE_OS::printf("Can't create Java VM\n");
            return 0;
	}
    
    //this->interpret("Simple", "main", "([Ljava/lang/String;)V", "");
    //this->interpret("GUI", "main", "([Ljava/lang/String;)V", "");
    //this->interpret("video/pull/JMFPlayer", "main", "([Ljava/lang/String;)V", "");
    return 0;
}

// make it return a string in the future (char *)
unsigned char 
JVM_Strategy::interpret(char *classname, char *method, char *signature, char *argstr)
{   
    ACE_OS::printf("JVM::interpret- - - class:%s\n", classname);
    ACE_OS::printf("JVM::interpret- - - method:%s\n", method);
    ACE_OS::printf("JVM::Env --- %d\n",env);
    
    cls = (*env).FindClass(classname);
    
    if (cls == 0) 
	{
            ACE_OS::printf("Can't find class %s\n", classname);
            return 0;
        }
    
    mid = (*env).GetStaticMethodID(cls, method, signature);
    
    if (mid == 0) 
	{
            ACE_OS::printf("Can't find method %s with signature %s\n", method, signature);
            return 0;
	}
    
    jstr = (*env).NewStringUTF(argstr);
    
    if (jstr == 0) 
	{
            cerr<< "Out of memory\n";
            return 0;
	}
    
    args = (*env).NewObjectArray(1, (*env).FindClass("java/lang/String"), jstr);
    
    if (args == 0) 
	{
            ACE_OS::printf("Out of memory\n");
            return 0;
	}
    
    
    unsigned char r=(*env).CallStaticBooleanMethod(cls, mid, args);
    
    return r;
}


int 
JVM_Strategy::fini (void)
{
    if (jvm)
        (*jvm).DestroyJavaVM();
    
    return 0;
}
