/******************************************************************************
*                                                                             *
*  File: JVM_Strategy_Factory.cpp                                             *
*                                                                             *
*  Description: A Factory for the Java Virtual Machine                        *
*                                                                             *
*                                                                             *
*  Project: 2K - dynamicTAO                                                   *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************

   Revision Control Information

   $Revision: 1.1.1.1 $

   $Log: JVM_Strategy_Factory.cpp,v $
   Revision 1.1.1.1  1999/10/12 00:43:30  f-kon
   Version compatible with TAO 1.0

   Revision 1.2  1999/05/12 19:09:17  f-kon
   *** empty log message ***

   Revision 1.1  1999/04/23 22:33:58  f-kon
   A new, more flexible, component encapsulating the JVM

******************************************************************************/


#include "tao/corba.h"
#include "tao/ORB_Core.h"
#include "JVM_Strategy_Factory.h"

const char * const 
JVM_Strategy_Factory::infoStr 
    = "An implementation of the JVM that can be \r\n used to execute Java bytecode";

JVM_Strategy_Factory::JVM_Strategy_Factory (void) 
{
}

JVM_Strategy_Factory::~JVM_Strategy_Factory(void)
{
}


int JVM_Strategy_Factory::init (int argc, char *argv[])
{	
  configurator (new SimpleConfigurator ("JVM Strategy", this));
  configurator ()->info (infoStr);
  jvm_strategy_.open();
  return 0;
}


int 
JVM_Strategy_Factory::configure (char *message_string) 
{
    if (!ACE_OS::strncmp (message_string, "shutdown", sizeof("shutdown")))
        {
            jvm_strategy_.fini (); // this is not killing it.
            return 0;
        }
    else
        return (int) jvm_strategy_.interpret (message_string, "main", "([Ljava/lang/String;)V", "");
}


int
JVM_Strategy_Factory::open (void)
{
  return 0;
}



int JVM_Strategy_Factory::info (char **info_string, size_t length) const
{
  if (*info_string == 0)
    return -1;

  ACE_OS::strncpy(*info_string, "JVM strategy\r\n", length);
  return ACE_OS::strlen (*info_string);
}



void *
JVM_Strategy_Factory::strategy (void)
{
  return &jvm_strategy_;
}


ACE_FACTORY_DEFINE (ACE, JVM_Strategy_Factory)


