//**************************************************************************
// Author: 
//      Jina Mao      
// 
// Date:
//      May 3, 1999
//
// File:
//      Monitoring_Strategy.cpp
//
// Description:
//      The monitoring strategy inplements the interceptor which
// gets the request and send it to the storage server.
// 
//***************************************************************************
#include "tao/corba.h"
#include "tao/GIOP_Server_Request.h"
#include "Monitoring_Strategy.h"	
#include "dynamicTAO/src/DynamicConfigurator/dynamicConfiguratorC.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ace/Thread_Manager.h"
// #include "dynamicTAO/src/ace/ACE_2K_OS.h"

#if 0
MONITORING_STRATEGY_Export M_STORAGE::Storage_var Monitoring_Strategy::storage_;
MONITORING_STRATEGY_Export M_STORAGE::entrySeq *Monitoring_Strategy::entries;
MONITORING_STRATEGY_Export ACE_Semaphore* Monitoring_Strategy::sem_;
MONITORING_STRATEGY_Export ACE_Mutex Monitoring_Strategy::mutex_;
MONITORING_STRATEGY_Export int Monitoring_Strategy::firsttime;
#else

M_STORAGE::Storage_var Monitoring_Strategy::storage_;
M_STORAGE::entrySeq *Monitoring_Strategy::entries;
ACE_Semaphore* Monitoring_Strategy::sem_;
ACE_Mutex Monitoring_Strategy::mutex_;
int Monitoring_Strategy::firsttime;
#endif

int Monitoring_Strategy::open()
{

  TAO_TRY
    {
      // Get the existing ORB
      orb_ = TAO_ORB_Core_instance()->orb();

      TAO_CHECK_ENV;

      // Contact the name service
      CORBA::Object_var naming_obj2 =
        this->orb_->resolve_initial_references ("NameService");

      if (CORBA::is_nil (naming_obj2.in ()))
      	{
        ACE_OS::printf("(ERROR) Unable to resolve the Name Service\n");
        return -1;
      }

      CosNaming::NamingContext_var naming_context =
      CosNaming::NamingContext::_narrow (naming_obj2.in (),
                                         TAO_TRY_ENV);
      
      ACE_OS::printf("<INTERCEPTOR> Resolve name service done!\n");
      TAO_CHECK_ENV;

      // Ask for the address of the Storage server
      CosNaming::Name storage_name(2);
      storage_name.length (2);
      storage_name[0].id = CORBA::string_dup ("Storage");
      storage_name[1].id = CORBA::string_dup ("StorageServer");

      CORBA::Object_var factory_obj =
        naming_context->resolve (storage_name,TAO_TRY_ENV);
      TAO_CHECK_ENV;

      this->storage_ = M_STORAGE::Storage::_narrow (factory_obj.in (),TAO_TRY_ENV);
      TAO_CHECK_ENV;

      ACE_OS::printf("<INTERCEPTOR> After resolving Storage Server\n");
      
      if (CORBA::is_nil (this->storage_.in ()))
      ACE_ERROR_RETURN ((LM_ERROR,
      	 " could not resolve StorageServer in Naming service <%s>\n"), -1);
       
    }
  TAO_CATCHANY
    {
      TAO_TRY_ENV.print_exception ("Monitoring_Strategy::open");
      return -1;
    }
  TAO_ENDTRY;

  // Initialize data structures and variables
  names = new My_NameSet();
  sem_ = new ACE_Semaphore(1);
  counter=0;
  entries = new M_STORAGE::entrySeq;
  entries->length(30);
  Monitoring_Strategy::firsttime=0;
  Monitoring_Strategy::sem_->acquire();
  Monitoring_Strategy::mutex_.acquire();

  // Create a separate thread to handle the sending part.
  // We use Semaphore to control the thread.
  int result = ACE_Thread_Manager::instance () ->
    spawn(ACE_THR_FUNC (Monitoring_Strategy::entry_point),(void*)&env_);
  if (result == -1)
    ACE_ERROR((LM_ERROR, "Could not spawn the send_entries thread.\n"));
  
  ACE_OS::printf("<INTERCEPTOR> Open done\n");
  
  return 1;
}


void Monitoring_Strategy::analyze(TAO_ORB_Core *orb_core, TAO_GIOP_ServerRequest &request, ACE_SOCK_Stream &peer, 
				 CORBA::Environment &env)
{
//  char message[100];
  //  char obname[100];
  char c[2];
  char temp[100];
//  unsigned int tid;
  ACE_INET_Addr addr;
//  struct timeval starttime, endtime;
  ACE_Time_Value starttime, endtime;
  M_STORAGE::entry e;

  // Get the object name.

  // FIX: The following is broken. We must look at the definition of the TAO object key and
  // use that information to find the right info. By now, this is a hack.
  // We must take a look at Pluggable.cpp: TAO_Connector::make_mprofile ();

  int i, max=request.object_key().length();
  i=max-1;
  while(i > 0 && request.object_key()[i]!= 1 /* '/' (this was the old hack) */)
    {
//       ACE_OS::printf ("key[%d] = <%c>  0x%X\n", i, request.object_key()[i], 
//                       request.object_key()[i]);
        i--;
    }
  
  i++;
  //  i=i+8;  // IMPORTANT: This line is here due to a bug in the request being passed in.        //  int index=0;
  temp[0]='\0';
  c[1]='\0';
      
  for(;i<max;i++)
    {
//       obname[index]=request.object_key()[i];
//       c[0]=obname[index];
      c[0]=request.object_key()[i]; 
      strcat(temp, c);
      //      index++;
    }

  //  obname[index]='\0';

  //ACE_OS::printf("<INTERCEPTOR> Called %s %s \n", temp, request.operation());
 
  // Check if the object name is in the name set.
  findit = names->find(temp);
  if (findit==0)
    {  
      //ACE_OS::printf("<INTERCEPTOR> Object %s not in the name set.\n", temp);
    }
  else
    {
      // Needs to collect information about this method call.
       if ((counter==0) && (Monitoring_Strategy::firsttime==1))
	 {
	   Monitoring_Strategy::mutex_.acquire();
	 }
       Monitoring_Strategy::firsttime=1;
       
       //ACE_OS::printf("<INTERCEPTOR> Object %s is in the name set.\n", temp);

      (*entries)[counter].ob_name=(const char* )temp;
      (*entries)[counter].op_name=(const char* )(request.operation());
      peer.get_remote_addr(addr);
      (*entries)[counter].client_machine=(const char* )(addr.get_host_addr());
#if 0
      gettimeofday(&starttime,NULL);
      (*entries)[counter].timestamp = starttime.tv_sec*1000+starttime.tv_usec/1000;
#endif
      starttime = ACE_OS::gettimeofday();
      (*entries)[counter].timestamp = starttime.sec()*1000+starttime.usec()/1000;
    }

  // Re-dispatch the method call
#if 1	// New code
  orb_core->object_adapter ()->dispatch_servant (request.object_key (),
                                                  request,
                                                  0,
                                                  env);
#else // Old code
  TAO_POA *the_poa = TAO_ORB_Core_instance ()->root_poa ();  
  the_poa->dispatch_servant (request.object_key (),
                             request,
                             0,
                             TAO_ORB_Core_instance ());
#endif

  if (findit!=0)
    {
      // Get the delay on the server side 
#if 0
      gettimeofday(&endtime,NULL);
      (*entries)[counter].delay = (endtime.tv_sec*1000000+endtime.tv_usec) - 
	(starttime.tv_sec*1000000+starttime.tv_usec);
#endif
      endtime = ACE_OS::gettimeofday();
      (*entries)[counter].delay = (endtime.sec()*1000000+endtime.usec()) - 
	(starttime.sec()*1000000+starttime.usec());
          
      counter++;
      if (counter >= 30)
	{
	  //ACE_OS::printf("<INTERCEPTOR> Sending multiple entries.\n");
	  Monitoring_Strategy::mutex_.release();
	  Monitoring_Strategy::sem_->release();
	  counter=0;
	}
    }
}

void Monitoring_Strategy::entry_point(void* arg_env)
{
  //ACE_OS::printf("<INTERCEPTOR> New thread created to send the entries.\n");
  CORBA::Environment* env = (CORBA::Environment *)arg_env;
  while(1)
    {
      Monitoring_Strategy::sem_->acquire();
      Monitoring_Strategy::mutex_.acquire();
      storage_->add_multiple_entries(*(return_entries())); 
      Monitoring_Strategy::mutex_.release();
    }
}

void Monitoring_Strategy::reconfigure(char *message)
{
  char type[20];
  char name[50];

  ACE_OS::printf("config message is %s\n", message);
  strcpy(type, strtok(message, " "));

  // Add an object name to the name set.
  if ((strcmp(type, "ADD_OB")==0) || (strcmp(type, "add_ob")==0))
  {
    strcpy(name, strtok((NULL), " ")); 
  
    if ((names->insert(name)) >= 0)
      ACE_OS::printf("<INTERCEPTOR> Object %s inserted into the name set.\n\n", name);
    else
      ACE_OS::printf("<INTERCEPTOR> Insert object name failed.\n");
  }
  // Delete an object name from the name set.
  else if ((strcmp(type, "DEL_OB")==0) || (strcmp(type, "del_ob")==0))
    {
      strcpy(name, strtok((NULL), " "));
  
      names->remove(name);
      ACE_OS::printf("<INTERCEPTOR> Object %s removed from the name set.\n\n", name);
    } 
  // Remove all names in the name set.
  else if ((strcmp(type, "CLEAN")==0) || (strcmp(type, "clean")==0))
    {
      names->reset();
      ACE_OS::printf("<INTERCEPTOR> All objects removed from the name set.\n\n");
    }
  // Print all names in the name set.
  else if ((strcmp(type, "DUMP")==0) || (strcmp(type, "dump")==0))
    {
      ACE_OS::printf("<INTERCEPTOR> All object names in the set.\n\n");
      names->dump();
    }      
  // Set how often the storage server dumps to files.
  else if ((strcmp(type, "SET_LEVEL")==0) || (strcmp(type, "set_level")==0))
    {
     storage_->set_water_mark(atoi(strtok(NULL, " ")));
     ACE_OS::printf("<INTERCEPTOR> Storage server buffer level set to new value.\n\n");
    }
  else
    {
      ACE_OS::printf("<INTERCEPTOR> Invalid Configuration Message!\n\n");
    }
}

int Monitoring_Strategy::read_ior (char *filename)
{  

  // Open the file for reading.
  f_handle_ = ACE_OS::open (filename,0);

  if (f_handle_ == ACE_INVALID_HANDLE)
    {
      ACE_OS::printf("Unable to open file ior\n");
      return -1;
    } 
  ACE_OS::printf("file opened\n");

    //ACE_ERROR_RETURN ((LM_ERROR,
    //                   "Unable to open %s for writing: %p\n",
    //                   filename),
    //                  -1);  
  ACE_Read_Buffer ior_buffer (f_handle_);
  Storage_key_  = ior_buffer.read ();
  if (Storage_key_ == 0)
    ACE_ERROR_RETURN ((LM_ERROR,
                       "Unable to allocate memory to read ior: %p\n"),
                      -1);
  return 0;
}
