#include "ace/OS.h"
#include "ACE_2K_OS.h"

#include <stdio.h>


DIR_HANDLE ACE_2k_os::opendir(char *path)
{
	DIR_HANDLE dh;
	
#if !defined(WIN32)
	dh=::opendir(path);
#else
	BOOL ret;
	ret=::SetCurrentDirectory(path);
	if( ret )
	{
		dh=(DIR_HANDLE)ACE_OS::malloc(sizeof(HANDLE));
		*dh=(HANDLE)-1;
	}
	else
		dh=(HANDLE*)0;
#endif
	return dh;
}


BOOL ACE_2k_os::readdir(DIR_HANDLE dh,char *fileName)
{
	BOOL retVal;
	DIR_ENTRY dire;

#if !defined(WIN32)
	dire=::readdir(dh);
	if(dire==NULL)
		retVal=FALSE;
	else
	{
		retVal=TRUE;
		strcpy(fileName,dire->d_name);
	}
#else
	if(*dh==(HANDLE)-1) //first time that readdir is called
	{
		*dh=::FindFirstFile("*",&dire);
		strcpy(fileName,dire.cFileName);
		retVal=TRUE;
	}
	else
	{
		if(*dh!=(HANDLE)0) //Check that there was no error while opening the dir.
		{
			retVal=FindNextFile(*dh,&dire);
			strcpy(fileName,dire.cFileName);
		}
	}
#endif
	return retVal;
}


void ACE_2k_os::closedir(DIR_HANDLE dh)
{
#if !defined(WIN32)
	::closedir(dh);
	//ACE_OS::free(dh); needed??
#else
	ACE_OS::free(dh);
#endif
}

#if defined(sun)
#include <sys/systeminfo.h>
#include <sys/sysinfo.h>

#elif defined(linux)
#include <sys/sysinfo.h>

#elif defined(ACE_WIN32)

#endif


char *
ACE_2k_os::os_name()
{
  //
  // Operation system
  //
  static char buf[512];

#if defined(sun)
  ACE_OS::sysinfo(SI_SYSNAME, buf, 511);

#elif defined(linux)
  static struct utsname host_info;
  ACE_OS::uname (&host_info);

  return(host_info.sysname);

#elif defined (ACE_WIN32)
  // OS version information

  OSVERSIONINFO osvi;
  osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

  GetVersionEx (&osvi);

  switch(osvi.dwPlatformId) {
  case VER_PLATFORM_WIN32s:
  case VER_PLATFORM_WIN32_WINDOWS:
          ACE_OS::strcpy(buf,"windows");
          break;
  case VER_PLATFORM_WIN32_NT:
          ACE_OS::strcpy(buf,"windows-nt");
          break;
  default:
          ACE_OS::strcpy(buf,"windows-unknown");
          break;
  }

#else
  ACE_OS::strcpy(buf,"unknown");
#endif
  return(buf);
}

char *
ACE_2k_os::processor_name()
{
  //
  // Processor
  //
  static char buf[256];

#if defined(sun)
  ACE_OS::sysinfo(SI_ARCHITECTURE, buf, 255);
  return(buf);

#elif defined(linux)
  static struct utsname host_info;

  ACE_OS::uname (&host_info);
  return(host_info.machine);

#elif defined (ACE_WIN32)
  // Processor name
  SYSTEM_INFO si;

  GetSystemInfo(&si);

  if (ACE_OS::strcmp(ACE_2k_os::os_name(), "windows-nt")==0) {
        if (si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_INTEL) {
                switch (si.wProcessorLevel) {
                case 3: ACE_OS::strcpy(buf,"i386");
                            break;
                case 4: ACE_OS::strcpy(buf,"i486");
                                break;
                case 5: ACE_OS::strcpy(buf, "i586");
                                break;
                default:        ACE_OS::strcpy(buf,"intel");
                                break;
                }
        } else if (si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_MIPS) {
                ACE_OS::strcpy(buf,"mips");
        } else if (si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_ALPHA) {
                ACE_OS::sprintf(buf, "Alpha%d", si.wProcessorLevel);
        } else if (si.wProcessorArchitecture == PROCESSOR_ARCHITECTURE_PPC) {
                ACE_OS::sprintf(buf, "PPC 6%02d", si.wProcessorLevel);
        }
  } else {
        if (si.dwProcessorType == PROCESSOR_INTEL_386) {
                strcpy(buf,"i386");
        } else if (si.dwProcessorType == PROCESSOR_INTEL_486) {
                strcpy(buf,"i486");
        } else if (si.dwProcessorType == PROCESSOR_INTEL_PENTIUM) {
                strcpy(buf,"i586");
        } else ACE_OS::strcpy(buf,"intel");
  }

#else /* ACE_WIN32 */
   strcpy(buf,"unknown");
#endif
  return (buf);
}

