/******************************************************************************
*                                                                             *
*  File: ACE_Persistent_Repository.h                                          *
*                                                                             *
*  Description: A hierarchical repository of data implemented on top of the   *
*               ACE_OS file system.                                           *
*                                                                             *
*  Project: 2K - dynamically configurable ORB.                                *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************

   Revision Control Information

   $Revision: 1.1.1.3 $

   $Log: ACE_Persistent_Repository.h,v $
   Revision 1.1.1.3  2000/09/30 18:12:19  yamane
   Patches for Windows

   Revision 1.1.1.2  1999/10/17 18:43:38  yamane
   Cleanup for native exception and linux support

   Revision 1.1.1.1  1999/10/12 00:43:30  f-kon
   Version compatible with TAO 1.0

   Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
   This new 2K source tree contains the most up-to-date version of ACE and TAO
   and including the dynamicTAO code as well as some 2K tests and preliminary
   versions of some of its high-level systems.


   Revision 1.6  1999/02/08 21:13:59  mroman
   no message

   Revision 1.5  1999/01/11 23:13:47  f-kon
   Upgrade to DCP v1.0 and improvements to the PersistentRepository.

   Revision 1.4  1998/10/23 17:14:29  f-kon
   Solaris working version with Manuel's Thread-Pool

   Revision 1.3  1998/10/06 18:16:16  mroman
   Version that works fine on NT, Win98 and compiles fine

   Revision 1.2  1998/08/30 14:56:21  f-kon
   First modifications to try to make it work in Windows

******************************************************************************/

#ifndef ACE_PERSISTENT_REPOSITORY_H
#define ACE_PERSISTENT_REPOSITORY_H

#include "Persistent_Repository.h"
#include "ACE_2K_OS.h"


class ACE_SOCK_Stream;

class ACE_2K_Export ACE_Persistent_Repository:public Persistent_Repository
{
 public:
  ACE_Persistent_Repository ();
  ~ACE_Persistent_Repository ();
  
  int init (const char *rootDir);

  int get_contents (char *buf, int maxsize, const char *dir = 0);

  //Overloaded method that returns an array of strings.
  int get_contents (char **buf, int &strings, int maxsize, const char *dir=0);
  
  /*NOT YET IMPLEMENTED************************************/
  int create_dir (const char *path, const char *dirname){ ACE_UNUSED_ARG(path); ACE_UNUSED_ARG(dirname); return 0;};
  int delete_dir (const char *path, const char *dirname){ ACE_UNUSED_ARG(path); ACE_UNUSED_ARG(dirname); return 0;};
  /********************************************************/
  
  int create_entry_from_stream (ACE_SOCK_Stream *input, 
                                const char *path, 
                                const char *entryname, 
                                long len);
  int create_entry_from_buffer (char *input, 
                                const char *path, 
                                const char *entryname, 
                                long len);
  int delete_entry (const char *path, const char *entryname);

  void build_pathname (char *pathname, const char *path, 
                       const char *entryname = 0);

  char *clean_name (char *name);
};

#endif
