/* -*- C++ -*- */
/******************************************************************************
*                                                                             *
*  File: DCP_Broker.h                                                         *
*                                                                             *
*  Description: A Dynamic Config Broker that implements DCP, the              *
*               Distributed Configuration Protocol.                           *
*                                                                             *
*  Project: 2K - dynamically configurable ORB.                                *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************

   Revision Control Information

   $Revision: 1.1.1.2 $

   $Log: DCP_Broker.h,v $
   Revision 1.1.1.2  2000/09/30 18:12:54  yamane
   Patches for Windows

   Revision 1.1.1.1  1999/10/12 00:43:30  f-kon
   Version compatible with TAO 1.0

   Revision 1.2  1999/04/07 20:09:03  f-kon
   added new DCP commands, upgrading to version 1.1

   Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
   This new 2K source tree contains the most up-to-date version of ACE and TAO
   and including the dynamicTAO code as well as some 2K tests and preliminary
   versions of some of its high-level systems.


   Revision 1.8  1999/03/03 16:12:12  f-kon
   Implemented DCP operation configure_implementation <loaded_implementation> <configuration_string>

   Revision 1.7  1999/01/26 20:03:44  f-kon
   added call to netBroker->fini() to fix segmentation fault bug in client

   Revision 1.6  1999/01/11 23:13:52  f-kon
   Upgrade to DCP v1.0 and improvements to the PersistentRepository.

   Revision 1.5  1999/01/07 22:43:30  f-kon
   Added TAOConfigurator and new DCP commands

   Revision 1.4  1998/10/27 23:52:44  mroman
   Version Control and a strategy hooked cannot be removed

   Revision 1.3  1998/10/06 18:16:19  mroman
   Version that works fine on NT, Win98 and compiles fine

   Revision 1.2  1998/08/30 14:56:27  f-kon
   First modifications to try to make it work in Windows

******************************************************************************/

#if !defined (DCP_BROKER_H)
#define DCP_BROKER_H

//#define ACE_LACKS_INLINE_FUNCTIONS
//#define ACE_BUILD_DLL

#include "Net_Broker.h"

class ACE_2K_Export DCP_Broker : public Net_Broker
{

public:
  ACE_INLINE DCP_Broker (ACE_Reactor *reactor) 
    : Net_Broker (reactor), request_index_(0) {}

  //  int initialize (Dynamic_Service_Config *dsc); use this in the future
  int initialize ();

  // = Dynamic linking hooks.
  int info (char **info_string, size_t length) const;

  ACE_ALLOC_HOOK_DECLARE; 
  // Declare the dynamic allocation hooks.

private:
  int handle_request ();

  void list_loaded_implementations () const;

  void help() const;
  void quit(int sendACK);
  int  list_categories() const;
  void list_implementations (char *request) const;
  void get_latest_version (char *request) const;
  void get_impl_info (char *request) const;
  void list_domain_components () const;
  void list_hooks (char *request) const;
  void get_comp_info (char *request) const;
  void get_hooked_comp (char *request) const;

  void add_hook (char *request);
  void remove_hook (char *request);
  void load_implementation (char *request, char *end_of_request);
  void hook_implementation (char *request);
  void suspend_implementation (char *request);
  void resume_implementation (char *request);
  void remove_implementation (char *request);
  void configure_implementation (char *request, char *end_of_request);
  void upload_implementation (char *request);
  void delete_implementation (char *request);

  int get_params(char * request, 
                 char **arg1, char **arg2 = 0, char **arg3 = 0, char **arg4 = 0) const;

  void restart (void);
  void AckOrNack(int value) const;

  enum { BUFSIZE_ = 8192 };
  char buf_[BUFSIZE_];  

  char request_[BUFSIZE_];
  int request_index_;
};

#endif /* DCP_BROKER_H */

