/******************************************************************************
*                                                                             *
*  File: Dynamic_Service_Config.cpp                                           *
*                                                                             *
*  Description: The dynamicTAO Dynamic Service Configurator.                  *
*                                                                             *
*  Project: 2K - dynamicTAO                                                   *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************

   Revision Control Information

   $Revision: 1.4 $

   $Log: Dynamic_Service_Config.cpp,v $
   Revision 1.4  2000/11/03 16:56:28  f-kon
   Removed some old code that had '#if 0' and did small bug fix.

   Revision 1.3  2000/09/30 18:13:32  yamane
   Patches for Windows

   Revision 1.1.1.2  1999/10/17 18:47:25  yamane
   Cleanup for native exception and linux support

   Revision 1.1.1.1  1999/10/12 00:43:30  f-kon
   Version compatible with TAO 1.0

   Revision 1.2.1.2  1999/09/23 20:59:07  yamane
   de-couple libACE and libACE2k

   Revision 1.2.1.1  1999/09/22 00:08:50  yamane
   Uninitialized memory read on Dynamic_Service_Config::close(void)
           int ret;  ===> int ret = 0;

   class Dynamic_Service_Config make the use of singleton
   to avoid the order problem of initialization

   Change static -> normal class variable
                   domainConfigurator
                   versions_list_
                   ver_li_
                   imp_list_
                   imp_li_

   Modify constructors to initialise these members

   Revision 1.2  1999/04/07 20:09:04  f-kon
   added new DCP commands, upgrading to version 1.1

   Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
   This new 2K source tree contains the most up-to-date version of ACE and TAO
   and including the dynamicTAO code as well as some 2K tests and preliminary
   versions of some of its high-level systems.


   Revision 1.19  1999/03/03 16:12:13  f-kon
   Implemented DCP operation configure_implementation <loaded_implementation> <configuration_string>

   Revision 1.18  1999/02/23 21:21:00  mroman
   no message

   Revision 1.17  1999/02/22 22:21:03  f-kon
   Introduced NiceConfigurator and LockingConfigurator and performed several
   fixes on dynamic reconfiguration.

   Revision 1.16  1999/02/17 00:19:53  ckhess
   Added template instantiation code at bottom of file.

   Revision 1.15  1999/02/08 21:21:13  mroman
   no message

   Revision 1.14  1999/02/05 20:23:29  f-kon
   *** empty log message ***

   Revision 1.13  1999/02/04 17:01:19  mroman
   no message

   Revision 1.12  1999/02/02 22:42:58  f-kon
   fixed errors with #includes

   Revision 1.11  1999/01/28 18:01:09  mroman
   Fixed the version problem. Versions are only incremented when
   Dynamic_Service_Config.cpp
   loading and implementation succeeds.


   Revision 1.10  1999/01/26 20:03:45  f-kon
   added call to netBroker->fini() to fix segmentation fault bug in client

******************************************************************************/

// we're including a lot of stuff here. I bet we don't need everything. Fabio.
#include "ace/Svc_Conf.h" // important
#include "ace/Get_Opt.h"
#include "ace/ARGV.h"
#include "ace/Service_Repository.h"
#include "ace/Service_Types.h"


// the ones bellow are indeed needed. Fabio.

#include "ace/Reactor.h"
#include "ace/Thread_Manager.h"
#include "ace/Service_Config.h"
#include "ace/Dynamic_Service.h"


#include "Dynamic_Service_Config.h"

#include "DCP_Broker.h"
#include "SmartPacket_Broker.h"

#include "dynamicTAO/src/tao/TAO_Configurator.h"

// defines specific new and delete for this class
ACE_ALLOC_HOOK_DEFINE(Dynamic_Service_Config)

// define static member variables

ACE_2K_Export Net_Broker *Dynamic_Service_Config::netBroker_ = 0;
ACE_2K_Export Net_Broker *Dynamic_Service_Config::netBroker2_ = 0;

ACE_Persistent_Repository *Dynamic_Service_Config::pers_rep_=0;

Dynamic_Service_Config *Dynamic_Service_Config::instance_ = 0;
int Dynamic_Service_Config::delete_instance_ = 0;
const int Dynamic_Service_Config::VERSION_NUMBER_LENGTH = 5;


////////////////////////////////////////////////////////////////////////////////
// Checks whether the implementation has already been loaded into memory. If it
// has it returns -1. Otherwise it looks for the name of the Category and returns
// the tentative version number. Once the implementation is loaded into memory
// (not done in this method) the "Update Version" method must be called in order
// to commit the new version number.
////////////////////////////////////////////////////////////////////////////////
int Dynamic_Service_Config::getTentativeVersion (const char *catName, 
                                                 const char *impName, 
                                                 Version_Record **incrementLater,
                                                 int *alreadyLoaded)
{
  int version;
  Version_Record *tempCat = 0;
  Implementation_Record *tempImp = 0;
  *alreadyLoaded = 0;
    
  // Look for the implementation in the implementation list
  imp_li_->first();
  tempImp = imp_li_->next();
  while (tempImp != NULL)
    {
      if(!ACE_OS::strcmp (tempImp->implementation_name(), impName))
        { 
          *alreadyLoaded = 1;
          return tempImp->version ();
        }
      else
        {
          imp_li_->advance ();
          tempImp = imp_li_->next ();  
        }
    }

  // The implementation is not already loaded.
  // Now, look for the category in the category list.
  ver_li_->first();
  tempCat = ver_li_->next();
  
  while (tempCat != NULL)
  {
    if (!ACE_OS::strcmp(tempCat->category_name(), catName))
      break;  // found it
    else
    {
      ver_li_->advance();
      tempCat = ver_li_->next();
    }
  }	
  
  
  if(tempCat == NULL) // If implementation is not loaded and
    {                 // we didn't find the category name.
      version = 1;
    }  
  else
    version = tempCat->version () + 1;
  
  *incrementLater = tempCat;
  
  return version;
}// getTentativeVersion ()



///////////////////////////////////////////////////////////////////
// Stores the new version number with the appropriate category name
///////////////////////////////////////////////////////////////////
int Dynamic_Service_Config::updateVersion(Version_Record *tempCat, 
                                          const char *catName, const char *impName)
{
  // char stemp[MAXNAMLEN];
  int version;
  
  Implementation_Record *tempImp = NULL;
  
  if(tempCat == NULL)
    version = 1;
  else
    version = tempCat->version() + 1;
  
  // ACE_OS::sprintf(stemp, "%s:%d", catName, version);

  tempImp = new Implementation_Record (impName, catName, version);
  
  if(tempImp == NULL)
    return -1;
  
  if(tempCat == NULL)
    {
      tempCat = new Version_Record (catName);
      
      if(tempCat == NULL)
        return -1;
      
      versions_list_.insert_tail (tempCat);
    }
  
  imp_list_.insert_tail(tempImp);
  tempCat->version()++;
  return 1;

}// updateVersion ()

void
Dynamic_Service_Config::dump (void) const
{
  ACE_TRACE ("Dynamic_Service_Config::dump");

  // implement a real dump here.
}


////////////////////////////////////////////////////////////////////////////////
// All the support for static services and for reading service configuration  //
// files are left in the ACE_Service_Config class.                            //
////////////////////////////////////////////////////////////////////////////////

// = Set by command-line options.
char Dynamic_Service_Config::verbose_ = 0;
int  Dynamic_Service_Config::debug_ = 0;


//////////////////////////////////////////////////////////////////////////////////
// Remove <svc_name> from the process by removing it from the
// ACE_Service_Repository, and our internal lists.
// Returns: 0  if successful
//          -1 if something failed
//          +1 if the removal was deferred because the implementation has clients
///////////////////////////////////////////////////////////////////////////////////
int
Dynamic_Service_Config::remove (const char *implementationName)
{
  ACE_TRACE ("Dynamic_Service_Config::remove");

  // First check whether the service to remove is attached to one of the TAO
  // hooks.  If it is, return an error and leave it as it is.  Note that it's
  // kind of ugly to treat TAO in a different way here. We must think of a
  // better way of doing it. Maybe, deferring the removal for when its
  // reference count reaches 0.
  // Another option is to store TAO in the list of clients for this
  // strategy. And only remove it if the list of clients is empty.

  Simple_Strategy_Factory *ssf =
    ACE_Dynamic_Service<Simple_Strategy_Factory>::instance (implementationName);
  
  if (ssf == 0)
    return -1;
  else
    {
      if(ssf->localConfigurator () && ssf->localConfigurator ()->numberOfClients () > 0)
        // There's at least one client using the implementation. We can either
        // remove the component anyway which will trigger DELETED events sent
        // to the clients or just return an error code. By now, we'll return
        // the error code.  The best thing to do is to have a flag in the
        // ComponentConfigurator telling which of the two approaches should be
        // taken. So, different components could use different approaches.
        // More info on that in Fabio's TODO file.
        
        // if (ssf->localConfigurator ()-> hasHardHooks ())
        {
          // schedules the removal for when the # of clients reaches 0.
          ssf->localConfigurator ()-> toBeRemoved();
          return +1;
        }
    }

  // At this point, implementationName contains something of the form
  // categoryName:v (where v is the version number). The tempImp records
  // contain implementation_name_, category_name_, and version_. So we
  // need to do some parsing before comparing them.
  int   version;
  char *versionStr;
  char *category_name = ACE_OS::strdup (implementationName);
  category_name = ACE_OS::strtok_r (category_name, ":", &versionStr);

  if (versionStr != NULL)
    version = ACE_OS::atoi (versionStr);
  else 
    version = 0;
  
  // First, delete it from the loaded implementations list.
  imp_li_->first();
  Implementation_Record *tempImp = imp_li_->next();
  while (tempImp != NULL )
  {
    if (!ACE_OS::strcmp (tempImp->category_name(), category_name) &&
        tempImp->version() == version)
      {
        imp_list_.remove(tempImp);
        delete tempImp;
        break;
      }
    else
      {
        imp_li_->advance();
        tempImp = imp_li_->next();  
      }
  }
  if (tempImp == NULL)
    {
      ACE_DEBUG ((LM_DEBUG, "Error, didn't find entry in the implementation list\n"));  
    }
  
  ACE_OS::free (category_name);

  // Now, try to remove it from the service repository.
  return ACE_Service_Repository::instance ()->remove (implementationName);
}

////////////////////////////////////////////////////////////////////////////////
// Suspend SVC_NAME.  Note that this will not unlink the service from
// the daemon if it was dynamically linked, it will mark it as being
// suspended in the Service Repository and call the suspend() member
// function on the appropriate ACE_Service_Object.  A service can be
// resumed later on by calling the RESUME() member function...
////////////////////////////////////////////////////////////////////////////////
int
Dynamic_Service_Config::suspend (const char svc_name[])
{
  ACE_TRACE ("Dynamic_Service_Config::suspend");
  return ACE_Service_Repository::instance ()->suspend (svc_name);
}

////////////////////////////////////////////////////////////////////////////////
// Resume a SVC_NAME that was previously suspended or has not yet
// been resumed (e.g., a static service).
////////////////////////////////////////////////////////////////////////////////
int
Dynamic_Service_Config::resume (const char svc_name[])
{
  ACE_TRACE ("Dynamic_Service_Config::resume");
  return ACE_Service_Repository::instance ()->resume (svc_name);
}



////////////////////////////////////////////////////////////////////////////////
// CTOR. Initialize the Service Repository. 
////////////////////////////////////////////////////////////////////////////////
// NOTE: The respective constructor in the ACE_Service_Config is NOT called.
Dynamic_Service_Config::Dynamic_Service_Config (size_t size)
  : domainConfigurator_ ("Domain Configurator", 0)
{
  ACE_TRACE ("Dynamic_Service_Config::Dynamic_Service_Config");

  // Initialize the Service Repository.
  ACE_Service_Repository::instance (size);

  // Initialize the ACE_Reactor (the ACE_Reactor should be the same
  // size as the ACE_Service_Repository).
  ACE_Reactor::instance ();

  ver_li_ 
    = new ACE_Double_Linked_List_Iterator<Version_Record>(versions_list_);

  imp_li_ 
    =  new ACE_Double_Linked_List_Iterator<Implementation_Record>(imp_list_);

}//CTOR

////////////////////////////////////////////////////////////////////////////////
// Constructor.
////////////////////////////////////////////////////////////////////////////////
// NOTE: The respective constructor in the ACE_Service_Config is NOT called.
Dynamic_Service_Config::Dynamic_Service_Config (const char program_name[],
                                                LPCTSTR logger_key)
  : domainConfigurator_ ("Domain Configurator", 0)
{
  ACE_TRACE ("Dynamic_Service_Config::Dynamic_Service_Config");

  if (this->open (program_name, logger_key) == -1
      && errno != ENOENT)
    // Only print out an error if it wasn't the svc.conf file that was
    // missing.
    ACE_ERROR ((LM_ERROR, "%p\n", program_name));
  ACE_OS::printf("We called Dynamic_Service_Config::Dynamic_Service_Config(program_name = %s)\n", 
                 program_name);
  ver_li_ 
    = new ACE_Double_Linked_List_Iterator<Version_Record>(versions_list_);

  imp_li_ 
    =  new ACE_Double_Linked_List_Iterator<Implementation_Record>(imp_list_);
}// constructor


////////////////////////////////////////////////////////////////////////////////
// Handles the command-line options intended for the Dynamic_Service_Config.
// Forwards other command-line options to the ACE_Service_Config (the static one)
////////////////////////////////////////////////////////////////////////////////
void
Dynamic_Service_Config::parse_args (int argc, char *argv[])
{

  // MODIFY THIS SO WE FIRST PARSE THE Dynamic_Service_Config-SPECIFIC
  // ARGUMENTS AND WHEN WE FIND AN ARGUMENT THAT'S NOT OURS, WE CALL
  // ACE_Service_Config::parse_args TO TAKE CARE OF THE REMAINING ARGUMENTS.
  
  ACE_TRACE ("Dynamic_Service_Config::parse_args");
  ACE_Get_Opt getopt (argc, argv, "bdf:k:ns:", 1); // Start at argv[1]

  for (int c; (c = getopt ()) != -1; )
    switch (c)
      {
      case 'v':
        Dynamic_Service_Config::verbose_ = 1;
        // set verbose in the the ACE_Service Config, too?
        break;
      default:
	// We don't know this option. So we assume this is the start of the
	// ACE_Service_Config options.
	ACE_Service_Config::parse_args(getopt.argc_, getopt.argv_);

	// if they ever hide the getopt member variables, we could use the
	// following call instead.
	//	ACE_Service_Config::parse_args (getopt);
        return;
      }
}// parse_args()

////////////////////////////////////////////////////////////////////////////////
// Dynamically loads a service and initializes it.
////////////////////////////////////////////////////////////////////////////////
// If we ever have problems here, 
// take a look at TAO_Internal::fake_service_entries_i
int
Dynamic_Service_Config::initialize (const char *pathname,
                                    const char *factoryFunc,
                                    const char *serviceName,
                                    int active,
                                    char parameters[])
{
  ACE_Function_Node *creator;
  ACE_NEW_RETURN (creator, ACE_Function_Node (pathname,factoryFunc), -1);

  // DELETE_THIS tells the ACE_Service_Config to delete the Service_Type
  // object and DELETE_OBJ tells it to delete the object implementation
  // (although I suspect the latter is not working).
  u_int flags =  ACE_Service_Type::DELETE_THIS | ACE_Service_Type::DELETE_OBJ;

  // The call to creator->symbol() is the one who dynamically links.
  void *symbol = creator->symbol();

  if (symbol == 0)
    {
      delete creator;
      return -1;
    }
  
  ACE_Service_Type_Impl *stp = ace_create_service_type (serviceName,
                                                        ACE_SVC_OBJ_T, 
                                                        symbol,
                                                        flags);

  ACE_Service_Type *service; 

  ACE_NEW_RETURN (service, ACE_Service_Type (serviceName, stp,
                                             creator->handle(),
                                             active), -1);
  int ret = initialize(service, parameters);
  
  if (ret == -1)
    {
      delete service;
      delete creator; // shall I always delete creator?
    }
  
  return ret;

}// initialize(pathname, factoryFunc, serviceName, parameters)

////////////////////////////////////////////////////////////////////////////////
// Inserts this service in the ACE_Service_Repository and initalizes it.
////////////////////////////////////////////////////////////////////////////////
int
Dynamic_Service_Config::initialize (const ACE_Service_Type *sr,
                                    char parameters[])
{
  // delegates this operation to <ACE_Service_Config>. We might change this in
  // the future if we think we need.

  return ACE_Service_Config::initialize (sr, parameters);
}// initialize(serviceType, parameters)


////////////////////////////////////////////////////////////////////////////////
//Dynamically links the library specified by path/implementation, creates from
//it an object of the specified category, and initializes the strategy with
//the given parameters.  If compConfig is non-null, requests the implementation
//installation in the proper component hook.  Returns -1 in case of failure.
////////////////////////////////////////////////////////////////////////////////
int
Dynamic_Service_Config::install (const char *path, const char *implementation,
                                 const char *categoryName,
                                 char parameters[], const char *hookName, 
                                 ComponentConfigurator *compConfig)
{
  int  version, alreadyLoaded;
  char factoryFunc[MAXNAMELEN];
  char pathname[MAXPATHLEN]; 
  char *newServName = new char[ACE_OS::strlen(categoryName)+
                                 VERSION_NUMBER_LENGTH+2];

  char *newImplServName= new char[ACE_OS::strlen(path)+ACE_OS::strlen(implementation) + 2];
  
  Version_Record *tempIncrementer;
  
  // Concatenate the path and the name of the implementation.
  // In this way it is possible to have implementations of different
  // categories with the same name.
  ACE_OS::strcpy(newImplServName, path);
  ACE_OS::strcat(newImplServName, "/");
  ACE_OS::strcat(newImplServName, implementation);

  // Requests a new version number for this implementation.
  version = getTentativeVersion(categoryName, newImplServName, &tempIncrementer, &alreadyLoaded);

  if (version == -1) 
    return -1;         // Something failed.

  ACE_OS::sprintf (newServName, "%s:%d", categoryName, version);

  if (!alreadyLoaded) // If it's not loaded, load it.
    {
      // Dynamically link the implementation and add it to the ACE service repository.
      ACE_OS::sprintf (pathname, "%s%s%s", path, ACE_DIRECTORY_SEPARATOR_STR_A, implementation);

      char *imp = ACE_OS::strdup (implementation);
      ACE_OS::sprintf (factoryFunc, "%s%s%s", 
                       "_make_", pers_rep_->clean_name (imp), "_Factory");
      ACE_OS::free (imp);
      
      if (initialize(pathname, factoryFunc, newServName, 1, parameters) == -1)
        return -1;
      
      // The implementation has been loaded. Increment the version number.
      updateVersion(tempIncrementer, categoryName, newImplServName);
      
      Simple_Strategy_Factory *ssf =
        ACE_Dynamic_Service<Simple_Strategy_Factory>::instance (newServName);
      
      if (ssf->localConfigurator ())
        ssf->localConfigurator ()-> name (newServName);
    }

  // At this point, the implementation is loaded.  If a compConfig was
  // specified, we attach the loaded implementation to the proper hook.
  if(compConfig)
    {
      if (hook (newServName, hookName, compConfig))
        return version;
      else
        return -1;
    }
  else
    return version;
}// install ()

///////////////////////////////////////////////////////////////////////////
// Adds a hook to the given component. Returns -1 if failed, 0 o/w.
///////////////////////////////////////////////////////////////////////////
int 
Dynamic_Service_Config::add_hook (char *componentName, const char *hookName)
{
  ComponentConfigurator *c = getConfigurator (componentName);
  if (c == 0)
    return -1;
  else
    return c->addHook(hookName);
}//add_hook

///////////////////////////////////////////////////////////////////////////
// Removes a hook from the given component. Returns -1 if failed, 0 o/w.
///////////////////////////////////////////////////////////////////////////
int 
Dynamic_Service_Config::delete_hook (char *componentName, const char *hookName)
{
  ComponentConfigurator *c = getConfigurator (componentName);
  if (c == 0)
    return -1;
  else
    return c->deleteHook(hookName);
}// delete_hook



///////////////////////////////////////////////////////////////////////////
// Assigns a loaded implementation to a particular hook in the specified
// application.
///////////////////////////////////////////////////////////////////////////
int
Dynamic_Service_Config::hook (const char *implementationName,
                              const char *hookName,
                              ComponentConfigurator *compConfig)
{
  int ret = 0;
  
  Simple_Strategy_Factory *ssf =
    ACE_Dynamic_Service<Simple_Strategy_Factory>::instance (implementationName);
  
  if (ssf == 0)
    return -1;
  
  // if (compConfig->lock ())
  //  compConfig->lock ()->acquire_write();

  ComponentConfigurator *oldComponent = compConfig->getHookedComponent (hookName);
  ComponentConfigurator *delete_me = ssf->localConfigurator ();

  // set the new dependencies
  ret  = compConfig->hook (hookName, ssf->localConfigurator ());
  ret += ssf->localConfigurator ()->registerClient (compConfig, hookName);
  
  // remove the old dependency
  if (oldComponent != 0) 
    {
      // First gather some information about the oldComponent.
      int wasRemoved;
      Simple_Strategy_Factory *oldComponentImplementation = 
        (Simple_Strategy_Factory *) oldComponent->implementation ();
      char *oldImplementationName = ACE_OS::strdup(oldComponent-> name());
      
      oldComponent->unregisterClient (compConfig, hookName, &wasRemoved);
      
      if (wasRemoved)
        {
          // If the old component was removed in the call to unregisterClient,
          // then we should remove it from the ACE_Service_Configurator, too.
          oldComponentImplementation-> localConfigurator (0);
          remove (oldImplementationName);
        }
      
      ACE_OS::free (oldImplementationName);
      //compConfig->unhook (hookName);
    }

//   if (compConfig->lock ())
//     compConfig->lock ()->release ();
  return ret;
}// hook ()


///////////////////////////////////////////////////////////////////////////
// Assigns a loaded implementation to a particular hook in the specified
// component. It uses the previous method.
///////////////////////////////////////////////////////////////////////////
int
Dynamic_Service_Config::hook (const char *implementationName,
                              const char *hookName,
                              char *compName)
{
  ComponentConfigurator *c = getConfigurator (compName);

  if (c == 0)
    return -1;
  else
    return hook(implementationName, hookName, c);
}// hook ()

////////////////////////////////////////////////////////////////////////////////
// Returns the latest version among the implementations of categoryName.
// Makes the output parameter implementationName point to a string with the 
// name of the implementation in the ACE_Service_Repository
////////////////////////////////////////////////////////////////////////////////
int Dynamic_Service_Config::get_latest_version (const char *categoryName, char **implementationName)
{
  Implementation_Record *tempImp;
  imp_li_->first();
  // The following ugly thing is required because the
  // ACE_Double_Linked_List_Iterator is currently asymmetric.  After porting
  // to the latest ACE, should use ACE_Double_Linked_List_Reverse_Iterator instead.
  tempImp = imp_li_->next(); // points to the first element
  if (tempImp != NULL)
    tempImp = tempImp->prev_->prev_; // now, it points to the tail.

  while (tempImp != NULL && tempImp->category_name() != NULL)
    {
      char *category = tempImp->category_name();
      if(category != NULL && !ACE_OS::strcmp (category, categoryName))
        { 
          // found it
          *implementationName = tempImp->implementation_name();
          return tempImp->version ();
        }
      else
        {
          tempImp = tempImp->prev_;
        }
    }

  // Didn't find any implementation for this category.
  *implementationName = 0;
  return 0;

}// get_latest_version ()





/////////////////////////////////////////////////////////////////////////////
// Opens the Dynamic Service Configurator and the ACE Service Configurator.
// Creates 2 new threads and run new network brokers on them. Establishes the
// links between the Dynamic Service Configurator and the Net brokers. 
// (It does not call parse_args, the latter is called by the open in
// Dynamic_Service_Config.i).
/////////////////////////////////////////////////////////////////////////////
int
Dynamic_Service_Config::open (const ASYS_TCHAR program_name[],
                              LPCTSTR logger_key,
                              int ignore_static_svcs,
                              int ignore_default_svc_conf)
{
  // Note that at this point the command-line arguments were already parsed.
  ACE_TRACE ("Dynamic_Service_Config::open");

  //  ACE_Log_Msg::disable_debug_messages(LM_DEBUG);
  ACE_Log_Msg::disable_debug_messages(LM_WARNING);

  // First open the ACE_Service_Config class which initializes the service
  // repository and the reactor, initializes static services, and evaluates
  // the service configuration file dynamic loading the services specified there.

  if (ACE_Service_Config::open(program_name, logger_key,
                               ignore_static_svcs, ignore_default_svc_conf) == -1)
    ACE_ERROR((LM_ERROR, "Could not open ACE_Service_Config\n"));

  //Creates the ACE_Persistent_Repository. This variable will be used by the
  //Net_Brokers and the DynamicConfigurator
  ACE_NEW_RETURN(Dynamic_Service_Config::pers_rep_, ACE_Persistent_Repository, -1);
  
  char path[MAXNAMELEN];

  //Now, find where the local repository is located.
  char *retEnv=ACE_OS::getenv("DTAO_REPOSITORY");
  if(retEnv==0)
    {
      retEnv=ACE_OS::getenv("ACE_ROOT");
      
      if(retEnv==0)
        {
          ACE_OS::printf("Warning: ACE_ROOT not set. By default . assumed\n");
          retEnv=".";
        }
    }
  
  ACE_OS::strcpy (path, retEnv);
  ACE_OS::strcat (path, ACE_DIRECTORY_SEPARATOR_STR_A);
  ACE_OS::strcat (path, "repository");
  
  if(pers_rep_->init(path)==-1)
    return -1;
  
  // Initializes the netBroker member variables.
  // This should be later specified at the svc.conf file
  // I think we shouldn't use ACE_Reactor::instance(), we should our own reactor
  // instead.
  ACE_NEW_RETURN (netBroker_, DCP_Broker (ACE_Reactor::instance ()), -1); 
  ACE_NEW_RETURN (netBroker2_, SmartPacket_Broker(ACE_Reactor::instance ()), -1); 

  netBroker()->initialize();
  netBroker2()->initialize();

  domainConfigurator_.info ("The DomainConfigurator that holds the components of a domain");
  domainConfigurator_.addHook ("Interpreter");
  domainConfigurator_.addHook ("TAO");
  domainConfigurator_.hook ("TAO", new TAO_Configurator("dynamicTAO", 0));

  // Create a new thread to execute the first network broker.
  int result = ACE_Thread_Manager::instance () ->
    spawn(ACE_THR_FUNC (Dynamic_Service_Config::net_broker_entry_point));
  if (result == -1)
    ACE_ERROR((LM_ERROR, "Could not spawn the first Network Broker thread\n"));

  // Create a new thread to execute the second network broker.
  result = ACE_Thread_Manager::instance () ->
    spawn(ACE_THR_FUNC (Dynamic_Service_Config::net_broker2_entry_point));
  if (result == -1)
    ACE_ERROR((LM_ERROR, "Could not spawn the second Network Broker thread\n"));

  return 0;

}// open()

////////////////////////////////////////////////////////////////////////////////
// These are a static methods that are executed when the threads that take care
// of the network brokers are started.
////////////////////////////////////////////////////////////////////////////////
int Dynamic_Service_Config::net_broker_entry_point(){
  return netBroker()->run();
}
int Dynamic_Service_Config::net_broker2_entry_point(){
  return netBroker2()->run();
}


////////////////////////////////////////////////////////////////////////////////
// Finilizes the netBrokers and their threads. Note that we do not call 
// ACE_Service_Config::close (); It must be called elsewhere, like in
// ACE_Object_Manager::fini, for example.
////////////////////////////////////////////////////////////////////////////////
int
Dynamic_Service_Config::close (void)
{
  int ret = 0;
  
  ACE_TRACE ("Dynamic_Service_Config::close");

  if (netBroker ())
    ret += netBroker ()->fini ();
  
  if (netBroker2 ())
    ret += netBroker2 ()->fini ();
  
  return ret;
}// close()
	

////////////////////////////////////////////////////////////////////////////////
// Perform user-specified close activities and remove dynamic memory.
////////////////////////////////////////////////////////////////////////////////
Dynamic_Service_Config::~Dynamic_Service_Config (void)
{
  ACE_TRACE ("Dynamic_Service_Config::~Dynamic_Service_Config");
  Dynamic_Service_Config::close ();

  if (pers_rep_)
	delete pers_rep_;

  if (netBroker())
    delete netBroker();
  if (netBroker2())
    delete netBroker2();  
}// destructor


// ************************************************************


////////////////////////////////////////////////////////////////////////////////
// cleanp methods for Object Manager
// It will be called from the Object Manager so we no longer have to modify
// Object_Manager itself to call Dynamic_Service_Config::instance()->close()
////////////////////////////////////////////////////////////////////////////////
void
Dynamic_Service_Config::cleanup (void *param)
{
	close();
}

////////////////////////////////////////////////////////////////////////////////
// Creates a new process-wide <Dynamic_Service_Config> - if one does not exist
// yet - and returns a pointer to it.
////////////////////////////////////////////////////////////////////////////////
Dynamic_Service_Config *
Dynamic_Service_Config::instance (void)
{
  ACE_TRACE ("Dynamic_Service_Config::instance");

  if (Dynamic_Service_Config::instance_ == 0)
    {
      // Perform Double-Checked Locking Optimization.
      ACE_MT (ACE_GUARD_RETURN (ACE_Recursive_Thread_Mutex, ace_mon,
				*ACE_Static_Object_Lock::instance (), 0));
      
      if (Dynamic_Service_Config::instance_ == 0)
	{
	  ACE_NEW_RETURN (Dynamic_Service_Config::instance_, Dynamic_Service_Config, 0);
	  Dynamic_Service_Config::delete_instance_ = 1;
	  ACE_Object_Manager::at_exit (Dynamic_Service_Config::instance_);
	}
    }
  return Dynamic_Service_Config::instance_;
}



////////////////////////////////////////////////////////////////////////////////
// Deletes the singleton. Use with caution.
////////////////////////////////////////////////////////////////////////////////
void
Dynamic_Service_Config::close_singleton (void)
{
  ACE_TRACE ("Dynamic_Service_Config::close_singleton");

  ACE_MT (ACE_GUARD (ACE_Recursive_Thread_Mutex, ace_mon,
		     *ACE_Static_Object_Lock::instance ()));

  if (Dynamic_Service_Config::delete_instance_)
    {
      delete Dynamic_Service_Config::instance_;
      Dynamic_Service_Config::instance_ = 0;
      Dynamic_Service_Config::delete_instance_ = 0;
    }
}// close_singleton ()



////////////////////////////////////////////////////////////////////////////////
// Fills buf with a list of components registered with this
// Dynamic_Service_Config.
////////////////////////////////////////////////////////////////////////////////
void
Dynamic_Service_Config::get_components (char *buf, int maxsize)
{
  listHooksOnString (&domainConfigurator_, buf, maxsize);
}// get_components ()



////////////////////////////////////////////////////////////////////////////////
// Fills "components" with the names of the components
////////////////////////////////////////////////////////////////////////////////
void Dynamic_Service_Config::get_components (char **components, int &size,int maxsize)
{
  listHooksOnString (&domainConfigurator_, components, size, maxsize);

}

////////////////////////////////////////////////////////////////////////////////
// Fills buf with a list of hooks for this component. 
////////////////////////////////////////////////////////////////////////////////
int
Dynamic_Service_Config::get_hooks (char *comp, char *buf, int maxsize)
{
  ComponentConfigurator *c = getConfigurator (comp);

  if (c == 0)
    return -1;
  
  listHooksOnString (c, buf, maxsize);
  
  return 0;
}// get_hooks ()



////////////////////////////////////////////////////////////////////////////////
// Fills hooks with the names of the hooks
////////////////////////////////////////////////////////////////////////////////
int Dynamic_Service_Config::get_hooks (char *comp, char **hooks, int &size,int maxsize)
{
  ComponentConfigurator *c = getConfigurator (comp);

  if (c == 0)
    return -1;
  
  listHooksOnString (c, hooks, size, maxsize);
  
  return 0;
}



////////////////////////////////////////////////////////////////////////////////
// Fills buf with information about this component (application).
////////////////////////////////////////////////////////////////////////////////
int
Dynamic_Service_Config::get_comp_info (char *comp, char *buf, int maxsize)
{
  ComponentConfigurator *c = getConfigurator (comp);

  if (c == 0)
    return -1;

  ACE_OS::strncpy(buf, c->info (), maxsize);
  buf[maxsize-1] = '\0';
  
  return 0;
}// get_comp_info ()

////////////////////////////////////////////////////////////////////////////////
// Returns a pointer to the name of the component attached to the given hook.
////////////////////////////////////////////////////////////////////////////////
const char *
Dynamic_Service_Config::get_hooked_comp (const char *hookName, char *comp)
{
  ComponentConfigurator *c = getConfigurator (comp);

  if (c == 0)
    return 0;

  ComponentConfigurator *hooked = c->getHookedComponent (hookName);
  if(hooked != 0)
    return hooked->name ();
  else
    return 0;
}// get_comp_info ()


////////////////////////////////////////////////////////////////////////////////
// Fills buf with a list of hooks for the given ComponentConfigurator.
////////////////////////////////////////////////////////////////////////////////
void
Dynamic_Service_Config::listHooksOnString (ComponentConfigurator *c, char *buf, 
                                           int maxsize)
{
  DependencySpecIterator hsi (*c->listHooks ());

  int size = 0;
  *buf = '\0';

  for (DependencySpecification *ap = hsi.next ();
       ap != 0;
       hsi.advance (), ap = hsi.next ())
    {
      const char *name = ap->name ();
      ACE_OS::strcat(buf, name);
      ACE_OS::strcat(buf, "\r\n");
      size += (ACE_OS::strlen(name) +2);
      if (size > maxsize - MAXNAMLEN)
        {
          ACE_OS::strcat(buf, "The list is too long, truncating...\r\n");
          break;
        }
    }
}// listHooksOnString ()


////////////////////////////////////////////////////////////////////////////////
// Fills the array of strings "buf" with a list of hooks for the given ComponentConfigurator.
////////////////////////////////////////////////////////////////////////////////
void
Dynamic_Service_Config::listHooksOnString (ComponentConfigurator *c, char **buf, int &elements, int maxsize)
{
  DependencySpecIterator hsi (*c->listHooks ());

  elements=0;

  for (DependencySpecification *ap = hsi.next ();
       ap != 0;
       hsi.advance (), ap = hsi.next ())
    {
      const char *name = ap->name ();
      
      buf[elements]=new char[ACE_OS::strlen(name)+1];
      ACE_OS::strcpy(buf[elements], name);
      ACE_OS::strcat(buf[elements++], "\0");
      
      if (elements == maxsize - 1)
        {
          ACE_OS::strcpy(buf[elements], "The list is too long, truncating...");
          break;
        }
    }
}// listHooksOnString ()

////////////////////////////////////////////////////////////////////////////////
// Returns a pointer to the ComponentConfigurator associated with the component
// specified by its pathname in relation to the domainConfigurator.
// Note that this is not a file system pathname, it's a in-memory path name;
// each pathname component is the name of a hook in a ComponentConfigurator.
// Note that the path separator is "/" no matter what the underlying OS is.
// A pointer to the domainConfigurator can be obtained by passing "/" as the 
// pathname.
////////////////////////////////////////////////////////////////////////////////
ComponentConfigurator *
Dynamic_Service_Config::getConfigurator (char *componentPathName)
{
  char *lasts = componentPathName;
  
  ComponentConfigurator *c = domainConfigurator ();
  if (c == 0)
    return 0;

  char *pathComponent = ACE_OS::strtok_r (componentPathName, "/", &lasts);
  
  while (pathComponent != 0)
    {      
      c = c->getHookedComponent (pathComponent);
      if (c == 0) 
        return 0;
      pathComponent = ACE_OS::strtok_r (NULL, "/", &lasts);
    }

  return c;
}// getConfigurator ()



///////////////////////////////////////////////////////////////////////////////
// Accessor method for the persistent repository. Note that Windows does not //
// allow accessing static attributes directly	                             //
///////////////////////////////////////////////////////////////////////////////
ACE_Persistent_Repository *Dynamic_Service_Config::pers_rep()
{
	return Dynamic_Service_Config::pers_rep_;
}


void Dynamic_Service_Config::pers_rep(ACE_Persistent_Repository *rep)
{
	Dynamic_Service_Config::pers_rep_=rep;
}



#if !defined (__ACE_INLINE__)
#include "Dynamic_Service_Config.i"
#endif /* __ACE_INLINE__ */

#if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)
template class ACE_Dynamic_Service<Simple_Strategy_Factory>;
template class ACE_Double_Linked_List<Version_Record>;
template class ACE_Double_Linked_List<Implementation_Record>;
template class ACE_Double_Linked_List<DependencySpecification>;
template class ACE_Double_Linked_List_Iterator<Version_Record>;
template class ACE_Double_Linked_List_Iterator<Implementation_Record>;
template class ACE_Double_Linked_List_Iterator<DependencySpecification>;
template class ACE_Double_Linked_List_Iterator_Base<Version_Record>;
template class ACE_Double_Linked_List_Iterator_Base<Implementation_Record>;
template class ACE_Double_Linked_List_Iterator_Base<DependencySpecification>;
#elif defined (ACE_HAS_TEMPLATE_INSTANTIATION_PRAGMA)
#pragma instantiate ACE_Dynamic_Service<Simple_Strategy_Factory>
#pragma instantiate ACE_Double_Linked_List<Version_Record>
#pragma instantiate ACE_Double_Linked_List<Implementation_Record>
#pragma instantiate ACE_Double_Linked_List<DependencySpecification>
#pragma instantiate ACE_Double_Linked_List_Iterator<Version_Record>
#pragma instantiate ACE_Double_Linked_List_Iterator<Implementation_Record>
#pragma instantiate ACE_Double_Linked_List_Iterator<DependencySpecification>
#pragma instantiate ACE_Double_Linked_List_Iterator_Base<Version_Record>
#pragma instantiate ACE_Double_Linked_List_Iterator_Base<Implementation_Record>
#pragma instantiate ACE_Double_Linked_List_Iterator_Base<DependencySpecification>
#endif /* ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION */
