/******************************************************************************
*                                                                             *
*  File: Net_Broker.cpp                                                       *
*                                                                             *
*  Description: Implements some of the methods of this abstract class.        *
*                                                                             *
*  Project: 2K - dynamically configurable ORB.                                *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************

   Revision Control Information

   $Revision: 1.1.1.3 $

   $Log: Net_Broker.cpp,v $
   Revision 1.1.1.3  1999/10/17 18:48:59  yamane
   Cleanup for native exception and linux support

   Revision 1.1.1.2  1999/10/17 18:47:47  yamane
   Cleanup for native exception and linux support

   Revision 1.1.1.1  1999/10/12 00:43:30  f-kon
   Version compatible with TAO 1.0

   Revision 1.1.1.2  1999/09/22 00:23:51  yamane
   class Dynamic_Service_Config make the use of singleton
   to avoid the order problem of initialization

   In Net_Broker::open()
        Dynamic_Service_Config::pers_rep()
                  --> Dynamic_Service_Config::instance()->pers_rep()

   Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
   This new 2K source tree contains the most up-to-date version of ACE and TAO
   and including the dynamicTAO code as well as some 2K tests and preliminary
   versions of some of its high-level systems.


   Revision 1.9  1999/02/05 23:22:38  f-kon
   *** empty log message ***

   Revision 1.8  1999/02/05 23:16:07  f-kon
   Modified the network broker so it binds to the first available local port
   between 10000 and 10100.

   Revision 1.7  1999/02/04 17:01:21  mroman
   no message

   Revision 1.6  1999/02/02 22:43:02  f-kon
   fixed errors with #includes

   Revision 1.5  1999/01/28 18:02:05  mroman
   Fixed the version problem. Versions are only incremented when 
   loading and implementation succeds.

   Revision 1.4  1998/10/06 18:16:23  mroman
   Version that works fine on NT, Win98 and compiles fine

   Revision 1.3  1998/08/30 14:56:36  f-kon
   First modifications to try to make it work in Windows

******************************************************************************/

#include "Net_Broker.h"

//#define ACE_BUILD_DLL
#include "ace/Get_Opt.h"
#include "ace/Service_Config.h"
#include "ace/Reactor.h"

#include "dynamicTAO/src/ace/Dynamic_Service_Config.h"
// #if !defined (__ACE_INLINE__)
// #include "ace/Net_Broker.i"
// #endif /* __ACE_INLINE__ */

#include "tao/corba.h"


ACE_ALLOC_HOOK_DEFINE(Net_Broker)

void
Net_Broker::dump (void) const
{
  ACE_TRACE ("Net_Broker::dump");
}

// Static variables. 

u_short Net_Broker::DEFAULT_PORT_ = 10000;

Net_Broker::Net_Broker (ACE_Reactor *reactor)
  : reactor_ (reactor),
    debug_ (0)
    //    signum_ (SIGHUP)
{
  ACE_TRACE ("Net_Broker::Net_Broker");
}

Net_Broker::~Net_Broker (void)
{
  ACE_TRACE ("Net_Broker::~Net_Broker");
}

int
Net_Broker::suspend (void)
{
  ACE_TRACE ("Net_Broker::suspend");
  return reactor_->suspend_handler (this);
}
     
int
Net_Broker::resume (void)
{
  ACE_TRACE ("Net_Broker::resume");
  return reactor_->resume_handler (this);
}

int
Net_Broker::open (const ACE_INET_Addr &sia)
{
  ACE_TRACE ("Net_Broker::open");
  if (this->acceptor_.open (sia, 0) == -1)
    return -1;

   this->client_stream_.set_handle(ACE_INVALID_HANDLE);

   pers_rep_ = Dynamic_Service_Config::instance()->pers_rep();
   return 0;
}// open ()


int 
Net_Broker::info (char **strp, size_t length) const
{
  ACE_TRACE ("Net_Broker::info");
  ACE_INET_Addr sa;
  char buf[BUFSIZ];

  if (this->acceptor_.get_local_addr (sa) == -1)
    return -1;
  
  ACE_OS::sprintf (buf, "%d/%s %s", sa.get_port_number (), "tcp", 
	     "# Dynamic Service Configurator Network Broker\n");

  if (*strp == 0 && (*strp = ACE_OS::strdup (buf)) == 0)
    return -1;
  else
    ACE_OS::strncpy (*strp, buf, length);
  return ACE_OS::strlen (buf);
}

int
Net_Broker::init (int argc, char *argv[])
{
  return init (argc, argv, "");
}

int
Net_Broker::init (int argc, char *argv[], const char *type)
{
  ACE_TRACE ("Net_Broker::init");
  u_short port = Net_Broker::DEFAULT_PORT_;
  ACE_INET_Addr local_addr (port);

  ACE_Get_Opt	getopt (argc, argv, "dp:s:", 0); // Start at argv[0]

  for (int c; (c = getopt ()) != -1; )
     switch (c)
       {
       case 'd':
	 this->debug_ = 1;
	 break;
       case 'p': 
	 local_addr.set ((u_short) ACE_OS::atoi (getopt.optarg));
	 break;
       default:
	 break;
       }
  
  // Try to bind the network broker to one of the 100 ports starting at
  // Net_Broker::DEFAULT_PORT_
  int ret, i;
  for (ret = -1, i = 0; i <= 100; i++)
    {
      local_addr.set (port + i);
      if ((ret = this->open (local_addr)) == 0)
        break;
    }  

  if (ret >= 0)
    ACE_DEBUG ((LM_NOTICE, 
                "\nThe Configuration Service %s interface is available at port %d\n\n", 
		type, port +i));
  else
    ACE_ERROR_RETURN ((LM_ERROR, "%p\n", "Net_Broker was not activated"), -1);

  if (reactor_->register_handler (this, ACE_Event_Handler::ACCEPT_MASK) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, "registering network broker with ACE_Reactor\n"), -1);

  return 0;
}


///////////////////////////////////////////////////////////////////////////
// This is the entry point of the thread that runs the reactor event loop.
///////////////////////////////////////////////////////////////////////////
int 
Net_Broker::run (void)
{
  my_main_thread_ = ACE_Thread::self (); // maybe we need to use the thread manager here.

  reactor_->owner (my_main_thread_); // Sets the current thread as the
                                     // reactor owner.
  return reactor_->run_event_loop ();
}


int
Net_Broker::handle_close (ACE_HANDLE, ACE_Reactor_Mask)
{
  ACE_TRACE ("Net_Broker::handle_close");
  return this->acceptor_.close ();
}

int
Net_Broker::fini (void)
{
  int ret;
  ACE_TRACE ("Net_Broker::fini");

  ret = reactor_->end_event_loop ();
  if (this->get_handle () != ACE_INVALID_HANDLE)
    ret += reactor_->remove_handler (this, ACE_Event_Handler::ACCEPT_MASK);
  return ret;
}
  
ACE_HANDLE
Net_Broker::get_handle (void) const
{ 
  ACE_TRACE ("Net_Broker::get_handle");
  return this->acceptor_.get_handle (); 
}

///////////////////////////////////////////////////////////////////////////
// Trigger a remote reconfiguration of the Service Configurator.
///////////////////////////////////////////////////////////////////////////
void
Net_Broker::reconfigure_services (void)
{
  ACE_TRACE ("Net_Broker::reconfigure_services");

  // Flag the main event loop that a reconfiguration should occur.
  // The next trip through the ACE_Reactor::run_event_loop()
  // should pick this up and cause a reconfiguration!
  // In other words, ACE_Service_Config will process the config file.
  ACE_Service_Config::reconfig_occurred ((sig_atomic_t) 1);
}

///////////////////////////////////////////////////////////////////////////
// Accepts a new connection from a client and calls the subclasse's
// handle_request() method to process it.
///////////////////////////////////////////////////////////////////////////
// Note, at this moment, only one concurrent connection to the Net_Broker
// is supported. In the future, we will spawn multiple threads to handle
// multiple connections.
// We must fix this and register different event handlers with the reactor.
// One with ACCEPT_MASK to accept new connections and several event handler
// objects with READ_MASK to read different streams. 
// But, when we do this, we must make sure that the code will run well with
// multiple threads. In particular, the Persistent_Repository is not
// multithread-safe, right now. But it's easy to fix it.
int 
Net_Broker::handle_input (ACE_HANDLE fd)
{
  ACE_TRACE ("Net_Broker::handle_input");

  // First check wether we are accepting a new connection or if this is more
  // data to an existing connection.

  if(fd == this->client_stream_.get_handle())
    return handle_request(); // we received more data to an existing connection.
  

  if(this->client_stream_.get_handle() != ACE_INVALID_HANDLE)
    return 0; // Do not accept this new connection yet, they'll have to wait.

  // If we reached this point, we have a new connection to accept.

  // Try to find out if the implementation of the reactor that we are
  // using requires us to reset the event association for the newly
  // created handle. This is because the newly created handle will
  // inherit the properties of the listen handle, including its event
  // associations.
  int reset_new_handle = reactor_->uses_event_associations ();

  if (this->acceptor_.accept (this->client_stream_, // stream
                              0, // remote address
                              0, // timeout
                              1, // restart
                              reset_new_handle  // reset new handler
                              ) == -1)
    return -1;

  // Now, register this client stream with the Reactor.
  if (reactor_->register_handler (this->client_stream_.get_handle(),
                                  this, 
                                  ACE_Event_Handler::READ_MASK) == -1)
    ACE_ERROR_RETURN ((LM_ERROR, 
                       "registering netbroker stream with ACE_Reactor\n"), 
                      -1);
  


  if (this->debug_)
    {
      ACE_DEBUG ((LM_DEBUG, "client_stream fd = %d\n",
		 this->client_stream_.get_handle ()));
      ACE_INET_Addr sa;
      if (this->client_stream_.get_remote_addr (sa) == -1)
	return -1;

      ACE_DEBUG ((LM_NOTICE, "Accepted reconfiguration connection from host %s at port %d\n", 
		sa.get_host_name (), sa.get_port_number ()));
    }

  return 0; 

}// handle_input()


