/******************************************************************************
*                                                                             *
*  File: Net_Broker.h                                                         *
*                                                                             *
*  Description: An abstract class for a Dynamic Service Configurator's        *
*               Network Broker.                                               *
*                                                                             *
*  Project: 2K - dynamically configurable ORB.                                *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************

   Revision Control Information

   $Revision: 1.1.1.2 $

   $Log: Net_Broker.h,v $
   Revision 1.1.1.2  2000/09/30 18:14:37  yamane
   Patches for Windows

   Revision 1.1.1.1  1999/10/12 00:43:30  f-kon
   Version compatible with TAO 1.0

   Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
   This new 2K source tree contains the most up-to-date version of ACE and TAO
   and including the dynamicTAO code as well as some 2K tests and preliminary
   versions of some of its high-level systems.


   Revision 1.6  1999/02/02 22:43:03  f-kon
   fixed errors with #includes

   Revision 1.5  1999/01/11 23:13:56  f-kon
   Upgrade to DCP v1.0 and improvements to the PersistentRepository.

   Revision 1.4  1998/10/06 18:16:24  mroman
   Version that works fine on NT, Win98 and compiles fine

   Revision 1.3  1998/08/30 14:56:37  f-kon
   First modifications to try to make it work in Windows

******************************************************************************/

#ifndef NETBROKER_H
#define NETBROKER_H

#include "ace/Service_Object.h"
#include "ace/SOCK_Stream.h"
#include "ace/SOCK_Acceptor.h"
#include "ace/INET_Addr.h"

#include "ACE_Persistent_Repository.h"

class ACE_Reactor;
class Dynamic_Service_Config;

class ACE_2K_Export Net_Broker : public ACE_Service_Object
{
  // = TITLE
  //    An abstract class defining a bridge between remote network clients and
  //    the local Dynamic Service Configurator.
public:
  Net_Broker (ACE_Reactor *reactor);
  virtual ~Net_Broker(void);

  virtual int run(); // called by the reactor thread to perform the event loop.
  virtual void reconfigure_services (void);

  // = Dynamic linking hooks.
  virtual int init (int argc, char *argv[]);
  virtual int init (int argc, char *argv[], const char *type);
  virtual int info (char **info_string, size_t length) const;
  virtual int fini (void);
  
  // = Scheduling hooks.
  virtual int suspend (void);
  virtual int resume (void);  

  void dump (void) const;
  // Dump the state of an object.
  
  ACE_ALLOC_HOOK_DECLARE;
  // Declare the dynamic allocation hooks.

protected:
  friend class Dynamic_Service_Config;
	
  ACE_SOCK_Stream client_stream_;
  ACE_SOCK_Acceptor acceptor_;
  ACE_Reactor *reactor_;
  ACE_thread_t my_main_thread_;

  int debug_;
  //  int signum_;
  static u_short DEFAULT_PORT_;

  ACE_Persistent_Repository *pers_rep_;
  Dynamic_Service_Config *dsc_;

  //  virtual int initialize(Dynamic_Service_Config *dsc) = 0; use this in the future
  virtual int initialize() = 0;
  
  ACE_INLINE void dynamic_service_config (Dynamic_Service_Config *dsc)
    { dsc_ = dsc; }

private:
  int open (const ACE_INET_Addr &sia);

  // = Demultiplexing hooks.
  virtual ACE_HANDLE get_handle () const;
  virtual int handle_input (ACE_HANDLE fd);
  virtual int handle_close (ACE_HANDLE fd, ACE_Reactor_Mask);
  virtual int handle_request() = 0;

};


#endif
