/******************************************************************************
*                                                                             *
*  File: Persistent_Repository.h                                              *
*                                                                             *
*  Description: An abstract class defining an interface to a hierarchical     *
*               repository of data.                                           *
*                                                                             *
*  Project: 2K - dynamically configurable ORB.                                *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************

   Revision Control Information

   $Revision: 1.1.1.1 $

   $Log: Changed the Persistent_Repository class into an abstract class. 
         Created a subclass of Persistent Repository based on ACE_OS file system$
******************************************************************************/

#ifndef PERSISTENT_REPOSITORY_H
#define PERSISTENT_REPOSITORY_H

class ACE_SOCK_Stream;

class Persistent_Repository
{
 public:

  Persistent_Repository (){};
  virtual ~Persistent_Repository (){};

  virtual int init (const char *rootDir)=0;

  virtual int get_contents (char *buf, int maxsize, const char *dir = 0)=0;
  virtual int create_dir (const char *path, const char *dirname)=0;
  virtual int delete_dir (const char *path, const char *dirname)=0;
  virtual int create_entry_from_stream (ACE_SOCK_Stream *input, 
                                        const char *path, 
                                        const char *entryname, 
                                        long len)=0;
  virtual int create_entry_from_buffer (char *input, 
                                        const char *path, 
                                        const char *entryname, 
                                        long len)=0;
  virtual int delete_entry (const char *path, const char *entryname)=0;

  virtual void build_pathname (char *pathname, const char *path, 
                       const char *entryname = 0)=0;

  ACE_INLINE char *root_dir () { return root_dir_; }

 protected:
  char *root_dir_;

};


#endif
