/* class Set
 * Provides routines for managing the history buffer
 */

#ifndef __SET_H_
#define __SET_H

class HistorySetElement; // forward declaration

class HistorySetElement
{
public:
  HistorySetElement(char *a, int b, int c)
    {
      ACE_OS::strcpy(hostname,a);
      port = b;
      seqno = c;
      next = NULL;
    }
  char hostname[32]; // name of the host
  int port;          // port number
  int seqno;         // sequence number
  HistorySetElement *next;  // pointer to next element in the list
};

class HistorySet 
{
public:
  HistorySet() { head = NULL; }

  // Adds <host,port> to the set. If it a new entry, it return -1. If an old
  // entry is over written, it returns the old seqno
  int update(char *host, int port, int seq)
    {
      ACE_DEBUG((LM_DEBUG,"Set: Update called with <%s:%d,%d>\n",
		 host, port, seq));
      if (this->head == NULL) 
	{
	  // there are no elements in the set
	  ACE_DEBUG((LM_DEBUG,"Set: Creating a new History Element\n"));
	  this->head = new HistorySetElement(host,port,seq);
	  return -1;
	}
      else 
	{
	  ACE_DEBUG((LM_DEBUG,"Set: Searching ... "));
	  // there is atleast one element in the set - check f it is the same
	  HistorySetElement *curr = this->head;
	  int found = 0;
	  do 
	    {
	      if ((ACE_OS::strcmp(curr->hostname,host) == 0) 
		  && (curr->port == port)) 
		{
		  found =1;
		  break;
		}
	      else if (curr->next == NULL)
		{
		  found = 0;
		  break;
		}
	      else
		curr = curr->next;
	    }
	  while (1);
	  if (found) 
	    {
	      ACE_DEBUG((LM_DEBUG," found\n"));
	      // set element already exists and is pointed to by curr
	      int ret = curr->seqno;
	      curr->seqno = seq;
	      return ret;
	    }
	  else 
	    {
	      ACE_DEBUG((LM_DEBUG," not found\n"));
	      // we have to make a new history entry
	      curr->next = new HistorySetElement(host,port,seq);
	      return -1;
	    }
	}
      
    }
  
private:
  HistorySetElement *head;
};

#endif
