/* SmartPacket.cpp
Implements the method for the SmartPacket Class
Done by Arun Viswanathan, Balaji Srinivasan and Ramesh Chandra UV
Last modified on Dec 15 1998.
*/

/* The "final version" of the SmartPacket format is as follows :
 
   hostname1\0port1\0hostname2\0port2\0...hostnamen\0portn\0\0AdjacencyMatrix\0\0DCPCommand1\0DCPCommand2\0...DCPCommandk\0\0(sourceHostname=\0)(seqNo=\0)hopcount\0
 
  
*/ 

#define ACE_NDEBUG 1
#define ACE_NLOGGING 1

#include "Net_Broker.h"
#include "SmartPacket.h"

SmartPacket::SmartPacket() 
{
  this->major_version = 0;
  this->minor_version = 1;
  this->incomingLen = 0;
}


// Initialize takes a stream and extracts information about the smartpacket

int SmartPacket::initialize(char * stream, int len)
{
  ACE_DEBUG((LM_DEBUG,"SmartPacket: Initializing new pkt of size %d\n",len));

  // allocate space for the stream
//   if (this->incoming != NULL)
//     delete incoming;
  //   incoming = new char[len];

  //  char incoming[512*1024]; // FIX: Must find a better solution for this!!!!!
  
  incomingLen = len;

  ACE_OS::memcpy(incoming, stream, len);

  int cur_ptr;
  if ((stream == NULL) || (!len))
    { 
      //initialize with void
      this->seqno = 0;
      this->num_dcp_cmds = 0;
      this->num_hosts = 0;
      return 0;
    }
 
  cur_ptr = get_hostnames(stream,len);
  ACE_DEBUG((LM_DEBUG,"SmartPacket: Hostnames end at offset %d\n",
	     cur_ptr));
  ACE_DEBUG((LM_DEBUG,"%d hostname(s) read\n",num_hosts));

  cur_ptr = get_adjacencymatrix(stream,len,cur_ptr);
  ACE_DEBUG((LM_DEBUG,"SmartPacket: Adjacency Matrix ends at offset %d\n",
	     cur_ptr));

  cur_ptr =  get_dcpcommand(stream, len,cur_ptr);
  ACE_DEBUG((LM_DEBUG,"SmartPacket: DCP commands end at offset %d\n",
	     cur_ptr));
  ACE_DEBUG((LM_DEBUG,"%d DCP command(s) read\n",num_dcp_cmds));
  this->endOfDCP = cur_ptr;

  cur_ptr =  get_source(stream,len,cur_ptr);
  ACE_DEBUG((LM_DEBUG,"SmartPacket: SeqNo ends at offset %d\n",
	     cur_ptr));
  return 0;
}


// This method gets hostnames from the smartpacket stream
int SmartPacket::get_hostnames(char * stream,int len)
{
  int cur_ptr = 0;
  char cur_hostname[ACE_MAX_FULLY_QUALIFIED_NAME_LEN] ;
  int cur_hostnamelen = 0;
  int nowhost = 1;
  num_hosts = 0;
  while (!( ( *(stream+cur_ptr) == '\0' ) && ( *(stream+cur_ptr+1) == '\0') ) )
    { 
//      if (stream[cur_ptr] == '\0')
// 	ACE_DEBUG((LM_DEBUG,"*"));
//       else
// 	ACE_DEBUG((LM_DEBUG,"%c",stream[cur_ptr]));
   
      if ( *(stream+cur_ptr) == '\0')
	{ 
	  //change and store attributes 
	  if (nowhost)  
	    { 
	      cur_hostname[cur_hostnamelen] = '\0';
              ACE_OS::strcpy(hostnames[num_hosts].name,cur_hostname);
	      ACE_DEBUG((LM_DEBUG,"Host name %s\n",cur_hostname));
            }
	  else  
	    { 
	      cur_hostname[cur_hostnamelen] = '\0';
              hostnames[num_hosts].port = atoi(cur_hostname);
	      ACE_DEBUG((LM_DEBUG,"Port %s\n",cur_hostname));
	      num_hosts++; // Changed
            }  
	  nowhost = 1 - nowhost;
	  cur_hostnamelen = 0;
	  cur_ptr++;
	}
      else 
	{ 
	  cur_hostname[cur_hostnamelen] = *(stream+cur_ptr);
          cur_hostnamelen++;
	  cur_ptr++;
	}
    }        
  cur_hostname[cur_hostnamelen] = '\0';
  hostnames[num_hosts].port = atoi(cur_hostname);
  ACE_DEBUG((LM_DEBUG,"Port %s\n",cur_hostname));
  num_hosts++; // Changed
    
  ACE_DEBUG((LM_DEBUG,"SmartPacket: There are %d hosts listed in the packet\n",
	     num_hosts));
  return (cur_ptr+2);
}



//This method gets the adjacency matrix from the stream
int SmartPacket::get_adjacencymatrix(char * stream,int len, int start_posn)
{
  int cur_ptr = start_posn;
  int i,j;
  
//   ACE_DEBUG((LM_DEBUG,"Adjacency Matrix: "));
//   for (i = 0; i < this->num_hosts * this->num_hosts; i++)
//     ACE_DEBUG((LM_DEBUG,"%c",stream[start_posn+i]));
//   ACE_DEBUG((LM_DEBUG,"\n"));

  for (i = 0; i < this->num_hosts; i++) 
    for (j = 0; j < this->num_hosts; j++)
      { 
	if (stream[cur_ptr++] == '1')
	  this->adjacency_matrix[i][j] = 1;
	else 
	  this->adjacency_matrix[i][j] = 0;
// 	ACE_DEBUG((LM_DEBUG,"A[%d][%d] = %d\n",i,j,
// 		   this->adjacency_matrix[i][j]));
      }
  return (cur_ptr+2);
}


//This packet gets the set of DCP commands in the smartpacket.
// special case is the uplocad implementation which has code as its parameter
int SmartPacket::get_dcpcommand(char *stream, int len, int start_posn)
{
  // (Dec 8) currently written under the assumption that no bad input arrives
  this->num_dcp_cmds = 0; // initially there are no DCP commands
  int state = 0; // start state for the parser's state machine
  char *currentOffset = stream + start_posn; // first character

 STATE_MACHINE:
  switch (state)
    {
    case 0: // start state
      if ((*currentOffset != '\r') && (*currentOffset != '\n') &&
	  (*currentOffset != '\0') && (*currentOffset != ' '))
	{
	  this->num_dcp_cmds++; // we have a new DCP command
	  this->dcp_cmd_len[this->num_dcp_cmds-1] = 1;
	  this->dcp_cmds[this->num_dcp_cmds-1][this->dcp_cmd_len[this->num_dcp_cmds-1]-1] = *currentOffset;
	  currentOffset++;
	  state = 1; // change to state 1
	}
      break;

    case 1: // reading in a DCP command 
      while ((*currentOffset != '\r') && (*currentOffset != '\n') &&
	     (*currentOffset != '\0') && (*currentOffset != ' '))
	{
	  // the command is still continuing
	  this->dcp_cmds[this->num_dcp_cmds-1][this->dcp_cmd_len[this->num_dcp_cmds-1]] = *currentOffset;
	  this->dcp_cmd_len[this->num_dcp_cmds-1]++;
	  currentOffset++;
	}
      // we have read in the entire command name 
      if ((*currentOffset == '\r') && (*(currentOffset+1) == '\n'))
	{
	  // we got a command like 'help' which has no args
	  this->dcp_cmds[this->num_dcp_cmds-1][this->dcp_cmd_len[this->num_dcp_cmds-1]] = '\r';
	  this->dcp_cmds[this->num_dcp_cmds-1][this->dcp_cmd_len[this->num_dcp_cmds-1]+1] = '\n';
	  this->dcp_cmd_len[this->num_dcp_cmds-1] += 2;
	  currentOffset += 3;
	  if (*currentOffset == '\0')
	    // we are done with the packet
	    return (int)(currentOffset + 1 - stream);
	  else
	    state = 0; // next command
	}
      else 
	{
	  // we got a blank, so the arguments are going to follow
	  this->dcp_cmds[this->num_dcp_cmds-1][this->dcp_cmd_len[this->num_dcp_cmds-1]] = *currentOffset;
	  this->dcp_cmd_len[this->num_dcp_cmds-1]++;
	  currentOffset++;
	  state = 2; // go on to state 2 to read the arguments
	}	  
      break;
      
    case 2: // have read the command name, now read the args - treat
            // upload implemetation separately
      if (ACE_OS::strncmp(this->dcp_cmds[this->num_dcp_cmds-1], 
			  "upload_implementation",
			  ACE_OS::strlen("upload_implementation")) == 0)
	{
	  // we have to treat upload_implementation separately
	  do
	    {
	      // read the service name and implementation name
	      this->dcp_cmds[this->num_dcp_cmds-1][this->dcp_cmd_len[this->num_dcp_cmds-1]] = *currentOffset;
	      this->dcp_cmd_len[this->num_dcp_cmds-1]++;
	      currentOffset++;
	    }
	  while (*currentOffset != '\r');

	  this->dcp_cmds[this->num_dcp_cmds-1][this->dcp_cmd_len[this->num_dcp_cmds-1]] = ' '; // replace the \r\n with b
	  currentOffset++; // skip over \r\n

	  this->dcp_cmds[this->num_dcp_cmds-1][this->dcp_cmd_len[this->num_dcp_cmds-1]] = '\t';
	  this->dcp_cmd_len[this->num_dcp_cmds-1]++;
	  currentOffset += 2; // skip over \n and \t
	  
	  int implLen = 0;
	  while (*currentOffset != ' ')
	    {
	      // read the length
	      this->dcp_cmds[this->num_dcp_cmds-1][this->dcp_cmd_len[this->num_dcp_cmds-1]] = *currentOffset;
	      implLen = implLen * 10 + (*currentOffset - '0');
	      this->dcp_cmd_len[this->num_dcp_cmds-1]++;
	      currentOffset++;

	    }
	  ACE_DEBUG((LM_DEBUG,"Implementation length = %d bytes\n",implLen));
	  currentOffset += 4; // skip over b\r\nb
	  this->dcp_cmds[this->num_dcp_cmds-1][this->dcp_cmd_len[this->num_dcp_cmds-1]++] = ' '; // put a space between length and implementation
	  
	  // read the actual implementation
	  for (int cnt = 0; cnt < implLen; cnt++)
	    {
	      this->dcp_cmds[this->num_dcp_cmds-1][this->dcp_cmd_len[this->num_dcp_cmds-1]++] = *currentOffset;
// 	      if (cnt < 200)
// 		if (*currentOffset == '\0')
// 		  ACE_DEBUG((LM_DEBUG,"NULL"));
// 		else
// 		  ACE_DEBUG((LM_DEBUG,"%c",*currentOffset));

	      currentOffset++;
	    }
	  this->dcp_cmds[this->num_dcp_cmds-1][this->dcp_cmd_len[this->num_dcp_cmds-1]++] = '\r'; 
	  this->dcp_cmds[this->num_dcp_cmds-1][this->dcp_cmd_len[this->num_dcp_cmds-1]++] = '\n';
	  currentOffset += 3; // skip \r\n\0
	}
      else
	{
	  while ((*currentOffset != '\r') && (*(currentOffset+1) != '\n'))
	    {
	      // read in the rest of a normal ASCII command
	      this->dcp_cmds[this->num_dcp_cmds-1][this->dcp_cmd_len[this->num_dcp_cmds-1]] = *currentOffset;
	      this->dcp_cmd_len[this->num_dcp_cmds-1]++;
	      currentOffset++;
	    }
	  
	  // read in the '\r' and '\n'
	  this->dcp_cmds[this->num_dcp_cmds-1][this->dcp_cmd_len[this->num_dcp_cmds-1]] = '\r';
	  this->dcp_cmds[this->num_dcp_cmds-1][this->dcp_cmd_len[this->num_dcp_cmds-1]+1] = '\n';
	  this->dcp_cmd_len[this->num_dcp_cmds-1] += 2;
	  currentOffset += 3;
	}
      // assuming no errors, the next one should be a '\0' 
      // which is why we can skip over it
      if (*currentOffset == '\0')
	// we are done with the DCP commands
	return (int)(currentOffset + 1 - stream);
      else
	state = 0; // next command
      break;
    }
  goto STATE_MACHINE;

  /* should not be reached */
  return 0;

}

// this method gets the source(root) host name. name is <machine name,port>
int SmartPacket::get_source(char *stream, int len, int start_posn)
{
  int cur_ptr = start_posn;
  char cur_hostname[ACE_MAX_FULLY_QUALIFIED_NAME_LEN];
  int cur_hostnamelen = 0;

  // THIS CODE IS TERRIBLY INNEFICIENT! MUST USE A LIBRARY FUNCTION!!! Fabio.

  while ( !((*(stream+cur_ptr)) == '\0'))
    {
     cur_hostname[cur_hostnamelen] = *(stream+cur_ptr);
     cur_hostnamelen++;
     cur_ptr++;
    }
  cur_hostname[cur_hostnamelen] = '\0';
  ACE_OS::strcpy(this->root.name, cur_hostname);
  cur_hostnamelen = 0;
  cur_ptr++;
  while ( !((*(stream+cur_ptr)) == '\0'))
    {
     cur_hostname[cur_hostnamelen] = *(stream+cur_ptr);
     cur_hostnamelen++;
     cur_ptr++;
    }
   
  cur_hostname[cur_hostnamelen] = '\0';
  root.port = ACE_OS::atoi(cur_hostname);
 cur_hostnamelen = 0;
  cur_ptr++;
  while ( !((*(stream+cur_ptr)) == '\0'))
    {
     cur_hostname[cur_hostnamelen] = *(stream+cur_ptr);
     cur_hostnamelen++;
     cur_ptr++;
    }
  cur_hostname[cur_hostnamelen] = '\0';
  seqno = ACE_OS::atoi(cur_hostname);

  // read the hop count
  this->hopCount = ACE_OS::atoi(stream+cur_ptr+1);
  ACE_DEBUG((LM_DEBUG,"Hop count on packet = %d\n",this->hopCount));

  return cur_ptr; // what is this supposed to return anyway?
}


// this method marshals the smartpacket into a stream
int SmartPacket::toStream(char* stream)
{
  // keep everything upto the DCP commands as before

  ACE_OS::memcpy(stream, incoming, this->endOfDCP);

  // append the sequence number and hop count
  int lenOfStream = this->endOfDCP;
  ACE_OS::strcpy(stream + lenOfStream,this->root.name);
  lenOfStream += ACE_OS::strlen(this->root.name) + 1;
  ACE_OS::sprintf(stream + lenOfStream,"%d",this->root.port);
  lenOfStream += ACE_OS::strlen(stream + lenOfStream) + 1;
  ACE_OS::sprintf(stream + lenOfStream,"%d",this->seqno);
  lenOfStream += ACE_OS::strlen(stream + lenOfStream) + 1;

  // make sure that hop count is incremented before sending
  ACE_OS::sprintf(stream + lenOfStream,"%d",this->hopCount + 1);
  this->hopCount++;
  lenOfStream += ACE_OS::strlen(stream + lenOfStream) + 1;
  stream[lenOfStream++] = '\0';
  stream[lenOfStream++] = '\0';
  return lenOfStream;
}


//returns the root host name
void SmartPacket::getRoot(char* name, int* port)
{
  ACE_OS::strcpy(name, root.name);
  *port = root.port;
}

//returns the sequence number. <rootname,seqno> uniquely identifies the packet
int SmartPacket::getSeqNo()
{
  return seqno;
}

//sets the root name , done by the root broker
void SmartPacket::setRoot(char *name, int port)
{
  ACE_OS::strcpy(root.name, name);
  root.port = port;
}

// sets seqno, done by the root broker
void SmartPacket::setSeqNo(int seqn)
{ 
  seqno = seqn; 
}

// returns the number of DCP commands
int SmartPacket::numDCPcmds()
{
  return num_dcp_cmds;
}

// gets the cmdno'th DCP commands.Each broker calls this repeatedly and 
// executes all the DCP commands locally
int SmartPacket::getDCPcmd(int cmdno, char* buf)
{
  ACE_OS::memcpy(buf, dcp_cmds[cmdno-1], dcp_cmd_len[cmdno-1]);
  return dcp_cmd_len[cmdno-1];
}

//returns the number of children of the broker in the graph
int SmartPacket::numChildren(char *parent, int port)
{
  int i = 0;
  ACE_INET_Addr addr1, addr2;

  for ( i = 0; i < this->num_hosts; i++) 
    {
      addr1.set(hostnames[i].port,hostnames[i].name);
      addr2.set(port,parent);      
      if ((addr1.get_ip_address() == addr2.get_ip_address())
	  && (hostnames[i].port == port))
	break;
    }
  if ( i == num_hosts) return -1;
  ACE_DEBUG((LM_DEBUG,"NumChildren: ORB %s:%d is the %d'th of %d ORBs\n",
	     parent, port, i+1, this->num_hosts));
  int j , num_child = 0;
  for ( j = 0; j < this->num_hosts ; j++)
    if (j != i) // we don't want to count the parent
      num_child += this->adjacency_matrix[i][j];
  return num_child;
}

//gets the child_no'th child in the graph of a broker. each node calls this
// repeatedly and sends the smartpacket to all the children
int SmartPacket::getchild(char *parent, int port, int child_no, char* buf, int* portno)
{
  int i = 0;
  ACE_INET_Addr addr1, addr2;

  for ( i = 0; i < this->num_hosts; i++) 
    {
      addr1.set(hostnames[i].port,hostnames[i].name);
      addr2.set(port,parent);      
      if ((addr1.get_ip_address() == addr2.get_ip_address())
	  && (port == hostnames[i].port))
	break;
    }
  if ( i == num_hosts) return -1;
  ACE_DEBUG((LM_DEBUG,"GetChild: ORB %s:%d is the %d'th of %d ORBs\n",
	     parent, port, i, this->num_hosts));
  int j , num_child = 0;
  for ( j = 0; j < num_hosts ; j++)
    { 
      num_child += this->adjacency_matrix[i][j];
      if (num_child == child_no) 
	{ 
	  ACE_DEBUG((LM_DEBUG,"getChild: ORB %s:%d is the %d'th child\n",
		     hostnames[j].name,hostnames[j].port,num_child));
	  ACE_OS::strcpy(buf, hostnames[j].name);
	  *portno = hostnames[j].port;
	  return 0;
	}
    }
  return -1;
}



void SmartPacket::myName(char *nme, int prt, char *myname, int *myport)
{
  ACE_DEBUG((LM_DEBUG,"My name is %s:%d and I am here to find my real name!\n",
	     nme,prt));

  ACE_DEBUG((LM_DEBUG,"These are the %d current hosts:\n",this->num_hosts));
  int i; // i will also be used outside of this loop
//   for (i = 0;i < this->num_hosts;i++)
//     ACE_DEBUG((LM_DEBUG,"\t%s:%d\n",hostnames[i].name,hostnames[i].port));

  ACE_INET_Addr addr1, addr2;
  for (i = 0; i < this->num_hosts; i++) 
    {
      ACE_DEBUG((LM_DEBUG,"%s:%d\n",hostnames[i].name,hostnames[i].port));
      addr1.set(hostnames[i].port,hostnames[i].name);
      addr2.set(prt,nme);      
      if ((addr1.get_ip_address() == addr2.get_ip_address())
	  && (prt == hostnames[i].port))
	{
	  ACE_OS::strcpy(myname,hostnames[i].name);
	  *myport = hostnames[i].port;
	  ACE_DEBUG((LM_DEBUG,"My real name is %s:%d\n",myname,*myport));
	  return;
	}
    }  
  ACE_DEBUG((LM_DEBUG,"SmartPacket: You are not listed in the smart packet\n"));
  ACE_OS::strcpy(myname,"");
  *myport = 0;

}

int SmartPacket::getHopCount() 
{
  return this->hopCount;
}



