
/* SmartPacketReply.cpp
Implements the method for the SmartPacketReply Class
Done by Arun Viswanathan, Balaji Srinivasan and Ramesh Chandra UV
Last modified on Dec 15 1998.
*/

#define ACE_NDEBUG 1
#define ACE_NLOGGING 1

#include "Net_Broker.h"
#include "SmartPacketReply.h"

SmartPacketReply::SmartPacketReply() 
{
    ACE_DEBUG((LM_DEBUG,"SmartPacketReply: Constructor called\n"));
//   this->sprstream = new char[MAX_REPLY_LEN];
}


//initializes the reply class from the stream. called when the first reply
//comes in.
int SmartPacketReply::initialize(char *stream, int len)
{
  ACE_DEBUG((LM_DEBUG,"SmartPacketReply: Initializing with %d bytes\n",
	     len));
  if ((stream == NULL) || (!len))
    { 
      this->sprstreamlen = 0;
      this->host_dcp_present = 0;
      return 0;
    }
  
  for ( int i = 0; i < len ; i++)
    this->sprstream[i] = stream[i];
  this->sprstreamlen = len;
  this->host_dcp_present = 0;
  return 0;
}


// sets the host of the reply packet.
void SmartPacketReply::sethost(char *buf, int port)
{
  ACE_DEBUG((LM_DEBUG,"SmartPacketReply: Setting ORB to <%s:%d>\n",
	     buf,port));
  ACE_OS::strcpy(cur_host.name, buf);
  this->cur_host.port = port;
}

// appends the reply from the child to the replypacket.
void SmartPacketReply::append(SmartPacketReply & incomingspr)
{ 
  ACE_DEBUG((LM_DEBUG,"SmartPacketReply: Appending another reply\n"));
  char *temp = new char[MAX_REPLY_LEN];

  // get the contents of the reply being appended
  int len = incomingspr.toStream(temp);

  // append it and update the stream length
  // note that we assume that the smart packet reply is terminated by
  // two \0 characters; so we retain only one

  // FIX: THE FOLLOWING IS EXTREMELY INEFFICIENT. USE LIB FUNCTIONS INSTEAD. Fabio.

  for (int i = 0; i < len-1; i++)
   this->sprstream[this->sprstreamlen++] = temp[i];

  delete temp;
}

//same as above, but using streams rather than SmartPacketReply class
void SmartPacketReply::append(char *incoming, int incominglen)
{
 // FIX: THE FOLLOWING IS EXTREMELY INEFFICIENT. USE LIB FUNCTIONS INSTEAD. Fabio.
  for (int i = 0; i < incominglen - 1 && sprstreamlen < MAX_REPLY_LEN;i++)
    this->sprstream[this->sprstreamlen++] = incoming[i];
}


// marshals the smartpacketreply class and creates a stream, which can be sent
// to the parent/client.
int SmartPacketReply::toStream(char *stream)
{
  int i;
  ACE_DEBUG((LM_DEBUG,"SmartPacketReply: Converting to character stream of length %d\n",
             this->sprstreamlen+1));

  for (i = 0; i < this->sprstreamlen; i++)
    stream[i] = this->sprstream[i];
  stream[i] = '\0'; // put in the extra \0 to indicate end of pkt

  // for debugging purposes dump the packet to screen
  for (i = 0; i < this->sprstreamlen+1;i++)
    if (stream[i] == '\0')
      ACE_DEBUG((LM_DEBUG,"*"));
    else
      ACE_DEBUG((LM_DEBUG,"%c",stream[i]));
  ACE_DEBUG((LM_DEBUG,"\n"));

  return this->sprstreamlen+1;
}

// appends the reply from the execution of the local DCP command
void SmartPacketReply::appendDCPoutput(const char* buf, int len)
{
  ACE_DEBUG((LM_DEBUG,"SmartPacketReply: Appending DCP output (%d bytes)\n",
	     len));
  int i;
  // check if we have already added the ORB name for the DCP command
  // outputs at this ORB
  if (!this->host_dcp_present)
    { // FIX: Use ACE_OS::strcpy instead.
      // we need to add the host name first ...
      for (i = 0;i < (int)ACE_OS::strlen(cur_host.name);i++)
	this->sprstream[this->sprstreamlen++] = cur_host.name[i];
      this->sprstream[this->sprstreamlen++] = '\0';
      
      // ... followed by the port number
      char portstr[5];
      int portnum = cur_host.port, portstrlen = 0;
      if (portnum == 0)
	portstr[portstrlen++] = '0';
      else 
	{
	  while (portnum != 0) // FIX: use ACE_OS::sprintf instead
	    {
	      int digit = portnum % 10;
	      portnum = portnum / 10;
	      portstr[portstrlen++] = digit + '0';
	    }
	}
      for (i = portstrlen - 1; i >= 0; i--)
	this->sprstream[this->sprstreamlen++] = portstr[i];
      this->sprstream[this->sprstreamlen++] = '\0';

      // make note that we have the host name added
      this->host_dcp_present = 1;
    }

  // FIX: THE FOLLOWING IS EXTREMELY INEFFICIENT. USE LIB FUNCTIONS INSTEAD. Fabio.

  // now we just add the output
  for (i = 0;i < len; i++)
    this->sprstream[this->sprstreamlen++] = buf[i];
  // adn terminate it with a \0
  this->sprstream[this->sprstreamlen++] = '\0';

}
