/******************************************************************************
*                                                                             *
*  File: SmartPacket_Broker.cpp                                               *
*                                                                             *
*  Description: A Dynamic Service Config Broker that implements SmartPackets
*                                                                             *
*  Project: 2K - dynamically configurable ORB - SmartPackets                  *
*  Author: Arun K Viswanathan (akviswan@uiuc.edu)                             *
*                                                                             *
*******************************************************************************

   Revision Control Information

   $Revision: 1.7 $

   $Log: SmartPacket_Broker.cpp,v $
   Revision 1.7  2000/09/30 18:17:02  yamane
   Patches for Windows

   Revision 1.5  1999/12/01 16:39:36  f-kon
   bug fixes

   Revision 1.4  1999/11/22 22:25:15  f-kon
   more optimizations and bug fixes

   Revision 1.3  1999/11/22 16:20:53  f-kon
   bug fixes and performance optimizations

   Revision 1.2  1999/11/08 23:57:44  f-kon
   did some performance optimizations

   Revision 1.1.1.3  1999/10/17 18:49:03  yamane
   Cleanup for native exception and linux support

   Revision 1.1.1.1  1999/10/12 00:43:30  f-kon
   Version compatible with TAO 1.0

   Revision 1.5.1.1  1999/09/22 00:25:19  yamane
   class Dynamic_Service_Config make the use of singleton
   to avoid the order problem of initialization

   Revision 1.5  1999/04/28 17:11:02  f-kon
   cleaned-up comments

   Revision 1.4  1999/04/28 17:03:31  f-kon
   fixed bug in upload_implementation of binary files

   Revision 1.3  1999/04/23 22:34:23  f-kon
   A new, more flexible, component encapsulating the JVM

   Revision 1.2  1999/04/07 20:09:06  f-kon
   added new DCP commands, upgrading to version 1.1

   Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
   This new 2K source tree contains the most up-to-date version of ACE and TAO
   and including the dynamicTAO code as well as some 2K tests and preliminary
   versions of some of its high-level systems.


   Revision 1.1.1.1  1998/11/28 21:37:00  akviswan
   Initial version containing ACE/TAO, dynamicTAO with smart packets,
   and RemoteControl

******************************************************************************/

#define ACE_NDEBUG 1
#define ACE_NLOGGING 1

#include "SmartPacket_Broker.h"

// do I need the following line?
//#define ACE_BUILD_DLL

ACE_ALLOC_HOOK_DEFINE(SmartPacket_Broker)

#include "ace/Reactor.h"
#include "ace/Service_Repository.h"
#include "ace/Dynamic_Service.h"

#include "DCP.h"
#include "SmartPacket.h"
#include "SmartPacketReply.h"
#include "Dynamic_Service_Config.h"
#include "Persistent_Repository.h"
#include "Simple_Strategy_Factory.h"

int 
SmartPacket_Broker::info (char **strp, size_t length) const
{
  ACE_TRACE ("SmartPacket_Broker::info");
  ACE_INET_Addr sa;
  char buf[BUFSIZ];

  if (this->acceptor_.get_local_addr (sa) == -1)
    return -1;
  
  ACE_OS::sprintf (buf, "%d/%s %s %d.%d", sa.get_port_number (), "tcp", 
                   "# SmartPacket Broker version", 
                   this->sp.major_version, this->sp.minor_version);

  if (*strp == 0 && (*strp = ACE_OS::strdup (buf)) == 0)
    return -1;
  else
    ACE_OS::strncpy (*strp, buf, length);
  return ACE_OS::strlen (buf);
}// info ()


///////////////////////////////////////////////////////////////////////////
// Calls the superclass init method to parse arguments, open the server 
// socket, and register us with the Reactor.
///////////////////////////////////////////////////////////////////////////
int
SmartPacket_Broker::initialize() 
{
  int argc = 0;
  char *argv[1] = {0}; 

  request_index_= 0;

  return this->init(argc, argv, "reconfiguration agents"); // hard-coded parameters by now.
}


///////////////////////////////////////////////////////////////////////////
// Handles one client request and returns.
///////////////////////////////////////////////////////////////////////////
int 
SmartPacket_Broker::handle_request ()
{
  int ret;
  char size[PKT_LEN_SIZE+1]; 
  
  if (!pktLenRead) // FIX: ugly global variable!!! Fabio.
    {
      ret = client_stream_.recv (size, PKT_LEN_SIZE);
      if (ret != PKT_LEN_SIZE)
        ACE_DEBUG((LM_ERROR,"Received only %d bytes instead of %d\n", ret, PKT_LEN_SIZE));
      if (ret < 0)
        return 0;
      size[PKT_LEN_SIZE] = '\0';
      pktLen = ACE_OS::atoi (size);
      pktLenRead = 1;
    }
  
//   if (!pktLenRead) // FIX: ugly global variable!!! Fabio.
//     {
//       // we are reading the pkt len. Add error handling code
//       char digit;
//       ret = client_stream_.recv(&digit, 1);
//       if (digit != '\0')
// 	pktLen = pktLen * 10 + (digit - '0');
//       else
// 	pktLenRead = 1;
//       return 0;
//     }

  ACE_DEBUG((LM_DEBUG,"Awaiting packet of length %d bytes\n",pktLen));
  ACE_DEBUG((LM_DEBUG,"Entering SmartPacket_Broker::handle_request\n"));
  char myName[ACE_MAX_FULLY_QUALIFIED_NAME_LEN];
  int myPort;
  char *p;
  int buflen;
  ACE_INET_Addr local_addr;
  // get local address 
  this->acceptor_.get_local_addr(local_addr);
  ACE_DEBUG((LM_DEBUG,"Local address: %s:%d\n", local_addr.get_host_name(),
	     local_addr.get_port_number()));

  // Read request from client.
//   ret = client_stream_.recv (request_ + request_index_, 
//                              sizeof request_ - request_index_);

  ret = client_stream_.recv (request_ + request_index_, 
                             ACE_MIN(pktLen - request_index_,
                                     ACE_DEFAULT_MAX_SOCKET_BUFSIZ));

  ACE_DEBUG((LM_DEBUG, "request_index_ = %d, ret = %d\n", request_index_, ret));

  switch (ret)
    {
    case -1:
      if (this->debug_)
        {
          ACE_DEBUG ((LM_ERROR, "%p\n", "handle_request recv"));
        }
      pktLen = 0; pktLenRead = 0;
      this->quit (0); // quit connection but do not send ACK.
      break;

    case 0:
      // recv returned 0, this is suspicious. 
      // Let's send an ACK to the client to check if the connection
      // is still alive.
      this->spr.initialize(NULL, 0);
      this->sp.myName((char*)local_addr.get_host_name(), 
		  local_addr.get_port_number(), myName, &myPort);
      this->spr.sethost(myName, myPort);
      this->spr.appendDCPoutput((char *)DCP_Reply::ACK,
				ACE_OS::strlen(DCP_Reply::ACK));
      buflen = this->spr.toStream(buf_);
      if (this->client_stream_.send_n (buf_,buflen) < 0)
        {
          this->quit (0); // quit connection but do not send ACK.
        }
      return 0;
      /* NOTREACHED */

    default:
      {
	request_index_ += ret;
	if (request_index_ < pktLen)
	  return 0;
	
        // We did receive a whole packet. Now I can safely initialize
	// the SmartPacket class and check for errors ...

	ACE_DEBUG((LM_DEBUG,"Entire SmartPacket received\n"
                   "Initializing SmartPacket object with stream of length %d bytes\n", 
                   pktLen));

        request_[pktLen] = '\0';// This shouldn't be necessary!

	if (sp.initialize(request_, pktLen) < 0)
	  {
	    // The packet was malformed.
	    ACE_DEBUG((LM_DEBUG,"SmartParket initialization was unsuccessful\n"));
	    this->spr.initialize(NULL, 0);
	    this->sp.myName((char*)local_addr.get_host_name(), 
			    local_addr.get_port_number(), myName, &myPort);
	    this->spr.sethost(myName, myPort);
	    this->spr.appendDCPoutput((char *)DCP_Reply::NACK,
				      ACE_OS::strlen(DCP_Reply::NACK));
	    int blen = this->spr.toStream(buf_);
	    if (this->client_stream_.send_n(buf_, blen) < 0)
	      {
		this->quit (0); // quit connection but don't send NACK
	      }
	    request_index_ = 0;
	    this->quit (0);
	    return 0;
	  }
	// ... and reset the request_index_ for the next packet
        request_index_ = 0;

	ACE_DEBUG((LM_DEBUG,"SmartPacket initialization successful\n"));


	/*************** Check if packet has been received before ***********/
	// TO DO: Check for duplicates
	int port, seq, oldseq;
	this->sp.getRoot (buf_, &port);
	seq = this->sp.getSeqNo ();
	ACE_DEBUG((LM_DEBUG,"Root ORB = %s:%d, SeqNo = %d\n", buf_, port,seq));
	
	if (seq == 0) 
	  {
	    // this is coming from the GUI, so I have to add my seqno
	    
	    ACE_DEBUG((LM_DEBUG,"Resetting root ORB = %s:%d SeqNo = %d\n",
		       local_addr.get_host_name(), 
		       local_addr.get_port_number(),sequenceCount));
	    this->sp.setRoot((char*)local_addr.get_host_name(),
			     local_addr.get_port_number());
	    this->sp.setSeqNo(sequenceCount);
	    
	    seq = sequenceCount;
	    sequenceCount++;

	    // get the new <host:port,seqno>
	    this->sp.getRoot(buf_,&port);
	    seq = this->sp.getSeqNo();
	  }

	ACE_DEBUG((LM_DEBUG,"Adding <%s:%d,%d> to the history set\n",
		   buf_,port,seq));
	oldseq = history.update(buf_,port,seq); // update the history DS
	ACE_DEBUG((LM_DEBUG,
		   "Old sequence number returned by history update = %d\n",
		   oldseq));

	if (oldseq >= 0) 
	  if (oldseq == seq) // duplicate packet - send empty message
	    {
	      ACE_DEBUG((LM_DEBUG,"Duplicate packet!\n"));
	      ACE_Time_Value tv(1,0);
	      //this->client_stream_.send_n("\0\0\0",3,0,&tv); 
	      this->quit(0);
              pktLen = 0; pktLenRead = 0; request_index_ = 0;
	      return 0;
	    }

	// Why do we need to do this signal_handler stuff????? Fabio.
        // Note that ACE_Reactor::instance is not necessarily the reactor 
        // we use. 
	ACE_Event_Handler *old_signal_handler = 0;
	ACE_Reactor::instance ()->register_handler (SIGPIPE, this, 0,
						    &old_signal_handler);

        /*************** Process SmartPacket Requests Locally ***************/

	this->spr.initialize(NULL, 0);
	this->sp.myName((char*)local_addr.get_host_name(), 
			local_addr.get_port_number(), myName, &myPort);
	this->spr.sethost(myName, myPort);

	ACE_DEBUG((LM_DEBUG,"Number of DCP commands = %d\n", sp.numDCPcmds()));

	for (int i = 0;i < sp.numDCPcmds(); i++)
	  {
	    int splen = this->sp.getDCPcmd(i+1,buf_);
	    ACE_DEBUG((LM_DEBUG,"Received a DCP command %d bytes long\n",
		       splen));

	    buf_[splen - 2] = '\0'; // snip off the \r\n
            ACE_DEBUG((LM_DEBUG,"DCP command %d\n",i+1/*,buf_*/));
	    
	    // The following chain of if's must be sorted so that common 
	    // requests come first.

	    if (ACE_OS::strncmp (buf_, "help", ACE_OS::strlen("help")) == 0)
	      this->help ();

	    else if (!ACE_OS::strncmp (buf_, DCP::LIST_CATEGORIES, 
				       ACE_OS::strlen (DCP::LIST_CATEGORIES)))
	      this->list_categories();

	    else if (!ACE_OS::strncmp (buf_, DCP::LIST_IMPLEMENTATIONS, 
				       ACE_OS::strlen (DCP::LIST_IMPLEMENTATIONS)))
	      this->list_implementations (buf_ + ACE_OS::strlen (DCP::LIST_IMPLEMENTATIONS));

	    else if (!ACE_OS::strncmp (buf_, DCP::LIST_LOADED_IMPLEMENTATIONS, 
				       ACE_OS::strlen (DCP::LIST_LOADED_IMPLEMENTATIONS)))
	      this->list_loaded_implementations ();

	    else if (!ACE_OS::strncmp (buf_, DCP::GET_IMPL_INFO, 
				       ACE_OS::strlen (DCP::GET_IMPL_INFO)))
	      this->get_impl_info (buf_+ACE_OS::strlen (DCP::GET_IMPL_INFO)); 

	    else if (!ACE_OS::strncmp (buf_, DCP::LIST_DOMAIN_COMPONENTS, 
				       ACE_OS::strlen (DCP::LIST_DOMAIN_COMPONENTS)))
	      this->list_domain_components ();

	    else if (!ACE_OS::strncmp (buf_, DCP::LIST_HOOKS, 
				       ACE_OS::strlen (DCP::LIST_HOOKS)))
	      this->list_hooks (buf_+ACE_OS::strlen (DCP::LIST_HOOKS));

            else if (!ACE_OS::strncmp (buf_, DCP::GET_COMP_INFO, 
                                       ACE_OS::strlen (DCP::GET_COMP_INFO)))
                this->get_comp_info (buf_+ACE_OS::strlen (DCP::GET_COMP_INFO));

	    else if (!ACE_OS::strncmp (buf_, DCP::LOAD_IMPLEMENTATION, 
				       ACE_OS::strlen (DCP::LOAD_IMPLEMENTATION)))
	      {
		// VERIFY: value of p is end of request, does it have the correct value here?
		p = buf_ + ACE_OS::strlen(buf_);
		this->load_implementation (buf_+ACE_OS::strlen(DCP::LOAD_IMPLEMENTATION), p);
	      }

	    else if (!ACE_OS::strncmp (buf_, DCP::HOOK_IMPLEMENTATION, 
				       ACE_OS::strlen (DCP::HOOK_IMPLEMENTATION)))
	      this->hook_implementation (buf_+ACE_OS::strlen (DCP::HOOK_IMPLEMENTATION));

	    else if (!ACE_OS::strncmp (buf_, DCP::SUSPEND_IMPLEMENTATION, 
				       ACE_OS::strlen (DCP::SUSPEND_IMPLEMENTATION)))
	      this->suspend_implementation(buf_+ACE_OS::strlen(DCP::SUSPEND_IMPLEMENTATION));

	    else if (!ACE_OS::strncmp (buf_, DCP::RESUME_IMPLEMENTATION, 
				       ACE_OS::strlen (DCP::RESUME_IMPLEMENTATION)))
	      this->resume_implementation (buf_+ACE_OS::strlen (DCP::RESUME_IMPLEMENTATION));

	    else if (!ACE_OS::strncmp (buf_, DCP::REMOVE_IMPLEMENTATION, 
				       ACE_OS::strlen (DCP::REMOVE_IMPLEMENTATION)))
	      this->remove_implementation (buf_+ACE_OS::strlen (DCP::REMOVE_IMPLEMENTATION));

 	    else if (!ACE_OS::strncmp (buf_, DCP::CONFIGURE_IMPLEMENTATION, 
				       ACE_OS::strlen(DCP::CONFIGURE_IMPLEMENTATION)))
	      this->configure_implementation (buf_+ ACE_OS::strlen(DCP::CONFIGURE_IMPLEMENTATION));

	    else if (!ACE_OS::strncmp (buf_, DCP::UPLOAD_IMPLEMENTATION, 
				       ACE_OS::strlen(DCP::UPLOAD_IMPLEMENTATION)))
	      this->upload_implementation (buf_ + ACE_OS::strlen(DCP::UPLOAD_IMPLEMENTATION));

	    else if (!ACE_OS::strncmp (buf_, DCP::DELETE_IMPLEMENTATION, 
				       ACE_OS::strlen(DCP::DELETE_IMPLEMENTATION)))
	      this->delete_implementation (buf_ + ACE_OS::strlen(DCP::DELETE_IMPLEMENTATION));

	    else if (ACE_OS::strcmp (buf_, DCP::RESTART) == 0)
	      this->restart ();

	    /************************     QUIT      ************************/
	    else if (ACE_OS::strcmp (buf_, "quit") == 0)
	      this->quit (1);

	    else if (ACE_OS::strlen (buf_) == 0){/* just ignore it */ }

	    else // unknown request
	      {
		if (this->debug_)
		  ACE_DEBUG ((LM_ERROR, "SmartPacket Broker received unknown request.\n"));
		
		this->spr.appendDCPoutput ((char *)DCP_Reply::UNKNOWN_REQUEST, 
                                           ACE_OS::strlen (DCP_Reply::UNKNOWN_REQUEST));
	      }
	  } // end of for loop

	ACE_DEBUG((LM_DEBUG,"Local execution of the SmartPacket complete\n"));

	/*************** Distribute Smart Packet to Neighbours ************/
	char hostname[256];

        // IMPROVE: The following is unnecessary. Just leave a space of size PKT_LEN_SIZE=10
	// add packet length and '\0' to head of packet
	ACE_OS::sprintf(buf_,"%10d",pktLen);
	ACE_DEBUG((LM_DEBUG,"Adding packet length %d to head of stream %s\n",
		   pktLen, buf_));
	int pktLenFieldLen = ACE_OS::strlen(buf_);

	// convert packet to stream
	int buf_len = sp.toStream(buf_ + pktLenFieldLen); 

        // We may have the wrong packet length in the beginning of the buffer,
        // so fix it. Fabio.
        char deletedChar = buf_[pktLenFieldLen];
        ACE_OS::sprintf(buf_,"%10d", buf_len);
        buf_[pktLenFieldLen] = deletedChar;

        for (int k = 0; buf_[k] == ' '; k++) // replace leading spaces by 0s.
          buf_[k] = '0';
                
	ACE_DEBUG((LM_DEBUG,"Fixed packet length to %d: <%s>.\n", 
		   buf_len, buf_));

	int numchld = sp.numChildren((char*)local_addr.get_host_name(),
				     local_addr.get_port_number());
	ACE_DEBUG((LM_DEBUG,"Number of neighbours = %d\n",numchld));

	// send the smart packet to each of the neighbours

	ACE_DEBUG((LM_DEBUG,"Hop count %d => ", this->sp.getHopCount()));

	double timeout = (double) SmartPacket_Broker::TIMEOUT;

	for (int c = 0; c < this->sp.getHopCount() - 1; c++)
	  timeout /= 2;
	long sec = (long)timeout, usec = (long)((timeout - sec) * 1.0E6);
	ACE_DEBUG((LM_DEBUG," Timeout: %f %l %l\n",timeout,sec,usec));
 	ACE_Time_Value tval(sec,usec);

	int j;
	for (j = 1; j <= numchld; j++)
	  {
	    sp.getchild((char*)local_addr.get_host_name(), 
			local_addr.get_port_number(),j,hostname,&port);
	    ACE_DEBUG((LM_DEBUG,"Forwarding packet of size %d to %s:%d\n",
		       buf_len + pktLenFieldLen, hostname, port));
		       
	    ACE_INET_Addr remote_addr(port,hostname);
	    // create a stream - see ace/SOCK_Connector.h
	    // we can add a time out to this if required
            //ACE_Time_Value wfc(10);
	    if (connector[j].connect(streams[j], remote_addr, &tval) < 0)
	      {
		ACE_DEBUG((LM_DEBUG,"Failed to connect to neighbour %d\n",j));
		status[j] = 0;
	      }
	    else 
	      {
		status[j] = 1;
		streams[j].send_n(buf_, buf_len + pktLenFieldLen);
	      }
	  }
	    
	// TO DO: get back replies from each of the neighbours
	// Choose the timeout appropriately!

        // IMPROVE: Note that this blocks waiting for each neighbour at a time.
        //          Modify it so it processes in a FIFO way.
        
//	ssize_t numread;

	//SmartPacketReply tmp;
	for (j = 1; j <= numchld; j++) 
	  {
	    if (status[j] == 1)
              {
                ACE_DEBUG((LM_DEBUG,"Waiting for reply from neighbour %d\n",j));
                buflen = streams[j].recv/*_n*/ (buf_, 
                                                ACE_DEFAULT_MAX_SOCKET_BUFSIZ/*sizeof(buf_)*/, 
                                                0, 0/*&tval*/);
                if (buflen < 0)
                  ACE_OS::perror("Could not receive reply from neighbour");
                else
                  ACE_DEBUG((LM_DEBUG,"Got a reply of length %d bytes from neighbour %d\n",buflen,j));
              }
            else 
	      buflen = 0;

	    if (buflen >= 3)
	      {
		// BUG/FEATURE: Ignore small packets
		if (!((buf_[buflen-1] == '\0') && (buf_[buflen-2] == '\0')
		      && (buf_[buflen-3] == '\0')))
		  {
		    // ORB will just return "\0\0\0" if packet rcvd 2nd time 
		    // tmp.initialize(buf_,buflen);
		    // this->spr.append(tmp);
		    this->spr.append(buf_,buflen);
		  }
	      }
            status[j] =0;
            streams[j].close();
	  }

	// return the reply to parent
	buflen = this->spr.toStream(buf_);
	ACE_DEBUG((LM_DEBUG,"Sending a reply of length %d bytes\n",buflen));
	int numsnt = this->client_stream_.send_n(buf_,buflen);
	ACE_DEBUG((LM_DEBUG,"%d bytes sent\n",numsnt));
	this->quit(0);

	// reset the packet length and state as reading packet length (!)
	pktLen = 0;
	pktLenRead = 0;

	// Restore existing SIGPIPE handler
	ACE_Reactor::instance()->register_handler(SIGPIPE, old_signal_handler);
      } // default for switch
    } // switch
  
  return 0;
}// handle_request()



///////////////////////////////////////////////////////////////////////////
// Determine all the loaded implementations offered by this daemon and
// return the information back to the client.
///////////////////////////////////////////////////////////////////////////
void
SmartPacket_Broker::list_loaded_implementations (void)
{
  int listed_something = 0;
  ACE_TRACE ("SmartPacket_Broker::list_loaded_implementations");
  ACE_DEBUG((LM_DEBUG,"'List loaded implementations' called\n"));
  ACE_Service_Repository_Iterator sri (*ACE_Service_Repository::instance ());

  buf_[0] = '\0';  //  ACE_OS::sprintf(buf_,"");
  for (const ACE_Service_Type *sr; 
       sri.next (sr) != 0; 
       sri.advance ())
    {
      ACE_OS::strncat(buf_, sr->name (), ACE_OS::strlen (sr->name ()));
      listed_something++;
    }

  if (listed_something == 0)
    {
      ACE_DEBUG((LM_DEBUG,"There are no loaded implementations\n"));
      this->spr.appendDCPoutput((char *)DCP_Reply::EMPTY, 
				ACE_OS::strlen(DCP_Reply::EMPTY));
    }
  else
    {
      ACE_DEBUG((LM_DEBUG,"Appending output of 'List loaded implementations' (%d bytes)\n",
                 ACE_OS::strlen(buf_)));
      this->spr.appendDCPoutput(buf_,ACE_OS::strlen(buf_));
    }
}// list_loaded_implementations()


///////////////////////////////////////////////////////////////////////////
// Sends a help message containing the protocol version number and the 
// URL of the proocol specification.
///////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::help () 
{
  ACE_DEBUG((LM_DEBUG,"'Help' called\n"));
  int size = 
    ACE_OS::sprintf (buf_, "SmartPacket version %d.%d.\r\n"
                     "Visit http://www.ews.uiuc.edu/~akviswan/SmartPackets.html"
                     " to get the SmartPacket specification.\r\n", 
                     this->sp.major_version, this->sp.minor_version);
  
#define help_msg "The following operations are currently supported:\r\n" \
    "\tlist_categories\r\n" \
    "\tlist_implementations <category_name>\r\n" \
    "\tlist_loaded_implementations\r\n" \
    "\tget_impl_info <loaded_implementation>\r\n" \
    "\tlist_domain_components\r\n" \
    "\tlist_hooks <component_name>\r\n" \
    "\tget_comp_info          <component_name>\r\n" \
    "\tload_implementation    <category_name> <implementation_name> [<init_parameters>]\r\n" \
    "\thook_implementation    <loaded_implementation> <component_name> <hook_name>\r\n" \
    "\tsuspend_implementation <loaded_implementation>\r\n" \
    "\tresume_implementation  <loaded_implementation>\r\n" \
    "\tremove_implementation  <loaded_implementation>\r\n" \
    "\tconfigure_implementation  <loaded_implementation> <configuration_string>\r\n" \
    "\tupload_implementation  <category_name> <implementation_name>\r\n" \
    "\t                       <length> CR/LF <binary-code>\r\n" \
    "\tdelete_implementation  <category_name> <implementation_name>\r\n" \
    "\trestart\r\n" \
    "\tquit\r\n" \
    "\thelp\r\n\r\n"
    
  ACE_OS::strcat(buf_,help_msg);
  ACE_DEBUG((LM_DEBUG,"Appending output of 'Help' (%d bytes)\n",
	     ACE_OS::strlen(buf_)));
  this->spr.appendDCPoutput(buf_,ACE_OS::strlen(buf_));
  
}// help()


///////////////////////////////////////////////////////////////////////////
// Send an ACK, remove this handler from the reactor and closes the stream
///////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::quit (int sendACK)
{
  ACE_DEBUG((LM_DEBUG,"'Quit' called with argument %d\n",sendACK));
  if (sendACK)
    {
      this->spr.appendDCPoutput((char*)DCP_Reply::ACK,
			  ACE_OS::strlen(DCP_Reply::ACK));
    }
   // remove this handler from the reactor 
   // Note: this DONT_CALL is important to sign that we don't want
   // remove_handler to close the event_handler, i.e., ourselves.
   this->reactor_->remove_handler
     (this->client_stream_.get_handle (),
      ACE_Event_Handler::READ_MASK |
      ACE_Event_Handler::DONT_CALL );
  
   // close the client stream and set handle to ACE_INVALID_HANDLE
   if (this->client_stream_.close () == -1 && this->debug_)
     {
       ACE_DEBUG ((LM_ERROR, "%p\n", "close"));
     }
}// quit()


/////////////////////////////////////////////////////////////////////////
// Sets arg1, arg2, arg3, and arg4 to point to the space-separated words
// in the request strings. It might ignore additional words if arg2,
// arg3, or arg4 is 0.  
// Should return -1 if an error occurs (by now, it returns 0 always).
/////////////////////////////////////////////////////////////////////////
int 
SmartPacket_Broker::get_params (char *request, char **arg1, char **arg2, 
                        char **arg3, char **arg4)
{
  *arg1 = ACE_OS::strtok (request, " \t");
  if(arg2)
    {
      *arg2 = ACE_OS::strtok (NULL, " \t");
      if(arg3)
        {
          *arg3 = ACE_OS::strtok (NULL, " \t");
          if(arg4)
            {
              *arg4 = ACE_OS::strtok (NULL, " \t");
            }
        }
    }
  return 0;
}// get_params()

///////////////////////////////////////////////////////////////////////////
// List categories in the persistent repository. By now, it is implemented by
// listing all the entities in the repository root directory.
// It sends a string containing the category names separated by "\r\n". 
// The list of categories is terminated by "\r\n\r\n".
///////////////////////////////////////////////////////////////////////////
int
SmartPacket_Broker::list_categories ()
{
  ACE_DEBUG((LM_DEBUG,"'List categories' called\n"));
  if (pers_rep_->get_contents (buf_, BUFSIZE_) == -1)
    {
      this->spr.appendDCPoutput((char*)DCP_Reply::NACK, 
				ACE_OS::strlen(DCP_Reply::NACK));
    }

  else if (buf_[0] == '\0')
    {
      ACE_DEBUG((LM_DEBUG,"There are no categories\n"));
      this->spr.appendDCPoutput((char*)DCP_Reply::EMPTY, 
				ACE_OS::strlen(DCP_Reply::EMPTY));
    }
  else
    {
      ACE_DEBUG((LM_DEBUG,"Appending output of 'List categories' (%d bytes)\n",
		 ACE_OS::strlen(buf_)));
      this->spr.appendDCPoutput(buf_, ACE_OS::strlen(buf_));
    }
  
  return 0;
}// list_categories ()

///////////////////////////////////////////////////////////////////////////
// List the implementations of a particular category in the persistent
// repository. By now, it is implemented by listing all the entities in
// the subdirectory named after the category in the repository root
// directory.  It sends a string containing the implementation names
// separated by "\r\n".  The list is terminated by "\r\n\r\n".
///////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::list_implementations (char *request)
{
  ACE_DEBUG((LM_DEBUG,"'List implementations' called with parameter %s\n",
	     request));
  char *category;

  this->get_params (request, &category);

  if(category == 0)
    {
      this->spr.appendDCPoutput((char*)DCP_Reply::MALFORMED_REQUEST, 
				ACE_OS::strlen(DCP_Reply::MALFORMED_REQUEST));
      return;
    }
  
  if (this->pers_rep_->get_contents ((char *)buf_, BUFSIZE_, category) != -1)
    {
      if (buf_[0] == '\0')
        {
	  ACE_DEBUG((LM_DEBUG,"There are no implementations\n"));
          this->spr.appendDCPoutput((char*)DCP_Reply::EMPTY, 
				    ACE_OS::strlen(DCP_Reply::EMPTY));
        }
      else
        {
	  ACE_DEBUG((LM_DEBUG,"Appending output of 'List implementations' (%d bytes)\n",
                     ACE_OS::strlen(buf_)));
          this->spr.appendDCPoutput(buf_, ACE_OS::strlen(buf_));
        }
    }
  else // could not find entity
    {
      this->spr.appendDCPoutput((char*)DCP_Reply::UNKNOWN_ENTITY, 
				ACE_OS::strlen(DCP_Reply::UNKNOWN_ENTITY));
    }
  
}// list_implementations ()

///////////////////////////////////////////////////////////////////////////
// List the components registered with the DomainConfigurator.
// It sends a string containing the componentnames
// separated by "\r\n".  The list is terminated by "\r\n\r\n".
///////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::list_domain_components ()
{
  ACE_DEBUG((LM_DEBUG,"'List domain_components' called\n"));
  Dynamic_Service_Config::instance()->get_components((char *)buf_, BUFSIZE_);
  
  if (buf_[0] == '\0')
    {
      ACE_DEBUG((LM_DEBUG,"There are no domain_components\n"));
      this->spr.appendDCPoutput((char*)DCP_Reply::EMPTY, 
				ACE_OS::strlen(DCP_Reply::EMPTY));
    }
  else
    {
      ACE_DEBUG((LM_DEBUG,"Appending output of 'List domain_components' (%d bytes)\n",
                 ACE_OS::strlen(buf_)));
      this->spr.appendDCPoutput(buf_, ACE_OS::strlen(buf_));
    }  
}// list_domain_components ()


///////////////////////////////////////////////////////////////////////////
// List the hooks of a given component registered with the 
// domain configurator.
// It sends a string containing the implementation names
// separated by "\r\n".  The list is terminated by "\r\n\r\n".
///////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::list_hooks (char *request)
{
  ACE_DEBUG((LM_DEBUG,"'List hooks' called with argument %s\n",request));
  char *component;

  this->get_params (request, &component);

  if(component == 0)
    {
      this->spr.appendDCPoutput((char*)DCP_Reply::MALFORMED_REQUEST, 
				ACE_OS::strlen(DCP_Reply::MALFORMED_REQUEST));
      return;
    }
  
  if (Dynamic_Service_Config::instance()->get_hooks (component, (char *)buf_, BUFSIZE_) == -1)
    {
      this->spr.appendDCPoutput((char*)DCP_Reply::UNKNOWN_ENTITY, 
				ACE_OS::strlen(DCP_Reply::UNKNOWN_ENTITY));
    }
  else
    {
      if (buf_[0] == '\0')
        {
          this->spr.appendDCPoutput((char*)DCP_Reply::EMPTY, 
				    ACE_OS::strlen(DCP_Reply::EMPTY));
        }
      else
        {
          this->spr.appendDCPoutput(buf_, ACE_OS::strlen(buf_));
        }
    }

}// list_hooks ()


//////////////////////////////////////////////////////////////////////////////
// Sends to the client an information string about the given domain component.
//////////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::get_comp_info (char *request)
{
  char *component;

  this->get_params (request, &component);

  if(component == 0)
    {
      this->spr.appendDCPoutput((char*)DCP_Reply::MALFORMED_REQUEST, 
                                ACE_OS::strlen(DCP_Reply::MALFORMED_REQUEST));
      return;
    }
  
  if (Dynamic_Service_Config::instance()->get_comp_info (component, (char*)buf_, BUFSIZE_) == -1)
    {
      this->spr.appendDCPoutput((char*)DCP_Reply::UNKNOWN_ENTITY, 
				ACE_OS::strlen(DCP_Reply::UNKNOWN_ENTITY));
    }
  else
    {
      if (buf_[0] == '\0')
        {
          this->spr.appendDCPoutput((char*)DCP_Reply::EMPTY, 
                                    ACE_OS::strlen(DCP_Reply::EMPTY));
        }
      else
        {
          this->spr.appendDCPoutput(buf_, ACE_OS::strlen(buf_));
        }
    }

}// get_comp_info ()


////////////////////////////////////////////////////////////////////////////
// Returns the name of the component attached to a given hook.
////////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::get_hooked_comp (char *request)
{
 char *componentName;
 char *hookName;
 this->get_params (request, &componentName, &hookName);

 if (componentName == 0 || hookName == 0)
   {
     this->spr.appendDCPoutput ((char*)DCP_Reply::MALFORMED_REQUEST, 
                                ACE_OS::strlen(DCP_Reply::MALFORMED_REQUEST));
     return;
   }
 
 const char *msg = Dynamic_Service_Config::instance()->get_hooked_comp (hookName, componentName);
 
 if (msg != 0)
   {
     this->spr.appendDCPoutput (msg, ACE_OS::strlen(msg));
   }
  else
   this->spr.appendDCPoutput ((char *)DCP_Reply::NACK, ACE_OS::strlen(DCP_Reply::NACK));

}// get_hooked_component ()

////////////////////////////////////////////////////////////////////////////
// Adds a new hook to a component
////////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::add_hook (char *request) 
{
 char *componentName;
 char *hookName;
 this->get_params (request, &componentName, &hookName);

 if (componentName == 0 || hookName == 0)
   {
      this->spr.appendDCPoutput ((char*)DCP_Reply::MALFORMED_REQUEST, 
                                ACE_OS::strlen(DCP_Reply::MALFORMED_REQUEST));
     return;
   }
 
  this->AckOrNack (Dynamic_Service_Config::instance()->add_hook (componentName, hookName));
 
}// add_hook ()


////////////////////////////////////////////////////////////////////////////
// Removes a hook from a component
////////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::remove_hook (char *request) 
{
 char *componentName;
 char *hookName;
 this->get_params (request, &componentName, &hookName);

 if (componentName == 0 || hookName == 0)
   {
     this->spr.appendDCPoutput ((char*)DCP_Reply::MALFORMED_REQUEST, 
                                ACE_OS::strlen(DCP_Reply::MALFORMED_REQUEST));
     return;
   }
 
  this->AckOrNack (Dynamic_Service_Config::instance()->delete_hook (componentName, hookName));
 
}// remove_hook ()


/////////////////////////////////////////////////////////////////////////////
// Sends information about a given loaded component stored in the ACE service
// repository.
/////////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::get_impl_info (char *request)
{
 char *implementationName;
 char buf[BUFSIZ];
 int len;
 this->get_params (request, &implementationName);
 
 if(implementationName == 0)
   {
     this->spr.appendDCPoutput((char*)DCP_Reply::MALFORMED_REQUEST, 
                               ACE_OS::strlen(DCP_Reply::MALFORMED_REQUEST));
     return;
   }

 // Now, an ugly hack to recognize ACE Services that are not 2K components: 2K
 // components must have the ":" to separate the category name from the version
 // number. If it doesn't have the ":", we assume it's not a 2K component.
 if (!strchr(implementationName, ':')) 
   {
     // Not a 2K component.
     const ACE_Service_Type *srvc = 0;
     int ret = ACE_Service_Repository::instance ()->find(implementationName, 
                                                         &srvc, 1);
     if (ret == -1 || srvc == 0) // Didn't find service in the repository.
       {
         this->spr.appendDCPoutput((char*)DCP_Reply::UNKNOWN_ENTITY, 
                                   ACE_OS::strlen(DCP_Reply::UNKNOWN_ENTITY));
         return;
       }

     // The service was found, get its info.
     char *bufP = buf;
     len = srvc->type ()->info (&bufP, BUFSIZ - 4);
   }
 else 
   {
     // It is a 2K component.
     
     // Try to find implementationName in the repository
     Simple_Strategy_Factory *ssf =
       ACE_Dynamic_Service<Simple_Strategy_Factory>::instance (implementationName);
     
     // IMPORTANT: The previous call should return 0 if implementationName is not
     // of type Simple_Strategy_Factory but the implementation of
     // ACE_Dynamic_Service<>::instance does NOT use RTTI yet. While this is not
     // fixed, this won't be completely safe.
     if (ssf == 0) // Didn't find a valid implementation in the repository.
       {
         this->spr.appendDCPoutput((char*)DCP_Reply::UNKNOWN_ENTITY, 
                                   ACE_OS::strlen(DCP_Reply::UNKNOWN_ENTITY));
         return;
       }
 
     // The implementationName was found, get its info.
     ACE_OS::strcpy (buf, ssf->localConfigurator ()->info ());
     len = ACE_OS::strlen(buf);
   }
 
 if (len > 0)
   {
     this->spr.appendDCPoutput(buf, len);
   }
 else 
   {
     this->spr.appendDCPoutput((char*)DCP_Reply::EMPTY, 
                               ACE_OS::strlen(DCP_Reply::EMPTY));
   }
}//get_impl_info()



////////////////////////////////////////////////////////////////////////////
// Asks the dynamic service config to load the file <implementation_name> and
// register as service <category_name>. 
////////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::load_implementation (char *request, char *end_of_request)
{
  ACE_DEBUG((LM_DEBUG,"'Load implementation' called\n"));
  char *categoryName;
  char *implementationName;
  char *parameters = 0;
  
  this->get_params (request, &categoryName, &implementationName);
  
  if(implementationName)
    parameters = implementationName + ACE_OS::strlen(implementationName);
  
  if (categoryName == 0 || implementationName == 0)
    {
      this->spr.appendDCPoutput((char*)DCP_Reply::MALFORMED_REQUEST, 
				ACE_OS::strlen(DCP_Reply::MALFORMED_REQUEST));
      return;
    }
  
  char pathname[MAXPATHLEN]; 
  
  pers_rep_->build_pathname (pathname, categoryName);
  
  if (parameters >= end_of_request)
    parameters = 0;
  

  int version = Dynamic_Service_Config::instance()->install (pathname, 
                                                 implementationName, 
                                                 categoryName, parameters); 
  if (version > 0)
    {
      char temp[10];
          
      ACE_OS::sprintf(temp, "%d", version);
      this->spr.appendDCPoutput (temp, ACE_OS::strlen(temp));
    }
  else
    {
      this->spr.appendDCPoutput ((char *)DCP_Reply::NACK, ACE_OS::strlen (DCP_Reply::NACK));
    }
}// load_implementation()


////////////////////////////////////////////////////////////////////////////
// Attaches a loaded implementation to the given component hook.
////////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::hook_implementation (char *request)
{
  ACE_DEBUG((LM_DEBUG,"'Hook implementation' called with argument %s\n",request));
  char *implementationName;
  char *componentName;
  char *hookName;
  this->get_params (request, &implementationName, &componentName, &hookName);
  
  if (implementationName == 0 || componentName == 0 || hookName == 0)
    {
      this->spr.appendDCPoutput((char*)DCP_Reply::MALFORMED_REQUEST, 
				ACE_OS::strlen(DCP_Reply::MALFORMED_REQUEST));
      return;
    }
  
  this->AckOrNack (Dynamic_Service_Config::instance()->hook (implementationName, hookName, componentName));
  
}// hook_implementation ()


///////////////////////////////////////////////////////////////////////////
// Suspends the execution of a loaded implementation
///////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::suspend_implementation  (char *request)
{  
  ACE_DEBUG((LM_DEBUG,"'Suspend implementation' called with argument %s\n",request));
  char *loadedImplementation;
  
  this->get_params (request, &loadedImplementation);
  
  if (loadedImplementation == 0)
    {
      this->spr.appendDCPoutput((char*)DCP_Reply::MALFORMED_REQUEST, 
				ACE_OS::strlen(DCP_Reply::MALFORMED_REQUEST));
      return;
    }
  
  this->AckOrNack (Dynamic_Service_Config::instance()->suspend (loadedImplementation)); 
}// suspend_implementation()


///////////////////////////////////////////////////////////////////////////
// Resumes a previously-suspended loaded Implementaion.
///////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::resume_implementation  (char *request)
{  
  ACE_DEBUG((LM_DEBUG,"'Resume implementation' called with argument %s\n", request));
  char *loadedImplementation;
  
  this->get_params (request, &loadedImplementation);
  
  if (loadedImplementation == 0)
    {
      this->spr.appendDCPoutput((char*)DCP_Reply::MALFORMED_REQUEST, 
				ACE_OS::strlen(DCP_Reply::MALFORMED_REQUEST));
      return;
    }
  
  this->AckOrNack (Dynamic_Service_Config::instance()->resume (loadedImplementation)); 
}// resume_implementation()

/////////////////////////////////////////////////////////////////////////////
// Removes a loaded implementation from the in-memory ACE service repository.
/////////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::remove_implementation (char *request)
{  
  ACE_DEBUG((LM_DEBUG,"'Remove service' called with argument %s\n", request));
  char *implementationName;
  
  this->get_params (request, &implementationName);

  if (implementationName == 0)
    {
      this->spr.appendDCPoutput((char*)DCP_Reply::MALFORMED_REQUEST, 
				ACE_OS::strlen(DCP_Reply::MALFORMED_REQUEST));
      return;
    }

  int ret = Dynamic_Service_Config::instance()->remove (implementationName); 
  if (ret == 1)
   {
     this->spr.appendDCPoutput((char*)DCP_Reply::DEFERRED, 
                               ACE_OS::strlen(DCP_Reply::DEFERRED));
     return;
   }
  else
   {          
     this->AckOrNack (ret);
   }
}// remove_implementation()



///////////////////////////////////////////////////////////////////////////
// Sends a configuration message to a loaded implementation.
///////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::configure_implementation (char *request)
{  
 char *implementationName;
 char *messageString = 0;
 
 this->get_params (request, &implementationName);

 if(implementationName)
   messageString = implementationName + ACE_OS::strlen(implementationName)+1;

 if (implementationName == 0 || messageString == 0)
   {
      this->spr.appendDCPoutput((char*)DCP_Reply::MALFORMED_REQUEST, 
				ACE_OS::strlen(DCP_Reply::MALFORMED_REQUEST));
      return;
   }
 
 // Now, an ugly hack to recognize ACE Services that are not 2K components: 2K
 // components must have the ":" to separate the category name from the version
 // number. If it doesn't have the ":", we assume it's not a 2K component.
 if (!strchr(implementationName, ':')) 
   {
     // Not a 2K component.
     const char *const msg = "configuration of non-2K component not implemented yet\r\n\r\n";
     this->spr.appendDCPoutput (msg, ACE_OS::strlen (msg));
     return;
   }
 else 
   {
     // It is a 2K component.
     // Try to find implementationName in the repository
     Simple_Strategy_Factory *ssf =
       ACE_Dynamic_Service<Simple_Strategy_Factory>::instance (implementationName);
     
     // IMPORTANT: The previous call should return 0 if implementationName is not
     // of type Simple_Strategy_Factory but the implementation of
     // ACE_Dynamic_Service<>::instance does NOT use RTTI yet. While this is not
     // fixed, this won't be completely safe.
     if (ssf == 0) // Didn't find a valid implementation in the repository.
       {
         this->spr.appendDCPoutput((char*)DCP_Reply::UNKNOWN_ENTITY, 
                                   ACE_OS::strlen(DCP_Reply::UNKNOWN_ENTITY));
         return;
       }
 
     // The implementationName was found, sends it the messageString
     AckOrNack (ssf->configure (messageString));
   }
}//configure_implementation()



///////////////////////////////////////////////////////////////////////////
// Uploads a category implementation to the local persistent repository.
///////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::upload_implementation (char *request)
{  
  ACE_DEBUG((LM_DEBUG,"'Upload implementation' called\n"));
  long len = -1;
  char *categoryName;
  char *implementationName;
  char *length;
  char *code;

  this->get_params (request, &categoryName, &implementationName, &length);
  
  ACE_DEBUG((LM_DEBUG,"The implementation is %s bytes long\n", length));

  if (length)
    len = ACE_OS::strtol(length, 0, 10);
  
  if (categoryName == 0 || implementationName == 0 || len <= 0)
    {
      this->spr.appendDCPoutput((char*)DCP_Reply::MALFORMED_REQUEST, 
				ACE_OS::strlen(DCP_Reply::MALFORMED_REQUEST));
      return;
    }
  
  // locates the beginning of the binary code
  code = length + ACE_OS::strlen(length) +1;

  this->AckOrNack (pers_rep_->create_entry_from_buffer(code,
						       categoryName, 
						       implementationName, 
						       len));
}// upload_implementation ()


////////////////////////////////////////////////////////////////////////////
// Removes an implementations from the local persistent repository.
////////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::delete_implementation (char *request)
{  
  ACE_DEBUG((LM_DEBUG,"'Delete implementation' called\n"));
  char *categoryName;
  char *implementationName;
  this->get_params (request, &categoryName, &implementationName);
  
  if (categoryName == 0 || implementationName == 0)
    {
      this->spr.appendDCPoutput((char*)DCP_Reply::MALFORMED_REQUEST, 
				ACE_OS::strlen(DCP_Reply::MALFORMED_REQUEST));
      return;
    }

  this->AckOrNack (pers_rep_-> delete_entry(categoryName, implementationName));

}// delete_implementation ()

////////////////////////////////////////////////////////////////////////////
// Restarts the ORB processing the configuration file again.
////////////////////////////////////////////////////////////////////////////
void 
SmartPacket_Broker::restart ()
{  
  ACE_DEBUG((LM_DEBUG,"'Restart' called\n"));
  this->reconfigure_services ();
  this->AckOrNack (1);
}// delete_implementation ()


////////////////////////////////////////////////////////////////////////////
// Sends an ACK if value >= 0 and sends a NACK otherwise.
////////////////////////////////////////////////////////////////////////////
void
SmartPacket_Broker::AckOrNack(int value)
{
  ACE_DEBUG((LM_DEBUG,"'AckOrNack' called with paramenter %d\n", value));
  if (value >= 0)
    {
      this->spr.appendDCPoutput((char*)DCP_Reply::ACK, 
				ACE_OS::strlen(DCP_Reply::ACK));
    }
  else
    {
      this->spr.appendDCPoutput((char*)DCP_Reply::NACK, 
				ACE_OS::strlen(DCP_Reply::NACK));
    }
}// AckOrNack
