/* -*- C++ -*- */
/******************************************************************************
*                                                                             *
*  File: SmartPacket_Broker.h                                                 *
*                                                                             *
*  Description: A Dynamic Service Config Broker that implements SmartPackets, *
*                                                                             *
*  Project: 2K - dynamically configurable ORB - SmartPackets                  *
*  Author: Arun K Viswanathan (akviswan@uiuc.edu)                             *
*                                                                             *
*******************************************************************************
   Revision Control Information
   $Revision: 1.3 $
   $Log: SmartPacket_Broker.h,v $
   Revision 1.3  2000/09/30 18:17:39  yamane
   Patches for Windows

   Revision 1.2  1999/11/22 16:20:55  f-kon
   bug fixes and performance optimizations

   Revision 1.1.1.1  1999/10/12 00:43:30  f-kon
   Version compatible with TAO 1.0

   Revision 1.3  1999/04/23 22:34:24  f-kon
   A new, more flexible, component encapsulating the JVM

   Revision 1.2  1999/04/07 20:09:08  f-kon
   added new DCP commands, upgrading to version 1.1

   Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
   This new 2K source tree contains the most up-to-date version of ACE and TAO
   and including the dynamicTAO code as well as some 2K tests and preliminary
   versions of some of its high-level systems.


******************************************************************************/

#if !defined (SMARTPACKET_BROKER_H)
#define SMARTPACKET_BROKER_H

#include "Net_Broker.h"
#include "SmartPacket.h"
#include "SmartPacketReply.h"
#include "Set.h"
#include "ace/SOCK_Connector.h"
#include "ace/SOCK_Stream.h"


// FIX: All this defines and static variables should be inside the class SmartPacketBroker!!!
//      Global variables should be against the law :-)

#define MAX_STREAMS 32
// Note: PKT_LEN_SIZE is hard-coded to 10 in two places in SmartPacket_Broker.cpp.
#define PKT_LEN_SIZE 10 

static int sequenceCount = 1; // sequence number for this broker
static int pktLen = 0;        // smart packet length
static int pktLenRead = 0;    // has packet lenght been read?

class ACE_2K_Export SmartPacket_Broker : public Net_Broker
{

public:
  ACE_INLINE SmartPacket_Broker (ACE_Reactor *reactor) 
    : Net_Broker (reactor), request_index_(0) {}

  //  int initialize (Dynamic_Service_Config *dsc); use this in the future
  int initialize ();

  // = Dynamic linking hooks.
  int info (char **info_string, size_t length) const;

  ACE_ALLOC_HOOK_DECLARE; 
  // Declare the dynamic allocation hooks.

private:
  int handle_request ();

  void list_loaded_implementations ();

  void help();
  void quit (int sendACK);
  int  list_categories();
  void list_implementations (char *request);
  void get_impl_info (char *request);
  void list_domain_components ();
  void list_hooks (char *request);
  void get_comp_info (char *request);
  void get_hooked_comp (char *request);

  void add_hook (char *request);
  void remove_hook (char *request);
  void load_implementation (char *request, char *end_of_request);
  void hook_implementation (char *request);
  void suspend_implementation (char *request);
  void resume_implementation (char *request);
  void remove_implementation (char *request);
  void configure_implementation (char *request);
  void upload_implementation (char *request);
  void delete_implementation (char *request);

  int get_params(char * request, char **arg1, char **arg2 = 0, 
		 char **arg3 = 0, char **arg4 = 0);

  void restart (void);
  void AckOrNack(int value);

  enum { TIMEOUT = 10, BUFSIZE_ = 512*1024 }; // MUST FIX THIS!!!!
  char buf_[BUFSIZE_];  
  
  char request_[BUFSIZE_];
  int request_index_;

  SmartPacket sp;                    // the incoming smart packet
  SmartPacketReply spr;              // smart packet replies

  ACE_SOCK_Stream streams[MAX_STREAMS];      // outgoing streams
  ACE_SOCK_Connector connector[MAX_STREAMS]; // outgoing connectors
  char status[MAX_STREAMS];                  // status of outgoing stream

  HistorySet history;
    
};

#endif /* SMARTPACKET_BROKER_H */

