#if !defined (VERSIONS_LIST_H)
#define VERSIONS_LIST_H

#include "ace/Containers.h"

class Version_Record
{
	friend class ACE_Double_Linked_List<Version_Record>;
	friend class ACE_Double_Linked_List_Iterator<Version_Record>;
	// Add friend declaration to reflect ACE change
	friend class ACE_Double_Linked_List_Iterator_Base<Version_Record>;
	friend class Dynamic_Service_Config;

public:
  ACE_INLINE Version_Record (const char *name=0, int lv=0)
  {
	version_       = lv;
	category_name_ = 0;
	if(name != 0) 
          category_name_ = ACE_OS::strdup(name);
  }

  ACE_INLINE ~Version_Record()
  {
    if(category_name_!=0) 
      ACE_OS::free (category_name_);
  }

  char *category_name() { return category_name_; }

  int &version() { return version_; }
  
protected:
  Version_Record *next_;
  Version_Record *prev_;

private:
  	char *category_name_;
	int version_;
};


typedef ACE_Double_Linked_List<Version_Record> Versions_List;



class Implementation_Record
{
	friend class ACE_Double_Linked_List<Implementation_Record>;
	friend class ACE_Double_Linked_List_Iterator<Implementation_Record>;

	// Add friend declaration to reflect ACE change
	friend class ACE_Double_Linked_List_Iterator_Base<Implementation_Record>;

	friend class Dynamic_Service_Config;

public:
  ACE_INLINE Implementation_Record (const char *impname = 0, const char *catname = 0, int version = 0)
  {
	implementation_name_=0;
	category_name_=0;

	if(impname!=0)
	{
		implementation_name_ = ACE_OS::strdup (impname);
	}

	if(catname!=0) 
	{
		category_name_ = ACE_OS::strdup (catname);
	}

        version_ = version;
  }

  ACE_INLINE ~Implementation_Record()
  {
	  if(implementation_name_!=0) ACE_OS::free (implementation_name_);
	  if(category_name_!=0) ACE_OS::free (category_name_);
  }

  char *implementation_name(){return implementation_name_;} // the DLL name without extensions
  char *category_name(){return category_name_;}
  int  version() {return version_;}
  
protected:
  Implementation_Record *next_;
  Implementation_Record *prev_;

private:
  char *implementation_name_;
  char *category_name_;
  int version_;
};


typedef ACE_Double_Linked_List<Implementation_Record> Implementations_List;

#endif



