/******************************************************************************
*                                                                             *
*  File: TAO_Configurator.cpp                                                 *
*                                                                             *
*  Description: A subclass of SimpleConfigurator to manage TAO's              *
*               internal and external dependencies.                           *
*                                                                             *
*  Project: 2K - dynamicTAO                                                   *
*  Library: TAO2k                                                             *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************

   Revision Control Information

   $Revision: 1.1.1.1 $

   $Log: TAO_Configurator.cpp,v $
   Revision 1.1.1.1  1999/10/12 00:43:30  f-kon
   Version compatible with TAO 1.0

   Revision 1.2.1.1  1999/09/22 23:07:12  yamane
   dynamicTAO for TAO1.0

   Revision 1.2  1999/04/13 17:02:59  f-kon
   Fixed a couple of bugs in error messages.

   Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
   This new 2K source tree contains the most up-to-date version of ACE and TAO
   and including the dynamicTAO code as well as some 2K tests and preliminary
   versions of some of its high-level systems.


   Revision 1.2  1999/03/24 22:13:23  mroman
   Last version with New Strategies

   Revision 1.6  1999/02/24 15:07:09  f-kon
   these files were deleted accidentally

   Revision 1.3  1999/01/26 18:22:37  mroman
   DomainConfigurator running on Windows NT

   Revision 1.2  1999/01/11 23:14:01  f-kon
   Upgrade to DCP v1.0 and improvements to the PersistentRepository.

   Revision 1.1  1999/01/07 22:43:40  f-kon
   Added TAOConfigurator and new DCP commands

******************************************************************************/

//#define TAO_BUILD_DLL

#include "TAO_Configurator.h"
#include "tao/default_server.h"
#include "dynamicTAO/src/StrategyFactories/Security/client_security_strategy.h"
#include "dynamicTAO/src/StrategyFactories/Security/server_security_strategy.h"
#include "dynamicTAO/src/StrategyFactories/PerformanceMonitor/performance_monitoring_strategy.h"
#include "dynamicTAO/src/ace/Dynamic_Service_Config.h"


// TAO_Configurator encapsulates TAO-specific knowledge of how to do the
// reconfiguration of Services.

TAO_Configurator::TAO_Configurator(const char *name, Object *implementation)
    : SimpleConfigurator (name, implementation)
{
  addHook ("Acceptor");
  addHook ("Concurrency_Strategy");
  addHook ("Connector");
  addHook ("Demarshalling_Strategy");
  addHook ("Demultiplexing");
  addHook ("Dispatching");
  addHook ("Marshalling_Strategy");
  addHook ("Scheduling_Strategy");
  addHook ("Client_Security_Strategy");
  addHook ("Server_Security_Strategy");
  addHook ("Performance_Monitoring_Strategy");
  info ("A CORBA-compliant reflective ORB supporting runtime reconfiguration");
}// contructor


////////////////////////////////////////////////////////////////////////////////
//
// If the given concurrency strategy is different from the one currently in
// use, sets it as the new concurrency strategies in the server factory and in
// the acceptor.
// Returns 0 in case of success and -1 otherwise.
//
////////////////////////////////////////////////////////////////////////////////
int TAO_Configurator::new_concurrency_strategy (Simple_Strategy_Factory *ssf,
                                                  const char *strategyName)
{
  int activate_server_connections = (int) ssf->strategy();
 
  if(activate_server_connections == server_factory()->activate_server_connections())
    {
        ACE_DEBUG ((LM_ERROR, "(%P|%t) Switch to the same strategy.\n"));
        return 0; // We're already using this strategy. Nothing to be done.
    }

  ((TAO_Default_Server_Strategy_Factory *)server_factory ())
    ->activate_server_connections (activate_server_connections);

  // How can I change the current thread policy?
  
#if 0
  ACE_Concurrency_Strategy<TAO_Server_Connection_Handler> *cs=
    (ACE_Concurrency_Strategy<TAO_Server_Connection_Handler>*) ssf->strategy();
  
  if(cs == server_factory()->concurrency_strategy())
    {
        ACE_DEBUG ((LM_ERROR, "(%P|%t) Switch to the same strategy.\n"));
        return 0; // We're already using this strategy. Nothing to be done.
    }
  
  // We should probably take some action in order to delete the old strategy
  // if it's going to be replaced. Probably decrement a reference counter.
  
  if(ssf->open() == -1)
    {
      ACE_DEBUG ((LM_ERROR, 
                  "(%P|%t) Could not open new Concurrency Strategy \"%s\".\n",
                  strategyName));
      return -1; 
    }

  // Registers the new strategy with the server_factory.
  // This will only affect Acceptors created from now on.
  ((TAO_Default_Server_Strategy_Factory *)server_factory ())
    ->concurrency_strategy (cs);

  // Locates the acceptor the ORB is currently using 
  ACE_Strategy_Acceptor<TAO_Server_Connection_Handler, TAO_SOCK_ACCEPTOR> *asa = 
    (ACE_Strategy_Acceptor<TAO_Server_Connection_Handler, TAO_SOCK_ACCEPTOR> *)
        TAO_ORB_Core_instance()->acceptor()->acceptor();
  // and change its concurrency strategy.
  asa->concurrency_strategy(cs);
  
  // log strategy change
  ACE_DEBUG ((LM_DEBUG, "(%P|%t) Switched Concurrency Strategy to \"%s\".\n",
              strategyName));
  return 0;
#endif
}//new_concurrency_strategy


int TAO_Configurator::new_client_security_strategy (Simple_Strategy_Factory *ssf,
                                             const char *strategyName)
{
   Client_Security_Strategy *css = (Client_Security_Strategy *) ssf->strategy();

   if(css == ((Client_Security_Strategy*)(Dynamic_Service_Config::instance()->domainConfigurator()->getHookedComponent("TAO")->
	   getHookedComponent("Client_Security_Strategy")))
	)
   {
     ACE_OS::printf("It seems that we are using the same existing strategy\n");
       return 0; // We're already using this strategy. Nothing to be done.
   }

//   // We should probably take some action in order to delete the old strategy
//   // if it's going to be replaced.
   ACE_OS::printf("New security: Before open\n");
   if(ssf->open() == -1)
   {
       ACE_DEBUG ((LM_ERROR, 
                   "(%P|%t) Could not open new Client Security Strategy \"%s\".\n",
                   strategyName));
       return -1; 
   }

   // registers the strategy in the TAOConfigurator
//   Dynamic_Service_Config::domainConfigurator()->getHookedComponent("TAO")
//	   ->hook("Client_Security_Strategy",ssf->configurator());

   ACE_DEBUG ((LM_DEBUG, "(%P|%t) Switched Security Strategy to \"%s\".\n", strategyName));
  return 1;
}//new_security_strategy



int TAO_Configurator::new_server_security_strategy (Simple_Strategy_Factory *ssf,
                                             const char *strategyName)
{
   Server_Security_Strategy *css = (Server_Security_Strategy *) ssf->strategy();

   if(css == ((Server_Security_Strategy*)(Dynamic_Service_Config::instance()->domainConfigurator()->getHookedComponent("TAO")->
	   getHookedComponent("Server_Security_Strategy")))
	)
   {
     ACE_OS::printf("It seems that we are using the same existing strategy\n");
       return 0; // We're already using this strategy. Nothing to be done.
   }

//   // We should probably take some action in order to delete the old strategy
//   // if it's going to be replaced.
   ACE_OS::printf("New security: Before open\n");
   if(ssf->open() == -1)
   {
       ACE_DEBUG ((LM_ERROR, 
                   "(%P|%t) Could not open new Server Security Strategy \"%s\".\n",
                   strategyName));
       return -1; 
   }

   ACE_DEBUG ((LM_DEBUG, "(%P|%t) Switched Security Strategy to \"%s\".\n", strategyName));
  return 1;
}//new_server_security_strategy


int TAO_Configurator::new_performance_monitoring_strategy (Simple_Strategy_Factory *ssf,
                                             const char *strategyName)
{
   Performance_Monitoring_Strategy *css = (Performance_Monitoring_Strategy *) ssf->strategy();

   if(css == ((Performance_Monitoring_Strategy*)
              (Dynamic_Service_Config::instance()->domainConfigurator()->getHookedComponent("TAO")->
               getHookedComponent("Performance_Monitoring_Strategy")))
	)
   {
     ACE_OS::printf("It seems that we are using the same existing strategy\n");
       return 0; // We're already using this strategy. Nothing to be done.
   }

//   // We should probably take some action in order to delete the old strategy
//   // if it's going to be replaced.
   ACE_OS::printf("New Monitoring: Before open\n");
   if(ssf->open() == -1)
   {
       ACE_DEBUG ((LM_ERROR, 
                   "(%P|%t) Could not open new Performance Monitoring Strategy \"%s\".\n",
                   strategyName));
       return -1; 
   }

   ACE_DEBUG ((LM_DEBUG, "(%P|%t) Switched Monitoring Strategy to \"%s\".\n", strategyName));
   return 1;
}//new_performance_monitoring_strategy


////////////////////////////////////////////////////////////////////////////////
// Installs a strategy in the given hook. 
////////////////////////////////////////////////////////////////////////////////
int
TAO_Configurator::hook (const char *hookName, ComponentConfigurator *strategy)
{
  unhook (hookName);
  if (SimpleConfigurator::hook(hookName, strategy) == -1)
    {
      if (hookName)
        ACE_DEBUG ((LM_ERROR, 
                    "(%P|%t) Could not attach new strategy to hook %s.\n", 
                    hookName));
      return -1;
    }
  
  Simple_Strategy_Factory *ssf = (Simple_Strategy_Factory*) 
    ((SimpleConfigurator*)strategy)->implementation ();
  const char *strategyName = strategy->name();
 
  // Connect it to the hook.

  if (ACE_OS::strcmp (hookName, "Concurrency_Strategy") == 0)
    {
      return this->new_concurrency_strategy (ssf, strategyName);
    }
  else if (ACE_OS::strcmp (hookName, "Client_Security_Strategy") == 0)
    {
	  return this->new_client_security_strategy (ssf, strategyName);
    }
  else if (ACE_OS::strcmp (hookName, "Server_Security_Strategy") == 0)
    {
	  return this->new_server_security_strategy (ssf, strategyName);
    }
  else if (ACE_OS::strcmp (hookName, "Performance_Monitoring_Strategy") == 0)
    {
	  return this->new_performance_monitoring_strategy (ssf, strategyName);
    }
  // test for other hooks here.
  else 
    {
      if (hookName)
        ACE_DEBUG ((LM_ERROR, 
                    "(%P|%t) Could not attach strategy to hook %s: unknown hook name.\n", 
                    hookName));
      return -1; 
    }

}// install ()



