// This may look like C, but it's really -*- C++ -*-
/******************************************************************************
*                                                                             *
*  File: TAO_Configurator.h                                                   *
*                                                                             *
*  Description: A subclass of SimpleConfigurator to manage TAO's              *
*               internal and external dependencies.                           *
*                                                                             *
*  Project: 2K - dynamicTAO                                                   *
*  Library: TAO2k                                                             *
*  Author: Fabio Kon (f-kon@cs.uiuc.edu)                                      *
*                                                                             *
*******************************************************************************

   Revision Control Information

   $Revision: 1.1.1.3 $

   $Log: TAO_Configurator.h,v $
   Revision 1.1.1.3  2000/09/30 19:55:05  yamane
   Patches for Windows

   Revision 1.1.1.2  2000/09/30 18:19:52  yamane
   Patches for Windows

   Revision 1.1.1.1  1999/10/12 00:43:30  f-kon
   Version compatible with TAO 1.0

   Revision 1.1.1.1  1999/04/01 18:16:36  f-kon
   This new 2K source tree contains the most up-to-date version of ACE and TAO
   and including the dynamicTAO code as well as some 2K tests and preliminary
   versions of some of its high-level systems.


   Revision 1.2  1999/03/24 22:13:24  mroman
   Last version with New Strategies

   Revision 1.5  1999/02/24 15:07:10  f-kon
   these files were deleted accidentally

   Revision 1.3  1999/01/26 18:22:38  mroman
   DomainConfigurator running on Windows NT

   Revision 1.3  1999/01/25 8:40:00	 mroman
   Added code to hook security strategies.

   Revision 1.2  1999/01/11 23:14:02  f-kon
   Upgrade to DCP v1.0 and improvements to the PersistentRepository.

   Revision 1.1  1999/01/07 22:43:40  f-kon
   Added TAOConfigurator and new DCP commands

******************************************************************************/

#if !defined (TAO_CONFIGURATOR_H)
#define TAO_CONFIGURATOR_H

#include "tao/corba.h" // This includes too much stuff. FIX IT!
#include "dynamicTAO/src/ComponentConfigurator/SimpleConfigurator.h"
#include "dynamicTAO/src/ace/Simple_Strategy_Factory.h"
#include "tao2k_export.h"

class TAO_2K_Export TAO_Configurator : public SimpleConfigurator
  // Encapsulates TAO-specific knowledge of how to do the 
  // reconfiguration of Services.
{
public:
  // = Initialization and termination methods.
  TAO_Configurator (const char *name, Object *implementation);

  int hook (const char *hookName, ComponentConfigurator *strategy);
  // Installs a strategy into a hook in the ORB.

  int strategy_being_used(const char *strategyName);
  // Return !=0 if the strategy called strategyName is currently attached to
  // some hook.

  // we might want to override unhook in the future for garbage collection.

private:
  int new_concurrency_strategy (Simple_Strategy_Factory *ssf,
                                const char *strategyName);


  int new_client_security_strategy (Simple_Strategy_Factory *ssf,
                                const char *strategyName);


  int new_server_security_strategy (Simple_Strategy_Factory *ssf,
                                const char *strategyName);

  int new_performance_monitoring_strategy (Simple_Strategy_Factory *ssf,
                                const char *strategyName);
  
  TAO_Server_Strategy_Factory *server_factory(void);
};

#include "TAO_Configurator.i" // is this they way that TAO uses to include .i files?


#endif /* TAO_CONFIGURATOR_H */
